@@ -29,11 +29,21 @@ STATIC_ASSERT(sizeof(long double)==8);
 
 // the following does both 32 and 64 bit architectures
 
-STATIC_ASSERT(sizeof(void *)==sizeof(long int));
+// Note that there are architectures (e.g., x32) with
+// 64-bit long int and 32-bit pointers. However,
+// a long int should always be able to hold a pointer.
+STATIC_ASSERT(sizeof(void *)<=sizeof(long int));
+
 STATIC_ASSERT(sizeof(int)==4);
 STATIC_ASSERT(sizeof(long int)==4 || sizeof(long int)==8);
 STATIC_ASSERT(sizeof(long long int)==8);
+
+#ifdef __CYGWIN__
+STATIC_ASSERT(sizeof(wchar_t)==2);
+#else
 STATIC_ASSERT(sizeof(wchar_t)==4);
+#endif
+
 STATIC_ASSERT(sizeof(float)==4);
 STATIC_ASSERT(sizeof(double)==8);
 STATIC_ASSERT(sizeof(long double)>=sizeof(double));
