@@ -6,8 +6,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <std_types.h>
-#include <std_expr.h>
+#include <util/std_types.h>
+#include <util/std_expr.h>
 
 #include "anonymous_member.h"
 
