@@ -9,10 +9,10 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_ANSI_C_CONVERT_H
 #define CPROVER_ANSI_C_CONVERT_H
 
-#include <message_stream.h>
-#include <std_code.h>
-#include <hash_cont.h>
-#include <string_hash.h>
+#include <util/message_stream.h>
+#include <util/std_code.h>
+#include <util/hash_cont.h>
+#include <util/string_hash.h>
 
 #include "c_storage_spec.h"
 #include "ansi_c_parse_tree.h"
