@@ -9,11 +9,11 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_ANSI_C_CONVERT_TYPE_H
 #define CPROVER_ANSI_C_CONVERT_TYPE_H
 
-#include <message_stream.h>
+#include <util/message_stream.h>
 
-#include <ansi-c/c_types.h>
-#include <ansi-c/c_qualifiers.h>
-#include <ansi-c/c_storage_spec.h>
+#include "c_types.h"
+#include "c_qualifiers.h"
+#include "c_storage_spec.h"
 
 class ansi_c_convert_typet:public message_streamt
 {
@@ -26,8 +26,9 @@ public:
   // extensions
   unsigned int8_cnt, int16_cnt, int32_cnt, int64_cnt,
            ptr32_cnt, ptr64_cnt,
-           gcc_float128_cnt, gcc_int128_cnt, bv_cnt, bv_width;
-  bool gcc_mode_QI, gcc_mode_HI, gcc_mode_SI, gcc_mode_DI;
+           gcc_float128_cnt, gcc_int128_cnt, bv_cnt, bv_width,
+           floatbv_cnt, mantissa_width;
+  bool gcc_mode_QI, gcc_mode_HI, gcc_mode_SI, gcc_mode_DI, gcc_mode_TI;
            
   bool packed, aligned;
   exprt vector_size, alignment;
@@ -56,11 +57,11 @@ public:
     long_cnt=double_cnt=float_cnt=c_bool_cnt=proper_bool_cnt=complex_cnt=
     int8_cnt=int16_cnt=int32_cnt=int64_cnt=
     ptr32_cnt=ptr64_cnt=
-    gcc_float128_cnt=gcc_int128_cnt=bv_cnt=0;
+    gcc_float128_cnt=gcc_int128_cnt=bv_cnt=floatbv_cnt=0;
     vector_size.make_nil();
     alignment.make_nil();
     bv_width=0;
-    gcc_mode_QI=gcc_mode_HI=gcc_mode_SI=gcc_mode_DI=false;
+    gcc_mode_QI=gcc_mode_HI=gcc_mode_SI=gcc_mode_DI=gcc_mode_TI=false;
     
     packed=aligned=false;
 
