@@ -9,9 +9,9 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_ANSI_C_DECLARATION_H
 #define CPROVER_ANSI_C_DECLARATION_H
 
-#include <assert.h>
+#include <cassert>
 
-#include <symbol.h>
+#include <util/symbol.h>
 
 class ansi_c_declarationt:public exprt
 {
@@ -88,14 +88,14 @@ public:
     set(ID_is_static, is_static);
   }
   
-  bool get_is_argument() const
+  bool get_is_parameter() const
   {
-    return get_bool(ID_is_argument);
+    return get_bool(ID_is_parameter);
   }
   
-  void set_is_argument(bool is_argument)
+  void set_is_parameter(bool is_parameter)
   {
-    set(ID_is_argument, is_argument);
+    set(ID_is_parameter, is_parameter);
   }
   
   bool get_is_global() const
