@@ -6,14 +6,13 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <string.h>
-
+#include <cstring>
 #include <sstream>
 #include <fstream>
 
-#include <expr_util.h>
-#include <replace_symbol.h>
-#include <config.h>
+#include <util/expr_util.h>
+#include <util/replace_symbol.h>
+#include <util/config.h>
 
 #include <linking/linking.h>
 #include <linking/remove_internal_symbols.h>
@@ -26,7 +25,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "expr2c.h"
 #include "trans_unit.h"
 #include "c_preprocess.h"
-#include "internal_additions.h"
+#include "ansi_c_internal_additions.h"
 
 /*******************************************************************\
 
@@ -85,9 +84,9 @@ bool ansi_c_languaget::preprocess(
 {
   // stdin?
   if(path=="")
-    return c_preprocess(PREPROCESS_C, instream, outstream, message_handler);
+    return c_preprocess(instream, outstream, message_handler);
 
-  return c_preprocess(PREPROCESS_C, path, outstream, message_handler);  
+  return c_preprocess(path, outstream, message_handler);  
 }
              
 /*******************************************************************\
@@ -127,30 +126,32 @@ bool ansi_c_languaget::parse(
   std::istringstream codestr(code);
 
   ansi_c_parser.clear();
-  ansi_c_parser.filename=ID_built_in;
+  ansi_c_parser.set_file(ID_built_in);
   ansi_c_parser.in=&codestr;
   ansi_c_parser.set_message_handler(message_handler);
   ansi_c_parser.grammar=ansi_c_parsert::LANGUAGE;
+  ansi_c_parser.for_has_scope=config.ansi_c.for_has_scope;
 
   switch(config.ansi_c.mode)
   {
-  case configt::ansi_ct::MODE_CODEWARRIOR:
+  case configt::ansi_ct::MODE_CODEWARRIOR_C_CPP:
     ansi_c_parser.mode=ansi_c_parsert::CW;
     break;
    
-  case configt::ansi_ct::MODE_VISUAL_STUDIO:
+  case configt::ansi_ct::MODE_VISUAL_STUDIO_C_CPP:
     ansi_c_parser.mode=ansi_c_parsert::MSC;
     break;
     
-  case configt::ansi_ct::MODE_ANSI:
+  case configt::ansi_ct::MODE_ANSI_C_CPP:
     ansi_c_parser.mode=ansi_c_parsert::ANSI;
     break;
     
-  case configt::ansi_ct::MODE_GCC:
+  case configt::ansi_ct::MODE_GCC_C:
+  case configt::ansi_ct::MODE_GCC_CPP:
     ansi_c_parser.mode=ansi_c_parsert::GCC;
     break;
     
-  case configt::ansi_ct::MODE_ARM:
+  case configt::ansi_ct::MODE_ARM_C_CPP:
     ansi_c_parser.mode=ansi_c_parsert::ARM;
     break;
     
@@ -164,8 +165,8 @@ bool ansi_c_languaget::parse(
 
   if(!result)
   {
-    ansi_c_parser.line_no=0;
-    ansi_c_parser.filename=path;
+    ansi_c_parser.set_line_no(0);
+    ansi_c_parser.set_file(path);
     ansi_c_parser.in=&i_preprocessed;
     ansi_c_scanner_init();
     result=ansi_c_parser.parse();
@@ -193,21 +194,21 @@ Function: ansi_c_languaget::typecheck
 \*******************************************************************/
 
 bool ansi_c_languaget::typecheck(
-  contextt &context,
+  symbol_tablet &symbol_table,
   const std::string &module,
   message_handlert &message_handler)
 {
   if(ansi_c_convert(parse_tree, module, message_handler))
     return true;
 
-  contextt new_context;
+  symbol_tablet new_symbol_table;
 
-  if(ansi_c_typecheck(parse_tree, new_context, module, message_handler))
+  if(ansi_c_typecheck(parse_tree, new_symbol_table, module, message_handler))
     return true;
 
-  remove_internal_symbols(new_context);
+  remove_internal_symbols(new_symbol_table);
   
-  if(linking(context, new_context, message_handler))
+  if(linking(symbol_table, new_symbol_table, message_handler))
     return true;
     
   return false;
@@ -226,10 +227,10 @@ Function: ansi_c_languaget::final
 \*******************************************************************/
 
 bool ansi_c_languaget::final(
-  contextt &context,
+  symbol_tablet &symbol_table,
   message_handlert &message_handler)
 {
-  if(entry_point(context, "c::main", message_handler)) return true;
+  if(entry_point(symbol_table, "c::main", message_handler)) return true;
   
   return false;
 }
@@ -338,7 +339,7 @@ bool ansi_c_languaget::to_expr(
   // parsing
 
   ansi_c_parser.clear();
-  ansi_c_parser.filename="";
+  ansi_c_parser.set_file(irep_idt());
   ansi_c_parser.in=&i_preprocessed;
   ansi_c_parser.set_message_handler(message_handler);
   ansi_c_parser.grammar=ansi_c_parsert::EXPRESSION;
