@@ -11,7 +11,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 /*! \defgroup gr_ansi_c ANSI-C front-end */
 
-#include <language.h>
+#include <util/language.h>
 
 #include "ansi_c_parse_tree.h"
 
@@ -33,12 +33,12 @@ public:
     message_handlert &message_handler);
              
   virtual bool typecheck(
-    contextt &context,
+    symbol_tablet &symbol_table,
     const std::string &module,
     message_handlert &message_handler);
 
   virtual bool final(
-    contextt &context,
+    symbol_tablet &symbol_table,
     message_handlert &message_handler);
 
   virtual void show_parse(std::ostream &out);
