@@ -6,16 +6,17 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#ifndef CPROVER_SPECC_PARSER_H
-#define CPROVER_SPECC_PARSER_H
+#ifndef CPROVER_ANSI_C_PARSER_H
+#define CPROVER_ANSI_C_PARSER_H
 
-#include <assert.h>
+#include <cassert>
 
-#include <parser.h>
-#include <expr.h>
-#include <hash_cont.h>
-#include <string_hash.h>
-#include <i2string.h>
+#include <util/parser.h>
+#include <util/expr.h>
+#include <util/hash_cont.h>
+#include <util/string_hash.h>
+#include <util/i2string.h>
+#include <util/mp_arith.h>
 
 #include "ansi_c_parse_tree.h"
 
@@ -29,7 +30,9 @@ class ansi_c_parsert:public parsert
 public:
   ansi_c_parse_treet parse_tree;
   
-  ansi_c_parsert():cpp(false)
+  ansi_c_parsert():
+    cpp(false),
+    for_has_scope(false)
   {
   }
   
@@ -43,11 +46,12 @@ public:
     parsert::clear();
     parse_tree.clear();
     
-    // scanner
-    string_literal.clear();
+    // scanner state
     tag_following=false;
     asm_block_following=false;
     parenthesis_counter=0;
+    string_literal.clear();
+    pragma_pack=0;
     
     // setup global scope
     scopes.clear();
@@ -57,10 +61,11 @@ public:
   }
 
   // internal state of the scanner
-  std::string string_literal;
   bool tag_following;
   bool asm_block_following;
   unsigned parenthesis_counter;
+  std::string string_literal;
+  mp_integer pragma_pack;
   
   enum { LANGUAGE, EXPRESSION } grammar;
 
@@ -74,6 +79,9 @@ public:
 
   // recognize C++ keywords  
   bool cpp;
+  
+  // in C99 and upwards, for(;;) has a scope
+  bool for_has_scope;
 
   class identifiert
   {
@@ -93,8 +101,11 @@ public:
     name_mapt name_map;
     
     std::string prefix;
+    irep_idt last_declarator;
     
-    unsigned compound_counter, anon_counter;
+    // for(;;) and { } scopes are numbered
+    unsigned compound_counter;
+    unsigned anon_counter;
     
     scopet():compound_counter(0), anon_counter(0) { }
     
@@ -102,6 +113,7 @@ public:
     {
       name_map.swap(scope.name_map);
       prefix.swap(scope.prefix);
+      last_declarator.swap(scope.last_declarator);
       std::swap(compound_counter, scope.compound_counter);
     }
     
@@ -136,13 +148,14 @@ public:
     irept &declarator,
     const typet &type,
     irept &identifier);
+    
+  typedef enum { TAG, MEMBER, PARAMETER, OTHER } decl_typet;
 
   void new_declaration(
     const irept &type,
     irept &declarator,
     exprt &declaration,
-    bool is_tag=false,
-    bool put_into_scope=true);
+    decl_typet decl_type);
    
   void copy_item(const ansi_c_declarationt &declaration)
   {
