@@ -58,12 +58,12 @@ Function: ansi_c_typecheck
 
 bool ansi_c_typecheck(
   ansi_c_parse_treet &ansi_c_parse_tree,
-  contextt &context,
+  symbol_tablet &symbol_table,
   const std::string &module,
   message_handlert &message_handler)
 {
   ansi_c_typecheckt ansi_c_typecheck(
-    ansi_c_parse_tree, context, module, message_handler);
+    ansi_c_parse_tree, symbol_table, module, message_handler);
   return ansi_c_typecheck.typecheck_main();
 }
 
@@ -84,19 +84,19 @@ bool ansi_c_typecheck(
   message_handlert &message_handler,
   const namespacet &ns)
 {
-  contextt context;
+  symbol_tablet symbol_table;
   ansi_c_parse_treet ansi_c_parse_tree;
 
   ansi_c_typecheckt ansi_c_typecheck(
-    ansi_c_parse_tree, context,
-    ns.get_context(), "", message_handler);
+    ansi_c_parse_tree, symbol_table,
+    ns.get_symbol_table(), "", message_handler);
 
   try
   {
     ansi_c_typecheck.typecheck_expr(expr);
   }
 
-  catch(int e)
+  catch(int)
   {
     ansi_c_typecheck.error();
   }
