@@ -6,25 +6,25 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 
-#ifdef __linux__
-#include <unistd.h>
-#endif
-
-#ifdef __APPLE__
+#if defined(__linux__) || \
+    defined(__FreeBSD_kernel__) || \
+    defined(__GNU__) || \
+    defined(__CYGWIN__) || \
+    defined(__MACH__)
 #include <unistd.h>
 #endif
 
 #include <fstream>
 
-#include <config.h>
-#include <i2string.h>
-#include <message_stream.h>
-#include <tempfile.h>
-#include <unicode.h>
+#include <util/config.h>
+#include <util/i2string.h>
+#include <util/message_stream.h>
+#include <util/tempfile.h>
+#include <util/unicode.h>
 
 #include "c_preprocess.h"
 
@@ -171,7 +171,6 @@ Function: c_preprocess
 \*******************************************************************/
 
 bool c_preprocess(
-  preprocess_modet mode,
   std::istream &instream,
   std::ostream &outstream,
   message_handlert &message_handler)
@@ -185,7 +184,7 @@ bool c_preprocess(
 
   fclose(tmp);
 
-  bool result=c_preprocess(mode, file, outstream, message_handler);
+  bool result=c_preprocess(file, outstream, message_handler);
   
   unlink(file.c_str());
   
@@ -225,34 +224,33 @@ Function: c_preprocess
 
 \*******************************************************************/
 
-bool c_preprocess_codewarrior(preprocess_modet, const std::string &, std::ostream &, message_handlert &);
-bool c_preprocess_arm(preprocess_modet, const std::string &, std::ostream &, message_handlert &);
-bool c_preprocess_gcc(preprocess_modet, const std::string &, std::ostream &, message_handlert &);
-bool c_preprocess_none(preprocess_modet, const std::string &, std::ostream &, message_handlert &);
-bool c_preprocess_visual_studio(preprocess_modet, const std::string &, std::ostream &, message_handlert &);
+bool c_preprocess_codewarrior(const std::string &, std::ostream &, message_handlert &);
+bool c_preprocess_arm(const std::string &, std::ostream &, message_handlert &);
+bool c_preprocess_gcc(const std::string &, std::ostream &, message_handlert &);
+bool c_preprocess_none(const std::string &, std::ostream &, message_handlert &);
+bool c_preprocess_visual_studio(const std::string &, std::ostream &, message_handlert &);
 
 bool c_preprocess(
-  preprocess_modet mode,
   const std::string &path,
   std::ostream &outstream,
   message_handlert &message_handler)
 {
-  switch(config.ansi_c.mode)
+  switch(config.ansi_c.preprocessor)
   {
-  case configt::ansi_ct::MODE_CODEWARRIOR:
-    return c_preprocess_codewarrior(mode, path, outstream, message_handler);
+  case configt::ansi_ct::PP_CODEWARRIOR:
+    return c_preprocess_codewarrior(path, outstream, message_handler);
   
-  case configt::ansi_ct::MODE_GCC:
-    return c_preprocess_gcc(mode, path, outstream, message_handler);
+  case configt::ansi_ct::PP_GCC:
+    return c_preprocess_gcc(path, outstream, message_handler);
   
-  case configt::ansi_ct::MODE_VISUAL_STUDIO:
-    return c_preprocess_visual_studio(mode, path, outstream, message_handler);
+  case configt::ansi_ct::PP_VISUAL_STUDIO:
+    return c_preprocess_visual_studio(path, outstream, message_handler);
   
-  case configt::ansi_ct::MODE_ARM:
-    return c_preprocess_arm(mode, path, outstream, message_handler);
+  case configt::ansi_ct::PP_ARM:
+    return c_preprocess_arm(path, outstream, message_handler);
   
-  default:
-    assert(false);
+  case configt::ansi_ct::NO_PP:
+    return c_preprocess_none(path, outstream, message_handler);
   }
 
   // not reached  
@@ -272,19 +270,13 @@ Function: c_preprocess_visual_studio
 \*******************************************************************/
 
 bool c_preprocess_visual_studio(
-  preprocess_modet mode,
   const std::string &file,
   std::ostream &outstream,
   message_handlert &message_handler)
 {
   // check extension
   if(is_dot_i_file(file))
-    return c_preprocess_none(mode, file, outstream, message_handler);
-
-  #ifndef _WIN32
-  // we fall back to gcc
-  return c_preprocess_gcc(mode, file, outstream, message_handler);
-  #endif
+    return c_preprocess_none(file, outstream, message_handler);
 
   message_streamt message_stream(message_handler);
 
@@ -335,6 +327,12 @@ bool c_preprocess_visual_studio(
         it++)
       command_file << "/I" << shell_quote(*it) << std::endl;
 
+       for(std::list<std::string>::const_iterator
+               it=config.ansi_c.include_files.begin();
+               it!=config.ansi_c.include_files.end();
+               it++)
+         command_file << "/FI" << shell_quote(*it) << std::endl;
+
     // Finally, the file to be preprocessed
     // (this is already in UTF-8).
     command_file << shell_quote(file) << std::endl;
@@ -342,7 +340,7 @@ bool c_preprocess_visual_studio(
   
   std::string tmpi=get_temporary_file("tmp.cl", "");
   
-  std::string command="CL @"+command_file_name;
+  std::string command="CL @\""+command_file_name+"\"";
   command+=" > \""+tmpi+"\"";
   command+=" 2> \""+stderr_file+"\"";
 
@@ -362,9 +360,9 @@ bool c_preprocess_visual_studio(
   }
 
   {
-    char ch;
+    int ch;
     while((ch=fgetc(stream))!=EOF)
-      outstream << ch;
+      outstream << (unsigned char)ch;
   }
 
   fclose(stream);
@@ -453,14 +451,13 @@ Function: c_preprocess_codewarrior
 \*******************************************************************/
 
 bool c_preprocess_codewarrior(
-  preprocess_modet mode,
   const std::string &file,
   std::ostream &outstream,
   message_handlert &message_handler)
 {
   // check extension
   if(is_dot_i_file(file))
-    return c_preprocess_none(mode, file, outstream, message_handler);
+    return c_preprocess_none(file, outstream, message_handler);
 
   // preprocessing
   message_streamt message_stream(message_handler);
@@ -475,19 +472,19 @@ bool c_preprocess_codewarrior(
       it=config.ansi_c.defines.begin();
       it!=config.ansi_c.defines.end();
       it++)
-    command+=" -D'"+*it+"'";
+    command+=" -D"+shell_quote(*it);
 
   for(std::list<std::string>::const_iterator
       it=config.ansi_c.include_paths.begin();
       it!=config.ansi_c.include_paths.end();
       it++)
-    command+=" -I'"+*it+"'";
+    command+=" -I"+shell_quote(*it);
 
   for(std::list<std::string>::const_iterator
       it=config.ansi_c.include_files.begin();
       it!=config.ansi_c.include_files.end();
       it++)
-    command+=" -include '"+*it+"'";
+    command+=" -include "+shell_quote(*it);
 
   for(std::list<std::string>::const_iterator
       it=config.ansi_c.preprocessor_options.begin();
@@ -556,14 +553,13 @@ Function: c_preprocess_gcc
 \*******************************************************************/
 
 bool c_preprocess_gcc(
-  preprocess_modet mode,
   const std::string &file,
   std::ostream &outstream,
   message_handlert &message_handler)
 {
   // check extension
   if(is_dot_i_file(file))
-    return c_preprocess_none(mode, file, outstream, message_handler);
+    return c_preprocess_none(file, outstream, message_handler);
 
   // preprocessing
   message_streamt message_stream(message_handler);
@@ -678,24 +674,16 @@ bool c_preprocess_gcc(
       command+=GCC_DEFINES_32;
     }
   }
-      
+  
   switch(config.ansi_c.os)
   {
   case configt::ansi_ct::OS_LINUX:
-    if(config.ansi_c.arch==configt::ansi_ct::ARCH_I386)
-       command+=" -Di386 -D__i386 -D__i386__";
-    else if(config.ansi_c.arch==configt::ansi_ct::ARCH_X86_64)
-       command+=" -D__LP64__ -D__x86_64 -D__x86_64__ -D_LP64";
     command+=" -Dlinux -D__linux -D__linux__ -D__gnu_linux__";
     command+=" -Dunix -D__unix -D__unix__";
     command+=" -D__USE_UNIX98";
     break;
 
   case configt::ansi_ct::OS_MACOS:
-    if(config.ansi_c.arch==configt::ansi_ct::ARCH_I386)
-      command+=" -Di386 -D__i386 -D__i386__ -D__LITTLE_ENDIAN__";
-    else if(config.ansi_c.arch==configt::ansi_ct::ARCH_POWER)
-      command+=" -D__BIG_ENDIAN__";
     command+=" -D__APPLE__ -D__MACH__";
     // needs to be __APPLE_CPP__ for C++
     command+=" -D__APPLE_CC__";
@@ -754,10 +742,10 @@ bool c_preprocess_gcc(
   int result;
   
   // the following forces the mode
-  switch(mode)
+  switch(config.ansi_c.mode)
   {
-  case PREPROCESS_C: command+=" -x c"; break;
-  case PREPROCESS_CPP: command+=" -x c++"; break;
+  case configt::ansi_ct::MODE_GCC_C: command+=" -x c"; break;
+  case configt::ansi_ct::MODE_GCC_CPP: command+=" -x c++"; break;
   default:;
   }
 
@@ -785,9 +773,9 @@ bool c_preprocess_gcc(
 
   if(stream!=NULL)
   {
-    char ch;
+    int ch;
     while((ch=fgetc(stream))!=EOF)
-      outstream << ch;
+      outstream << (unsigned char)ch;
 
     fclose(stream);
     unlink(tmpi.c_str());
@@ -806,9 +794,9 @@ bool c_preprocess_gcc(
 
   if(stream!=NULL)
   {
-    char ch;
+    int ch;
     while((ch=fgetc(stream))!=EOF)
-      outstream << ch;
+      outstream << (unsigned char)ch;
 
     result=pclose(stream);
   }
@@ -854,14 +842,13 @@ Function: c_preprocess_arm
 \*******************************************************************/
 
 bool c_preprocess_arm(
-  preprocess_modet mode,
   const std::string &file,
   std::ostream &outstream,
   message_handlert &message_handler)
 {
   // check extension
   if(is_dot_i_file(file))
-    return c_preprocess_none(mode, file, outstream, message_handler);
+    return c_preprocess_none(file, outstream, message_handler);
 
   // preprocessing using armcc
   message_streamt message_stream(message_handler);
@@ -908,13 +895,13 @@ bool c_preprocess_arm(
       it=config.ansi_c.defines.begin();
       it!=config.ansi_c.defines.end();
       it++)
-    command+=" \"-D"+*it+"\"";
+    command+=" "+shell_quote("-D"+*it);
 
   for(std::list<std::string>::const_iterator
       it=config.ansi_c.include_paths.begin();
       it!=config.ansi_c.include_paths.end();
       it++)
-    command+=" \"-I"+*it+"\"";
+    command+=" "+shell_quote("-I"+*it);
 
   int result;
 
@@ -932,9 +919,9 @@ bool c_preprocess_arm(
 
   if(stream!=NULL)
   {
-    char ch;
+    int ch;
     while((ch=fgetc(stream))!=EOF)
-      outstream << ch;
+      outstream << (unsigned char)ch;
 
     fclose(stream);
     unlink(tmpi.c_str());
@@ -954,9 +941,9 @@ bool c_preprocess_arm(
 
   if(stream!=NULL)
   {
-    char ch;
+    int ch;
     while((ch=fgetc(stream))!=EOF)
-      outstream << ch;
+      outstream << (unsigned char)ch;
 
     result=pclose(stream);
   }
@@ -1003,7 +990,6 @@ Function: c_preprocess_none
 \*******************************************************************/
 
 bool c_preprocess_none(
-  preprocess_modet mode,
   const std::string &file,
   std::ostream &outstream,
   message_handlert &message_handler)
@@ -1021,7 +1007,7 @@ bool c_preprocess_none(
     return true;
   }
   
-  if(config.ansi_c.mode==configt::ansi_ct::MODE_CODEWARRIOR)
+  if(config.ansi_c.mode==configt::ansi_ct::MODE_CODEWARRIOR_C_CPP)
   {
     // special treatment for "/* #line"
     postprocess_codewarrior(infile, outstream);
