@@ -9,20 +9,17 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_C_PREPROCESS_H
 #define CPROVER_C_PREPROCESS_H
 
-#include <iostream>
+#include <iosfwd>
 #include <string>
-#include <message.h>
 
-typedef enum { PREPROCESS_AUTO, PREPROCESS_C, PREPROCESS_CPP } preprocess_modet;
+#include <util/message.h>
 
 bool c_preprocess(
-  preprocess_modet mode,
   const std::string &path,
   std::ostream &outstream,
   message_handlert &message_handler);
  
 bool c_preprocess(
-  preprocess_modet mode,
   std::istream &instream,
   std::ostream &outstream,
   message_handlert &message_handler);
