@@ -9,9 +9,9 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_ANSI_C_C_QUALIFIERS_H
 #define CPROVER_ANSI_C_C_QUALIFIERS_H
 
-#include <iostream>
+#include <iosfwd>
 
-#include <expr.h>
+#include <util/expr.h>
 
 class c_qualifierst
 {
