@@ -8,11 +8,11 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <cassert>
 
-#include <config.h>
-#include <expr_util.h>
-#include <std_expr.h>
-#include <base_type.h>
-#include <symbol.h>
+#include <util/config.h>
+#include <util/expr_util.h>
+#include <util/std_expr.h>
+#include <util/base_type.h>
+#include <util/symbol.h>
 
 #include "c_typecast.h"
 #include "c_types.h"
@@ -219,6 +219,22 @@ bool check_c_implicit_typecast(
     if(dest_type.id()==ID_fixedbv) return false;
     if(dest_type.id()==ID_complex) return false;
   }
+  else if(src_type_id==ID_complex)
+  {
+    if(dest_type.id()==ID_complex)
+      return check_c_implicit_typecast(src_type.subtype(), dest_type.subtype());
+    else
+    {
+      // 6.3.1.7, par 2:
+
+      // When a value of complex type is converted to a real type, the
+      // imaginary part of the complex value is discarded and the value of the
+      // real part is converted according to the conversion rules for the
+      // corresponding real type.
+      
+      return check_c_implicit_typecast(src_type.subtype(), dest_type);
+    }
+  }
   else if(src_type_id==ID_array ||
           src_type_id==ID_pointer)
   {
@@ -421,12 +437,12 @@ void c_typecastt::implicit_typecast_arithmetic(
   case UCHAR:      assert(false); // should always be promoted to int
   case SHORT:      assert(false); // should always be promoted to int
   case USHORT:     assert(false); // should always be promoted to int
-  case INT:        new_type=int_type(); break;
-  case UINT:       new_type=uint_type(); break;
-  case LONG:       new_type=long_int_type(); break;
-  case ULONG:      new_type=long_uint_type(); break;
-  case LONGLONG:   new_type=long_long_int_type(); break;
-  case ULONGLONG:  new_type=long_long_uint_type(); break;
+  case INT:        new_type=signed_int_type(); break;
+  case UINT:       new_type=unsigned_int_type(); break;
+  case LONG:       new_type=signed_long_int_type(); break;
+  case ULONG:      new_type=unsigned_long_int_type(); break;
+  case LONGLONG:   new_type=signed_long_long_int_type(); break;
+  case ULONGLONG:  new_type=unsigned_long_long_int_type(); break;
   case SINGLE:     new_type=float_type(); break;
   case DOUBLE:     new_type=double_type(); break;
   case LONGDOUBLE: new_type=long_double_type(); break;
@@ -735,46 +751,45 @@ Function: c_typecastt::do_typecast
 
 \*******************************************************************/
 
-void c_typecastt::do_typecast(exprt &expr, const typet &type)
+void c_typecastt::do_typecast(exprt &expr, const typet &dest_type)
 {
   // special case: array -> pointer is actually
   // something like address_of
   
-  const typet &expr_type=ns.follow(expr.type());
+  const typet &src_type=ns.follow(expr.type());
 
-  if(expr_type.id()==ID_array)
+  if(src_type.id()==ID_array)
   {
     index_exprt index;
     index.array()=expr;
     index.index()=gen_zero(index_type());
-    index.type()=expr_type.subtype();
+    index.type()=src_type.subtype();
     expr=address_of_exprt(index);
-    if(ns.follow(expr.type())!=ns.follow(type))
-      expr.make_typecast(type);
+    if(ns.follow(expr.type())!=ns.follow(dest_type))
+      expr.make_typecast(dest_type);
     return;
   }
 
-  if(expr_type!=type)
+  if(src_type!=dest_type)
   {
     // C booleans are special: we compile to ?0:1
     
-    if(type.get(ID_C_c_type)==ID_bool)
+    if(dest_type.get(ID_C_c_type)==ID_bool)
     {
-      if(expr_type.id()==ID_bool) // bool -> _Bool
+      if(src_type.id()==ID_bool) // bool proper -> _Bool
       {
-        exprt result=if_exprt(expr, gen_one(type), gen_zero(type));
-        expr.swap(result);
+        expr.make_typecast(dest_type);
       }
       else // * -> _Bool
       {
-        equal_exprt equal_zero(expr, gen_zero(expr_type));
-        exprt result=if_exprt(equal_zero, gen_zero(type), gen_one(type));
-        expr.swap(result);
+        notequal_exprt notequal_zero(expr, gen_zero(src_type));
+        expr=notequal_zero;
+        expr.make_typecast(dest_type);
       }
     }
     else
     {    
-      expr.make_typecast(type);
+      expr.make_typecast(dest_type);
     }
   }
 }
