@@ -6,14 +6,14 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <std_types.h>
-#include <prefix.h>
-#include <config.h>
+#include <util/std_types.h>
+#include <util/prefix.h>
+#include <util/config.h>
+#include <util/std_types.h>
 
 #include "c_typecheck_base.h"
 #include "expr2c.h"
 #include "type2name.h"
-#include "std_types.h"
 
 /*******************************************************************\
 
@@ -87,11 +87,11 @@ void c_typecheck_baset::move_symbol(symbolt &symbol, symbolt *&new_symbol)
   symbol.mode=mode;
   symbol.module=module;
 
-  if(context.move(symbol, new_symbol))
+  if(symbol_table.move(symbol, new_symbol))
   {
     err_location(symbol.location);
     throw "failed to move symbol `"+id2string(symbol.name)+
-          "' into context";
+          "' into symbol table";
   }
 }
 
@@ -109,6 +109,8 @@ Function: c_typecheck_baset::typecheck_symbol
 
 void c_typecheck_baset::typecheck_symbol(symbolt &symbol)
 {
+  current_symbol_id=symbol.name;
+
   // first of all, we typecheck the type
   typecheck_type(symbol.type);
 
@@ -117,7 +119,7 @@ void c_typecheck_baset::typecheck_symbol(symbolt &symbol)
   const typet &final_type=follow(symbol.type);
   
   // set a few flags
-  symbol.lvalue=!symbol.is_type && !symbol.is_macro;
+  symbol.is_lvalue=!symbol.is_type && !symbol.is_macro;
   
   std::string prefix="c::";
   std::string root_name=prefix+id2string(symbol.base_name);
@@ -133,14 +135,14 @@ void c_typecheck_baset::typecheck_symbol(symbolt &symbol)
     
     id_replace_map[symbol.name]=new_name;    
 
-    contextt::symbolst::const_iterator it=context.symbols.find(new_name);
-    if(it!=context.symbols.end())
+    symbol_tablet::symbolst::const_iterator it=symbol_table.symbols.find(new_name);
+    if(it!=symbol_table.symbols.end())
       return; // bail out, we have an appropriate symbol already.
 
     irep_idt newtag=std::string("#anon#")+typestr;
     symbol.type.set(ID_tag, newtag);
   }
-  else if(symbol.file_local)
+  else if(symbol.is_file_local)
   {
     // file-local stuff -- stays as is
     // collisions are resolved during linking
@@ -150,7 +152,7 @@ void c_typecheck_baset::typecheck_symbol(symbolt &symbol)
     // variables mared as "extern" go into the global namespace
     // and have static lifetime
     new_name=root_name;
-    symbol.static_lifetime=true;
+    symbol.is_static_lifetime=true;
   }
   else if(!is_function && symbol.value.id()==ID_code)
   {
@@ -206,11 +208,11 @@ void c_typecheck_baset::typecheck_symbol(symbolt &symbol)
   }
   
   // see if we have it already
-  contextt::symbolst::iterator old_it=context.symbols.find(symbol.name);
+  symbol_tablet::symbolst::iterator old_it=symbol_table.symbols.find(symbol.name);
   
-  if(old_it==context.symbols.end())
+  if(old_it==symbol_table.symbols.end())
   {
-    // just put into context
+    // just put into symbol_table
     symbolt *new_symbol;
     move_symbol(symbol, new_symbol);
     
@@ -247,8 +249,8 @@ Function: c_typecheck_baset::typecheck_new_symbol
 
 void c_typecheck_baset::typecheck_new_symbol(symbolt &symbol)
 {
-  if(symbol.is_actual)
-    adjust_function_argument(symbol.type);
+  if(symbol.is_parameter)
+    adjust_function_parameter(symbol.type);
 
   // check initializer, if needed
 
@@ -269,7 +271,9 @@ void c_typecheck_baset::typecheck_new_symbol(symbolt &symbol)
   }
   else
   {
-    if(symbol.type.id()==ID_array)
+    if(symbol.type.id()==ID_array &&
+       to_array_type(symbol.type).size().is_nil() &&
+       !symbol.is_type)
     {
       // Insert a new type symbol for the array.
       // We do this because we want a convenient way
@@ -288,7 +292,7 @@ void c_typecheck_baset::typecheck_new_symbol(symbolt &symbol)
       symbol.type=symbol_typet(new_symbol.name);
     
       symbolt *new_sp;
-      context.move(new_symbol, new_sp);
+      symbol_table.move(new_symbol, new_sp);
     }
 
     // check the initializer
@@ -462,14 +466,25 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
     // do body
     
     if(new_symbol.value.is_not_nil())
-    {      
+    {  
       if(old_symbol.value.is_not_nil())
       {
-        err_location(new_symbol.location);
-        str << "function `" << new_symbol.display_name()
-            << "' defined twice";
-        error();
-        throw 0;
+        // gcc allows re-definition if the first
+        // definition is marked as "extern inline"
+        
+        if(old_symbol.type.get_bool(ID_C_inlined) &&
+           (config.ansi_c.mode==configt::ansi_ct::MODE_GCC_C ||
+            config.ansi_c.mode==configt::ansi_ct::MODE_ARM_C_CPP))
+        {
+        }
+        else
+        {
+          err_location(new_symbol.location);
+          str << "function body `" << new_symbol.display_name()
+              << "' defined twice";
+          error();
+          throw 0;
+        }
       }
 
       typecheck_function_body(new_symbol);
@@ -511,9 +526,9 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
         const irep_idt identifier=
           to_symbol_type(old_symbol.type).get_identifier();
 
-        contextt::symbolst::iterator s_it=context.symbols.find(identifier);
+        symbol_tablet::symbolst::iterator s_it=symbol_table.symbols.find(identifier);
   
-        if(s_it==context.symbols.end())
+        if(s_it==symbol_table.symbols.end())
         {
           err_location(old_symbol.location);
           str << "typecheck_redefinition_non_type: "
@@ -535,6 +550,27 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
     {
       // this is ok for now
     }
+    else if(final_old.id()==ID_pointer &&
+            follow(final_old).subtype().id()==ID_code &&
+            to_code_type(follow(final_old).subtype()).has_ellipsis() &&
+            final_new.id()==ID_pointer &&
+            follow(final_new).subtype().id()==ID_code)
+    {
+      // to allow 
+      // int (*f) ();
+      // int (*f) (int)=0;
+      old_symbol.type=new_symbol.type;
+    }
+    else if(final_old.id()==ID_pointer &&
+            follow(final_old).subtype().id()==ID_code &&
+            final_new.id()==ID_pointer &&
+            follow(final_new).subtype().id()==ID_code &&
+            to_code_type(follow(final_new).subtype()).has_ellipsis())
+    {
+      // to allow 
+      // int (*f) (int)=0;
+      // int (*f) ();
+    }
     else
     {
       err_location(new_symbol.location);
@@ -590,6 +626,13 @@ void c_typecheck_baset::typecheck_redefinition_non_type(
       old_symbol.type=new_symbol.type;
     }
   }
+  
+  // take care of some flags
+  old_symbol.is_extern=old_symbol.is_extern && new_symbol.is_extern;
+  
+  // We should likely check is_volatile and
+  // is_thread_local for consistency. GCC complains if these
+  // mismatch.
 }
 
 /*******************************************************************\
