@@ -6,9 +6,6 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <i2string.h>
-#include <expr_util.h>
-
 #include <linking/zero_initializer.h>
 
 #include "c_typecheck_base.h"
@@ -118,6 +115,17 @@ void c_typecheck_baset::typecheck_code(codet &code)
     typecheck_expr(code.op0());
     typecheck_expr(code.op1());
   }
+  else if(statement==ID_CPROVER_try_catch ||
+          statement==ID_CPROVER_try_finally)
+  {
+    assert(code.operands().size()==2);
+    typecheck_code(to_code(code.op0()));
+    typecheck_code(to_code(code.op1()));
+  }
+  else if(statement==ID_CPROVER_throw)
+  {
+    assert(code.operands().size()==0);
+  }
   else
   {
     err_location(code);
@@ -348,9 +356,10 @@ void c_typecheck_baset::typecheck_decl_type(codet &code)
   if(!clean_code.empty())
   {
     // build a decl-block
-    codet decl_block(ID_decl_block);
-    decl_block.copy_to_operands(code_blockt(clean_code), code);
-    code.swap(decl_block);
+    code_blockt code_block(clean_code);
+    code_block.set_statement(ID_decl_block);
+    code_block.copy_to_operands(code);
+    code.swap(code_block);
   }
 }
 
@@ -374,9 +383,10 @@ void c_typecheck_baset::typecheck_decl(codet &code)
   if(!clean_code.empty())
   {
     // build a decl-block
-    codet decl_block(ID_decl_block);
-    decl_block.copy_to_operands(code_blockt(clean_code), code);
-    code.swap(decl_block);    
+    code_blockt code_block(clean_code);
+    code_block.set_statement(ID_decl_block);
+    code_block.copy_to_operands(code);
+    code.swap(code_block);    
   }
 }
 
@@ -423,12 +433,12 @@ void c_typecheck_baset::typecheck_decl(
   // look it up
   const irep_idt &identifier=to_symbol_expr(code.op0()).get_identifier();
 
-  contextt::symbolst::iterator s_it=context.symbols.find(identifier);
+  symbol_tablet::symbolst::iterator s_it=symbol_table.symbols.find(identifier);
 
-  if(s_it==context.symbols.end())
+  if(s_it==symbol_table.symbols.end())
   {
     err_location(code);
-    str << "failed to find decl symbol `" << identifier << "' in context";
+    str << "failed to find decl symbol `" << identifier << "' in symbol table";
     throw 0;
   }
 
@@ -443,7 +453,7 @@ void c_typecheck_baset::typecheck_decl(
   // or static
   if(symbol.is_type ||
      symbol.type.id()==ID_code ||
-     symbol.static_lifetime)
+     symbol.is_static_lifetime)
   {
     locationt location=code.location();
     code=code_skipt();
@@ -536,6 +546,9 @@ void c_typecheck_baset::typecheck_expression(codet &code)
   exprt &op=code.op0();
   typecheck_expr(op);
 
+  #if 0
+  // Goes away since stuff like x=({y=1;});
+  // needs the inner side-effect.
   if(op.id()==ID_sideeffect)
   {
     const irep_idt &statement=op.get(ID_statement);
@@ -576,6 +589,7 @@ void c_typecheck_baset::typecheck_expression(codet &code)
       code.swap(function_call);
     }
   }
+  #endif
 }
 
 /*******************************************************************\
@@ -609,7 +623,7 @@ void c_typecheck_baset::typecheck_for(codet &code)
   if(code.op0().is_nil())
   {
     if(code.op1().is_nil())
-      code.op1().make_true();
+      code.op1()=true_exprt();
     else
     {
       typecheck_expr(code.op1());
@@ -780,9 +794,8 @@ Function: c_typecheck_baset::typecheck_ifthenelse
 
 void c_typecheck_baset::typecheck_ifthenelse(code_ifthenelset &code)
 {
-  if(code.operands().size()!=2 &&
-     code.operands().size()!=3)
-    throw "ifthenelse expected to have two or three operands";
+  if(code.operands().size()!=3)
+    throw "ifthenelse expected to have three operands";
 
   exprt &cond=code.cond();
 
@@ -801,8 +814,7 @@ void c_typecheck_baset::typecheck_ifthenelse(code_ifthenelset &code)
 
   typecheck_code(to_code(code.then_case()));
 
-  if(code.operands().size()==3 &&
-     !code.else_case().is_nil())
+  if(!code.else_case().is_nil())
     typecheck_code(to_code(code.else_case()));
 }
 
