@@ -6,24 +6,26 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
-#include <arith_tools.h>
-#include <bitvector.h>
-#include <config.h>
-#include <expr_util.h>
-#include <std_types.h>
-#include <prefix.h>
-#include <cprover_prefix.h>
-#include <simplify_expr.h>
-#include <base_type.h>
-#include <std_expr.h>
-#include <i2string.h>
+#include <util/arith_tools.h>
+#include <util/bitvector.h>
+#include <util/config.h>
+#include <util/std_types.h>
+#include <util/expr_util.h>
+#include <util/prefix.h>
+#include <util/cprover_prefix.h>
+#include <util/simplify_expr.h>
+#include <util/base_type.h>
+#include <util/std_expr.h>
+#include <util/i2string.h>
+#include <util/pointer_offset_size.h>
 
 #include "c_types.h"
 #include "c_typecast.h"
 #include "c_typecheck_base.h"
 #include "c_sizeof.h"
+#include "c_qualifiers.h"
 #include "string_constant.h"
 #include "anonymous_member.h"
 #include "padding.h"
@@ -70,6 +72,68 @@ Function: c_typecheck_baset::typecheck_expr_main
 
 \*******************************************************************/
 
+bool c_typecheck_baset::gcc_types_compatible_p(
+  const typet &type1,
+  const typet &type2)
+{
+  // read
+  // http://gcc.gnu.org/onlinedocs/gcc-3.3.6/gcc/Other-Builtins.html
+
+  if(type1.id()==ID_symbol)
+    return gcc_types_compatible_p(follow(type1), type2);
+  else if(type2.id()==ID_symbol)
+    return gcc_types_compatible_p(type1, follow(type2));
+
+  // check qualifiers first
+  if(c_qualifierst(type1)!=c_qualifierst(type2))
+    return false;
+
+  if(type1.id()==ID_c_enum)
+  {
+    if(type2==signed_int_type())
+      return true;
+    else if(type2==type1) // compares the tag
+      return true;
+  }
+  else if(type2.id()==ID_c_enum)
+  {
+    if(type1==signed_int_type())
+      return true;
+    else if(type1==type2) // compares the tag
+      return true;
+  }
+  else if(type1.id()==ID_pointer && type2.id()==ID_pointer)
+    return gcc_types_compatible_p(type1.subtype(), type2.subtype());
+  else if(type1.id()==ID_array && type2.id()==ID_array)
+    return gcc_types_compatible_p(type1.subtype(), type2.subtype()); // ignore size
+  else
+  {
+    if(type1==type2)
+    {
+      // Need to distinguish e.g. long int from int or
+      // long long int from long int.
+      // The rules appear to match those of C++.
+      
+      if(type1.get(ID_C_c_type)==type2.get(ID_C_c_type))
+        return true;
+    }
+  }
+  
+  return false;
+}
+
+/*******************************************************************\
+
+Function: c_typecheck_baset::typecheck_expr_main
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
 void c_typecheck_baset::typecheck_expr_main(exprt &expr)
 {
   if(expr.id()==ID_sideeffect)
@@ -88,7 +152,7 @@ void c_typecheck_baset::typecheck_expr_main(exprt &expr)
     typecheck_expr_unary_arithmetic(expr);
   else if(expr.id()==ID_not)
     typecheck_expr_unary_boolean(expr);
-  else if(expr.id()==ID_and || expr.id()==ID_or)
+  else if(expr.id()==ID_and || expr.id()==ID_or || expr.id()==ID_implies)
     typecheck_expr_binary_boolean(expr);
   else if(expr.id()==ID_address_of)
     typecheck_expr_address_of(expr);
@@ -111,14 +175,15 @@ void c_typecheck_baset::typecheck_expr_main(exprt &expr)
     typecheck_expr_typecast(expr);
   else if(expr.id()==ID_sizeof)
     typecheck_expr_sizeof(expr);
-  else if(expr.id()==ID_builtin_alignof)
+  else if(expr.id()==ID_alignof)
     typecheck_expr_alignof(expr);
   else if(expr.id()==ID_plus || expr.id()==ID_minus ||
           expr.id()==ID_mult || expr.id()==ID_div ||
           expr.id()==ID_mod ||
-          expr.id()==ID_shl || expr.id()==ID_shr ||
           expr.id()==ID_bitand || expr.id()==ID_bitxor || expr.id()==ID_bitor)
     typecheck_expr_binary_arithmetic(expr);
+  else if(expr.id()==ID_shl || expr.id()==ID_shr)
+    typecheck_expr_shifts(to_shift_expr(expr));
   else if(expr.id()==ID_comma)
     typecheck_expr_comma(expr);
   else if(expr.id()==ID_if)
@@ -141,7 +206,16 @@ void c_typecheck_baset::typecheck_expr_main(exprt &expr)
     typecheck_type(subtypes[0]);
     typecheck_type(subtypes[1]);
     locationt location=expr.location();
-    expr.make_bool(follow(subtypes[0])==follow(subtypes[1]));
+    
+    // ignores top level qualifiers
+    subtypes[0].remove(ID_C_constant);
+    subtypes[0].remove(ID_C_volatile);
+    subtypes[0].remove(ID_C_restricted);
+    subtypes[1].remove(ID_C_constant);
+    subtypes[1].remove(ID_C_volatile);
+    subtypes[1].remove(ID_C_restricted);
+    
+    expr.make_bool(gcc_types_compatible_p(subtypes[0], subtypes[1]));
     expr.location()=location;
   }
   else if(expr.id()==ID_builtin_offsetof)
@@ -238,6 +312,12 @@ void c_typecheck_baset::typecheck_expr_main(exprt &expr)
       throw "_Generic expects one operand";
     }
 
+    // This is one of the few places where it's detectable
+    // that we are using "bool" for boolean operators instead
+    // of "int". We convert for this reason.
+    if(follow(expr.op0().type()).id()==ID_bool)
+      expr.op0().make_typecast(signed_int_type());
+
     irept::subt &generic_associations=
       expr.add(ID_generic_associations).get_sub();
       
@@ -344,6 +424,8 @@ void c_typecheck_baset::typecheck_expr_builtin_va_arg(exprt &expr)
   assert(expr.operands().size()==1);  
   exprt arg=expr.op0();
 
+  implicit_typecast(arg, pointer_typet(empty_typet()));
+
   // turn into function call
   side_effect_expr_function_callt result;
   result.location()=expr.location();
@@ -361,7 +443,7 @@ void c_typecheck_baset::typecheck_expr_builtin_va_arg(exprt &expr)
   symbol.name=ID_gcc_builtin_va_arg;
   symbol.type=new_type;
   
-  context.move(symbol);
+  symbol_table.move(symbol);
 }
 
 /*******************************************************************\
@@ -390,7 +472,7 @@ void c_typecheck_baset::typecheck_expr_cw_va_arg_typeof(exprt &expr)
   typecheck_type(type);
 
   // these return an integer
-  expr.type()=int_type();
+  expr.type()=signed_int_type();
 }
 
 /*******************************************************************\
@@ -528,7 +610,7 @@ void c_typecheck_baset::typecheck_expr_builtin_offsetof(exprt &expr)
     else if(m_it->id()==ID_index)
     {
       assert(m_it->operands().size()==1);
-
+      
       if(type.id()!=ID_array)
       {
         err_location(expr);
@@ -536,6 +618,10 @@ void c_typecheck_baset::typecheck_expr_builtin_offsetof(exprt &expr)
       }
 
       exprt index=m_it->op0();
+
+      // still need to typecheck index
+      typecheck_expr(index);
+
       exprt sub_size=c_sizeof(type.subtype(), *this);
       if(index.type()!=size_type()) index.make_typecast(size_type());
       result=plus_exprt(result, mult_exprt(sub_size, index));
@@ -545,7 +631,8 @@ void c_typecheck_baset::typecheck_expr_builtin_offsetof(exprt &expr)
     }
   }
 
-  // we make an effort to produce a constant
+  // We make an effort to produce a constant,
+  // but this may depend on variables
   simplify(result, *this);
   result.location()=expr.location();
 
@@ -615,6 +702,16 @@ void c_typecheck_baset::typecheck_expr_symbol(exprt &expr)
   replace_symbol(expr);
   
   const irep_idt &identifier=to_symbol_expr(expr).get_identifier();
+  
+  // Is it a parameter? We do this while checking parameter lists.
+  id_type_mapt::const_iterator p_it=parameter_map.find(identifier);
+  if(p_it!=parameter_map.end())
+  {
+    // yes
+    expr.type()=p_it->second;
+    expr.set(ID_C_lvalue, true);
+    return;
+  }
 
   // look it up
   const symbolt *symbol_ptr;
@@ -647,7 +744,7 @@ void c_typecheck_baset::typecheck_expr_symbol(exprt &expr)
   }
   else if(has_prefix(id2string(identifier), CPROVER_PREFIX "constant_infinity"))
   {
-    expr=exprt(ID_infinity, symbol.type);
+    expr=infinity_exprt(symbol.type);
 
     // put it back
     expr.location()=location;
@@ -658,20 +755,19 @@ void c_typecheck_baset::typecheck_expr_symbol(exprt &expr)
   {
     // __func__ is an ANSI-C standard compliant hack to get the function name
     // __FUNCTION__ and __PRETTY_FUNCTION__ are GCC-specific
-    string_constantt s;
-    s.set_value(location.get_function());
+    string_constantt s(location.get_function());
     s.location()=location;
     s.set(ID_C_lvalue, true);
     expr.swap(s);
   }
   else
   {
-    expr=symbol_expr(symbol);
+    expr=symbol.symbol_expr();
 
     // put it back
     expr.location()=location;
 
-    if(symbol.lvalue)
+    if(symbol.is_lvalue)
       expr.set(ID_C_lvalue, true);
 
     if(expr.type().id()==ID_code) // function designator
@@ -925,6 +1021,12 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
   {
     // This is a GCC extension. It's either a 'temporary union',
     // where the argument is one of the member types.
+    
+    // This is one of the few places where it's detectable
+    // that we are using "bool" for boolean operators instead
+    // of "int". We convert for this reason.
+    if(follow(op.type()).id()==ID_bool)
+      op.make_typecast(signed_int_type());
 
     // we need to find a member with the right type
     const union_typet &union_type=to_union_type(expr_type);
@@ -947,6 +1049,12 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
         return;
       }
     }
+    
+    // not found, complain
+    err_location(expr);
+    str << "type cast to union: type `"
+        << to_string(op.type()) << "' not found in union";
+    throw 0;
   }
 
   // We allow (TYPE){ expression }
@@ -979,6 +1087,10 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
       return;
   }
   
+  // cast to same type?
+  if(base_type_eq(expr_type, op_type, *this))
+    return; // it's ok
+  
   if(!is_number(expr_type) &&
      expr_type.id()!=ID_bool &&
      expr_type.id()!=ID_pointer &&
@@ -993,20 +1105,6 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
     throw 0;
   }
 
-  // Casts to booleans have particular meaning
-  if(expr_type.get(ID_C_c_type)==ID_bool)
-  {
-    // we replace (_Bool)x by x!=0; use ieee_float_notequal for floats
-    irep_idt id=
-      op_type.id()==ID_floatbv?ID_ieee_float_notequal:ID_notequal;
-
-    binary_exprt comparison(expr.op0(), id, gen_zero(expr.op0().type()), bool_typet());
-
-    comparison.location()=expr.location();
-    expr.swap(comparison);
-    return;
-  }
-
   if(is_number(op_type) ||
      op_type.id()==ID_c_enum ||
      op_type.id()==ID_incomplete_c_enum ||
@@ -1034,10 +1132,24 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
   }
   else if(op_type.id()==ID_vector)
   {
-    // we are generous -- any vector to integer is fine
-    if(expr_type.id()==ID_signedbv ||
-       expr_type.id()==ID_unsignedbv)
-      return;
+    const vector_typet &op_vector_type=
+      to_vector_type(op_type);
+      
+    // gcc allows conversion of a vector of size 1 to
+    // an integer/float of the same size
+    if((expr_type.id()==ID_signedbv ||
+        expr_type.id()==ID_unsignedbv) &&
+       pointer_offset_size(*this, expr_type)==
+       pointer_offset_size(*this, op_vector_type))
+    {
+    }
+    else
+    {
+      err_location(expr);
+      str << "type cast from vector to `"
+          << to_string(expr.type()) << "' not permitted";
+      throw 0;
+    }
   }
   else
   {
@@ -1047,8 +1159,24 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
     throw 0;
   }
 
-  // special case: NULL
+  // Casts to C/C++ booleans have particular meaning
+  if(expr_type.get(ID_C_c_type)==ID_bool)
+  {
+    // we replace (_Bool)x by x!=0; use ieee_float_notequal for floats
+    irep_idt id=
+      op_type.id()==ID_floatbv?ID_ieee_float_notequal:ID_notequal;
+      
+    exprt zero=gen_zero(expr.op0().type());
+    assert(zero.is_not_nil());
 
+    binary_exprt comparison(expr.op0(), id, zero, bool_typet());
+
+    comparison.location()=expr.location();
+    expr.swap(comparison);
+    return;
+  }
+
+  // special case: NULL
   if(expr_type.id()==ID_pointer &&
      op.is_zero())
   {
@@ -1059,9 +1187,16 @@ void c_typecheck_baset::typecheck_expr_typecast(exprt &expr)
     return;
   }
 
-  // the new thing is an lvalue if the previous one is
-  // an lvalue, and it's just a pointer type cast
-  // this isn't really standard conformant!
+  // The new thing is an lvalue if the previous one is
+  // an lvalue, and it's just a pointer type cast.
+  // This isn't really standard conformant!
+
+  // Note that gcc says "warning: target of assignment not really an lvalue;
+  // this will be a hard error in the future", i.e., we
+  // can hope that the below will one day go away.
+  
+  // Current versions of gcc in fact do not do this! Yay!
+  
   if(expr.op0().get_bool(ID_C_lvalue))
   {
     if(expr_type.id()==ID_pointer)
@@ -1831,11 +1966,11 @@ void c_typecheck_baset::typecheck_side_effect_function_call(
     const irep_idt &identifier=
       add_language_prefix(to_symbol_expr(f_op).get_identifier());
 
-    if(context.symbols.find(identifier)==context.symbols.end())
+    if(symbol_table.symbols.find(identifier)==symbol_table.symbols.end())
     {
       // This is an undeclared function. Let's just add it.
       // We do a bit of return-type guessing, but just a bit.
-      typet return_type=int_type();
+      typet return_type=signed_int_type();
       
       // The following isn't really right and sound, but there
       // are too many idiots out there who use malloc and the like
@@ -1943,7 +2078,20 @@ void c_typecheck_baset::do_special_functions(
         throw "same_object expects two operands";
       }
 
-      exprt same_object_expr("same-object", bool_typet());
+      predicate_exprt same_object_expr(ID_same_object);
+      same_object_expr.operands()=expr.arguments();
+      same_object_expr.location()=location;
+      expr.swap(same_object_expr);
+    }
+    else if(identifier==CPROVER_PREFIX "invalid_pointer")
+    {
+      if(expr.arguments().size()!=1)
+      {
+        err_location(f_op);
+        throw "invalid_pointer expects one operand";
+      }
+
+      predicate_exprt same_object_expr(ID_invalid_pointer);
       same_object_expr.operands()=expr.arguments();
       same_object_expr.location()=location;
       expr.swap(same_object_expr);
@@ -1956,7 +2104,7 @@ void c_typecheck_baset::do_special_functions(
         throw "buffer_size expects one operand";
       }
 
-      exprt buffer_size_expr("buffer_size", uint_type());
+      exprt buffer_size_expr("buffer_size", size_type());
       buffer_size_expr.operands()=expr.arguments();
       buffer_size_expr.location()=location;
       expr.swap(buffer_size_expr);
@@ -1969,7 +2117,7 @@ void c_typecheck_baset::do_special_functions(
         throw "is_zero_string expects one operand";
       }
 
-      exprt is_zero_string_expr("is_zero_string", bool_typet());
+      predicate_exprt is_zero_string_expr("is_zero_string");
       is_zero_string_expr.operands()=expr.arguments();
       is_zero_string_expr.set(ID_C_lvalue, true); // make it an lvalue
       is_zero_string_expr.location()=location;
@@ -1983,7 +2131,7 @@ void c_typecheck_baset::do_special_functions(
         throw "zero_string_length expects one operand";
       }
 
-      exprt zero_string_length_expr("zero_string_length", uint_type());
+      exprt zero_string_length_expr("zero_string_length", size_type());
       zero_string_length_expr.operands()=expr.arguments();
       zero_string_length_expr.set(ID_C_lvalue, true); // make it an lvalue
       zero_string_length_expr.location()=location;
@@ -2059,6 +2207,14 @@ void c_typecheck_baset::do_special_functions(
       inff_expr.location()=location;
       expr.swap(inff_expr);
     }
+    else if(identifier==CPROVER_PREFIX "infl")
+    {
+      floatbv_typet type=to_floatbv_type(long_double_type());
+      constant_exprt infl_expr=
+        ieee_floatt::plus_infinity(ieee_float_spect(type)).to_expr();
+      infl_expr.location()=location;
+      expr.swap(infl_expr);
+    }
     else if(identifier==CPROVER_PREFIX "abs" ||
             identifier==CPROVER_PREFIX "labs" ||
             identifier==CPROVER_PREFIX "fabs" ||
@@ -2242,8 +2398,25 @@ void c_typecheck_baset::do_special_functions(
       // try to produce constant
       exprt tmp1=expr.arguments().front();
       simplify(tmp1, *this);
-
-      exprt tmp2=from_integer(tmp1.is_constant(), expr.type());      
+      
+      bool is_constant=false;
+      
+      // Need to do some special treatment for string literals,
+      // which are (void *)&("lit"[0])
+      if(tmp1.id()==ID_typecast &&
+         tmp1.operands().size()==1 &&
+         tmp1.op0().id()==ID_address_of &&
+         tmp1.op0().operands().size()==1 &&
+         tmp1.op0().op0().id()==ID_index &&
+         tmp1.op0().op0().operands().size()==2 &&
+         tmp1.op0().op0().op0().id()==ID_string_constant)
+      {
+        is_constant=true;
+      }
+      else
+        is_constant=tmp1.is_constant();
+      
+      exprt tmp2=from_integer(is_constant, expr.type());
       tmp2.location()=location;
       expr.swap(tmp2);
     }
@@ -2540,97 +2713,137 @@ void c_typecheck_baset::typecheck_expr_binary_arithmetic(exprt &expr)
     }
   }
 
-  if(expr.id()==ID_shl || expr.id()==ID_shr)
+  // promote!
+  
+  implicit_typecast_arithmetic(op0, op1);
+
+  const typet &type0=follow(op0.type());
+  const typet &type1=follow(op1.type());
+
+  if(expr.id()==ID_plus || expr.id()==ID_minus ||
+     expr.id()==ID_mult || expr.id()==ID_div)
   {
-    // must do the promotions _separately_!
-    implicit_typecast_arithmetic(op0);
-    implicit_typecast_arithmetic(op1);
-
-    if(is_number(op0.type()) &&
-       is_number(op1.type()))
+    if(type0.id()==ID_pointer || type1.id()==ID_pointer)
     {
-      expr.type()=op0.type();
-
-      if(expr.id()==ID_shr) // shifting operation depends on types
+      typecheck_expr_pointer_arithmetic(expr);
+      return;
+    }
+    else if(type0==type1)
+    {
+      if(is_number(type0))
       {
-        const typet &op0_type=follow(op0.type());
-
-        if(op0_type.id()==ID_unsignedbv)
-        {
-          expr.id(ID_lshr);
-          return;
-        }
-        else if(op0_type.id()==ID_signedbv)
-        {
-          expr.id(ID_ashr);
-          return;
-        }
+        expr.type()=type0;
+        return;
       }
+    }
+  }
+  else if(expr.id()==ID_mod)
+  {
+    if(type0==type1)
+    {
+      if(type0.id()==ID_signedbv || type0.id()==ID_unsignedbv)
+      {
+        expr.type()=type0;
+        return;
+      }
+    }
+  }
+  else if(expr.id()==ID_bitand || 
+          expr.id()==ID_bitxor ||
+          expr.id()==ID_bitor)
+  {
+    if(type0==type1)
+    {
+      if(is_number(type0))
+      {
+        expr.type()=type0;
+        return;
+      }
+      else if(type0.id()==ID_bool)
+      {
+        if(expr.id()==ID_bitand)
+          expr.id(ID_and);
+        else if(expr.id()==ID_bitor)
+          expr.id(ID_or);
+        else if(expr.id()==ID_bitxor)
+          expr.id(ID_xor);
+        else
+          assert(false);
+        expr.type()=type0;
+        return;
+      }
+    }
+  }
 
+  err_location(expr);
+  str << "operator `" << expr.id()
+      << "' not defined for types `"
+      << to_string(o_type0) << "' and `"
+      << to_string(o_type1) << "'";
+  throw 0;
+}
+
+/*******************************************************************\
+
+Function: c_typecheck_baset::typecheck_expr_shifts
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void c_typecheck_baset::typecheck_expr_shifts(shift_exprt &expr)
+{
+  assert(expr.id()==ID_shl || expr.id()==ID_shr);
+  
+  exprt &op0=expr.op0();
+  exprt &op1=expr.op1();
+
+  const typet o_type0=follow(op0.type());
+  const typet o_type1=follow(op1.type());
+
+  if(o_type0.id()==ID_vector &&
+     o_type1.id()==ID_vector)
+  {
+    if(follow(o_type0.subtype())==follow(o_type1.subtype()) &&
+       is_number(follow(o_type0.subtype())))
+    {
+      // {a0, a1, ..., an} >> {b0, b1, ..., bn} == {a0 >> b0, a1 >> b1, ..., an >> bn}
+      // Fairly strict typing rules, no promotion
+      expr.type()=op0.type();
       return;
     }
   }
-  else
+
+  // must do the promotions _separately_!
+  implicit_typecast_arithmetic(op0);
+  implicit_typecast_arithmetic(op1);
+
+  if(is_number(op0.type()) &&
+     is_number(op1.type()))
   {
-    // promote!
-    
-    implicit_typecast_arithmetic(op0, op1);
+    expr.type()=op0.type();
 
-    const typet &type0=follow(op0.type());
-    const typet &type1=follow(op1.type());
-
-    if(expr.id()==ID_plus || expr.id()==ID_minus ||
-       expr.id()==ID_mult || expr.id()==ID_div)
+    if(expr.id()==ID_shr) // shifting operation depends on types
     {
-      if(type0.id()==ID_pointer || type1.id()==ID_pointer)
+      const typet &op0_type=follow(op0.type());
+
+      if(op0_type.id()==ID_unsignedbv)
       {
-        typecheck_expr_pointer_arithmetic(expr);
+        expr.id(ID_lshr);
         return;
       }
-      else if(type0==type1)
+      else if(op0_type.id()==ID_signedbv)
       {
-        if(is_number(type0))
-        {
-          expr.type()=type0;
-          return;
-        }
-      }
-    }
-    else if(expr.id()==ID_mod)
-    {
-      if(type0==type1)
-      {
-        if(type0.id()==ID_signedbv || type0.id()==ID_unsignedbv)
-        {
-          expr.type()=type0;
-          return;
-        }
-      }
-    }
-    else if(expr.id()==ID_bitand || expr.id()==ID_bitxor || expr.id()==ID_bitor)
-    {
-      if(type0==type1)
-      {
-        if(is_number(type0))
-        {
-          expr.type()=type0;
-          return;
-        }
-        else if(type0.id()==ID_bool)
-        {
-          if(expr.id()==ID_bitand)
-            expr.id(ID_and);
-          else if(expr.id()==ID_bitor)
-            expr.id(ID_or);
-          else if(expr.id()==ID_bitxor)
-            expr.id(ID_xor);
-          else
-            assert(false);
-          expr.type()=type0;
-          return;
-        }
+        expr.id(ID_ashr);
+        return;
       }
     }
+
+    return;
   }
 
   err_location(expr);
@@ -2685,11 +2898,12 @@ Function: c_typecheck_baset::typecheck_expr_pointer_arithmetic
 void c_typecheck_baset::typecheck_expr_pointer_arithmetic(exprt &expr)
 {
   assert(expr.operands().size()==2);
+
   exprt &op0=expr.op0();
   exprt &op1=expr.op1();
 
-  const typet &type0=op0.type();
-  const typet &type1=op1.type();
+  const typet &type0=follow(op0.type());
+  const typet &type1=follow(op1.type());
 
   if(expr.id()==ID_minus ||
      (expr.id()==ID_sideeffect && expr.get(ID_statement)==ID_assign_minus))
@@ -2697,13 +2911,19 @@ void c_typecheck_baset::typecheck_expr_pointer_arithmetic(exprt &expr)
     if(type0.id()==ID_pointer &&
        type1.id()==ID_pointer)
     {
+      // We should check the subtypes, and complain if
+      // they are really different.
       expr.type()=pointer_diff_type();
       typecheck_arithmetic_pointer(op0);
       typecheck_arithmetic_pointer(op1);
       return;
     }
 
-    if(type0.id()==ID_pointer)
+    if(type0.id()==ID_pointer &&
+       (type1.id()==ID_bool ||
+        type1.id()==ID_unsignedbv ||
+        type1.id()==ID_signedbv ||
+        type1.id()==ID_c_enum))
     {
       typecheck_arithmetic_pointer(op0);
       make_index_type(op1);
@@ -2729,10 +2949,18 @@ void c_typecheck_baset::typecheck_expr_pointer_arithmetic(exprt &expr)
     else
       assert(false);
 
-    typecheck_arithmetic_pointer(*p_op);
-    make_index_type(*int_op);
-    expr.type()=p_op->type();
-    return;
+    const typet &int_op_type=follow(int_op->type());
+
+    if(int_op_type.id()==ID_bool ||
+       int_op_type.id()==ID_unsignedbv ||
+       int_op_type.id()==ID_signedbv ||
+       int_op_type.id()==ID_c_enum)
+    {
+      typecheck_arithmetic_pointer(*p_op);
+      make_index_type(*int_op);
+      expr.type()=p_op->type();
+      return;
+    }
   }
 
   err_location(expr);
@@ -2843,6 +3071,15 @@ void c_typecheck_baset::typecheck_side_effect_assignment(exprt &expr)
         << "' is constant";
     throw 0;
   }
+  
+  // refuse to assign arrays
+  if(final_type0.id()==ID_array ||
+     final_type0.id()==ID_incomplete_array)
+  {
+    err_location(expr);
+    str << "error: direct assignments to arrays not permitted";
+    throw 0;
+  }
 
   if(statement==ID_assign)
   {
