@@ -6,15 +6,15 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <arith_tools.h>
-#include <config.h>
-#include <type_eq.h>
-#include <std_types.h>
-#include <expr_util.h>
-#include <simplify_expr.h>
-#include <cprover_prefix.h>
-#include <prefix.h>
-#include <std_types.h>
+#include <util/arith_tools.h>
+#include <util/config.h>
+#include <util/type_eq.h>
+#include <util/std_types.h>
+#include <util/expr_util.h>
+#include <util/simplify_expr.h>
+#include <util/cprover_prefix.h>
+#include <util/prefix.h>
+#include <util/std_types.h>
 
 #include <linking/zero_initializer.h>
 
@@ -232,24 +232,9 @@ void c_typecheck_baset::do_initializer(symbolt &symbol)
   if(has_prefix(id2string(symbol.name), CPROVER_PREFIX "constant_infinity"))
     return;
 
-  if(symbol.static_lifetime)
+  if(symbol.is_static_lifetime)
   {
-    if(symbol.value.is_nil())
-    {
-      const typet &final_type=follow(symbol.type);
-      
-      // We do some checks, but this is otherwise left
-      // to __CPROVER_initialize to do.
-      
-      if(final_type.id()==ID_empty)
-      {
-        err_location(symbol.location);
-        str << "cannot initialize type `"
-            << to_string(final_type) << "'";
-        throw 0;
-      }
-    }
-    else
+    if(symbol.value.is_not_nil())
     {
       typecheck_expr(symbol.value);
       do_initializer(symbol.value, symbol.type, true);
@@ -526,15 +511,20 @@ void c_typecheck_baset::do_designated_initializer(
     }
     else if(follow(value.type())==type)
     {
-      // a struct/union can be initialized directly with
+      // a struct/union/vector can be initialized directly with
       // an expression of the right type. This doesn't
       // work with arrays, unfortunately.
-      if(type.id()==ID_struct || type.id()==ID_union)
+      if(type.id()==ID_struct || type.id()==ID_union || type.id()==ID_vector)
       {
         *dest=value;
         return; // done
       }
     }
+
+    assert(type.id()==ID_struct ||
+           type.id()==ID_union ||
+           type.id()==ID_array ||
+           type.id()==ID_vector);
     
     // we are initializing a compound type, and enter it!
     designator_enter(type, designator);
@@ -582,8 +572,9 @@ void c_typecheck_baset::increment_designator(designatort &designator)
         struct_type.components();
       assert(components.size()==entry.size);
       
-      // we skip over padding
-      if(components[entry.index].get_is_padding())
+      // we skip over any padding
+      while(entry.index<entry.size &&
+            components[entry.index].get_is_padding())
         entry.index++;
 
       if(entry.index<entry.size)
@@ -818,7 +809,7 @@ exprt c_typecheck_baset::do_initializer_list(
   designatort current_designator;
   
   designator_enter(type, current_designator);
-    
+
   forall_operands(it, value)
   {
     do_designated_initializer(
@@ -828,9 +819,10 @@ exprt c_typecheck_baset::do_initializer_list(
     increment_designator(current_designator);
   }
   
-  if(type.id()==ID_array && to_array_type(type).size().is_nil())
+  if(type.id()==ID_array &&
+     to_array_type(type).size().is_nil())
   {
-    // make complete
+    // make complete by setting array size
     unsigned size=result.operands().size();
     result.type().id(ID_array);
     result.type().set(ID_size, from_integer(size, index_type()));
