@@ -6,13 +6,13 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <config.h>
-#include <simplify_expr.h>
-#include <arith_tools.h>
-#include <std_types.h>
-#include <i2string.h>
-#include <expr_util.h>
-#include <pointer_offset_size.h>
+#include <util/config.h>
+#include <util/simplify_expr.h>
+#include <util/arith_tools.h>
+#include <util/std_types.h>
+#include <util/i2string.h>
+#include <util/expr_util.h>
+#include <util/pointer_offset_size.h>
 
 #include "c_typecheck_base.h"
 #include "c_types.h"
@@ -67,6 +67,18 @@ void c_typecheck_baset::typecheck_type(typet &type)
     typecheck_symbol_type(type);
   else if(type.id()==ID_vector)
     typecheck_vector_type(to_vector_type(type));
+
+  // do a bit of rule checking
+
+  if(type.get_bool(ID_C_restricted) &&
+     type.id()!=ID_pointer &&
+     type.id()!=ID_array)
+  {
+    err_location(type);
+    error("only a pointer can be 'restrict'");
+    throw 0;
+  }
+  
 }
 
 /*******************************************************************\
@@ -83,29 +95,33 @@ Function: c_typecheck_baset::typecheck_code_type
 
 void c_typecheck_baset::typecheck_code_type(code_typet &type)
 {
-  code_typet::argumentst &arguments=type.arguments();
-
-  // if we don't have any arguments, we assume it's (...)
-  if(arguments.empty())
+  code_typet::parameterst &parameters=type.parameters();
+  
+  // if we don't have any parameters, we assume it's (...)
+  if(parameters.empty())
   {
     type.make_ellipsis();
   }
   else
   {
-    for(unsigned i=0; i<type.arguments().size(); i++)
+    // we do have parameters
+    
+    parameter_map.clear();
+  
+    for(unsigned i=0; i<type.parameters().size(); i++)
     {
-      code_typet::argumentt &argument=type.arguments()[i];
+      code_typet::parametert &parameter=type.parameters()[i];
 
       // first fix type
-      typet &type=argument.type();
+      typet &type=parameter.type();
 
       typecheck_type(type);
       
-      adjust_function_argument(type);
+      adjust_function_parameter(type);
       
       // adjust the identifier
 
-      irep_idt identifier=argument.get_identifier();
+      irep_idt identifier=parameter.get_identifier();
 
       if(identifier!=irep_idt())
       {
@@ -117,19 +133,42 @@ void c_typecheck_baset::typecheck_code_type(code_typet &type)
         if(m_it!=id_replace_map.end())
           identifier=m_it->second;
 
-        argument.set_identifier(identifier);
+        parameter.set_identifier(identifier);
+
+        // make visible now, later parameters might use it
+        parameter_map[identifier]=type;
       }
     }
+    
+    parameter_map.clear();
   
-    if(arguments.size()==1 &&
-       follow(arguments[0].type()).id()==ID_empty)
+    if(parameters.size()==1 &&
+       follow(parameters[0].type()).id()==ID_empty)
     {
-      // if we just have one argument of type void, remove it
-      arguments.clear();
+      // if we just have one parameter of type void, remove it
+      parameters.clear();
     }
   }
 
   typecheck_type(type.return_type());
+  
+  // 6.7.6.3:
+  // "A function declarator shall not specify a return type that
+  // is a function type or an array type."
+  
+  const typet &return_type=follow(type.return_type());
+  
+  if(return_type.id()==ID_array)
+  {
+    err_location(type);
+    throw "function must not return array";
+  }
+  
+  if(return_type.id()==ID_code)
+  {
+    err_location(type);
+    throw "function must not return function type";
+  }
 }
 
 /*******************************************************************\
@@ -151,6 +190,14 @@ void c_typecheck_baset::typecheck_array_type(array_typet &type)
 
   // check subtype
   typecheck_type(type.subtype());
+  
+  // we don't allow void as subtype
+  if(follow(type.subtype()).id()==ID_empty)
+  {
+    err_location(type);
+    str << "array of voids";
+    throw 0;
+  }
 
   // check size, if any
   
@@ -247,7 +294,7 @@ void c_typecheck_baset::typecheck_vector_type(vector_typet &type)
   
   // the subtype must have constant size
   exprt size_expr=c_sizeof(type.subtype(), *this);
-  
+
   simplify(size_expr, *this);
 
   mp_integer sub_size;
@@ -259,12 +306,22 @@ void c_typecheck_baset::typecheck_vector_type(vector_typet &type)
         << to_string(type.subtype()) << "'";
     throw 0;
   }
+
+  if(sub_size==0)
+  {
+    err_location(size_location);
+    str << "type had size 0: `"
+        << to_string(type.subtype()) << "'";
+    throw 0;
+  }
   
   // adjust by width of base type
   if(s%sub_size!=0)
   {
     err_location(size_location);
-    str << "vector size expected to be multiple of base type size";
+    str << "vector size (" << s
+        << ") expected to be multiple of base type size (" << sub_size
+        << ")";
     throw 0;
   }
   
@@ -288,7 +345,7 @@ Function: c_typecheck_baset::typecheck_compound_type
 void c_typecheck_baset::typecheck_compound_type(struct_union_typet &type)
 {
   struct_union_typet::componentst &components=type.components();
-  
+
   // mark bit-fields
   for(struct_union_typet::componentst::iterator
       it=components.begin();
@@ -580,10 +637,10 @@ void c_typecheck_baset::typecheck_symbol_type(typet &type)
   const irep_idt &identifier=
     to_symbol_type(type).get_identifier();
 
-  contextt::symbolst::const_iterator s_it=
-    context.symbols.find(identifier);
+  symbol_tablet::symbolst::const_iterator s_it=
+    symbol_table.symbols.find(identifier);
 
-  if(s_it==context.symbols.end())
+  if(s_it==symbol_table.symbols.end())
   {
     err_location(type);
     str << "type symbol `" << identifier << "' not found";
@@ -600,11 +657,18 @@ void c_typecheck_baset::typecheck_symbol_type(typet &type)
   
   if(symbol.is_macro)
   {
-    // overwrite, but preserve (add) any qualifiers
+    // overwrite, but preserve (add) any qualifiers and other flags
+
     c_qualifierst c_qualifiers(type);
+    bool is_packed=type.get_bool(ID_C_packed);
+    irept alignment=type.find(ID_C_alignment);
+    
     c_qualifiers+=c_qualifierst(symbol.type);
     type=symbol.type;
     c_qualifiers.write(type);
+    
+    if(is_packed) type.set(ID_C_packed, true);
+    if(alignment.is_not_nil()) type.set(ID_C_alignment, alignment);
   }
     
   // CPROVER extensions
@@ -620,7 +684,7 @@ void c_typecheck_baset::typecheck_symbol_type(typet &type)
 
 /*******************************************************************\
 
-Function: c_typecheck_baset::adjust_function_argument
+Function: c_typecheck_baset::adjust_function_parameter
 
   Inputs:
 
@@ -630,7 +694,7 @@ Function: c_typecheck_baset::adjust_function_argument
 
 \*******************************************************************/
 
-void c_typecheck_baset::adjust_function_argument(typet &type) const
+void c_typecheck_baset::adjust_function_parameter(typet &type) const
 {
   if(type.id()==ID_array)
   {
@@ -648,7 +712,7 @@ void c_typecheck_baset::adjust_function_argument(typet &type) const
   else if(type.id()==ID_KnR)
   {
     // any KnR args without type yet?
-    type=int_type(); // the default is integer!
+    type=signed_int_type(); // the default is integer!
   }
 }
 
@@ -675,8 +739,8 @@ void c_typecheck_baset::clean_type(
     irep_idt identifier=to_symbol_type(type).get_identifier();
     if(already_cleaned.insert(identifier).second)
     {
-      contextt::symbolst::iterator s_it=context.symbols.find(identifier);
-      assert(s_it!=context.symbols.end());
+      symbol_tablet::symbolst::iterator s_it=symbol_table.symbols.find(identifier);
+      assert(s_it!=symbol_table.symbols.end());
       clean_type(identifier, s_it->second.type, code);
     }
   }
@@ -693,45 +757,67 @@ void c_typecheck_baset::clean_type(
     
     if(size.is_not_nil() &&
        !size.is_constant() &&
-       size.id()!=ID_infinity)
+       size.id()!=ID_infinity &&
+       !(size.id()==ID_symbol && size.type().get_bool(ID_C_constant)))
     {
-      const symbolt &base_symbol=lookup(base_symbol_identifier);
+      // The criterion above can be tricked:
+      // Of course we can modify a 'const' symbol, e.g.,
+      // using a pointer type cast. Interestingly,
+      // at least gcc 4.2.1 makes the very same mistake!
+    
+      assert(current_symbol_id!=irep_idt());
+        
+      const symbolt &base_symbol=
+        lookup(
+          base_symbol_identifier!=irep_idt()?
+          base_symbol_identifier:
+          current_symbol_id);
       
       // Need to pull out! We insert new symbol.
       locationt location=size.find_location();
       unsigned count=0;
       irep_idt temp_identifier;
       std::string suffix;
+
       do
       {
-        suffix="#array_size"+i2string(count);
+        suffix="$array_size"+i2string(count);
         temp_identifier=id2string(base_symbol.name)+suffix;
         count++;
       }
-      while(context.symbols.find(temp_identifier)!=context.symbols.end());
+      while(symbol_table.symbols.find(temp_identifier)!=symbol_table.symbols.end());
 
-      // add the symbol to context
+      // add the symbol to symbol table
       symbolt new_symbol;
       new_symbol.name=temp_identifier;
       new_symbol.pretty_name=id2string(base_symbol.pretty_name)+suffix;
       new_symbol.base_name=id2string(base_symbol.base_name)+suffix;
       new_symbol.type=size.type();
-      new_symbol.file_local=true;
+      new_symbol.type.set(ID_C_constant, true);
+      new_symbol.is_file_local=true;
       new_symbol.is_type=false;
+      new_symbol.is_thread_local=true;
+      new_symbol.is_static_lifetime=false;
       new_symbol.value.make_nil();
       new_symbol.location=location;
-      context.add(new_symbol);
+      
+      symbol_table.add(new_symbol);
 
-      // produce the code that initializes the symbol      
+      // produce the code that declares and initializes the symbol
       symbol_exprt symbol_expr;
       symbol_expr.set_identifier(temp_identifier);
-      symbol_expr.type()=size.type();
+      symbol_expr.type()=new_symbol.type;
+      
+      code_declt declaration(symbol_expr);
+      declaration.location()=location;
+
       code_assignt assignment;
       assignment.lhs()=symbol_expr;
       assignment.rhs()=size;
       assignment.location()=location;
 
       // store the code
+      code.push_back(declaration);
       code.push_back(assignment);
 
       // fix type
