@@ -11,12 +11,12 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <set>
 
-#include <context.h>
-#include <message.h>
+#include <util/symbol_table.h>
+#include <util/message.h>
 
 void add_cprover_library(
   const std::set<irep_idt> &functions,
-  contextt &context,
+  symbol_tablet &symbol_table,
   message_handlert &message_handler);
 
 #endif
