@@ -12,17 +12,17 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <set>
 #include <map>
 
-#include <expr.h>
-#include <std_code.h>
-#include <std_expr.h>
-#include <namespace.h>
+#include <util/expr.h>
+#include <util/std_code.h>
+#include <util/std_expr.h>
+#include <util/namespace.h>
 
 #include "c_qualifiers.h"
 
 class expr2ct
 {
 public:
-  expr2ct(const namespacet &_ns):ns(_ns) { }
+  expr2ct(const namespacet &_ns):ns(_ns), sizeof_nesting(0) { }
   virtual ~expr2ct() { }
 
   virtual std::string convert(const typet &src);
@@ -35,19 +35,21 @@ protected:
 
   virtual std::string convert_rec(
     const typet &src,
-    const c_qualifierst &qualifiers);
+    const c_qualifierst &qualifiers,
+    const std::string &declarator);
 
   static std::string indent_str(unsigned indent);
 
   std::set<exprt> symbols;
   std::map<irep_idt, exprt> shorthands;
   std::set<irep_idt> ns_collision;
+  unsigned sizeof_nesting;
 
   void get_symbols(const exprt &expr);
   std::string id_shorthand(const exprt &expr) const;
 
   std::string convert_typecast(
-    const exprt &src, unsigned &precedence);
+    const typecast_exprt &src, unsigned &precedence);
 
   std::string convert_pointer_arithmetic(const exprt &src,
     unsigned &precedence);
@@ -93,6 +95,15 @@ protected:
   std::string convert_with(
     const exprt &src, unsigned precedence);
 
+  std::string convert_update(
+    const exprt &src, unsigned precedence);
+
+  std::string convert_member_designator(
+    const exprt &src);
+
+  std::string convert_index_designator(
+    const exprt &src);
+
   std::string convert_index(
     const exprt &src, unsigned precedence);
 
@@ -124,6 +135,14 @@ protected:
     const exprt &src, const std::string &symbol,
     unsigned precedence);
 
+  std::string convert_complex(
+    const exprt &src, 
+    unsigned precedence);
+
+  std::string convert_comma(
+    const exprt &src,
+    unsigned precedence);
+
   std::string convert_Hoare(const exprt &src);
 
   std::string convert_code(const codet &src);
@@ -136,7 +155,7 @@ protected:
   std::string convert_code_ifthenelse(const code_ifthenelset &src, unsigned indent);
   std::string convert_code_for(const code_fort &src, unsigned indent);
   std::string convert_code_while(const code_whilet &src, unsigned indent);
-  std::string convert_code_dowhile(const codet &src, unsigned indent);
+  std::string convert_code_dowhile(const code_dowhilet &src, unsigned indent);
   std::string convert_code_block(const code_blockt &src, unsigned indent);
   std::string convert_code_expression(const codet &src, unsigned indent);
   std::string convert_code_return(const codet &src, unsigned indent);
@@ -147,6 +166,7 @@ protected:
   std::string convert_code_switch(const codet &src, unsigned indent);
   std::string convert_code_continue(const codet &src, unsigned indent);
   std::string convert_code_decl(const codet &src, unsigned indent);
+  std::string convert_code_dead(const codet &src, unsigned indent);
   std::string convert_code_function_call(const code_function_callt &src, unsigned indent);
   std::string convert_code_lock(const codet &src, unsigned indent);
   std::string convert_code_unlock(const codet &src, unsigned indent);
