@@ -543,9 +543,9 @@ __gcc_v2df __builtin_ia32_pd_pd256(__gcc_v4df);
 __gcc_v4df __builtin_ia32_pd256_pd(__gcc_v2df);
 __gcc_v4sf __builtin_ia32_ps_ps256(__gcc_v8sf);
 __gcc_v8sf __builtin_ia32_ps256_ps(__gcc_v4sf);
-int __builtin_ia32_ptestc256(__gcc_v4di,__gcc_v4di,void *);
-int __builtin_ia32_ptestnzc256(__gcc_v4di,__gcc_v4di,void *);
-int __builtin_ia32_ptestz256(__gcc_v4di,__gcc_v4di,void *);
+int __builtin_ia32_ptestc256(__gcc_v4di,__gcc_v4di,...);
+int __builtin_ia32_ptestnzc256(__gcc_v4di,__gcc_v4di,...);
+int __builtin_ia32_ptestz256(__gcc_v4di,__gcc_v4di,...);
 __gcc_v8sf __builtin_ia32_rcpps256(__gcc_v8sf);
 __gcc_v4df __builtin_ia32_roundpd256(__gcc_v4df,int);
 __gcc_v8sf __builtin_ia32_roundps256(__gcc_v8sf,int);
@@ -593,18 +593,18 @@ __gcc_v2df __builtin_ia32_vpermilvarpd(__gcc_v2df,__gcc_v2di);
 __gcc_v4df __builtin_ia32_vpermilvarpd256(__gcc_v4df,__gcc_v4di);
 __gcc_v4sf __builtin_ia32_vpermilvarps(__gcc_v4sf,__gcc_v4si);
 __gcc_v8sf __builtin_ia32_vpermilvarps256(__gcc_v8sf,__gcc_v8si);
-int __builtin_ia32_vtestcpd(__gcc_v2df,__gcc_v2df,void *);
-int __builtin_ia32_vtestcpd256(__gcc_v4df,__gcc_v4df,void *);
-int __builtin_ia32_vtestcps(__gcc_v4sf,__gcc_v4sf,void *);
-int __builtin_ia32_vtestcps256(__gcc_v8sf,__gcc_v8sf,void *);
-int __builtin_ia32_vtestnzcpd(__gcc_v2df,__gcc_v2df,void *);
-int __builtin_ia32_vtestnzcpd256(__gcc_v4df,__gcc_v4df,void *);
-int __builtin_ia32_vtestnzcps(__gcc_v4sf,__gcc_v4sf,void *);
-int __builtin_ia32_vtestnzcps256(__gcc_v8sf,__gcc_v8sf,void *);
-int __builtin_ia32_vtestzpd(__gcc_v2df,__gcc_v2df,void *);
-int __builtin_ia32_vtestzpd256(__gcc_v4df,__gcc_v4df,void *);
-int __builtin_ia32_vtestzps(__gcc_v4sf,__gcc_v4sf,void *);
-int __builtin_ia32_vtestzps256(__gcc_v8sf,__gcc_v8sf,void *);
+int __builtin_ia32_vtestcpd(__gcc_v2df,__gcc_v2df,...);
+int __builtin_ia32_vtestcpd256(__gcc_v4df,__gcc_v4df,...);
+int __builtin_ia32_vtestcps(__gcc_v4sf,__gcc_v4sf,...);
+int __builtin_ia32_vtestcps256(__gcc_v8sf,__gcc_v8sf,...);
+int __builtin_ia32_vtestnzcpd(__gcc_v2df,__gcc_v2df,...);
+int __builtin_ia32_vtestnzcpd256(__gcc_v4df,__gcc_v4df,...);
+int __builtin_ia32_vtestnzcps(__gcc_v4sf,__gcc_v4sf,...);
+int __builtin_ia32_vtestnzcps256(__gcc_v8sf,__gcc_v8sf,...);
+int __builtin_ia32_vtestzpd(__gcc_v2df,__gcc_v2df,...);
+int __builtin_ia32_vtestzpd256(__gcc_v4df,__gcc_v4df,...);
+int __builtin_ia32_vtestzps(__gcc_v4sf,__gcc_v4sf,...);
+int __builtin_ia32_vtestzps256(__gcc_v8sf,__gcc_v8sf,...);
 void __builtin_ia32_vzeroall(void);
 void __builtin_ia32_vzeroupper(void);
 __gcc_v4df __builtin_ia32_xorpd256(__gcc_v4df,__gcc_v4df);
