@@ -9,5 +9,7 @@ int fcntl(int fd, int cmd, ...)
 {
 __CPROVER_HIDE:;
   int return_value;
+  (void)fd;
+  (void)cmd;
   return return_value;
 }
