@@ -12,4 +12,8 @@
 
 int kill(pid_t pid, int sig)
 {
+  (void)pid;
+  (void)sig;
+  _Bool error;
+  return error ? -1 : 0;
 }
