@@ -16,6 +16,7 @@ inline long int labs(long int i) { return __CPROVER_labs(i); }
 
 inline void exit(int status)
 {
+  (void)status;
   __CPROVER_assume(0);
 }
 
@@ -42,7 +43,7 @@ inline void *calloc(__CPROVER_size_t nmemb, __CPROVER_size_t size)
   void *res;
   res=malloc(total_size);
   #ifdef __CPROVER_STRING_ABSTRACTION
-  __CPROVER_is_zero_string(res);
+  __CPROVER_is_zero_string(res)=1;
   __CPROVER_zero_string_length(res)=0;
   //for(int i=0; i<nmemb*size; i++) res[i]=0;
   #else
@@ -109,34 +110,114 @@ inline void free(void *ptr)
   }
 }
 
+/* FUNCTION: strtol */
+
+#ifndef __CPROVER_ERRNO_H_INCLUDED
+#include <errno.h>
+#define __CPROVER_ERRNO_H_INCLUDED
+#endif
+
+#undef strtol
+#undef isdigit
+#undef isspace
+
+int isspace(int);
+int isdigit(int);
+
+inline long strtol(const char *nptr, char **endptr, int base)
+{
+  __CPROVER_HIDE:;
+  #ifdef __CPROVER_STRING_ABSTRACTION
+  __CPROVER_assert(__CPROVER_is_zero_string(nptr),
+    "zero-termination of argument of strtol");
+  #endif
+
+  if(base==1 || base<0 || base>36)
+  {
+    errno=EINVAL;
+    return 0;
+  }
+
+  int res=0;
+  _Bool in_number=0;
+  char sign=0;
+
+  // 32 chars is an arbitrarily chosen limit
+  int i=0;
+  for( ; i<31; ++i)
+  {
+    char ch=nptr[i];
+    if(ch==0)
+      break;
+    else if((base==0 || base==16) && !in_number &&
+            ch=='0' && (nptr[i+1]=='x' || nptr[i+1]=='X'))
+    {
+      base=16;
+      in_number=1;
+      ++i;
+    }
+    else if(base==0 && !in_number && ch=='0')
+    {
+      base=8;
+      in_number=1;
+    }
+    else if(!in_number && !sign && isspace(ch))
+      continue;
+    else if(!in_number && !sign && (ch=='-' || ch=='+'))
+      sign=ch;
+    else if(base>10 && ch>='a' && ch-'a'<base-10)
+    {
+      in_number=1;
+      res=res*base+ch-'a'+10;
+    }
+    else if(base>10 && ch>='A' && ch-'A'<base-10)
+    {
+      in_number=1;
+      res=res*base+ch-'A'+10;
+    }
+    else if(isdigit(ch))
+    {
+      in_number=1;
+      base=base==0 ? 10 : base;
+      res=res*base+ch-'0';
+    }
+    else
+      break;
+  }
+
+  if(endptr!=0)
+    *endptr=(char*)nptr+i;
+
+  if(sign=='-')
+    res*=-1;
+
+  return res;
+}
+
 /* FUNCTION: atoi */
 
 #undef atoi
+#undef strtol
+
+long strtol(const char *nptr, char **endptr, int base);
 
 inline int atoi(const char *nptr)
 {
   __CPROVER_HIDE:;
-  int res;
-  #ifdef __CPROVER_STRING_ABSTRACTION
-  __CPROVER_assert(__CPROVER_is_zero_string(nptr),
-    "zero-termination of argument of atoi");
-  #endif
-  return res;
+  return (int)strtol(nptr, (char **)0, 10);
 }
 
 /* FUNCTION: atol */
 
 #undef atol
+#undef strtol
+
+long strtol(const char *nptr, char **endptr, int base);
 
 inline long atol(const char *nptr)
 {
   __CPROVER_HIDE:;
-  long res;
-  #ifdef __CPROVER_STRING_ABSTRACTION
-  __CPROVER_assert(__CPROVER_is_zero_string(nptr),
-    "zero-termination of argument of atol");
-  #endif
-  return res;
+  return strtol(nptr, (char **)0, 10);
 }
 
 /* FUNCTION: getenv */
@@ -147,6 +228,7 @@ inline char *getenv(const char *name)
 {
   __CPROVER_HIDE:;
 
+  (void)*name;
   #ifdef __CPROVER_STRING_ABSTRACTION
   __CPROVER_assert(__CPROVER_is_zero_string(name),
     "zero-termination of argument of getenv");
