@@ -6,10 +6,11 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
-#include <arith_tools.h>
-#include <i2string.h>
+#include <util/arith_tools.h>
+#include <util/i2string.h>
+#include <util/std_expr.h>
 
 #include <ansi-c/c_types.h>
 
@@ -85,7 +86,7 @@ exprt convert_character_literal(
       throw "empty character literal";
     else if(value.size()==1)
     {
-      typet type=force_integer_type?int_type():char_type();
+      typet type=force_integer_type?signed_int_type():char_type();
       result=from_integer(value[0], type);
     }
     else if(value.size()>=2 && value.size()<=4)
@@ -100,7 +101,7 @@ exprt convert_character_literal(
       }
 
       // always integer, never char!
-      result=from_integer(x, int_type());
+      result=from_integer(x, signed_int_type());
     }
     else
       throw "literals with "+i2string(value.size())+
