@@ -10,10 +10,11 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <cstdlib>
 #include <cctype>
 
-#include <arith_tools.h>
-#include <config.h>
-#include <std_types.h>
-#include <expr_util.h>
+#include <util/arith_tools.h>
+#include <util/config.h>
+#include <util/std_types.h>
+#include <util/std_expr.h>
+#include <util/expr_util.h>
 
 #include "convert_integer_literal.h"
 
@@ -67,6 +68,14 @@ exprt convert_integer_literal(const std::string &src)
     std::string without_prefix(src, 2, std::string::npos);
     value=string2integer(without_prefix, 16);
   }
+  else if(src.size()>=2 && src[0]=='0' && tolower(src[1])=='b')
+  {
+    // binary; strip "0x"
+    // see http://gcc.gnu.org/onlinedocs/gcc/Binary-constants.html
+    base=2;
+    std::string without_prefix(src, 2, std::string::npos);
+    value=string2integer(without_prefix, 2);
+  }
   else if(src.size()>=2 && src[0]=='0')
   {
     // octal
@@ -106,10 +115,10 @@ exprt convert_integer_literal(const std::string &src)
   if(value<0)
     value_abs.negate();
 
-  bool is_hex_or_oct=(base==8) || (base==16);
+  bool is_hex_or_oct_or_bin=(base==8) || (base==16) || (base==2);
   
   #define FITS(width, signed) \
-    ((signed?!is_unsigned:(is_unsigned || is_hex_or_oct)) && \
+    ((signed?!is_unsigned:(is_unsigned || is_hex_or_oct_or_bin)) && \
     (power(2, signed?width-1:width)>value_abs))
 
   unsigned width;
