@@ -10,7 +10,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #define CPROVER_CPP_CONVERT_INTEGER_LITERAL_H
 
 #include <string>
-#include <expr.h>
+
+#include <util/expr.h>
 
 exprt convert_integer_literal(const std::string &src);
 
