@@ -6,6 +6,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
+#include <cctype>
+
 #include "parse_float.h"
 
 /*******************************************************************\
@@ -25,7 +27,7 @@ void parse_float(
   mp_integer &significand,
   mp_integer &exponent,
   unsigned &exponent_base,
-  bool &is_float, bool &is_long)
+  bool &is_float, bool &is_long, bool &is_imaginary)
 {
   // {digits}{dot}{digits}{exponent}?{floatsuffix}?
   // {digits}{dot}{exponent}?{floatsuffix}?
@@ -46,7 +48,7 @@ void parse_float(
   
   // is this hex?
   
-  if(src.size()>=2 && src[0]=='0' && src[1]=='x')
+  if(src.size()>=2 && src[0]=='0' && tolower(src[1])=='x')
   {
     // skip the 0x
     p+=2;
@@ -88,16 +90,20 @@ void parse_float(
 
     std::string str_number=str_whole_number+
                            str_fraction_part;
+                           
+    // The significand part is interpreted as a (decimal or hexadecimal)
+    // rational number; the digit sequence in the exponent part is
+    // interpreted as a decimal integer.
 
     if(str_number.empty())
       significand=0;
     else
-      significand=string2integer(str_number, 16);
+      significand=string2integer(str_number, 16); // hex
 
     if(str_exponent.empty())
       exponent=0;
     else
-      exponent=string2integer(str_exponent, 16);
+      exponent=string2integer(str_exponent, 10); // decimal
 
     // adjust exponent
     exponent-=str_fraction_part.size()*4; // each digit has 4 bits
@@ -106,7 +112,8 @@ void parse_float(
   {
     // get whole number part
     while(*p!='.' && *p!=0 && *p!='e' && *p!='E' &&
-          *p!='f' && *p!='F' && *p!='l' && *p!='L')
+          *p!='f' && *p!='F' && *p!='l' && *p!='L' &&
+          *p!='i' && *p!='I' && *p!='j' && *p!='J')
     {
       str_whole_number+=*p;
       p++;
@@ -118,7 +125,8 @@ void parse_float(
 
     // get fraction part
     while(*p!=0 && *p!='e' && *p!='E' &&
-           *p!='f' && *p!='F' && *p!='l' && *p!='L')
+          *p!='f' && *p!='F' && *p!='l' && *p!='L' &&
+          *p!='i' && *p!='I' && *p!='j' && *p!='J')
     {
       str_fraction_part+=*p;
       p++;
@@ -133,7 +141,8 @@ void parse_float(
       p++;
 
     // get exponent
-    while(*p!=0 && *p!='f' && *p!='F' && *p!='l' && *p!='L')
+    while(*p!=0 && *p!='f' && *p!='F' && *p!='l' && *p!='L' &&
+          *p!='i' && *p!='I' && *p!='j' && *p!='J')
     {
       str_exponent+=*p;
       p++;
@@ -157,7 +166,7 @@ void parse_float(
   }
 
   // get flags
-  is_float=is_long=false;
+  is_float=is_long=is_imaginary=false;
 
   while(*p!=0)
   {
@@ -165,6 +174,8 @@ void parse_float(
       is_float=true;
     else if(*p=='l' || *p=='L')
       is_long=true;
+    else if(*p=='i' || *p=='I' || *p=='j' || *p=='J')
+      is_imaginary=true;
 
     p++;
   }
