@@ -6,10 +6,10 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 \*******************************************************************/
 
-#include <ctype.h>
+#include <cctype>
 
-#include <i2string.h>
-#include <std_types.h>
+#include <util/i2string.h>
+#include <util/std_types.h>
 
 #include "type2name.h"
 
@@ -44,9 +44,9 @@ std::string type2name(const typet &type)
   else if(type.id()==ID_empty)
     result+="V";   
   else if(type.id()==ID_signedbv)
-    result+="S" + type.get(ID_width).as_string(); 
+    result+="S" + type.get_string(ID_width);
   else if(type.id()==ID_unsignedbv)
-    result+="U" + type.get(ID_width).as_string(); 
+    result+="U" + type.get_string(ID_width);
   else if(type.id()==ID_bool) 
     result+="B";
   else if(type.id()==ID_integer) 
@@ -56,9 +56,9 @@ std::string type2name(const typet &type)
   else if(type.id()==ID_complex) 
     result+="C";
   else if(type.id()==ID_floatbv) 
-    result+="F" + type.get(ID_width).as_string();
+    result+="F" + type.get_string(ID_width);
   else if(type.id()==ID_fixedbv) 
-    result+="X" + type.get(ID_width).as_string(); 
+    result+="X" + type.get_string(ID_width);
   else if(type.id()==ID_natural)
     result+="N";
   else if(type.id()==ID_pointer)
@@ -71,15 +71,23 @@ std::string type2name(const typet &type)
     const code_typet::argumentst arguments=t.arguments();
     result+="P(";
 
-    for(code_typet::argumentst::const_iterator it=arguments.begin();
+    for(code_typet::argumentst::const_iterator
+        it=arguments.begin();
         it!=arguments.end();
         it++)
     {      
+      if(it!=arguments.begin()) result+="|";
       result+=type2name(it->type());
-      result+="'" + it->get_identifier().as_string() + "'|";
     }
-    result.resize(result.size()-1);
-    result+=")";
+
+    if(t.has_ellipsis())
+    {
+      if(!arguments.empty()) result+="|";
+      result+="...";
+    }
+
+    result+=")->";
+    result+=type2name(t.return_type());
   }
   else if(type.id()==ID_array)
   {
@@ -87,28 +95,29 @@ std::string type2name(const typet &type)
     if(t.size().is_nil())
       result+="ARR?";
     else
-      result+="ARR"+t.size().get(ID_value).as_string();
+      result+="ARR"+t.size().get_string(ID_value);
   }
   else if(type.id()==ID_symbol)
   {
-    result+="SYM#"+type.get(ID_identifier).as_string()+"#";
+    result+="SYM#"+type.get_string(ID_identifier)+"#";
   }
   else if(type.id()==ID_struct || 
           type.id()==ID_union)
   {
     if(type.id()==ID_struct) result+="ST";
     if(type.id()==ID_union) result+="UN";
-    const struct_typet &t = to_struct_type(type);
-    const struct_typet::componentst &components = t.components();
+    const struct_union_typet &t=to_struct_union_type(type);
+    const struct_union_typet::componentst &components = t.components();
     result+="[";
-    for(struct_typet::componentst::const_iterator it = components.begin();
+    for(struct_union_typet::componentst::const_iterator
+        it=components.begin();
         it!=components.end();
         it++)
     {            
+      if(it!=components.begin()) result+="|";
       result+=type2name(it->type());
-      result+="'"+it->get(ID_name).as_string() + "'|";
+      result+="'"+it->get_string(ID_name)+"'|";
     }
-    result.resize(result.size()-1);
     result+="]";
   }
   else if(type.id()==ID_incomplete_struct)
@@ -116,16 +125,16 @@ std::string type2name(const typet &type)
   else if(type.id()==ID_incomplete_union)
     result +="UN?";
   else if(type.id()==ID_c_enum)
-    result +="EN" + type.get(ID_width).as_string();
+    result +="EN"+type.get_string(ID_width);
   else if(type.id()==ID_incomplete_c_enum)
     result +="EN?";
   else if(type.id()==ID_c_bitfield)
-    result+="BF"+type.get(ID_size).as_string(); 
+    result+="BF"+type.get_string(ID_size);
   else if(type.id()==ID_vector)
-    result+="VEC"+type.get(ID_size).as_string(); 
+    result+="VEC"+type.get_string(ID_size);
   else
     throw (std::string("Unknown type '") + 
-           type.id().as_string() + 
+           type.id_string() + 
            "' encountered."); 
     
   if(type.has_subtype())
