@@ -6,14 +6,15 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <sstream>
 #include <fstream>
 #include <cstdlib>
+#include <iostream>
+#include <memory>
 
-#include <i2string.h>
-#include <location.h>
-#include <time_stopping.h>
-#include <message_stream.h>
+#include <util/i2string.h>
+#include <util/location.h>
+#include <util/time_stopping.h>
+#include <util/message_stream.h>
 
 #include <langapi/mode.h>
 #include <langapi/languages.h>
@@ -26,9 +27,11 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <goto-symex/slice.h>
 #include <goto-symex/slice_by_trace.h>
 #include <goto-symex/xml_goto_trace.h>
+#include <goto-symex/memory_model_sc.h>
+#include <goto-symex/memory_model_tso.h>
+#include <goto-symex/memory_model_pso.h>
 
 #include <solvers/sat/satcheck_minisat2.h>
-#include <solvers/prop/cover_goals.h>
 
 #include "bmc.h"
 #include "bv_cbmc.h"
@@ -64,7 +67,7 @@ Function: bmct::error_trace
 
 void bmct::error_trace(const prop_convt &prop_conv)
 {
-  status("Building error trace");
+  status() << "Building error trace" << eom;
 
   goto_tracet goto_trace;
   build_goto_trace(equation, prop_conv, ns, goto_trace);
@@ -85,19 +88,15 @@ void bmct::error_trace(const prop_convt &prop_conv)
   switch(ui)
   {
   case ui_message_handlert::PLAIN:
-    std::cout << std::endl << "Counterexample:" << std::endl;
+    std::cout << "\n" << "Counterexample:" << "\n";
     show_goto_trace(std::cout, ns, goto_trace);
     break;
   
-  case ui_message_handlert::OLD_GUI:
-    show_goto_trace_gui(std::cout, ns, goto_trace);
-    break;
-  
   case ui_message_handlert::XML_UI:
     {
       xmlt xml;
       convert(ns, goto_trace, xml);
-      std::cout << xml << std::endl;
+      std::cout << xml << "\n";
     }
     break;
   
@@ -146,7 +145,8 @@ Function: bmct::run_decision_procedure
 decision_proceduret::resultt
 bmct::run_decision_procedure(prop_convt &prop_conv)
 {
-  status("Passing problem to "+prop_conv.decision_procedure_text());
+  status() << "Passing problem to " 
+           << prop_conv.decision_procedure_text() << eom;
 
   prop_conv.set_message_handler(get_message_handler());
   prop_conv.set_verbosity(get_verbosity());
@@ -156,19 +156,15 @@ bmct::run_decision_procedure(prop_convt &prop_conv)
   
   do_conversion(prop_conv);  
 
-  status("Running "+prop_conv.decision_procedure_text());
+  status() << "Running " << prop_conv.decision_procedure_text() << eom;
 
   decision_proceduret::resultt dec_result=prop_conv.dec_solve();
   // output runtime
 
   {
-    std::ostringstream str;
     fine_timet sat_stop=current_time();
-
-    str << "Runtime decision procedure: ";
-    output_time(sat_stop-sat_start, str);
-    str << "s";
-    status(str.str());
+    status() << "Runtime decision procedure: "
+             << (sat_stop-sat_start) << "s" << eom;
   }
 
   return dec_result;
@@ -188,19 +184,10 @@ Function: bmct::report_success
 
 void bmct::report_success()
 {
-  status("VERIFICATION SUCCESSFUL");
+  result() << "VERIFICATION SUCCESSFUL" << eom;
 
   switch(ui)
   {
-  case ui_message_handlert::OLD_GUI:
-    std::cout << "SUCCESS" << std::endl
-              << "Verification successful" << std::endl
-              << ""     << std::endl
-              << ""     << std::endl
-              << ""     << std::endl
-              << ""     << std::endl;
-    break;
-    
   case ui_message_handlert::PLAIN:
     break;
     
@@ -209,7 +196,7 @@ void bmct::report_success()
       xmlt xml("cprover-status");
       xml.data="SUCCESS";
       std::cout << xml;
-      std::cout << std::endl;
+      std::cout << "\n";
     }
     break;
     
@@ -232,13 +219,10 @@ Function: bmct::report_failure
 
 void bmct::report_failure()
 {
-  status("VERIFICATION FAILED");
+  result() << "VERIFICATION FAILED" << eom;
 
   switch(ui)
   {
-  case ui_message_handlert::OLD_GUI:
-    break;
-    
   case ui_message_handlert::PLAIN:
     break;
     
@@ -247,7 +231,7 @@ void bmct::report_failure()
       xmlt xml("cprover-status");
       xml.data="FAILURE";
       std::cout << xml;
-      std::cout << std::endl;
+      std::cout << "\n";
     }
     break;
     
@@ -274,7 +258,7 @@ void bmct::show_program()
 
   languagest languages(ns, new_ansi_c_language());
   
-  std::cout << std::endl << "Program constraints:" << std::endl;
+  std::cout << "\n" << "Program constraints:" << "\n";
 
   for(symex_target_equationt::SSA_stepst::const_iterator
       it=equation.SSA_steps.begin();
@@ -284,13 +268,13 @@ void bmct::show_program()
     {
       std::string string_value;
       languages.from_expr(it->cond_expr, string_value);
-      std::cout << "(" << count << ") " << string_value << std::endl;
+      std::cout << "(" << count << ") " << string_value << "\n";
 
-      if(!it->guard_expr.is_true())
+      if(!it->guard.is_true())
       {
-        languages.from_expr(it->guard_expr, string_value);
+        languages.from_expr(it->guard, string_value);
         std::cout << std::string(i2string(count).size()+3, ' ');
-        std::cout << "guard: " << string_value << std::endl;
+        std::cout << "guard: " << string_value << "\n";
       }
       
       count++;
@@ -300,13 +284,13 @@ void bmct::show_program()
       std::string string_value;
       languages.from_expr(it->cond_expr, string_value);
       std::cout << "(" << count << ") ASSERT("
-                << string_value <<") " << std::endl;
+                << string_value <<") " << "\n";
 
-      if(!it->guard_expr.is_true())
+      if(!it->guard.is_true())
       {
-        languages.from_expr(it->guard_expr, string_value);
+        languages.from_expr(it->guard, string_value);
         std::cout << std::string(i2string(count).size()+3, ' ');
-        std::cout << "guard: " << string_value << std::endl;
+        std::cout << "guard: " << string_value << "\n";
       }
 
       count++;
@@ -328,12 +312,23 @@ Function: bmct::run
 
 bool bmct::run(const goto_functionst &goto_functions)
 {
+  const std::string mm=options.get_option("mm");
+  std::auto_ptr<memory_model_baset> memory_model(0);
+  if(mm.empty() || mm=="sc")
+    memory_model.reset(new memory_model_sct(ns));
+  else if(mm=="tso")
+    memory_model.reset(new memory_model_tsot(ns));
+  else if(mm=="pso")
+    memory_model.reset(new memory_model_psot(ns));
+  else
+    throw "Invalid memory model "+mm+" -- use one of sc, tso, pso";
+
   //symex.total_claims=0;
   symex.set_message_handler(get_message_handler());
   symex.set_verbosity(get_verbosity());
   symex.options=options;
 
-  status("Starting Bounded Model Checking");
+  status() << "Starting Bounded Model Checking" << eom;
 
   symex.last_location.make_nil();
 
@@ -342,7 +337,15 @@ bool bmct::run(const goto_functionst &goto_functions)
     // get unwinding info
     setup_unwind();
 
+    // perform symbolic execution
     symex(goto_functions);
+
+    // add a partial ordering, if required    
+    if(equation.has_threads())
+    {
+      memory_model->set_message_handler(get_message_handler());
+      (*memory_model)(equation);
+    }
   }
 
   catch(std::string &error_str)
@@ -363,14 +366,13 @@ bool bmct::run(const goto_functionst &goto_functions)
 
   catch(std::bad_alloc)
   {
-    message_streamt message_stream(get_message_handler());
-    message_stream.error("Out of memory");
+    error() << "Out of memory" << eom;
     return true;
   }
 
-  print(8, "size of program expression: "+
-           i2string(equation.SSA_steps.size())+
-           " assignments");
+  statistics() << "size of program expression: "
+               << equation.SSA_steps.size()
+               << " steps" << eom;
 
   try
   {
@@ -382,17 +384,27 @@ bool bmct::run(const goto_functionst &goto_functions)
 	(options.get_option("slice-by-trace"), equation);
     }
 
-    if(options.get_bool_option("slice-formula"))
+    if(equation.has_threads())
     {
-      slice(equation);
-      print(8, "slicing removed "+
-        i2string(equation.count_ignored_SSA_steps())+" assignments");
+      // we should build a thread-aware SSA slicer
+      statistics() << "no slicing due to threads" << eom;
     }
     else
     {
-      simple_slice(equation);
-      print(8, "simple slicing removed "+
-        i2string(equation.count_ignored_SSA_steps())+" assignments");
+      if(options.get_bool_option("slice-formula"))
+      {
+        slice(equation);
+        statistics() << "slicing removed "
+                     << equation.count_ignored_SSA_steps()
+                     << " assignments" << eom;
+      }
+      else
+      {
+        simple_slice(equation);
+        statistics() << "simple slicing removed "
+                     << equation.count_ignored_SSA_steps()
+                     << " assignments" << eom;
+      }
     }
 
     if(options.get_bool_option("program-only"))
@@ -402,11 +414,9 @@ bool bmct::run(const goto_functionst &goto_functions)
     }
 
     {
-      std::string msg;
-      msg="Generated "+i2string(symex.total_claims)+
-          " VCC(s), "+i2string(symex.remaining_claims)+
-          " remaining after simplification";
-      print(8, msg);
+      statistics() << "Generated " << symex.total_claims
+                   << " VCC(s), " << symex.remaining_claims
+                   << " remaining after simplification" << eom;
     }
 
     if(options.get_bool_option("show-vcc"))
@@ -470,161 +480,13 @@ bool bmct::run(const goto_functionst &goto_functions)
 
   catch(std::bad_alloc)
   {
-    error("Out of memory");
+    error() << "Out of memory" << eom;
     return true;
   }
 }
 
 /*******************************************************************\
 
-Function: bmct::all_claims
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-struct goalt
-{
-  bvt bv;
-  std::string description;
-
-  explicit goalt(const goto_programt::instructiont &instruction)
-  {
-    irep_idt claim_id=instruction.location.get_claim();
-    irep_idt comment=instruction.location.get_comment();
-    std::string text=id2string(claim_id);
-    if(comment!="")
-      text+=", "+id2string(comment);
-    description=text;
-  }
-  
-  goalt()
-  {
-  }
-};
-
-bool bmct::all_claims(const goto_functionst &goto_functions)
-{
-  satcheck_minisat_no_simplifiert satcheck;
-  satcheck.set_message_handler(get_message_handler());
-  satcheck.set_verbosity(get_verbosity());
-  
-  bv_cbmct bv_cbmc(ns, satcheck);
-  
-  if(options.get_option("arrays-uf")=="never")
-    bv_cbmc.unbounded_array=bv_cbmct::U_NONE;
-  else if(options.get_option("arrays-uf")=="always")
-    bv_cbmc.unbounded_array=bv_cbmct::U_ALL;
-    
-  prop_convt &prop_conv=bv_cbmc;
-
-  status("Passing problem to "+prop_conv.decision_procedure_text());
-
-  prop_conv.set_message_handler(get_message_handler());
-  prop_conv.set_verbosity(get_verbosity());
-
-  // stop the time
-  fine_timet sat_start=current_time();
-  
-  do_conversion(prop_conv);  
-  
-  // collect _all_ goals in `goal_map'
-  typedef std::map<irep_idt, goalt> goal_mapt;
-  goal_mapt goal_map;
-  
-  forall_goto_functions(f_it, goto_functions)
-    forall_goto_program_instructions(i_it, f_it->second.body)
-      if(i_it->is_assert())
-        goal_map[i_it->location.get_claim()]=goalt(*i_it);
-
-  // get the conditions for these goals from formula
-
-  for(symex_target_equationt::SSA_stepst::iterator
-      it=equation.SSA_steps.begin();
-      it!=equation.SSA_steps.end();
-      it++)
-  {
-    if(it->is_assert())
-    {
-      irep_idt claim_id=it->source.pc->location.get_claim();
-      goal_map[claim_id].bv.push_back(it->cond_literal);
-    }
-  }
-  
-  cover_goalst cover_goals(prop_conv);
-  
-  for(goal_mapt::const_iterator
-      it=goal_map.begin();
-      it!=goal_map.end();
-      it++)
-  {
-    // the following is TRUE if the bv is empty
-    literalt p=prop_conv.prop.lnot(prop_conv.prop.land(it->second.bv));
-    cover_goals.add(p, it->second.description);
-  }
-
-  status("Running "+prop_conv.decision_procedure_text());
-
-  cover_goals();  
-
-  // output runtime
-
-  {
-    std::ostringstream str;
-    fine_timet sat_stop=current_time();
-
-    str << "Runtime decision procedure: ";
-    output_time(sat_stop-sat_start, str);
-    str << "s";
-    status(str.str());
-  }
-  
-  // report
-  if(ui==ui_message_handlert::XML_UI)
-  {
-    std::list<cover_goalst::cover_goalt>::const_iterator g_it=
-      cover_goals.goals.begin();
-      
-    for(goal_mapt::const_iterator
-        it=goal_map.begin();
-        it!=goal_map.end();
-        it++, g_it++)
-    {
-      xmlt xml_result("result");
-      xml_result.set_attribute("claim", id2string(it->first));
-
-      xml_result.set_attribute("status",
-        g_it->covered?"FAILURE":"SUCCESS");
-      
-      std::cout << xml_result << std::endl;
-    }
-  }
-  else
-  {
-    status("");
-    status("** Results:");
-    for(std::list<cover_goalst::cover_goalt>::const_iterator
-        g_it=cover_goals.goals.begin();
-        g_it!=cover_goals.goals.end();
-        g_it++)
-      status(g_it->description+": "+(g_it->covered?"FAILED":"OK"));
-
-    status("");
-  }
-  
-  status("** "+i2string(cover_goals.number_covered())+
-         " of "+i2string(cover_goals.size())+" failed ("+
-         i2string(cover_goals.iterations())+" iterations)");
-  
-  return false;
-}
-
-/*******************************************************************\
-
 Function: bmct::decide
 
   Inputs:
@@ -659,7 +521,7 @@ bool bmct::decide(prop_convt &prop_conv)
     break;
 
   default:
-    error("decision procedure failed");
+    error() << "decision procedure failed" << eom;
   }
 
   return result;
