@@ -6,8 +6,9 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 \*******************************************************************/
 
-#include <location.h>
-#include <std_types.h>
+#include <util/location.h>
+#include <util/std_types.h>
+
 #include <ansi-c/c_types.h>
 
 #include "cpp_type2name.h"
@@ -50,14 +51,14 @@ Function: cpp_declarator_convertert::convert
 \*******************************************************************/
 
 symbolt &cpp_declarator_convertert::convert(
-  const typet &type,
+  const typet &declaration_type,
   const cpp_storage_spect &storage_spec,
   const cpp_member_spect &member_spec,
   cpp_declaratort &declarator)
 {
-  assert(type.is_not_nil());
+  assert(declaration_type.is_not_nil());
 
-  if(type.id()=="cpp-cast-operator")
+  if(declaration_type.id()=="cpp-cast-operator")
   {
     typet type;
     type.swap(declarator.name().get_sub().back());
@@ -70,11 +71,11 @@ symbolt &cpp_declarator_convertert::convert(
   }
 
   assert(declarator.id()==ID_cpp_declarator);
-  final_type=declarator.merge_type(type);
+  final_type=declarator.merge_type(declaration_type);
   assert(final_type.is_not_nil());
   
   cpp_template_args_non_tct template_args;
-
+  
   // run resolver on scope
   {
     cpp_save_scopet save_scope(cpp_typecheck.cpp_scopes);
@@ -85,10 +86,10 @@ symbolt &cpp_declarator_convertert::convert(
       declarator.name(), base_name, template_args);
 
     scope=&cpp_typecheck.cpp_scopes.current_scope();
+
+    // check the declarator-part of the type, in that scope
+    cpp_typecheck.typecheck_type(final_type);
   }
-  
-  // check the type
-  cpp_typecheck.typecheck_type(final_type);
 
   is_code=is_code_type(final_type);
 
@@ -116,10 +117,10 @@ symbolt &cpp_declarator_convertert::convert(
     }
 
     // try static first
-    contextt::symbolst::iterator c_it=
-      cpp_typecheck.context.symbols.find(final_identifier);
+    symbol_tablet::symbolst::iterator c_it=
+      cpp_typecheck.symbol_table.symbols.find(final_identifier);
 
-    if(c_it==cpp_typecheck.context.symbols.end())
+    if(c_it==cpp_typecheck.symbol_table.symbols.end())
     {
       // adjust type if it's a non-static member function
       if(final_type.id()==ID_code)
@@ -129,9 +130,9 @@ symbolt &cpp_declarator_convertert::convert(
       get_final_identifier();
 
       // try again
-      c_it=cpp_typecheck.context.symbols.find(final_identifier);
+      c_it=cpp_typecheck.symbol_table.symbols.find(final_identifier);
 
-      if(c_it==cpp_typecheck.context.symbols.end())
+      if(c_it==cpp_typecheck.symbol_table.symbols.end())
       {
         cpp_typecheck.err_location(declarator.name());
         cpp_typecheck.str << "member `" << base_name
@@ -141,7 +142,7 @@ symbolt &cpp_declarator_convertert::convert(
       }
     }
 
-    assert(c_it!=cpp_typecheck.context.symbols.end());
+    assert(c_it!=cpp_typecheck.symbol_table.symbols.end());
 
     symbolt &symbol=c_it->second;
 
@@ -208,10 +209,10 @@ symbolt &cpp_declarator_convertert::convert(
     }
 
     // already there?
-    contextt::symbolst::iterator c_it=
-      cpp_typecheck.context.symbols.find(final_identifier);
+    symbol_tablet::symbolst::iterator c_it=
+      cpp_typecheck.symbol_table.symbols.find(final_identifier);
 
-    if(c_it==cpp_typecheck.context.symbols.end())
+    if(c_it==cpp_typecheck.symbol_table.symbols.end())
       return convert_new_symbol(storage_spec, member_spec, declarator);
 
     symbolt &symbol=c_it->second;
@@ -325,11 +326,11 @@ void cpp_declarator_convertert::combine_types(
 
   cpp_typecheck.err_location(location);
   cpp_typecheck.str << "symbol `" << symbol.display_name()
-                    << "' already declared with different type"
+                    << "' already declared with different type:"
                     << std::endl;
-  cpp_typecheck.str << "previous type: "
+  cpp_typecheck.str << "original: "
                     << cpp_typecheck.to_string(symbol.type) << std::endl;
-  cpp_typecheck.str << "new type: "
+  cpp_typecheck.str << "     new: "
                     << cpp_typecheck.to_string(final_type);
   throw 0;
 }
@@ -445,18 +446,30 @@ void cpp_declarator_convertert::get_final_identifier()
 
   if(is_code)
   {
-    // is there already an `extern "C"' function with the same name?
-    
-    if(linkage_spec==ID_auto &&
-       scope->prefix=="" &&
-       cpp_typecheck.context.symbols.find("c::"+identifier)!=
-       cpp_typecheck.context.symbols.end())
+    if(linkage_spec==ID_C)
     {
+      // fine as is
     }
-    else if(linkage_spec!=ID_C)
+    else if(linkage_spec==ID_auto ||
+            linkage_spec==ID_cpp)
     {
-      // add C++ decoration
-      identifier+=id2string(cpp_typecheck.function_identifier(final_type));
+      // Is there already an `extern "C"' function with the same name
+      // and the same signature?
+      symbol_tablet::symbolst::const_iterator
+        c_it=cpp_typecheck.symbol_table.symbols.find("c::"+identifier);
+        
+      if(c_it!=cpp_typecheck.symbol_table.symbols.end() &&
+         c_it->second.type.id()==ID_code &&
+         cpp_typecheck.function_identifier(final_type)==
+         cpp_typecheck.function_identifier(c_it->second.type))
+      {
+        // leave as is, no decoration
+      }
+      else
+      {
+        // add C++ decoration
+        identifier+=id2string(cpp_typecheck.function_identifier(final_type));
+      }
     }
   }
 
@@ -508,18 +521,24 @@ symbolt &cpp_declarator_convertert::convert_new_symbol(
 
   if(!symbol.is_type)
   {
-    if(!is_code)
+    if(is_code)
+    {
+      // it is a function
+      if(storage_spec.is_static())
+        symbol.is_file_local=true;
+    }
+    else
     {
       // it is a variable
-      symbol.is_statevar=true;
-      symbol.lvalue = !is_reference(symbol.type) &&
-                      !(symbol.type.get_bool(ID_C_constant) &&
-                        is_number(symbol.type) &&
-                        symbol.value.id() == ID_constant);
+      symbol.is_state_var=true;
+      symbol.is_lvalue = !is_reference(symbol.type) &&
+                         !(symbol.type.get_bool(ID_C_constant) &&
+                         is_number(symbol.type) &&
+                         symbol.value.id() == ID_constant);
 
       if(cpp_typecheck.cpp_scopes.current_scope().is_global_scope())
       {
-        symbol.static_lifetime=true;
+        symbol.is_static_lifetime=true;
 
         if(storage_spec.is_extern())
           symbol.is_extern=true;
@@ -528,8 +547,8 @@ symbolt &cpp_declarator_convertert::convert_new_symbol(
       {
         if(storage_spec.is_static())
         {
-          symbol.static_lifetime=true;
-          symbol.file_local=true;
+          symbol.is_static_lifetime=true;
+          symbol.is_file_local=true;
         }
         else if(storage_spec.is_extern())
         {
@@ -538,18 +557,16 @@ symbolt &cpp_declarator_convertert::convert_new_symbol(
         }
       }
     }
-    else
-      symbol.theorem=true;
   }
 
-  if(symbol.static_lifetime)
+  if(symbol.is_static_lifetime)
     cpp_typecheck.dynamic_initializations.push_back(symbol.name);
 
   // move early, it must be visible before doing any value
   symbolt *new_symbol;
 
-  if(cpp_typecheck.context.move(symbol, new_symbol))
-    throw "cpp_typecheckt::convert_declarator: context.move() failed";
+  if(cpp_typecheck.symbol_table.move(symbol, new_symbol))
+    throw "cpp_typecheckt::convert_declarator: symbol_table.move() failed";
 
   if(!is_code)
   {
@@ -685,13 +702,14 @@ void cpp_declarator_convertert::main_function_rules(
     }
 
     const typet &return_type=
-      static_cast<const typet &>(
-        symbol.type.find(ID_return_type));
+      to_code_type(symbol.type).return_type();
 
-    if(return_type!=int_type())
+    if(return_type!=signed_int_type())
     {
-      cpp_typecheck.err_location(symbol.location);
-      throw "main must return int";
+      // Too many embedded compilers ignore this rule.
+      //
+      //cpp_typecheck.err_location(symbol.location);
+      //throw "main must return int";
     }
   }
 }
