@@ -9,7 +9,7 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 #ifndef CPROVER_CPP_DECLARATOR_CONVERTER_H
 #define CPROVER_CPP_DECLARATOR_CONVERTER_H
 
-#include <symbol.h>
+#include <util/symbol.h>
 
 #include "cpp_declarator.h"
 #include "cpp_declaration.h"
