@@ -6,13 +6,12 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 \*******************************************************************/
 
-#include <string.h>
-
+#include <cstring>
 #include <sstream>
 #include <fstream>
 
-#include <config.h>
-#include <replace_symbol.h>
+#include <util/config.h>
+#include <util/replace_symbol.h>
 
 #include <linking/linking.h>
 #include <linking/entry_point.h>
@@ -20,12 +19,11 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 #include <ansi-c/c_preprocess.h>
 #include <ansi-c/trans_unit.h>
 
-#include "internal_additions.h"
+#include "cpp_internal_additions.h"
 #include "cpp_language.h"
 #include "expr2cpp.h"
 #include "cpp_parser.h"
 #include "cpp_typecheck.h"
-#include "cpp_final.h"
 
 /*******************************************************************\
 
@@ -47,7 +45,6 @@ std::set<std::string> cpp_languaget::extensions() const
   s.insert("CPP");
   s.insert("cc");
   s.insert("c++");
-  s.insert("ipp");
   s.insert("ii");
   s.insert("cxx");
   
@@ -93,8 +90,11 @@ bool cpp_languaget::preprocess(
   std::ostream &outstream,
   message_handlert &message_handler)
 {
+  if(config.ansi_c.mode==configt::ansi_ct::MODE_GCC_C)
+    config.ansi_c.mode=configt::ansi_ct::MODE_GCC_CPP;
+
   if(path=="")
-    return c_preprocess(PREPROCESS_CPP, instream, outstream, message_handler);
+    return c_preprocess(instream, outstream, message_handler);
 
   // check extension
 
@@ -111,7 +111,7 @@ bool cpp_languaget::preprocess(
     return false;
   }
 
-  return c_preprocess(PREPROCESS_CPP, path, outstream, message_handler);
+  return c_preprocess(path, outstream, message_handler);
 }
 
 /*******************************************************************\
@@ -149,30 +149,31 @@ bool cpp_languaget::parse(
   // parsing
 
   cpp_parser.clear();
-  cpp_parser.filename=path;
+  cpp_parser.set_file(path);
   cpp_parser.in=&i_preprocessed;
   cpp_parser.set_message_handler(message_handler);
   cpp_parser.grammar=cpp_parsert::LANGUAGE;
 
   switch(config.ansi_c.mode)
   {
-  case configt::ansi_ct::MODE_CODEWARRIOR:
+  case configt::ansi_ct::MODE_CODEWARRIOR_C_CPP:
     cpp_parser.mode=cpp_parsert::CW;
     break;
    
-  case configt::ansi_ct::MODE_VISUAL_STUDIO:
+  case configt::ansi_ct::MODE_VISUAL_STUDIO_C_CPP:
     cpp_parser.mode=cpp_parsert::MSC;
     break;
     
-  case configt::ansi_ct::MODE_ANSI:
+  case configt::ansi_ct::MODE_ANSI_C_CPP:
     cpp_parser.mode=cpp_parsert::ANSI;
     break;
     
-  case configt::ansi_ct::MODE_GCC:
+  case configt::ansi_ct::MODE_GCC_C:
+  case configt::ansi_ct::MODE_GCC_CPP:
     cpp_parser.mode=cpp_parsert::GCC;
     break;
     
-  case configt::ansi_ct::MODE_ARM:
+  case configt::ansi_ct::MODE_ARM_C_CPP:
     cpp_parser.mode=cpp_parsert::ARM;
     break;
     
@@ -206,18 +207,18 @@ Function: cpp_languaget::typecheck
 \*******************************************************************/
 
 bool cpp_languaget::typecheck(
-  contextt &context,
+  symbol_tablet &symbol_table,
   const std::string &module,
   message_handlert &message_handler)
 {
   if(module=="") return false;
 
-  contextt new_context;
+  symbol_tablet new_symbol_table;
 
-  if(cpp_typecheck(cpp_parse_tree, new_context, module, message_handler))
+  if(cpp_typecheck(cpp_parse_tree, new_symbol_table, module, message_handler))
     return true;
 
-  return linking(context, new_context, message_handler);
+  return linking(symbol_table, new_symbol_table, message_handler);
 }
 
 /*******************************************************************\
@@ -233,11 +234,10 @@ Function: cpp_languaget::final
 \*******************************************************************/
 
 bool cpp_languaget::final(
-  contextt &context,
+  symbol_tablet &symbol_table,
   message_handlert &message_handler)
 {
-  if(cpp_final(context, message_handler)) return true;
-  if(entry_point(context, "c::main", message_handler)) return true;
+  if(entry_point(symbol_table, "c::main", message_handler)) return true;
 
   return false;
 }
@@ -416,7 +416,7 @@ bool cpp_languaget::to_expr(
   // parsing
 
   cpp_parser.clear();
-  cpp_parser.filename="";
+  cpp_parser.set_file(irep_idt());
   cpp_parser.in=&i_preprocessed;
   cpp_parser.set_message_handler(message_handler);
   cpp_parser.grammar=cpp_parsert::EXPRESSION;
