@@ -11,7 +11,7 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 /*! \defgroup gr_cpp C++ front-end */
 
-#include <language.h>
+#include <util/language.h>
 
 #include "cpp_parse_tree.h"
 
@@ -33,19 +33,19 @@ public:
     message_handlert &message_handler);
 
   virtual bool typecheck(
-    contextt &context,
+    symbol_tablet &symbol_table,
     const std::string &module,
     message_handlert &message_handler);
 
-  bool merge_context(
-    contextt &dest,
-    contextt &src,
+  bool merge_symbol_table(
+    symbol_tablet &dest,
+    symbol_tablet &src,
     message_handlert &message_handler,
     const std::string &module,
     class replace_symbolt &replace_symbol) const; 
 
   virtual bool final(
-    contextt &context,
+    symbol_tablet &symbol_table,
     message_handlert &message_handler);
 
   virtual void show_parse(std::ostream &out);
