@@ -9,7 +9,8 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 #ifndef CPROVER_CPP_TEMPLATE_TYPE_H
 #define CPROVER_CPP_TEMPLATE_TYPE_H
 
-#include <type.h>
+#include <util/type.h>
+#include <util/expr.h>
 
 #include "cpp_template_parameter.h"
 
