@@ -7,11 +7,12 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 \*******************************************************************/
 
 #include <algorithm>
-#include <expr_util.h>
-#include <arith_tools.h>
-#include <i2string.h>
-#include <location.h>
-#include <symbol.h>
+
+#include <util/expr_util.h>
+#include <util/arith_tools.h>
+#include <util/i2string.h>
+#include <util/location.h>
+#include <util/symbol.h>
 
 #include <linking/zero_initializer.h>
 #include <ansi-c/c_typecast.h>
@@ -102,6 +103,8 @@ void cpp_typecheckt::convert(cpp_itemt &item)
     convert(item.get_namespace_spec());
   else if(item.is_using())
     convert(item.get_using());
+  else if(item.is_static_assert())
+    convert(item.get_static_assert());
   else
   {
     err_location(item);
@@ -153,11 +156,11 @@ Function: cpp_typecheck
 
 bool cpp_typecheck(
   cpp_parse_treet &cpp_parse_tree,
-  contextt &context,
+  symbol_tablet &symbol_table,
   const std::string &module,
   message_handlert &message_handler)
 {
-  cpp_typecheckt cpp_typecheck(cpp_parse_tree, context, module, message_handler);
+  cpp_typecheckt cpp_typecheck(cpp_parse_tree, symbol_table, module, message_handler);
   return cpp_typecheck.typecheck_main();
 }
 
@@ -178,18 +181,18 @@ bool cpp_typecheck(
   message_handlert &message_handler,
   const namespacet &ns)
 {
-  contextt context;
+  symbol_tablet symbol_table;
   cpp_parse_treet cpp_parse_tree;
 
-  cpp_typecheckt cpp_typecheck(cpp_parse_tree, context,
-                               ns.get_context(), "", message_handler);
+  cpp_typecheckt cpp_typecheck(cpp_parse_tree, symbol_table,
+                               ns.get_symbol_table(), "", message_handler);
 
   try
   {
     cpp_typecheck.typecheck_expr(expr);
   }
 
-  catch(int e)
+  catch(int)
   {
     cpp_typecheck.error();
   }
@@ -240,11 +243,11 @@ void cpp_typecheckt::static_and_dynamic_initialization()
 
   // fill in any missing zero initializers
   // for static initialization
-  Forall_symbols(s_it, context.symbols)
+  Forall_symbols(s_it, symbol_table.symbols)
   {
     symbolt &symbol=s_it->second;
 
-    if(!symbol.static_lifetime)
+    if(!symbol.is_static_lifetime)
       continue;
       
     if(symbol.mode!=ID_cpp)
@@ -263,7 +266,7 @@ void cpp_typecheckt::static_and_dynamic_initialization()
     if(symbol.is_extern)
       continue;
 
-    if(!symbol.lvalue)
+    if(!symbol.is_lvalue)
       continue;
 
     if(cpp_is_pod(symbol.type))
@@ -285,7 +288,7 @@ void cpp_typecheckt::static_and_dynamic_initialization()
       d_it!=dynamic_initializations.end();
       d_it++)
   {
-    symbolt &symbol=context.symbols.find(*d_it)->second;
+    symbolt &symbol=symbol_table.symbols.find(*d_it)->second;
 
     if(symbol.is_extern)
       continue;
@@ -294,7 +297,7 @@ void cpp_typecheckt::static_and_dynamic_initialization()
     if(cpp_is_pod(symbol.type))
       continue;
 
-    assert(symbol.static_lifetime);
+    assert(symbol.is_static_lifetime);
     assert(!symbol.is_type);
     assert(symbol.type.id()!=ID_code);
 
@@ -343,9 +346,8 @@ void cpp_typecheckt::static_and_dynamic_initialization()
   init_symbol.type.set("initialization", true);
   init_symbol.is_type=false;
   init_symbol.is_macro=false;
-  init_symbol.theorem=true;
 
-  context.move(init_symbol);
+  symbol_table.move(init_symbol);
 
   disable_access_control=false;
 }
@@ -370,7 +372,7 @@ void cpp_typecheckt::do_not_typechecked()
   {
     cont = false;
 
-    Forall_symbols(s_it, context.symbols)
+    Forall_symbols(s_it, symbol_table.symbols)
     {
       symbolt &symbol=s_it->second;
 
@@ -403,7 +405,7 @@ void cpp_typecheckt::do_not_typechecked()
   }
   while(cont);
 
-  Forall_symbols(s_it, context.symbols)
+  Forall_symbols(s_it, symbol_table.symbols)
   {
     symbolt &symbol=s_it->second;
     if(symbol.value.id()=="cpp_not_typechecked")
@@ -425,18 +427,18 @@ Function: cpp_typecheckt::clean_up
 
 void cpp_typecheckt::clean_up()
 {
-  contextt::symbolst::iterator it=context.symbols.begin();
+  symbol_tablet::symbolst::iterator it=symbol_table.symbols.begin();
   
-  while(it!=context.symbols.end())
+  while(it!=symbol_table.symbols.end())
   {
-    contextt::symbolst::iterator cur_it = it;
+    symbol_tablet::symbolst::iterator cur_it = it;
     it++;
 
     symbolt &symbol = cur_it->second;
 
     if(symbol.type.get_bool(ID_is_template))
     {
-      context.symbols.erase(cur_it);
+      symbol_table.symbols.erase(cur_it);
       continue;
     }
     else if(symbol.type.id()==ID_struct ||
@@ -453,7 +455,7 @@ void cpp_typecheckt::clean_up()
 
       struct_union_typet::componentst &function_members=
         (struct_union_typet::componentst &)
-        (struct_union_type.add("methods").get_sub());
+        (struct_union_type.add(ID_methods).get_sub());
         
       function_members.reserve(components.size());
 
