@@ -8,10 +8,10 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 #include <algorithm>
 
-#include <i2string.h>
-#include <arith_tools.h>
-#include <expr_util.h>
-#include <simplify_expr.h>
+#include <util/i2string.h>
+#include <util/arith_tools.h>
+#include <util/expr_util.h>
+#include <util/simplify_expr.h>
 
 #include <ansi-c/c_qualifiers.h>
 
@@ -23,6 +23,60 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 /*******************************************************************\
 
+Function: cpp_typecheckt::has_const
+
+Inputs:
+
+Outputs:
+
+Purpose:
+
+\*******************************************************************/
+
+bool cpp_typecheckt::has_const(const typet &type)
+{
+  if(type.id()==ID_const)
+    return true;
+  else if(type.id()==ID_merged_type)
+  {
+    forall_subtypes(it, type)
+      if(has_const(*it)) return true;
+      
+    return false;
+  }
+  else
+    return false;
+}
+
+/*******************************************************************\
+
+Function: cpp_typecheckt::has_volatile
+
+Inputs:
+
+Outputs:
+
+Purpose:
+
+\*******************************************************************/
+
+bool cpp_typecheckt::has_volatile(const typet &type)
+{
+  if(type.id()==ID_volatile)
+    return true;
+  else if(type.id()==ID_merged_type)
+  {
+    forall_subtypes(it, type)
+      if(has_volatile(*it)) return true;
+      
+    return false;
+  }
+  else
+    return false;
+}
+
+/*******************************************************************\
+
 Function: cpp_typecheckt::tag_scope
 
 Inputs:
@@ -98,17 +152,19 @@ void cpp_typecheckt::typecheck_compound_type(
   type.remove(ID_C_restricted);
 
   // get the tag name
-  bool anonymous=type.find(ID_tag).is_nil();
+  bool has_tag=type.find(ID_tag).is_not_nil();
   irep_idt base_name;
   cpp_scopet *dest_scope=NULL;
   bool has_body=type.find(ID_body).is_not_nil();
   bool tag_only_declaration=type.get_bool(ID_C_tag_only_declaration);
 
-  if(anonymous)
+  if(!has_tag)
   {
-    base_name="#anon_"+type.id_string()+i2string(anon_counter++);
-    type.set("#is_anonymous", true);
-    // anonymous structs always go into the current scope
+    // most of these should be named by now; see
+    // cpp_declarationt::name_anon_struct_union()
+
+    base_name=std::string("#anon_")+i2string(++anon_counter);
+    type.set(ID_C_is_anonymous, true);
     dest_scope=&cpp_scopes.current_scope();
   }
   else
@@ -120,7 +176,12 @@ void cpp_typecheckt::typecheck_compound_type(
     if(cpp_name.is_simple_name())
     {
       base_name=cpp_name.get_base_name();
-      dest_scope=&tag_scope(base_name, has_body, tag_only_declaration);
+     
+      // anonymous structs always go into the current scope
+      if(type.get_bool(ID_C_is_anonymous))
+        dest_scope=&cpp_scopes.current_scope();
+      else
+        dest_scope=&tag_scope(base_name, has_body, tag_only_declaration);
     }
     else
     {
@@ -138,10 +199,10 @@ void cpp_typecheckt::typecheck_compound_type(
 
   // check if we have it already
 
-  contextt::symbolst::iterator previous_symbol=
-    context.symbols.find(symbol_name);
+  symbol_tablet::symbolst::iterator previous_symbol=
+    symbol_table.symbols.find(symbol_name);
 
-  if(previous_symbol!=context.symbols.end())
+  if(previous_symbol!=symbol_table.symbols.end())
   {
     // we do!
 
@@ -155,10 +216,14 @@ void cpp_typecheckt::typecheck_compound_type(
         symbol.type.swap(type);
         typecheck_compound_body(symbol);
       }
+      else if(symbol.type.get_bool(ID_C_is_anonymous))
+      {
+        // we silently ignore
+      }
       else
       {
         err_location(type.location());
-        str << "error: struct symbol `" << base_name
+        str << "error: compound tag `" << base_name
             << "' declared previously" << std::endl;
         str << "location of previous definition: "
             << symbol.location;
@@ -186,8 +251,8 @@ void cpp_typecheckt::typecheck_compound_type(
     // move early, must be visible before doing body
     symbolt *new_symbol;
  
-    if(context.move(symbol, new_symbol))
-      throw "cpp_typecheckt::typecheck_compound_type: context.move() failed";
+    if(symbol_table.move(symbol, new_symbol))
+      throw "cpp_typecheckt::typecheck_compound_type: symbol_table.move() failed";
 
     // put into dest_scope
     cpp_idt &id=cpp_scopes.put_into_scope(*new_symbol, *dest_scope);
@@ -283,7 +348,6 @@ void cpp_typecheckt::typecheck_compound_declarator(
     throw 0;
   }
 
-
   bool is_method=!is_typedef && final_type.id()==ID_code;
   bool is_constructor=declaration.is_constructor();
   bool is_destructor=declaration.is_destructor();
@@ -356,12 +420,27 @@ void cpp_typecheckt::typecheck_compound_declarator(
   // now do actual work
 
   struct_typet::componentt component;
-
-  irep_idt identifier=
-    language_prefix+
-    cpp_scopes.current_scope().prefix+
-    id2string(base_name);
-
+  irep_idt identifier;
+  
+  // the below is a temporary hack
+  //if(is_method || is_static)d
+  if(id2string(cpp_scopes.current_scope().prefix).find("#anon")==
+     std::string::npos ||
+     is_method || is_static)
+  {
+    // Identifiers for methods include the scope prefix.
+    // Identifiers for static members include the scope prefix.
+    identifier=
+      language_prefix+
+      cpp_scopes.current_scope().prefix+
+      id2string(base_name);
+  }
+  else
+  {
+    // otherwise, we keep them simple
+    identifier=base_name;
+  }
+  
   component.set(ID_name, identifier);
   component.type()=final_type;
   component.set(ID_access, access);
@@ -381,8 +460,9 @@ void cpp_typecheckt::typecheck_compound_declarator(
   if(declaration.member_spec().is_explicit())
     component.set("is_explicit", true);
 
-  typet &method_qualifier=
-    (typet &)declarator.add("method_qualifier");
+  // either blank, const, volatile, or const volatile
+  const typet &method_qualifier=
+    static_cast<const typet &>(declarator.add(ID_method_qualifier));
 
   if(is_static)
   {
@@ -408,10 +488,13 @@ void cpp_typecheckt::typecheck_compound_declarator(
     component.get_string(ID_base_name)+
       id2string(
         function_identifier(static_cast<const typet &>(component.find(ID_type))));
+        
+    if(has_const(method_qualifier))
+      virtual_name+="$const";
 
-    if(method_qualifier.id()==ID_const)
-      virtual_name += "$const";
-
+    if(has_volatile(method_qualifier))
+      virtual_name+="$virtual";
+      
     if(component.type().get(ID_return_type) == ID_destructor)
       virtual_name= "@dtor";
     
@@ -452,7 +535,7 @@ void cpp_typecheckt::typecheck_compound_declarator(
     }
     else // virtual
     {
-      component.type().set("#is_virtual", true);
+      component.type().set(ID_C_is_virtual, true);
       component.type().set("#virtual_name",virtual_name);
 
       // Check if it is a pure virtual method
@@ -480,17 +563,17 @@ void cpp_typecheckt::typecheck_compound_declarator(
         value);
 
       // get the virtual-table symbol type
-      irep_idt vt_name = "virtual_table::"+symbol.name.as_string();
+      irep_idt vt_name = "virtual_table::"+id2string(symbol.name);
 
-      contextt::symbolst::iterator vtit =
-        context.symbols.find(vt_name);
+      symbol_tablet::symbolst::iterator vtit =
+        symbol_table.symbols.find(vt_name);
 
-      if(vtit == context.symbols.end())
+      if(vtit == symbol_table.symbols.end())
       {
         // first time: create a virtual-table symbol type 
         symbolt vt_symb_type;
         vt_symb_type.name= vt_name;
-        vt_symb_type.base_name="virtual_table::"+symbol.base_name.as_string();
+        vt_symb_type.base_name="virtual_table::"+id2string(symbol.base_name);
         vt_symb_type.pretty_name = vt_symb_type.base_name;
         vt_symb_type.mode=ID_cpp;
         vt_symb_type.module=module;
@@ -499,16 +582,16 @@ void cpp_typecheckt::typecheck_compound_declarator(
         vt_symb_type.type.set(ID_name, vt_symb_type.name);
         vt_symb_type.is_type = true;
 
-        bool failed = context.move(vt_symb_type);
+        bool failed = symbol_table.move(vt_symb_type);
         assert(!failed);
-        vtit = context.symbols.find(vt_name);
+        vtit = symbol_table.symbols.find(vt_name);
 
         // add a virtual-table pointer 
         struct_typet::componentt compo;
         compo.type() = pointer_typet(symbol_typet(vt_name));
-        compo.set_name(symbol.name.as_string() +"::@vtable_pointer");
+        compo.set_name(id2string(symbol.name) +"::@vtable_pointer");
         compo.set(ID_base_name, "@vtable_pointer");
-        compo.set(ID_pretty_name, symbol.base_name.as_string() +"@vtable_pointer");
+        compo.set(ID_pretty_name, id2string(symbol.base_name) +"@vtable_pointer");
         compo.set("is_vtptr", true);
         compo.set(ID_access, ID_public);
         components.push_back(compo);
@@ -526,7 +609,7 @@ void cpp_typecheckt::typecheck_compound_declarator(
       // add an entry to the virtual table
       struct_typet::componentt vt_entry;
       vt_entry.type() = pointer_typet(component.type());
-      vt_entry.set_name(vtit->first.as_string()+"::"+virtual_name);
+      vt_entry.set_name(id2string(vtit->first)+"::"+virtual_name);
       vt_entry.set(ID_base_name, virtual_name);
       vt_entry.set(ID_pretty_name, virtual_name);
       vt_entry.set(ID_access, ID_public);
@@ -540,7 +623,7 @@ void cpp_typecheckt::typecheck_compound_declarator(
 
         // a new function that does 'late casting' of the 'this' parameter
         symbolt func_symb;
-        func_symb.name=component.get_name().as_string() + "::" +virtual_base.as_string();
+        func_symb.name=id2string(component.get_name()) + "::" +id2string(virtual_base);
         func_symb.base_name=component.get(ID_base_name);
         func_symb.pretty_name = component.get(ID_base_name);
         func_symb.mode=ID_cpp;
@@ -564,7 +647,7 @@ void cpp_typecheckt::typecheck_compound_declarator(
             base_name="arg"+i2string(i);
 
           symbolt arg_symb;
-          arg_symb.name = func_symb.name.as_string() + "::"+ base_name.as_string();
+          arg_symb.name = id2string(func_symb.name) + "::"+ id2string(base_name);
           arg_symb.base_name = base_name;
           arg_symb.pretty_name = base_name;
           arg_symb.mode=ID_cpp;
@@ -574,7 +657,7 @@ void cpp_typecheckt::typecheck_compound_declarator(
           arg.set(ID_C_identifier, arg_symb.name);
 
           // add the argument to the symbol table
-          bool failed = context.move(arg_symb);
+          bool failed = symbol_table.move(arg_symb);
           assert(!failed);
         }
 
@@ -582,8 +665,8 @@ void cpp_typecheckt::typecheck_compound_declarator(
         typecast_exprt late_cast(to_code_type(component.type()).arguments()[0].type());
 
         late_cast.op0()=
-          symbol_expr(namespacet(context).lookup(
-            args[0].get(ID_C_identifier)));
+          namespacet(symbol_table).lookup(
+            args[0].get(ID_C_identifier)).symbol_expr();
         
         if(code_type.return_type().id()!=ID_empty &&
            code_type.return_type().id()!=ID_destructor)
@@ -597,8 +680,7 @@ void cpp_typecheckt::typecheck_compound_declarator(
           for(unsigned i=1; i < args.size(); i++)
           {
             expr_call.arguments().push_back(
-              symbol_expr(namespacet(context).lookup(
-                args[i].get(ID_C_identifier))));
+              namespacet(symbol_table).lookup(args[i].get(ID_C_identifier)).symbol_expr());
           }
 
           code_returnt code_return;
@@ -616,8 +698,8 @@ void cpp_typecheckt::typecheck_compound_declarator(
           for(unsigned i=1; i < args.size(); i++)
           {
             code_func.arguments().push_back(
-              symbol_expr(namespacet(context).lookup(
-                args[i].get(ID_C_identifier))));
+              namespacet(symbol_table).lookup(
+                args[i].get(ID_C_identifier)).symbol_expr());
           }
 
           func_symb.value = code_func;
@@ -632,7 +714,7 @@ void cpp_typecheckt::typecheck_compound_declarator(
 
         // add the function to the symbol table
         {
-          bool failed = context.move(func_symb);
+          bool failed = symbol_table.move(func_symb);
           assert(!failed);
         }
 
@@ -644,14 +726,14 @@ void cpp_typecheckt::typecheck_compound_declarator(
   
   if(is_static && !is_method) // static non-method member
   {
-    // add as global variable to context
+    // add as global variable to symbol_table
     symbolt static_symbol;
     static_symbol.mode=symbol.mode;
     static_symbol.name=identifier;
     static_symbol.type=component.type();
     static_symbol.base_name=component.get(ID_base_name);
-    static_symbol.lvalue=true;
-    static_symbol.static_lifetime=true;
+    static_symbol.is_lvalue=true;
+    static_symbol.is_static_lifetime=true;
     static_symbol.location=cpp_name.location();
     static_symbol.is_extern=true;
     
@@ -659,11 +741,11 @@ void cpp_typecheckt::typecheck_compound_declarator(
     dynamic_initializations.push_back(static_symbol.name);
 
     symbolt *new_symbol;
-    if(context.move(static_symbol, new_symbol))
+    if(symbol_table.move(static_symbol, new_symbol))
     {
       err_location(cpp_name.location());
 	str << "redeclaration of static member `" 
-	    << static_symbol.base_name.as_string()
+	    << static_symbol.base_name
 	    << "'";
       throw 0;
     }
@@ -950,7 +1032,7 @@ void cpp_typecheckt::typecheck_compound_body(symbolt &symbol)
     typecheck_compound_bases(to_struct_type(type));
   }
 
-  exprt &body=(exprt &)type.add(ID_body);
+  exprt &body=static_cast<exprt &>(type.add(ID_body));
   struct_union_typet::componentst &components=type.components();
 
   symbol.type.set(ID_name, symbol.name);
@@ -998,6 +1080,7 @@ void cpp_typecheckt::typecheck_compound_body(symbolt &symbol)
         if(declaration.declarators().empty())
           declaration.type().set(ID_C_tag_only_declaration, true);
 
+      declaration.name_anon_struct_union();
       typecheck_type(declaration.type());
 
       bool is_static=declaration.storage_spec().is_static();
@@ -1016,7 +1099,7 @@ void cpp_typecheckt::typecheck_compound_body(symbolt &symbol)
 
       // anonymous member?
       if(declaration.declarators().empty() &&
-         final_type.get_bool("#is_anonymous"))
+         final_type.get_bool(ID_C_is_anonymous))
       {
         // we only allow this on struct/union types
         if(final_type.id()!=ID_union &&
@@ -1082,7 +1165,8 @@ void cpp_typecheckt::typecheck_compound_body(symbolt &symbol)
   }
 
   // setup virtual tables before doing the constructors
-  do_virtual_table(symbol);
+  if(symbol.type.id()==ID_struct)
+    do_virtual_table(symbol);
 
   if(!found_ctor && !cpp_is_pod(symbol.type))
   {
@@ -1252,7 +1336,7 @@ void cpp_typecheckt::move_member_initializers(
     exprt::operandst::iterator o_it=value.operands().begin();
     forall_irep(it, initializers.get_sub())
     {
-      o_it=value.operands().insert(o_it,(exprt&)*it);
+      o_it=value.operands().insert(o_it, static_cast<const exprt &>(*it));
       o_it++;
     }
   }
@@ -1274,7 +1358,7 @@ void cpp_typecheckt::typecheck_member_function(
   const irep_idt &compound_symbol,
   struct_typet::componentt &component,
   irept &initializers,
-  typet &method_qualifier,
+  const typet &method_qualifier,
   exprt &value)
 {
   symbolt symbol;
@@ -1283,7 +1367,7 @@ void cpp_typecheckt::typecheck_member_function(
 
   if(component.get_bool(ID_is_static))
   {
-    if(method_qualifier.id()!="")
+    if(method_qualifier.id()!=irep_idt())
     {
       err_location(component);
       throw "method is static -- no qualifiers allowed";
@@ -1326,21 +1410,20 @@ void cpp_typecheckt::typecheck_member_function(
   symbol.type=type;
   symbol.is_type=false;
   symbol.is_macro=false;
-  symbol.theorem=true;
   symbol.location=component.location();
 
   // move early, it must be visible before doing any value
   symbolt *new_symbol;
 
-  if(context.move(symbol, new_symbol))
+  if(symbol_table.move(symbol, new_symbol))
   {
     err_location(symbol.location);
     str << "failed to insert new symbol: " << symbol.name.c_str() << std::endl;
 
-    contextt::symbolst::iterator symb_it =
-      context.symbols.find(symbol.name);
+    symbol_tablet::symbolst::iterator symb_it =
+      symbol_table.symbols.find(symbol.name);
 
-    if(symb_it != context.symbols.end())
+    if(symb_it != symbol_table.symbols.end())
     {
       str << "name of previous symbol: " << symb_it->second.name << std::endl;
       str << "location of previous symbol: ";
@@ -1369,13 +1452,13 @@ Purpose:
 void cpp_typecheckt::adjust_method_type(
   const irep_idt &compound_symbol,
   typet &type,
-  typet &method_type)
+  const typet &method_qualifier)
 {
   irept &arguments=type.add(ID_arguments);
 
   arguments.get_sub().insert(arguments.get_sub().begin(), irept(ID_argument));
 
-  exprt &argument=(exprt &)arguments.get_sub().front();
+  exprt &argument=static_cast<exprt &>(arguments.get_sub().front());
   argument.type()=typet(ID_pointer);
 
   argument.type().subtype()=typet(ID_symbol);
@@ -1384,16 +1467,11 @@ void cpp_typecheckt::adjust_method_type(
   argument.set(ID_C_identifier, ID_this);
   argument.set(ID_C_base_name, ID_this);
 
-  if(method_type.id()=="" || method_type.is_nil())
-  {
-  }
-  else if(method_type.id()==ID_const)
-    argument.type().subtype().set("#constant", true);
-  else
-  {
-    err_location(method_type);
-    throw "invalid method qualifier";
-  }
+  if(has_const(method_qualifier))
+    argument.type().subtype().set(ID_C_constant, true);
+  
+  if(has_volatile(method_qualifier))
+    argument.type().subtype().set(ID_C_volatile, true);
 }
 
 /*******************************************************************\
@@ -1479,7 +1557,7 @@ void cpp_typecheckt::convert_anon_struct_union_member(
   struct_typet::componentst &components)
 {
   symbolt &struct_union_symbol=
-    context.symbols[follow(declaration.type()).get(ID_name)];
+    symbol_table.symbols[follow(declaration.type()).get(ID_name)];
 
   if(declaration.storage_spec().is_static() ||
      declaration.storage_spec().is_mutable())
@@ -1575,7 +1653,7 @@ bool cpp_typecheckt::get_component(
         if(disable_access_control)
         {
           member.set("#not_accessible", true);
-          member.set("#access", component.get(ID_access));
+          member.set(ID_C_access, component.get(ID_access));
         }
         else
         {
@@ -1583,7 +1661,8 @@ bool cpp_typecheckt::get_component(
           err_location(location);
           str << "error: member `" << component_name
               << "' is not accessible (" << component.get(ID_access) << ")";
-          str << "\nstruct name: " << final_type.get(ID_name);
+          str << std::endl
+              << "struct name: " << final_type.get(ID_name);
           throw 0;
           #endif
         }
@@ -1691,14 +1770,17 @@ bool cpp_typecheckt::check_component_access(
   }
 
   // check friendship
-  forall_irep(f_it, struct_union_type.find("#friends").get_sub())
+  const irept::subt &friends=
+    struct_union_type.find("#friends").get_sub();
+  
+  forall_irep(f_it, friends)
   {
-    const irept& friend_symb = *f_it;
+    const irept &friend_symb = *f_it;
 
-    const cpp_scopet& friend_scope =
+    const cpp_scopet &friend_scope =
       cpp_scopes.get_scope(friend_symb.get(ID_identifier));
 
-    cpp_scopet* pscope = &(cpp_scopes.current_scope());
+    cpp_scopet *pscope = &(cpp_scopes.current_scope());
 
     while(!(pscope->is_root_scope()))
     {
