@@ -6,10 +6,10 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 \*******************************************************************/
 
-#include <arith_tools.h>
-#include <std_code.h>
-#include <std_expr.h>
-#include <expr_util.h>
+#include <util/arith_tools.h>
+#include <util/std_code.h>
+#include <util/std_expr.h>
+#include <util/expr_util.h>
 
 #include <ansi-c/c_types.h>
 
@@ -153,7 +153,7 @@ static void copy_array(
   exprt &block)
 {
   // Build the index expression
-  exprt constant=from_integer(i, int_type());
+  exprt constant=from_integer(i, index_type());
 
   block.operands().push_back(exprt(ID_code));
   exprt& code = block.operands().back();
@@ -354,12 +354,13 @@ void cpp_typecheckt::default_cpctor(
       cppname.move_to_sub(name);
 
       const symbolt &virtual_table_symbol_type = 
-        namespacet(context).lookup(mem_it->type().subtype().get(ID_identifier));
+        namespacet(symbol_table).lookup(mem_it->type().subtype().get(ID_identifier));
 
       const symbolt &virtual_table_symbol_var  =
-        namespacet(context).lookup(virtual_table_symbol_type.name.as_string() + "@" + symbol.name.as_string());
+        namespacet(symbol_table).lookup(id2string(virtual_table_symbol_type.name) + "@" +
+        id2string(symbol.name));
 
-      exprt var = symbol_expr(virtual_table_symbol_var);
+      exprt var = virtual_table_symbol_var.symbol_expr();
       address_of_exprt address(var);
       assert(address.type() == mem_it->type());
 
@@ -924,10 +925,10 @@ void cpp_typecheckt::full_member_initialization(
         lookup(mem_it->type().subtype().get(ID_identifier));
 
       const symbolt& virtual_table_symbol_var  =
-        lookup(virtual_table_symbol_type.name.as_string() + "@" + 
-            struct_type.get(ID_name).as_string());
+        lookup(id2string(virtual_table_symbol_type.name) + "@" + 
+            id2string(struct_type.get(ID_name)));
 
-      exprt var =symbol_expr(virtual_table_symbol_var);
+      exprt var = virtual_table_symbol_var.symbol_expr();
       address_of_exprt address(var);
       assert(address.type() == mem_it->type());
 
@@ -1239,14 +1240,14 @@ codet cpp_typecheckt::dtor(const symbolt &symb)
       cppname.move_to_sub(name);
 
       const symbolt &virtual_table_symbol_type = 
-        namespacet(context).lookup(
+        namespacet(symbol_table).lookup(
           cit->type().subtype().get(ID_identifier));
 
       const symbolt &virtual_table_symbol_var  =
-        namespacet(context).lookup(
-          virtual_table_symbol_type.name.as_string() + "@" + symb.name.as_string());
+        namespacet(symbol_table).lookup(
+          id2string(virtual_table_symbol_type.name) + "@" + id2string(symb.name));
 
-      exprt var=symbol_expr(virtual_table_symbol_var);
+      exprt var=virtual_table_symbol_var.symbol_expr();
       address_of_exprt address(var);
       assert(address.type()==cit->type());
 
