@@ -8,11 +8,11 @@ Author:
 
 #include <cstdlib>
 
-#include <config.h>
-#include <arith_tools.h>
-#include <expr_util.h>
-#include <std_types.h>
-#include <std_expr.h>
+#include <util/config.h>
+#include <util/arith_tools.h>
+#include <util/expr_util.h>
+#include <util/std_types.h>
+#include <util/std_expr.h>
 
 #include <ansi-c/c_qualifiers.h>
 #include <ansi-c/c_types.h>
@@ -89,7 +89,7 @@ bool cpp_typecheckt::standard_conversion_array_to_pointer(
   assert(expr.type().id()==ID_array);
 
   exprt index(ID_index, expr.type().subtype());
-  index.copy_to_operands(expr, from_integer(0, int_type()));
+  index.copy_to_operands(expr, from_integer(0, index_type()));
   index.set(ID_C_lvalue, true);
 
   pointer_typet pointer;
@@ -826,6 +826,22 @@ bool cpp_typecheckt::standard_conversion_sequence(
 
   curr_expr.swap(new_expr);
 
+  // two enums are the same if the tag is the same,
+  // even if the width differs (enum bit-fields!)
+  if(follow(type).id()==ID_c_enum &&
+     follow(curr_expr.type()).id()==ID_c_enum)
+  {
+    if(follow(type).find(ID_tag)==
+       follow(curr_expr.type()).find(ID_tag))
+      return true;
+    else
+    {
+      // In contrast to C, we simply don't allow implicit conversions
+      // between enums.
+      return false;
+    }
+  }
+
   // need to consider #c_type
   if(follow(curr_expr.type())!=follow(type) ||
      curr_expr.type().get(ID_C_c_type)!=type.get(ID_C_c_type))
@@ -842,6 +858,7 @@ bool cpp_typecheckt::standard_conversion_sequence(
           if(!standard_conversion_floating_integral_conversion(curr_expr, type, new_expr))
             return false;
         }
+
         rank+=3;
       }
       else
@@ -1699,7 +1716,11 @@ void cpp_typecheckt::implicit_typecast(exprt &expr, const typet &type)
     err_location(e);
     str << "invalid implicit conversion from `"
         << to_string(e.type()) << "' to `"
-        << to_string(type) << "' ";
+        << to_string(type) << "'";
+    #if 0
+    str << "\n " << follow(e.type()).pretty() << std::endl;
+    str << "\n " << type.pretty() << std::endl;
+    #endif
     throw 0;
   }
 }
@@ -2180,7 +2201,7 @@ bool cpp_typecheckt::static_typecast(
         }
 
         exprt address_of(ID_address_of, pointer_typet());
-        address_of.type().subtype()==e.type();
+        address_of.type().subtype()=e.type();
         address_of.copy_to_operands(e);
         make_ptr_typecast(address_of ,type);
         new_expr.swap(address_of);
@@ -2197,16 +2218,16 @@ bool cpp_typecheckt::static_typecast(
     return true;
   }
 
-  if (follow(type).id()==ID_c_enum
-         && (e.type().id()==ID_signedbv
+  if(follow(type).id()==ID_c_enum && (
+                e.type().id()==ID_signedbv
              || e.type().id()==ID_unsignedbv
              || follow(e.type()).id()==ID_c_enum))
   {
-     new_expr = e;
-     new_expr.make_typecast(type);
-     if(new_expr.get_bool(ID_C_lvalue))
-       new_expr.remove(ID_C_lvalue);
-     return true;
+    new_expr = e;
+    new_expr.make_typecast(type);
+    if(new_expr.get_bool(ID_C_lvalue))
+      new_expr.remove(ID_C_lvalue);
+    return true;
   }
 
   if(implicit_conversion_sequence(e, type, new_expr))
