@@ -6,16 +6,17 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 \*******************************************************************/
 
+#include <iostream>
 #include <cstdlib>
 
-#include <expr_util.h>
-#include <i2string.h>
-#include <std_types.h>
-#include <arith_tools.h>
-#include <bitvector.h>
-#include <std_expr.h>
-#include <config.h>
-#include <simplify_expr.h>
+#include <util/expr_util.h>
+#include <util/i2string.h>
+#include <util/std_types.h>
+#include <util/arith_tools.h>
+#include <util/bitvector.h>
+#include <util/std_expr.h>
+#include <util/config.h>
+#include <util/simplify_expr.h>
 
 #include <ansi-c/c_types.h>
 #include <ansi-c/c_qualifiers.h>
@@ -24,7 +25,6 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 #include "cpp_type2name.h"
 #include "cpp_typecheck.h"
 #include "cpp_convert_type.h"
-#include "cpp_class_type.h"
 #include "cpp_exception_id.h"
 #include "expr2cpp.h"
 
@@ -128,16 +128,16 @@ void cpp_typecheckt::typecheck_expr_main(exprt &expr)
     follow_symbol(deriv);
 
     if(base.id()!=ID_struct || deriv.id()!=ID_struct)
-      expr.make_false();
+      expr=false_exprt();
     else
     {
       irep_idt base_name=base.get(ID_name);
       const class_typet &class_type=to_class_type(deriv);
 
       if(class_type.has_base(base_name))
-        expr.make_true();
+        expr=true_exprt();
       else
-        expr.make_false();
+        expr=false_exprt();
     }
   }
   else if(expr.id()==ID_msc_uuidof)
@@ -291,12 +291,12 @@ void cpp_typecheckt::typecheck_expr_trinary(if_exprt &expr)
       
       index_exprt index1;
       index1.array() = expr.op1();
-      index1.index() = from_integer(0,int_type());
+      index1.index() = from_integer(0, index_type());
       index1.type() = expr.op1().type().subtype();
 
       index_exprt index2;
       index2.array() = expr.op2();
-      index2.index() = from_integer(0,int_type());
+      index2.index() = from_integer(0, index_type());
       index2.type() = expr.op2().type().subtype();
 
       address_of_exprt addr1(index1);
@@ -368,8 +368,8 @@ void cpp_typecheckt::typecheck_expr_sizeof(exprt &expr)
       
     if(type.id()==ID_cpp_name)
     {
-      // this may be ambiguous -- it can be either a type or
-      // an expression
+      // sizeof(X) may be ambiguous -- X can be either a type or
+      // an expression.
 
       cpp_typecheck_fargst fargs;
 
@@ -695,7 +695,7 @@ bool cpp_typecheckt::operator_is_overloaded(exprt &expr)
       // 1. In the scope of a as a.operator(b, ...)
       // 2. Anywhere in scope as operator(a, b, ...)
       //
-      // Both are not allowed.
+      // Using both is not allowed.
       //
       // We try and fail silently, maybe conversions will work
       // instead.
@@ -1297,22 +1297,10 @@ void cpp_typecheckt::typecheck_expr_member(
   // The member operator will trigger template elaboration
   elaborate_class_template(op0.type());
 
-  if(op0.type().id()!=ID_symbol)
-  {
-    err_location(expr);
-    str << "error: member operator requires type symbol "
-           "on left hand side but got `"
-        << to_string(op0.type()) << "'";
-    throw 0;
-  }
-  
-  const irep_idt &struct_identifier=
-    to_symbol_type(op0.type()).get_identifier();
+  const typet &followed_op0_type=follow(op0.type());
 
-  const symbolt &struct_symbol=lookup(struct_identifier);
-
-  if(struct_symbol.type.id()==ID_incomplete_struct ||
-     struct_symbol.type.id()==ID_incomplete_union)
+  if(followed_op0_type.id()==ID_incomplete_struct ||
+     followed_op0_type.id()==ID_incomplete_union)
   {
     err_location(expr);
     str << "error: member operator got incomplete type "
@@ -1320,18 +1308,20 @@ void cpp_typecheckt::typecheck_expr_member(
     throw 0;
   }
 
-  if(struct_symbol.type.id()!=ID_struct &&
-     struct_symbol.type.id()!=ID_union)
+  if(followed_op0_type.id()!=ID_struct &&
+     followed_op0_type.id()!=ID_union)
   {
     err_location(expr);
     str << "error: member operator requires struct/union type "
            "on left hand side but got `"
-        << to_string(struct_symbol.type) << "'";
+        << to_string(followed_op0_type) << "'";
     throw 0;
   }
 
   const struct_union_typet &type=
-    to_struct_union_type(struct_symbol.type);
+    to_struct_union_type(followed_op0_type);
+    
+  irep_idt struct_identifier=type.get(ID_name);
 
   if(expr.find("component_cpp_name").is_not_nil())
   {
@@ -1389,7 +1379,7 @@ void cpp_typecheckt::typecheck_expr_member(
           str << "error: `"
               << symbol_expr.get(ID_identifier)
               << "' is not static member "
-              << "of class `" << struct_symbol.base_name << "'";
+              << "of class `" << to_string(type) << "'";
           throw 0;
         }
       }
@@ -1433,7 +1423,7 @@ void cpp_typecheckt::typecheck_expr_member(
   {
     err_location(expr);
     str << "error: member `" << component_name
-        << "' of `" << struct_symbol.pretty_name
+        << "' of `" << to_string(type)
         << "' not found";
     throw 0;
   }
@@ -1443,10 +1433,10 @@ void cpp_typecheckt::typecheck_expr_member(
   if(expr.type().id()==ID_code)
   {
     // Check if the function body has to be typechecked
-    contextt::symbolst::iterator it=
-      context.symbols.find(component_name);
+    symbol_tablet::symbolst::iterator it=
+      symbol_table.symbols.find(component_name);
 
-    assert(it!=context.symbols.end());
+    assert(it!=symbol_table.symbols.end());
 
     symbolt &func_symb=it->second;
 
@@ -1969,7 +1959,7 @@ void cpp_typecheckt::typecheck_side_effect_function_call(
 
       // get the virtual table
       typet this_type =  to_code_type(expr.function().type()).arguments().front().type();
-      irep_idt vtable_name = this_type.subtype().get(ID_identifier).as_string() +"::@vtable_pointer";
+      irep_idt vtable_name = this_type.subtype().get_string(ID_identifier) +"::@vtable_pointer";
 
       const struct_typet &vt_struct=
         to_struct_type(follow(this_type.subtype()));
@@ -1982,8 +1972,8 @@ void cpp_typecheckt::typecheck_side_effect_function_call(
       vtptr_member.set(ID_component_name, vtable_name);
 
       // look for the right entry
-      irep_idt vtentry_component_name = vt_compo.type().subtype().get(ID_identifier).as_string()
-        + "::" + expr.function().type().get("#virtual_name").as_string();
+      irep_idt vtentry_component_name = vt_compo.type().subtype().get_string(ID_identifier)
+        + "::" + expr.function().type().get_string("#virtual_name");
 
       exprt vtentry_member(ID_ptrmember);
       vtentry_member.copy_to_operands(vtptr_member);
@@ -2082,7 +2072,7 @@ void cpp_typecheckt::typecheck_side_effect_function_call(
     // special case for the initialization of parents
     if(member.get_bool("#not_accessible"))
     {
-      assert(id2string(member.get(ID_C_access))!="");
+      assert(member.get(ID_C_access)!="");
       tmp_object_expr.set("#not_accessible", true);
       tmp_object_expr.set(ID_C_access, member.get(ID_C_access));
     }
@@ -2312,7 +2302,7 @@ void cpp_typecheckt::typecheck_method_application(
   if(symbol.value.id()=="cpp_not_typechecked" &&
       !symbol.value.get_bool("is_used"))
   {
-    context.symbols[symbol.name].value.set("is_used", true);
+    symbol_table.symbols[symbol.name].value.set("is_used", true);
   }
 }
 
@@ -2489,7 +2479,7 @@ void cpp_typecheckt::typecheck_side_effect_inc_dec(
   // the odd C++ way to denote the post-inc/dec operator
   if(post)
     new_expr.arguments().push_back(
-      from_integer(mp_integer(0), int_type()));
+      from_integer(mp_integer(0), signed_int_type()));
       
   typecheck_side_effect_function_call(new_expr);
   expr.swap(new_expr);
@@ -2630,10 +2620,10 @@ void cpp_typecheckt::typecheck_expr_function_identifier(exprt &expr)
   if(expr.id()==ID_symbol)
   {
     // Check if the function body has to be typechecked
-    contextt::symbolst::iterator it=
-      context.symbols.find(expr.get(ID_identifier));
+    symbol_tablet::symbolst::iterator it=
+      symbol_table.symbols.find(expr.get(ID_identifier));
 
-    assert(it != context.symbols.end());
+    assert(it != symbol_table.symbols.end());
 
     symbolt &func_symb = it->second;
 
@@ -2661,8 +2651,8 @@ void cpp_typecheckt::typecheck_expr(exprt &expr)
   bool override_constantness=
     expr.get_bool("#override_constantness");
     
-  // we take care of an ambiguity in the C++ grammar
-  // needs to be done before the operands!
+  // We take care of an ambiguity in the C++ grammar.
+  // Needs to be done before the operands!
   explicit_typecast_ambiguity(expr);
   
   c_typecheck_baset::typecheck_expr(expr);
