@@ -6,9 +6,9 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 \*******************************************************************/
 
-#include <expr_util.h>
-#include <arith_tools.h>
-#include <std_expr.h>
+#include <util/expr_util.h>
+#include <util/arith_tools.h>
+#include <util/std_expr.h>
 
 #include <linking/zero_initializer.h>
 #include <ansi-c/c_types.h>
@@ -121,7 +121,8 @@ void cpp_typecheckt::convert_initializer(symbolt &symbol)
       else if(resolved_expr.id()==ID_member)
       {
         symbol.value =
-          address_of_exprt(symbol_expr(lookup(resolved_expr.get(ID_component_name))));
+          address_of_exprt(
+            lookup(resolved_expr.get(ID_component_name)).symbol_expr());
 
         symbol.value.type().add("to-member") = resolved_expr.op0().type();
       }
@@ -256,7 +257,7 @@ void cpp_typecheckt::zero_initializer(
       for(mp_integer i=0; i<size; ++i)
       {
         exprt index(ID_index);
-        index.copy_to_operands(object, from_integer(i, int_type()));
+        index.copy_to_operands(object, from_integer(i, index_type()));
         zero_initializer(index, array_type.subtype(), location, ops);
       }
     }
