@@ -8,13 +8,14 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 #include <cstdlib>
 #include <algorithm>
+#include <iostream>
 
-#include <expr_util.h>
-#include <std_types.h>
-#include <std_expr.h>
-#include <i2string.h>
-#include <arith_tools.h>
-#include <prefix.h>
+#include <util/expr_util.h>
+#include <util/std_types.h>
+#include <util/std_expr.h>
+#include <util/i2string.h>
+#include <util/arith_tools.h>
+#include <util/prefix.h>
 
 #include <ansi-c/c_types.h>
 #include <ansi-c/string_constant.h>
@@ -463,7 +464,7 @@ exprt cpp_typecheck_resolvet::convert_identifier(
 
 /*******************************************************************\
 
-Function: cpp_typecheck_resolvet::disambiguate
+Function: cpp_typecheck_resolvet::filter
 
 Inputs:
 
@@ -512,6 +513,42 @@ void cpp_typecheck_resolvet::filter(
 
 /*******************************************************************\
 
+Function: cpp_typecheck_resolvet::exact_match_functions
+
+Inputs:
+
+Outputs:
+
+Purpose:
+
+\*******************************************************************/
+
+void cpp_typecheck_resolvet::exact_match_functions(
+  resolve_identifierst &identifiers,
+  const cpp_typecheck_fargst &fargs)
+{
+  if(!fargs.in_use) return;
+
+  resolve_identifierst old_identifiers;
+  old_identifiers.swap(identifiers);
+  
+  identifiers.clear();
+
+  // put in the ones that match precisely
+  for(resolve_identifierst::const_iterator
+      it=old_identifiers.begin();
+      it!=old_identifiers.end();
+      it++)
+  {
+    unsigned distance;
+    if(disambiguate_functions(*it, distance, fargs))
+      if(distance<=0)
+        identifiers.push_back(*it);
+  }
+}
+
+/*******************************************************************\
+
 Function: cpp_typecheck_resolvet::disambiguate_functions
 
 Inputs:
@@ -570,14 +607,14 @@ void cpp_typecheck_resolvet::disambiguate_functions(
         it++)
       identifiers.push_back(it->second);
   }
-
+  
   if(identifiers.size()>1 && fargs.in_use)
   {
     // try to further disambiguate functions
 
     for(resolve_identifierst::iterator
-        it1 = identifiers.begin();
-        it1 != identifiers.end();
+        it1=identifiers.begin();
+        it1!=identifiers.end();
         it1++)
     {
       if(it1->type().id()!=ID_code) continue;
@@ -728,7 +765,7 @@ void cpp_typecheck_resolvet::make_constructors(
         code_typet t3;
         t3.return_type()=it->type();
         t3.arguments().resize(1);
-        t3.arguments()[0].type()=int_type();
+        t3.arguments()[0].type()=signed_int_type();
         exprt pod_constructor3("pod_constructor", t3);
         new_identifiers.push_back(pod_constructor3);
       }
@@ -1523,19 +1560,18 @@ exprt cpp_typecheck_resolvet::resolve(
   {
     if(base_name==ID_true)
     {
-      exprt result;
-      result.make_true();
+      exprt result=true_exprt();
       result.location()=location;
       return result;
     }
     else if(base_name==ID_false)
     {
-      exprt result;
-      result.make_false();
+      exprt result=false_exprt();
       result.location()=location;
       return result;
     }
-    else if(base_name=="__nullptr") // this is c++0x
+    else if(base_name=="__nullptr" ||
+            base_name=="nullptr") // this is c++0x
     {
       constant_exprt result;
       result.set_value(ID_NULL);
@@ -1596,7 +1632,7 @@ exprt cpp_typecheck_resolvet::resolve(
     }
 
     //cpp_typecheck.cpp_scopes.get_root_scope().print(std::cout);
-    cpp_typecheck.cpp_scopes.current_scope().print(std::cout);
+    //cpp_typecheck.cpp_scopes.current_scope().print(std::cout);
     throw 0;
   }
   
@@ -1670,9 +1706,9 @@ exprt cpp_typecheck_resolvet::resolve(
   
   exprt result;
 
-  // We may need to disambiguate functions,
-  // but don't want templates yet
+  // We disambiguate functions
   resolve_identifierst new_identifiers=identifiers;
+
   remove_templates(new_identifiers);
 
   #if 0
@@ -1681,16 +1717,32 @@ exprt cpp_typecheck_resolvet::resolve(
   std::cout << "\n";
   #endif
 
-  disambiguate_functions(new_identifiers, fargs);
+  // we only want _exact_ matches, without templates!
+  exact_match_functions(new_identifiers, fargs);
   
-  // no matches? Try again with function template guessing.
-  if(new_identifiers.empty() && template_args.is_nil())
+  #if 0
+  std::cout << "P2 " << base_name << " " << new_identifiers.size() << "\n";
+  show_identifiers(base_name, new_identifiers, std::cout);
+  std::cout << "\n";
+  #endif
+
+  // no exact matches? Try again with function template guessing.
+  if(new_identifiers.empty())
   {
     new_identifiers=identifiers;
-    guess_function_template_args(new_identifiers, fargs);
+    
+    if(template_args.is_nil())
+    {
+      guess_function_template_args(new_identifiers, fargs);
+      
+      if(new_identifiers.empty())
+        new_identifiers=identifiers;
+    }
+
+    disambiguate_functions(new_identifiers, fargs);
 
     #if 0
-    std::cout << "P2 " << base_name << " " << new_identifiers.size() << "\n";
+    std::cout << "P3 " << base_name << " " << new_identifiers.size() << "\n";
     show_identifiers(base_name, new_identifiers, std::cout);
     std::cout << "\n";
     #endif
@@ -1699,7 +1751,7 @@ exprt cpp_typecheck_resolvet::resolve(
   remove_duplicates(new_identifiers);
 
   #if 0
-  std::cout << "P3 " << base_name << " " << new_identifiers.size() << "\n";
+  std::cout << "P4 " << base_name << " " << new_identifiers.size() << "\n";
   show_identifiers(base_name, new_identifiers, std::cout);
   std::cout << "\n";
   #endif
@@ -2315,7 +2367,7 @@ bool cpp_typecheck_resolvet::disambiguate_functions(
       const code_typet::argumentst &arguments=type.arguments();
       const code_typet::argumentt &argument = arguments.front();
 
-      assert(argument.get("#base_name")==ID_this);
+      assert(argument.get(ID_C_base_name)==ID_this);
 
       if(type.return_type().id() == ID_constructor)
       {
@@ -2326,7 +2378,7 @@ bool cpp_typecheck_resolvet::disambiguate_functions(
 
         cpp_typecheck_fargst new_fargs(fargs);
         new_fargs.add_object(object);
-        return new_fargs.match(type, args_distance, cpp_typecheck);          
+        return new_fargs.match(type, args_distance, cpp_typecheck);
       }
       else
       {
