@@ -6,10 +6,12 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 \*******************************************************************/
 
-#include <expr_util.h>
-#include <i2string.h>
-#include <simplify_expr_class.h>
-#include <simplify_expr.h>
+#include <iostream>
+
+#include <util/expr_util.h>
+#include <util/i2string.h>
+#include <util/simplify_expr_class.h>
+#include <util/simplify_expr.h>
 
 #include "cpp_type2name.h"
 #include "cpp_typecheck.h"
@@ -157,10 +159,10 @@ void cpp_typecheckt::typecheck_class_template(
 
   // check if we have it already
   
-  contextt::symbolst::iterator previous_symbol=
-    context.symbols.find(symbol_name);
+  symbol_tablet::symbolst::iterator previous_symbol=
+    symbol_table.symbols.find(symbol_name);
 
-  if(previous_symbol!=context.symbols.end())
+  if(previous_symbol!=symbol_table.symbols.end())
   {
     // there already
     cpp_declarationt &previous_declaration=
@@ -232,8 +234,8 @@ void cpp_typecheckt::typecheck_class_template(
     cpp_scopes.current_scope().prefix+id2string(symbol.base_name);
 
   symbolt *new_symbol;
-  if(context.move(symbol, new_symbol))
-    throw "cpp_typecheckt::typecheck_compound_type: context.move() failed";
+  if(symbol_table.move(symbol, new_symbol))
+    throw "cpp_typecheckt::typecheck_compound_type: symbol_table.move() failed";
 
   // put into current scope
   cpp_idt &id=cpp_scopes.put_into_scope(*new_symbol);
@@ -297,10 +299,10 @@ void cpp_typecheckt::typecheck_function_template(
 
   // check if we have it already
 
-  contextt::symbolst::iterator previous_symbol=
-    context.symbols.find(symbol_name);
+  symbol_tablet::symbolst::iterator previous_symbol=
+    symbol_table.symbols.find(symbol_name);
 
-  if(previous_symbol!=context.symbols.end())
+  if(previous_symbol!=symbol_table.symbols.end())
   {
     bool previous_has_value =
      to_cpp_declaration(previous_symbol->second.type).
@@ -340,8 +342,8 @@ void cpp_typecheckt::typecheck_function_template(
     cpp_scopes.current_scope().prefix+id2string(symbol.base_name);
 
   symbolt *new_symbol;
-  if(context.move(symbol, new_symbol))
-    throw "cpp_typecheckt::typecheck_compound_type: context.move() failed";
+  if(symbol_table.move(symbol, new_symbol))
+    throw "cpp_typecheckt::typecheck_compound_type: symbol_table.move() failed";
 
   // put into scope
   cpp_idt &id=cpp_scopes.put_into_scope(*new_symbol);
@@ -440,7 +442,7 @@ void cpp_typecheckt::typecheck_class_template_member(
 
   const cpp_idt &cpp_id=**(id_set.begin());
   symbolt &template_symbol=
-    context.symbols.find(cpp_id.identifier)->second;
+    symbol_table.symbols.find(cpp_id.identifier)->second;
 
   exprt &template_methods=static_cast<exprt &>(
     template_symbol.value.add("template_methods"));
@@ -662,10 +664,10 @@ void cpp_typecheckt::convert_class_template_specialization(
     throw 0;
   }
   
-  contextt::symbolst::iterator s_it=
-    context.symbols.find((*id_set.begin())->identifier);
+  symbol_tablet::symbolst::iterator s_it=
+    symbol_table.symbols.find((*id_set.begin())->identifier);
     
-  assert(s_it!=context.symbols.end());
+  assert(s_it!=symbol_table.symbols.end());
   
   symbolt &template_symbol=s_it->second;
     
@@ -906,7 +908,7 @@ cpp_scopet &cpp_typecheckt::typecheck_template_parameters(
       parameter.type().location()=declaration.find_location();
     }
     else
-      parameter=symbol_expr(symbol);
+      parameter=symbol.symbol_expr();
       
     // set (non-typechecked) default value
     if(default_value.is_not_nil())
