@@ -6,7 +6,7 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 
 \*******************************************************************/
 
-#include <location.h>
+#include <util/location.h>
 
 #include "cpp_typecheck.h"
 
