@@ -104,6 +104,7 @@ TOK_REAL,
 TOK_IMAG,
 TOK_NULLPTR,
 TOK_ASM_STRING,
+TOK_ALIGNOF,
 
 TOK_Ignore,
 TOK_GCC_ASM,
@@ -118,6 +119,7 @@ TOK_FASTCALL,
 TOK_CLRCALL,
 TOK_STATIC_ASSERT,
 TOK_GCC_FLOAT128,
+TOK_GCC_INT128,
 TOK_CPROVER_BOOL,
 
 // MSC-specific
