@@ -9,20 +9,22 @@ Date: August 2012
 \*******************************************************************/
 
 #ifdef _WIN32
-
+#include <process.h>
 #else
 
-#include <string.h>
+#include <cstring>
 #include <unistd.h>
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cerrno>
+#include <cstdio>
+#include <cstdlib>
 
 #include <sys/wait.h>
 #include <sys/types.h>
 
 #endif
 
+#include <util/unicode.h>
+
 #include "run.h"
   
 /*******************************************************************\
@@ -42,7 +44,30 @@ int run(
   const std::vector<std::string> &argv)
 {
   #ifdef _WIN32
-  return 1;
+  // unicode version of the arguments
+  std::vector<std::wstring> wargv;
+  
+  wargv.resize(argv.size());
+  
+  for(unsigned i=0; i<argv.size(); i++)
+    wargv[i]=widen(argv[i]);
+
+  const wchar_t **_argv=new const wchar_t * [argv.size()+1];
+
+  for(unsigned i=0; i<wargv.size(); i++)
+    _argv[i]=wargv[i].c_str();
+  
+  _argv[argv.size()]=NULL;
+
+  // warning: the arguments may still need escaping
+
+  std::wstring wide_what=widen(what);
+
+  int status=_wspawnvp(_P_WAIT, wide_what.c_str(), _argv);
+
+  delete[] _argv;  
+
+  return status;
   
   #else
   pid_t childpid; /* variable to store the child's pid */
