@@ -11,7 +11,7 @@ Date: June 2006
 #include <sstream>
 #include <iostream>
 
-#include <xml_irep.h>
+#include <util/xml_irep.h>
 
 #include "xml_goto_program.h"
 
@@ -199,7 +199,7 @@ void convert(const goto_programt &goto_program,
     if (ins_it->function!="")
     {
       xmlt &fnc = ins.new_element("function");
-      fnc.data = ins_it->function.as_string();       
+      fnc.data = id2string(ins_it->function);
     }
   }
 }
