@@ -13,7 +13,7 @@ Date: July 2006
 #define XML_GOTO_PROGRAM_H_
 
 #include <goto-programs/goto_program.h>
-#include <xml.h>
+#include <util/xml.h>
 
 #include "xml_irep_hashing.h"
 
