@@ -9,12 +9,12 @@ Author:
 #ifndef CPROVER_ALIGNMENT_CHECKS_H
 #define CPROVER_ALIGNMENT_CHECKS_H
 
-#include <iostream>
+#include <ostream>
 
 #include <goto-programs/goto_functions.h>
 
 void print_struct_alignment_problems(
-  const contextt &context,
+  const symbol_tablet &symbol_table,
   std::ostream &out);
 
 #endif
