@@ -8,12 +8,11 @@ Date: February 2006
 
 \*******************************************************************/
 
-#include <hash_cont.h>
-#include <std_expr.h>
-#include <expr_util.h>
-#include <guard.h>
-#include <context.h>
-#include <prefix.h>
+#include <util/hash_cont.h>
+#include <util/std_expr.h>
+#include <util/guard.h>
+#include <util/symbol_table.h>
+#include <util/prefix.h>
 
 #include <pointer-analysis/value_sets.h>
 #include <goto-programs/remove_skip.h>
@@ -24,7 +23,7 @@ Date: February 2006
 class w_guardst
 {
 public:
-  w_guardst(contextt &_context):context(_context)
+  w_guardst(symbol_tablet &_symbol_table):symbol_table(_symbol_table)
   {
   }
   
@@ -34,7 +33,7 @@ public:
   
   const exprt get_guard_symbol_expr(const irep_idt &object)
   {
-    return symbol_expr(get_guard_symbol(object));
+    return get_guard_symbol(object).symbol_expr();
   }
   
   const exprt get_w_guard_expr(const rw_set_baset::entryt &entry)
@@ -44,13 +43,13 @@ public:
   
   const exprt get_assertion(const rw_set_baset::entryt &entry)
   {
-    return gen_not(get_guard_symbol_expr(entry.object));
+    return not_exprt(get_guard_symbol_expr(entry.object));
   }
   
   void add_initialization(goto_programt &goto_program) const;
   
 protected:
-  contextt &context;
+  symbol_tablet &symbol_table;
 };
 
 /*******************************************************************\
@@ -69,10 +68,10 @@ const symbolt &w_guardst::get_guard_symbol(const irep_idt &object)
 {
   const irep_idt identifier=id2string(object)+"$w_guard";
 
-  const contextt::symbolst::const_iterator it=
-    context.symbols.find(identifier);
+  const symbol_tablet::symbolst::const_iterator it=
+    symbol_table.symbols.find(identifier);
 
-  if(it!=context.symbols.end())
+  if(it!=symbol_table.symbols.end())
     return it->second;
     
   w_guards.push_back(identifier);
@@ -81,11 +80,11 @@ const symbolt &w_guardst::get_guard_symbol(const irep_idt &object)
   new_symbol.name=identifier;
   new_symbol.base_name=identifier;
   new_symbol.type=bool_typet();
-  new_symbol.static_lifetime=true;
-  new_symbol.value.make_false();
+  new_symbol.is_static_lifetime=true;
+  new_symbol.value=false_exprt();
   
   symbolt *symbol_ptr;
-  context.move(new_symbol, symbol_ptr);
+  symbol_table.move(new_symbol, symbol_ptr);
   return *symbol_ptr;
 }
 
@@ -104,14 +103,14 @@ Function: w_guardst::add_initialization
 void w_guardst::add_initialization(goto_programt &goto_program) const
 {
   goto_programt::targett t=goto_program.instructions.begin();
-  const namespacet ns(context);
+  const namespacet ns(symbol_table);
 
   for(std::list<irep_idt>::const_iterator
       it=w_guards.begin();
       it!=w_guards.end();
       it++)
   {
-    exprt symbol=symbol_expr(ns.lookup(*it));
+    exprt symbol=ns.lookup(*it).symbol_expr();
   
     t=goto_program.insert_before(t);
     t->type=ASSIGN;
@@ -170,12 +169,12 @@ bool is_shared(
      identifier=="c::stdout" ||
      identifier=="c::stderr" ||
      identifier=="c::sys_nerr" ||
-     has_prefix(identifier.as_string(), "symex::invalid_object") ||
-     has_prefix(identifier.as_string(), "symex_dynamic::dynamic_object"))
+     has_prefix(id2string(identifier), "symex::invalid_object") ||
+     has_prefix(id2string(identifier), "symex_dynamic::dynamic_object"))
     return false; // no race check
 
   const symbolt &symbol=ns.lookup(identifier);
-  return is_global(symbol);
+  return symbol.is_shared();
 }
 
 /*******************************************************************\
@@ -225,11 +224,11 @@ Function: race_check
 
 void race_check(
   value_setst &value_sets,
-  contextt &context,
+  symbol_tablet &symbol_table,
   goto_programt &goto_program,
   w_guardst &w_guards)
 {
-  namespacet ns(context);
+  namespacet ns(symbol_table);
 
   Forall_goto_program_instructions(i_it, goto_program)
   {
@@ -333,12 +332,12 @@ Function: race_check
 
 void race_check(
   value_setst &value_sets,
-  contextt &context,
+  symbol_tablet &symbol_table,
   goto_programt &goto_program)
 {
-  w_guardst w_guards(context);
+  w_guardst w_guards(symbol_table);
 
-  race_check(value_sets, context, goto_program, w_guards);
+  race_check(value_sets, symbol_table, goto_program, w_guards);
 
   w_guards.add_initialization(goto_program);
   goto_program.update();
@@ -358,19 +357,19 @@ Function: race_check
 
 void race_check(
   value_setst &value_sets,
-  contextt &context,
+  symbol_tablet &symbol_table,
   goto_functionst &goto_functions)
 {
-  w_guardst w_guards(context);
+  w_guardst w_guards(symbol_table);
 
   Forall_goto_functions(f_it, goto_functions)
     if(f_it->first!=ID_main &&
        f_it->first!="c::__CPROVER_initialize")
-      race_check(value_sets, context, f_it->second.body, w_guards);
+      race_check(value_sets, symbol_table, f_it->second.body, w_guards);
 
   // get "main"
   goto_functionst::function_mapt::iterator
-    m_it=goto_functions.function_map.find(goto_functions.main_id());
+    m_it=goto_functions.function_map.find(goto_functions.entry_point());
 
   if(m_it==goto_functions.function_map.end())
     throw "Race check instrumentation needs an entry point";
