@@ -8,10 +8,9 @@ Date: February 2006
 
 \*******************************************************************/
 
-#include <expr_util.h>
-#include <std_expr.h>
-#include <std_code.h>
-#include <namespace.h>
+#include <util/std_expr.h>
+#include <util/std_code.h>
+#include <util/namespace.h>
 
 #include <langapi/language_util.h>
 
@@ -179,6 +178,11 @@ void rw_set_loct::read_write_rec(
     
     read_write_rec(tmp, r, w, suffix, guard);
   }
+  else if(expr.id()==ID_typecast)
+  {
+    assert(expr.operands().size()==1);
+    read_write_rec(expr.op0(), r, w, suffix, guard);
+  }
   else if(expr.id()==ID_address_of)
   {
     assert(expr.operands().size()==1);
@@ -194,7 +198,7 @@ void rw_set_loct::read_write_rec(
     read_write_rec(expr.op1(), r, w, suffix, true_guard);
     
     guardt false_guard(guard);
-    false_guard.add(gen_not(expr.op0()));
+    false_guard.add(not_exprt(expr.op0()));
     read_write_rec(expr.op2(), r, w, suffix, false_guard);
   }
   else
