@@ -11,14 +11,15 @@ Date: February 2006
 #ifndef CPROVER_GOTO_PROGRAMS_RW_SET
 #define CPROVER_GOTO_PROGRAMS_RW_SET
 
-#include <iostream>
+#include <ostream>
 
-#include <hash_cont.h>
-#include <guard.h>
-#include <std_code.h>
-#include <namespace.h>
-#include <std_expr.h>
+#include <util/hash_cont.h>
+#include <util/guard.h>
+#include <util/std_code.h>
+#include <util/namespace.h>
+#include <util/std_expr.h>
 
+#include <goto-programs/goto_functions.h>
 #include <pointer-analysis/value_sets.h>
 
 // a container for read/write sets
@@ -78,7 +79,8 @@ protected:
   const namespacet &ns;
 };
 
-extern inline std::ostream & operator << (std::ostream &out, const rw_set_baset &rw_set)
+extern inline std::ostream & operator << (
+  std::ostream &out, const rw_set_baset &rw_set)
 {
   rw_set.output(out);
   return out;
