@@ -8,12 +8,12 @@ Date: November 2011
 
 \*******************************************************************/
 
-#include <context.h>
-#include <std_expr.h>
-#include <std_types.h>
-#include <arith_tools.h>
-#include <cprover_prefix.h>
-#include <i2string.h>
+#include <util/symbol_table.h>
+#include <util/std_expr.h>
+#include <util/std_types.h>
+#include <util/arith_tools.h>
+#include <util/cprover_prefix.h>
+#include <util/i2string.h>
 
 #include <goto-programs/goto_functions.h>
 
@@ -31,7 +31,7 @@ Function: add_stack_depth_symbol
 
 \*******************************************************************/
 
-symbol_exprt add_stack_depth_symbol(contextt &context)
+symbol_exprt add_stack_depth_symbol(symbol_tablet &symbol_table)
 {
   const irep_idt identifier="$stack_depth";
   signedbv_typet type(sizeof(int)*8);
@@ -41,13 +41,13 @@ symbol_exprt add_stack_depth_symbol(contextt &context)
   new_symbol.base_name=identifier;
   new_symbol.pretty_name=identifier;
   new_symbol.type=type;
-  new_symbol.static_lifetime=true;
+  new_symbol.is_static_lifetime=true;
   new_symbol.value=from_integer(0, type);
   new_symbol.mode=ID_C;
-  new_symbol.thread_local=true;
-  new_symbol.lvalue=true;
+  new_symbol.is_thread_local=true;
+  new_symbol.is_lvalue=true;
 
-  context.move(new_symbol);
+  symbol_table.move(new_symbol);
 
   return symbol_exprt(identifier, type);
 }
@@ -116,11 +116,11 @@ Function: stack_depth
 \*******************************************************************/
 
 void stack_depth(
-  contextt &context,
+  symbol_tablet &symbol_table,
   goto_functionst &goto_functions,
   const int depth)
 {
-  const symbol_exprt sym=add_stack_depth_symbol(context);
+  const symbol_exprt sym=add_stack_depth_symbol(symbol_table);
   const exprt depth_expr(from_integer(depth, sym.type()));
 
   Forall_goto_functions(f_it, goto_functions)
