@@ -6,25 +6,25 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
-#include <i2string.h>
-#include <replace_expr.h>
-#include <expr_util.h>
-#include <rational_tools.h>
-#include <location.h>
-#include <cprover_prefix.h>
-#include <prefix.h>
-#include <arith_tools.h>
-#include <simplify_expr.h>
-#include <std_code.h>
-#include <std_expr.h>
-#include <symbol.h>
+#include <util/i2string.h>
+#include <util/replace_expr.h>
+#include <util/expr_util.h>
+#include <util/rational_tools.h>
+#include <util/location.h>
+#include <util/cprover_prefix.h>
+#include <util/prefix.h>
+#include <util/arith_tools.h>
+#include <util/simplify_expr.h>
+#include <util/std_code.h>
+#include <util/std_expr.h>
+#include <util/symbol.h>
+#include <util/pointer_predicates.h>
 
 #include <ansi-c/c_types.h>
 
 #include "goto_convert_class.h"
-#include "dynamic_memory.h"
 
 /*******************************************************************\
 
@@ -228,6 +228,8 @@ void goto_convertt::do_printf(
       copy(to_code(printf_code), OTHER, dest);
     }
   }
+  else
+    assert(false);
 }
 
 /*******************************************************************\
@@ -454,8 +456,8 @@ void goto_convertt::do_cpp_new(
   if(rhs.operands().size()==0) // no, "regular" one
   {
     // call __new or __new_array
-    exprt new_symbol=symbol_expr(
-      ns.lookup(new_array?"c::__new_array":"c::__new"));
+    exprt new_symbol=
+      ns.lookup(new_array?"c::__new_array":"c::__new").symbol_expr();
     
     const code_typet &code_type=
       to_code_type(new_symbol.type());
@@ -469,7 +471,7 @@ void goto_convertt::do_cpp_new(
     const symbolt &tmp_symbol=
       new_tmp_symbol(return_type, "new", dest, rhs.location());
     
-    tmp_symbol_expr=symbol_expr(tmp_symbol);
+    tmp_symbol_expr=tmp_symbol.symbol_expr();
     
     code_function_callt new_call;
     new_call.function()=new_symbol;
@@ -484,8 +486,8 @@ void goto_convertt::do_cpp_new(
   else if(rhs.operands().size()==1)
   {
     // call __placement_new
-    exprt new_symbol=symbol_expr(
-      ns.lookup(new_array?"c::__placement_new_array":"c::__placement_new"));
+    exprt new_symbol=
+      ns.lookup(new_array?"c::__placement_new_array":"c::__placement_new").symbol_expr();
     
     const code_typet &code_type=
       to_code_type(new_symbol.type());
@@ -498,7 +500,7 @@ void goto_convertt::do_cpp_new(
     const symbolt &tmp_symbol=
       new_tmp_symbol(return_type, "new", dest, rhs.location());
 
-    tmp_symbol_expr=symbol_expr(tmp_symbol);
+    tmp_symbol_expr=tmp_symbol.symbol_expr();
 
     code_function_callt new_call;
     new_call.function()=new_symbol;
@@ -532,6 +534,73 @@ void goto_convertt::do_cpp_new(
 
 /*******************************************************************\
 
+Function: goto_convertt::do_java_new
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void goto_convertt::do_java_new(
+  const exprt &lhs,
+  const side_effect_exprt &rhs,
+  goto_programt &dest)
+{
+  if(lhs.is_nil())
+    throw "do_java_new without lhs is yet to be implemented";
+
+  // the object size is the first argument
+  bool new_array=rhs.get(ID_statement)==ID_java_new_array;
+
+  if(new_array)
+    assert(rhs.operands().size()==2);
+  else
+    assert(rhs.operands().size()==1);
+
+  typet object_type=rhs.type().subtype();
+
+  // build size expression
+  exprt object_size=rhs.op0();
+  exprt size;
+
+  if(new_array)
+  {
+    exprt count=rhs.op1();
+
+    // might have side-effect
+    clean_expr(count, dest);
+
+    if(count.type()!=object_size.type())
+      count.make_typecast(object_size.type());
+
+    // multiply to get final size
+    size=mult_exprt(object_size, count);
+  }
+  else
+    size=object_size;
+
+  // we produce a malloc side-effect, which stays
+  side_effect_exprt malloc_expr(ID_malloc);
+  
+  malloc_expr.copy_to_operands(size);
+  malloc_expr.set("#type", object_type);
+
+  goto_programt::targett t_n=dest.add_instruction(ASSIGN);
+  t_n->code=code_assignt(lhs, malloc_expr);
+  t_n->location=rhs.find_location();
+
+  // grab initializer
+  goto_programt tmp_initializer;
+  cpp_new_initializer(lhs, rhs, tmp_initializer);
+
+  dest.destructive_append(tmp_initializer);
+}
+
+/*******************************************************************\
+
 Function: goto_convertt::cpp_new_initializer
 
   Inputs:
@@ -759,6 +828,33 @@ bool is_lvalue(const exprt &expr)
 
 /*******************************************************************\
 
+Function: make_va_list
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+exprt make_va_list(const exprt &expr)
+{
+  // we first strip any typecast
+  if(expr.id()==ID_typecast)
+    return make_va_list(to_typecast_expr(expr).op());
+
+  // if it's an address of an lvalue, we take that
+  if(expr.id()==ID_address_of &&
+     expr.operands().size()==1 &&
+     is_lvalue(expr.op0()))
+    return expr.op0();
+
+  return expr;
+}
+
+/*******************************************************************\
+
 Function: goto_convertt::do_function_call_symbol
 
   Inputs:
@@ -841,9 +937,6 @@ void goto_convertt::do_function_call_symbol(
       throw "`"+id2string(identifier)+"' expected to have one argument";
     }
 
-    if(!options.get_bool_option("assumptions"))
-      return;
-
     goto_programt::targett t=dest.add_instruction(ASSUME);
     t->guard=arguments.front();
     t->location=function.location();
@@ -859,6 +952,29 @@ void goto_convertt::do_function_call_symbol(
       throw id2string(identifier)+" expected not to have LHS";
     }
   }
+  else if(has_prefix(id2string(identifier), "java::java.lang.AssertionError.<init>:"))
+  {
+    // insert function call anyway
+    code_function_callt function_call;
+    function_call.lhs()=lhs;
+    function_call.function()=function;
+    function_call.arguments()=arguments;
+    function_call.location()=function.location();
+
+    copy(function_call, FUNCTION_CALL, dest);
+
+    if(arguments.size()!=1 && arguments.size()!=2)
+    {
+      err_location(function);
+      throw "`"+id2string(identifier)+"' expected to have one or two arguments";
+    }
+
+    goto_programt::targett t=dest.add_instruction(ASSERT);
+    t->guard=false_exprt();
+    t->location=function.location();
+    t->location.set("user-provided", true);
+    t->location.set_property(ID_assertion);    
+  }
   else if(identifier=="c::assert")
   {
     if(arguments.size()!=1)
@@ -867,9 +983,6 @@ void goto_convertt::do_function_call_symbol(
       throw "`"+id2string(identifier)+"' expected to have one argument";
     }
 
-    if(!options.get_bool_option("assertions"))
-      return;
-
     goto_programt::targett t=dest.add_instruction(ASSERT);
     t->guard=arguments.front();
     t->location=function.location();
@@ -897,9 +1010,6 @@ void goto_convertt::do_function_call_symbol(
     const irep_idt description=
       get_string_constant(arguments[1]);
 
-    if(!options.get_bool_option("assertions"))
-      return;
-
     goto_programt::targett t=dest.add_instruction(ASSERT);
     t->guard=arguments[0];
     t->location=function.location();
@@ -962,6 +1072,7 @@ void goto_convertt::do_function_call_symbol(
 
     exprt rhs=side_effect_expr_nondett(lhs.type());
     rhs.location()=function.location();
+    rhs.set(ID_C_identifier, identifier);
 
     code_assignt assignment(lhs, rhs);
     assignment.location()=function.location();
@@ -996,10 +1107,12 @@ void goto_convertt::do_function_call_symbol(
   {
     do_array_copy(lhs, function, arguments, dest);
   }
-  else if(identifier=="c::printf" ||
+  else if(identifier=="c::printf")
+  /*
           identifier=="c::fprintf" ||
           identifier=="c::sprintf" ||
           identifier=="c::snprintf")
+  */
   {
     do_printf(lhs, function, arguments, dest);
   }
@@ -1018,9 +1131,6 @@ void goto_convertt::do_function_call_symbol(
     const irep_idt description=
       "assertion "+id2string(get_string_constant(arguments[0]));
 
-    if(!options.get_bool_option("assertions"))
-      return;
-
     goto_programt::targett t=dest.add_instruction(ASSERT);
     t->guard=false_exprt();
     t->location=function.location();
@@ -1044,9 +1154,6 @@ void goto_convertt::do_function_call_symbol(
     const irep_idt description=
       "assertion "+id2string(get_string_constant(arguments[3]));
 
-    if(!options.get_bool_option("assertions"))
-      return;
-
     goto_programt::targett t=dest.add_instruction(ASSERT);
     t->guard=false_exprt();
     t->location=function.location();
@@ -1055,6 +1162,28 @@ void goto_convertt::do_function_call_symbol(
     t->location.set_comment(description);
     // we ignore any LHS
   }
+  else if(identifier=="c::__assert_func")
+  {
+    // __assert_func is newlib (used by, e.g., cygwin)
+    // These take four arguments:
+    // "file.c", line, __func__, "expression"
+    if(arguments.size()!=4)
+    {
+      err_location(function);
+      throw "`"+id2string(identifier)+"' expected to have four arguments";
+    }
+
+    const irep_idt description=
+      "assertion "+id2string(get_string_constant(arguments[3]));
+      goto_programt::targett t=dest.add_instruction(ASSERT);
+
+    t->guard=false_exprt();
+    t->location=function.location();
+    t->location.set("user-provided", true);
+    t->location.set_property(ID_assertion);
+    t->location.set_comment(description);
+    // we ignore any LHS
+  }
   else if(identifier=="c::_wassert")
   {
     // This is Windows. The arguments are
@@ -1069,9 +1198,6 @@ void goto_convertt::do_function_call_symbol(
     const irep_idt description=
       "assertion "+id2string(get_string_constant(arguments[0]));
 
-    if(!options.get_bool_option("assertions"))
-      return;
-
     goto_programt::targett t=dest.add_instruction(ASSERT);
     t->guard=false_exprt();
     t->location=function.location();
@@ -1102,6 +1228,10 @@ void goto_convertt::do_function_call_symbol(
   {
     // does nothing
   }
+  else if(identifier=="c::__builtin_unreachable")
+  {
+    // says something like assert(false);
+  }
   else if(identifier==ID_gcc_builtin_va_arg)
   {
     // This does two things.
@@ -1116,12 +1246,15 @@ void goto_convertt::do_function_call_symbol(
       throw "`"+id2string(identifier)+"' expected to have one argument";
     }
     
+    exprt list_arg=make_va_list(arguments[0]);
+    
     {
-      sideeffect_exprt rhs(ID_gcc_builtin_va_arg_next, arguments[0].type());
-      rhs.copy_to_operands(arguments[0]);
+      sideeffect_exprt rhs(ID_gcc_builtin_va_arg_next, list_arg.type());
+      rhs.copy_to_operands(list_arg);
+      rhs.set("#va_arg_type", to_code_type(function.type()).return_type());
       goto_programt::targett t1=dest.add_instruction(ASSIGN);
       t1->location=function.location();
-      t1->code=code_assignt(arguments[0], rhs);
+      t1->code=code_assignt(list_arg, rhs);
     }
 
     if(lhs.is_not_nil())
@@ -1129,7 +1262,7 @@ void goto_convertt::do_function_call_symbol(
       typet t=pointer_typet();
       t.subtype()=lhs.type();
       dereference_exprt rhs(lhs.type());
-      rhs.op0()=typecast_exprt(arguments[0], t);
+      rhs.op0()=typecast_exprt(list_arg, t);
       rhs.location()=function.location();
       goto_programt::targett t2=dest.add_instruction(ASSIGN);
       t2->location=function.location();
@@ -1144,18 +1277,18 @@ void goto_convertt::do_function_call_symbol(
       throw "`"+id2string(identifier)+"' expected to have two arguments";
     }
     
-    exprt lhs=arguments[0];
-    exprt rhs=typecast_exprt(arguments[1], lhs.type());
+    exprt dest_expr=make_va_list(arguments[0]);
+    exprt src_expr=typecast_exprt(arguments[1], dest_expr.type());
 
-    if(!is_lvalue(lhs))
+    if(!is_lvalue(dest_expr))
     {
-      err_location(lhs);
+      err_location(dest_expr);
       throw "va_copy argument expected to be lvalue";
     }    
     
     goto_programt::targett t=dest.add_instruction(ASSIGN);
     t->location=function.location();
-    t->code=code_assignt(lhs, rhs);
+    t->code=code_assignt(dest_expr, src_expr);
   }
   else if(identifier=="c::__builtin_va_start")
   {
@@ -1167,19 +1300,19 @@ void goto_convertt::do_function_call_symbol(
       throw "`"+id2string(identifier)+"' expected to have two arguments";
     }
     
-    exprt lhs=arguments[0];
-    exprt rhs=typecast_exprt(
-      address_of_exprt(arguments[1]), lhs.type());
+    exprt dest_expr=make_va_list(arguments[0]);
+    exprt src_expr=typecast_exprt(
+      address_of_exprt(arguments[1]), dest_expr.type());
 
-    if(!is_lvalue(lhs))
+    if(!is_lvalue(dest_expr))
     {
-      err_location(lhs);
+      err_location(dest_expr);
       throw "va_start argument expected to be lvalue";
     }    
     
     goto_programt::targett t=dest.add_instruction(ASSIGN);
     t->location=function.location();
-    t->code=code_assignt(lhs, rhs);
+    t->code=code_assignt(dest_expr, src_expr);
   }
   else if(identifier=="c::__builtin_va_end")
   {
@@ -1190,15 +1323,17 @@ void goto_convertt::do_function_call_symbol(
       throw "`"+id2string(identifier)+"' expected to have one argument";
     }
     
-    if(!is_lvalue(arguments[0]))
+    exprt dest_expr=make_va_list(arguments[0]);
+    
+    if(!is_lvalue(dest_expr))
     {
-      err_location(lhs);
+      err_location(dest_expr);
       throw "va_end argument expected to be lvalue";
     }    
     
     goto_programt::targett t=dest.add_instruction(ASSIGN);
     t->location=function.location();
-    t->code=code_assignt(arguments[0], gen_zero(arguments[0].type()));
+    t->code=code_assignt(dest_expr, gen_zero(dest_expr.type()));
   }
   else if(identifier=="c::__sync_fetch_and_add" ||
           identifier=="c::__sync_fetch_and_sub" ||
