@@ -7,17 +7,14 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
-#include <std_expr.h>
-#include <std_code.h>
-#include <expr_util.h>
+#include <util/std_expr.h>
+#include <util/std_code.h>
+#include <util/expr_util.h>
 
 #include "flow_insensitive_analysis.h"
 
-//#include <fstream>
-//#include <i2string.h>
-
 /*******************************************************************\
 
 Function: flow_insensitive_abstract_domain_baset::get_guard
@@ -307,7 +304,7 @@ bool flow_insensitive_analysis_baset::visit(
     }
   }
   
-//  if (l->function.as_string().find("debug")!=std::string::npos)
+//  if (id2string(l->function).find("debug")!=std::string::npos)
 //    std::cout << l->function << std::endl; //=="c::messages::debug")
   
 //  {
@@ -454,10 +451,7 @@ bool flow_insensitive_analysis_baset::do_function_call_rec(
       goto_functions.function_map.find(identifier);
       
     if(it==goto_functions.function_map.end())
-    {      
-      std::cout << "failed to find function " << id2string(identifier) << std::endl;
       throw "failed to find function "+id2string(identifier);
-    }
     
     new_data = 
       do_function_call(
