@@ -6,11 +6,12 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <expr_util.h>
-#include <std_expr.h>
-#include <rename.h>
-#include <cprover_prefix.h>
-#include <i2string.h>
+#include <util/simplify_expr.h>
+#include <util/expr_util.h>
+#include <util/std_expr.h>
+#include <util/rename.h>
+#include <util/cprover_prefix.h>
+#include <util/i2string.h>
 
 #include <ansi-c/c_types.h>
 
@@ -42,22 +43,24 @@ symbol_exprt goto_convertt::make_static_symbol(
   {
     new_symbol.base_name="static_"+suffix+"$"+i2string(++temporary_counter);
     new_symbol.name=tmp_symbol_prefix+id2string(new_symbol.base_name);
-    new_symbol.lvalue=true;
-    new_symbol.thread_local=false;
-    new_symbol.static_lifetime=true;
-    new_symbol.file_local=true;
+    new_symbol.is_lvalue=true;
+    new_symbol.is_thread_local=false;
+    new_symbol.is_static_lifetime=true;
+    new_symbol.is_file_local=true;
     new_symbol.value=expr;
     new_symbol.type=expr.type();
+    new_symbol.location=location;
   }
-  while(context.move(new_symbol, symbol_ptr));    
+  while(symbol_table.move(new_symbol, symbol_ptr));    
 
   // The value might depend on a variable, thus
   // generate code for this.
 
-  symbol_exprt result=symbol_expr(*symbol_ptr);
+  symbol_exprt result=symbol_ptr->symbol_expr();
+  result.location()=location;
   
   code_assignt code_assign(result, expr);
-  code_assign.location()=expr.find_location();
+  code_assign.location()=location;
   convert(code_assign, dest);
 
   return result;
@@ -77,8 +80,7 @@ Function: goto_convertt::needs_cleaning
 
 bool goto_convertt::needs_cleaning(const exprt &expr)
 {
-  if(expr.id()==ID_index ||
-     expr.id()==ID_dereference ||
+  if(expr.id()==ID_dereference ||
      expr.id()==ID_sideeffect ||
      expr.id()==ID_struct ||
      expr.id()==ID_array ||
@@ -86,6 +88,17 @@ bool goto_convertt::needs_cleaning(const exprt &expr)
      expr.id()==ID_comma)
     return true;
 
+  if(expr.id()==ID_index)
+  {
+    // Will usually clean index because of possible memory violation.
+    // We do an exception for "string-lit"[0], which is safe.
+    if(to_index_expr(expr).array().id()==ID_string_constant &&
+       to_index_expr(expr).index().is_zero())
+      return false;
+    
+    return true;
+  }
+
   // We can't flatten quantified expressions by introducing new literals for
   // conditional expressions.  This is because the body of the quantified
   // may refer to bound variables, which are not visible outside the scope
@@ -203,48 +216,134 @@ void goto_convertt::clean_expr(
   }
   else if(expr.id()==ID_if)
   {
-    if(expr.operands().size()!=3)
-      throw "if takes three arguments";
+    // first clean condition
+    clean_expr(to_if_expr(expr).cond(), dest, true);
 
-    if(!expr.op0().is_boolean())
+    // possibly done now
+    if(!needs_cleaning(to_if_expr(expr).true_case()) &&
+       !needs_cleaning(to_if_expr(expr).false_case()))
+      return;
+
+    // copy expression
+    if_exprt if_expr=to_if_expr(expr);
+    
+    if(!if_expr.cond().is_boolean())
       throw "first argument of `if' must be boolean, but got "
-        +expr.op0().to_string();
+        +if_expr.cond().to_string();
 
-    // first pull out condition -- we need to prevent
-    // this getting destroyed by the side-effects in the other
-    // operands
-    make_temp_symbol(expr.op0(), "condition", dest);
+    const locationt location=expr.find_location();
+  
+    // We do some constant-folding here, to mimic
+    // what typical compilers do.
+    {
+      exprt tmp_cond=if_expr.cond();
+      simplify(tmp_cond, ns);
+      if(tmp_cond.is_true())
+      {
+        clean_expr(if_expr.true_case(), dest, result_is_used);
+        expr=if_expr.true_case();
+        return;
+      }
+      else if(tmp_cond.is_false())
+      {
+        clean_expr(if_expr.false_case(), dest, result_is_used);
+        expr=if_expr.false_case();
+        return;
+      }
+    }
 
-    // now clean arguments    
-    goto_programt tmp_true, tmp_false;
-    clean_expr(expr.op1(), tmp_true, result_is_used);
-    clean_expr(expr.op2(), tmp_false, result_is_used);
+    goto_programt tmp_true;
+    clean_expr(if_expr.true_case(), tmp_true, result_is_used);
+
+    goto_programt tmp_false;
+    clean_expr(if_expr.false_case(), tmp_false, result_is_used);
+    
+    if(result_is_used)
+    {
+      symbolt &new_symbol=
+        new_tmp_symbol(expr.type(), "if_expr", dest, location);
+
+      code_assignt assignment_true;
+      assignment_true.lhs()=new_symbol.symbol_expr();
+      assignment_true.rhs()=if_expr.true_case();
+      assignment_true.location()=location;
+      convert(assignment_true, tmp_true);
+
+      code_assignt assignment_false;
+      assignment_false.lhs()=new_symbol.symbol_expr();
+      assignment_false.rhs()=if_expr.false_case();
+      assignment_false.location()=location;
+      convert(assignment_false, tmp_false);
+
+      // overwrites expr
+      expr=new_symbol.symbol_expr();
+    }
+    else
+    {
+      // preserve the expressions for possible later checks
+      if(if_expr.true_case().is_not_nil())
+      {
+        code_expressiont code_expression(if_expr.true_case());
+        convert(code_expression, tmp_true);
+      }
+      
+      if(if_expr.false_case().is_not_nil())
+      {
+        code_expressiont code_expression(if_expr.false_case());
+        convert(code_expression, tmp_false);
+      }
+      
+      expr=nil_exprt();
+    }
 
     // generate guard for argument side-effects    
     generate_ifthenelse(
-      expr.op0(), tmp_true, tmp_false,
-      expr.location(), dest);
+      if_expr.cond(), tmp_true, tmp_false,
+      location, dest);
 
     return;
   }
   else if(expr.id()==ID_comma)
   {
-    exprt result;
-  
-    Forall_operands(it, expr)
+    if(result_is_used)
     {
-      bool last=(it==--expr.operands().end());
-      
-      if(last)
-      {
-        result.swap(*it);
-        clean_expr(result, dest, result_is_used);
-      }
-      else
-        clean_expr(*it, dest, false);
-    }
+      exprt result;
     
-    expr.swap(result);
+      Forall_operands(it, expr)
+      {
+        bool last=(it==--expr.operands().end());
+        
+        // special treatment for last one
+        if(last)
+        {
+          result.swap(*it);
+          clean_expr(result, dest, true);
+        }
+        else
+        {
+          clean_expr(*it, dest, false);
+
+          // remember these for later checks
+          if(it->is_not_nil())
+            convert(code_expressiont(*it), dest);
+        }
+      }
+
+      expr.swap(result);
+    }
+    else // result not used
+    {
+      Forall_operands(it, expr)
+      {
+        clean_expr(*it, dest, false);
+
+        // remember as expression statement for later checks
+        if(it->is_not_nil())
+          convert(code_expressiont(*it), dest);
+      }
+      
+      expr=nil_exprt();
+    }
     
     return;
   }
@@ -256,12 +355,15 @@ void goto_convertt::clean_expr(
     // preserve 'result_is_used'
     clean_expr(expr.op0(), dest, result_is_used);
     
+    if(expr.op0().is_nil())
+      expr.make_nil();
+    
     return;
   }
   else if(expr.id()==ID_sideeffect)
   {
     // some of the side-effects need special treatment!
-    const irep_idt statement=expr.get(ID_statement);
+    const irep_idt statement=to_side_effect_expr(expr).get_statement();
     
     if(statement==ID_gcc_conditional_expression)
     {
@@ -276,6 +378,51 @@ void goto_convertt::clean_expr(
       remove_statement_expression(to_side_effect_expr(expr), dest, result_is_used);
       return;
     }
+    else if(statement==ID_assign)
+    {
+      // we do a special treatment for x=f(...)
+      assert(expr.operands().size()==2);
+
+      if(expr.op1().id()==ID_sideeffect &&
+         to_side_effect_expr(expr.op1()).get_statement()==ID_function_call)
+      {
+        clean_expr(expr.op0(), dest);
+        exprt lhs=expr.op0();
+
+        // turn into code
+        code_assignt assignment;
+        assignment.lhs()=lhs;
+        assignment.rhs()=expr.op1();
+        assignment.location()=expr.location();
+        convert_assign(assignment, dest);
+
+        if(result_is_used)
+          expr.swap(lhs);
+        else
+          expr.make_nil();
+        return;
+      }
+    }
+    else if(statement==ID_function_call)
+    {
+      if(to_side_effect_expr_function_call(expr).function().id()==ID_symbol &&
+         to_symbol_expr(to_side_effect_expr_function_call(expr).function()).get_identifier()=="c::__noop")
+      {
+        // __noop needs special treatment, as arguments are not
+        // evaluated
+        to_side_effect_expr_function_call(expr).arguments().clear();
+      }
+    }
+  }
+  else if(expr.id()==ID_forall || expr.id()==ID_exists)
+  {
+    assert(expr.operands().size()==2);
+    // check if there are side-effects
+    goto_programt tmp;
+    clean_expr(expr.op1(), tmp, true);
+    if(tmp.instructions.empty())
+      throw "no side-effects in quantified expressions allowed";
+    return;
   }
 
   // TODO: evaluation order
