@@ -6,18 +6,20 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
-#include <i2string.h>
-#include <cprover_prefix.h>
-#include <expr_util.h>
-#include <prefix.h>
-#include <std_expr.h>
-#include <context.h>
-#include <simplify_expr.h>
+#include <util/i2string.h>
+#include <util/cprover_prefix.h>
+#include <util/expr_util.h>
+#include <util/prefix.h>
+#include <util/std_expr.h>
+#include <util/symbol_table.h>
+#include <util/simplify_expr.h>
+#include <util/rename.h>
 
 #include <ansi-c/c_types.h>
 
+#include "goto_convert.h"
 #include "goto_convert_class.h"
 #include "destructor.h"
 
@@ -55,7 +57,8 @@ void goto_convertt::finish_gotos()
         if(l_it==targets.labels.end())
         {
           err_location(i.code);
-          str << "goto label " << it->id_string() << " not found";
+          str << "goto label `" << it->id_string() << "' not found";
+          error();
           throw 0;
         }
           
@@ -72,7 +75,8 @@ void goto_convertt::finish_gotos()
       if(l_it==targets.labels.end())
       {
         err_location(i.code);
-        str << "goto label " << goto_label << " not found";
+        str << "goto label `" << goto_label << "' not found";
+        error();
         throw 0;
       }
 
@@ -87,7 +91,8 @@ void goto_convertt::finish_gotos()
       if(l_it==targets.labels.end())
       {
         err_location(i.code);
-        str << "goto label " << goto_label << " not found";
+        str << "goto label `" << goto_label << "' not found";
+        error();
         throw 0;
       }
 
@@ -260,30 +265,9 @@ void goto_convertt::convert_label(
   }
   else
     convert(to_code(code.op0()), tmp);
-
-  // magic ERROR label?
-
-  const std::string &error_label=options.get_option("error-label");
-
-  goto_programt::targett target;
-
-  if(error_label!="" && label==error_label)
-  {
-    goto_programt::targett t=dest.add_instruction(ASSERT);
-    t->guard.make_false();
-    t->location=code.location();
-    t->location.set(ID_property, "error label");
-    t->location.set(ID_comment, "error label");
-    t->location.set("user-provided", true);
-
-    target=t;
-    dest.destructive_append(tmp);
-  }
-  else
-  {
-    target=tmp.instructions.begin();
-    dest.destructive_append(tmp);
-  }
+  
+  goto_programt::targett target=tmp.instructions.begin();
+  dest.destructive_append(tmp);
 
   if(!label.empty())
   {
@@ -357,7 +341,7 @@ void goto_convertt::convert(
   const irep_idt &statement=code.get_statement();
   
   if(statement==ID_block)
-    convert_block(code, dest);
+    convert_block(to_code_block(code), dest);
   else if(statement==ID_decl)
     convert_decl(to_code_decl(code), dest);
   else if(statement==ID_decl_type)
@@ -379,13 +363,13 @@ void goto_convertt::convert(
   else if(statement==ID_gcc_local_label)
     convert_gcc_local_label(code, dest);
   else if(statement==ID_for)
-    convert_for(code, dest);
+    convert_for(to_code_for(code), dest);
   else if(statement==ID_while)
-    convert_while(code, dest);
+    convert_while(to_code_while(code), dest);
   else if(statement==ID_dowhile)
     convert_dowhile(code, dest);
   else if(statement==ID_switch)
-    convert_switch(code, dest);
+    convert_switch(to_code_switch(code), dest);
   else if(statement==ID_break)
     convert_break(to_code_break(code), dest);
   else if(statement==ID_return)
@@ -401,7 +385,7 @@ void goto_convertt::convert(
   else if(statement=="non-deterministic-goto")
     convert_non_deterministic_goto(code, dest);
   else if(statement==ID_ifthenelse)
-    convert_ifthenelse(code, dest);
+    convert_ifthenelse(to_code_ifthenelse(code), dest);
   else if(statement==ID_specc_notify)
     convert_specc_notify(code, dest);
   else if(statement==ID_specc_wait)
@@ -431,6 +415,12 @@ void goto_convertt::convert(
     convert_msc_leave(code, dest);
   else if(statement==ID_catch) // C++ try/catch
     convert_catch(code, dest);
+  else if(statement==ID_CPROVER_try_catch) // CPROVER-homemade
+    convert_CPROVER_try_catch(code, dest);
+  else if(statement==ID_CPROVER_throw) // CPROVER-homemade
+    convert_CPROVER_throw(code, dest);
+  else if(statement==ID_CPROVER_try_finally)
+    convert_CPROVER_try_finally(code, dest);
   else if(statement==ID_asm)
     convert_asm(code, dest);
   else if(statement==ID_static_assert)
@@ -444,6 +434,7 @@ void goto_convertt::convert(
       err_location(code.op0());
       str << "static assertion "
           << get_string_constant(code.op1());
+      error();
       throw 0;
     }
     else if(assertion.is_true())
@@ -454,6 +445,8 @@ void goto_convertt::convert(
       // we may wish to complain
     }
   }
+  else if(statement==ID_dead)
+    copy(code, DEAD, dest);
   else
     copy(code, OTHER, dest);
 
@@ -478,61 +471,26 @@ Function: goto_convertt::convert_block
 \*******************************************************************/
 
 void goto_convertt::convert_block(
-  const codet &code,
+  const code_blockt &code,
   goto_programt &dest)
 {
-  // this needs to be ordered to obtain
-  // the correct ordering for the destructor calls
-  std::list<irep_idt> locals;
+  const locationt &end_location=code.end_location();
+
+  // this saves the size of the destructor stack
+  unsigned old_stack_size=targets.destructor_stack.size();
   
+  // now convert block  
   forall_operands(it, code)
   {
-    const codet &code=to_code(*it);
-
-    if(code.get_statement()==ID_decl)
-    {
-      const exprt &op0=code.op0();
-      assert(op0.id()==ID_symbol);
-      const irep_idt &identifier=op0.get(ID_identifier);
-      const symbolt &symbol=lookup(identifier);
-      
-      if(!symbol.static_lifetime &&
-         symbol.type.id()!=ID_code)
-        locals.push_back(identifier);
-    }
-    
-    convert(code, dest);
-
-    for(tmp_symbolst::const_iterator
-        it=tmp_symbols.begin();
-        it!=tmp_symbols.end();
-        it++)
-      locals.push_back(*it);
-
-    tmp_symbols.clear();
+    const codet &b_code=to_code(*it);
+    convert(b_code, dest);
   }
 
-  // see if we need to call any destructors
+  // see if we need to do any destructors
+  unwind_destructor_stack(end_location, old_stack_size, dest);
 
-  while(!locals.empty())
-  {
-    const symbolt &symbol=ns.lookup(locals.back());
-
-    code_function_callt destructor=get_destructor(ns, symbol.type);
-
-    if(destructor.is_not_nil())
-    {
-      // add "this"
-      exprt this_expr(ID_address_of, pointer_typet());
-      this_expr.type().subtype()=symbol.type;
-      this_expr.copy_to_operands(symbol_expr(symbol));
-      destructor.arguments().push_back(this_expr);
-
-      convert(destructor, dest);
-    }
-    
-    locals.pop_back();
-  }
+  // remove those destructors
+  targets.destructor_stack.resize(old_stack_size);
 }
 
 /*******************************************************************\
@@ -564,7 +522,6 @@ void goto_convertt::convert_expression(
     // we do a special treatment for ?: 
     const if_exprt &if_expr=to_if_expr(expr);
     code_ifthenelset tmp_code;
-    tmp_code.operands().resize(3);
     tmp_code.location()=expr.location();
     tmp_code.cond()=if_expr.cond();
     tmp_code.then_case()=code_expressiont(if_expr.true_case());
@@ -615,7 +572,7 @@ void goto_convertt::convert_decl(
   
   const symbolt &symbol=lookup(identifier);
   
-  if(symbol.static_lifetime ||
+  if(symbol.is_static_lifetime ||
      symbol.type.id()==ID_code)
     return; // this is a SKIP!
 
@@ -625,23 +582,43 @@ void goto_convertt::convert_decl(
   }
   else
   {
+    // this is expected to go away
     exprt initializer;
   
     codet tmp=code;
     initializer=code.op1();
     tmp.operands().resize(1);
     
-    goto_programt sideeffects;
-    clean_expr(initializer, sideeffects);
-    dest.destructive_append(sideeffects);
-    
-    // break up into decl and assignment
+    // Break up into decl and assignment.
+    // Decl must be visible before initializer.
     copy(tmp, DECL, dest);
 
     code_assignt assign(code.op0(), initializer);
     assign.location()=tmp.location();
+
+    convert_assign(assign, dest);
+  }
+
+  // do destructor
+  const symbol_exprt symbol_expr(symbol.name, symbol.type);
+
+  code_function_callt destructor=get_destructor(ns, symbol.type);
+
+  if(destructor.is_not_nil())
+  {
+    // add "this"
+    exprt this_expr(ID_address_of, pointer_typet());
+    this_expr.type().subtype()=symbol.type;
+    this_expr.copy_to_operands(symbol_expr);
+    destructor.arguments().push_back(this_expr);
+
+    targets.destructor_stack.push_back(destructor);
+  }
     
-    copy(assign, ASSIGN, dest);
+  // now create a 'dead' instruction
+  {
+    code_deadt code_dead(symbol_expr);
+    targets.destructor_stack.push_back(code_dead);
   }
 }
 
@@ -701,13 +678,22 @@ void goto_convertt::convert_assign(
   }
   else if(rhs.id()==ID_sideeffect &&
           (rhs.get(ID_statement)==ID_cpp_new ||
-           rhs.get(ID_statement)=="cpp_new[]"))
+           rhs.get(ID_statement)==ID_cpp_new_array))
   {
     Forall_operands(it, rhs)
       clean_expr(*it, dest);
 
     do_cpp_new(lhs, to_side_effect_expr(rhs), dest);
   }
+  else if(rhs.id()==ID_sideeffect &&
+          (rhs.get(ID_statement)==ID_java_new ||
+           rhs.get(ID_statement)==ID_java_new_array))
+  {
+    Forall_operands(it, rhs)
+      clean_expr(*it, dest);
+
+    do_java_new(lhs, to_side_effect_expr(rhs), dest);
+  }
   else if(rhs.id()==ID_sideeffect &&
           rhs.get(ID_statement)==ID_malloc)
   {
@@ -836,7 +822,7 @@ void goto_convertt::convert_cpp_delete(
   }
   
   // now do "free"
-  exprt delete_symbol=symbol_expr(ns.lookup(delete_identifier));
+  exprt delete_symbol=ns.lookup(delete_identifier).symbol_expr();
   
   assert(to_code_type(delete_symbol.type()).arguments().size()==1);
 
@@ -872,9 +858,6 @@ void goto_convertt::convert_assert(
 
   clean_expr(cond, dest);
   
-  if(!options.get_bool_option("assertions"))
-    return;
-
   goto_programt::targett t=dest.add_instruction(ASSERT);
   t->guard.swap(cond);
   t->location=code.location();
@@ -941,15 +924,9 @@ Function: goto_convertt::convert_for
 \*******************************************************************/
 
 void goto_convertt::convert_for(
-  const codet &code,
+  const code_fort &code,
   goto_programt &dest)
 {
-  if(code.operands().size()!=4)
-  {
-    err_location(code);
-    throw "for takes four operands";
-  }
-  
   // turn for(A; c; B) { P } into
   //  A; while(c) { P; B; }
   //-----------------------------
@@ -963,13 +940,13 @@ void goto_convertt::convert_for(
 
   // A;
   code_blockt block;  
-  if(code.op0().is_not_nil())
+  if(code.init().is_not_nil())
   {
-    block.copy_to_operands(code.op0());
+    block.copy_to_operands(code.init());
     convert(block, dest);
   }
     
-  exprt cond=code.op1();
+  exprt cond=code.cond();
 
   goto_programt sideeffects;
   clean_expr(cond, sideeffects);
@@ -995,7 +972,7 @@ void goto_convertt::convert_for(
     tmp_x.add_instruction(SKIP);
   else
   {
-    exprt tmp_B=code.op2();
+    exprt tmp_B=code.iter();
 
     clean_expr(tmp_B, tmp_x, false);
 
@@ -1019,13 +996,13 @@ void goto_convertt::convert_for(
 
   // do the w label
   goto_programt tmp_w;
-  convert(to_code(code.op3()), tmp_w);
+  convert(code.body(), tmp_w);
   
   // y: goto u;
   goto_programt tmp_y;
   goto_programt::targett y=tmp_y.add_instruction();
   y->make_goto(u);
-  y->guard.make_true();
+  y->guard=true_exprt();
   y->location=code.location();
 
   dest.destructive_append(sideeffects);
@@ -1036,7 +1013,7 @@ void goto_convertt::convert_for(
   dest.destructive_append(tmp_z);
 
   // restore break/continue
-  targets.restore(old_targets);
+  old_targets.restore(targets);
 }
 
 /*******************************************************************\
@@ -1052,16 +1029,10 @@ Function: goto_convertt::convert_while
 \*******************************************************************/
 
 void goto_convertt::convert_while(
-  const codet &code,
+  const code_whilet &code,
   goto_programt &dest)
 {
-  if(code.operands().size()!=2)
-  {
-    err_location(code);
-    throw "while takes two operands";
-  }
-
-  const exprt &cond=code.op0();
+  const exprt &cond=code.cond();
   const locationt &location=code.location();
 
   //    while(c) P;
@@ -1081,7 +1052,7 @@ void goto_convertt::convert_while(
   z->make_skip();
 
   goto_programt tmp_branch;
-  generate_conditional_branch(gen_not(cond), z, location, tmp_branch);
+  generate_conditional_branch(not_exprt(cond), z, location, tmp_branch);
 
   // do the v label
   goto_programt::targett v=tmp_branch.instructions.begin();
@@ -1096,11 +1067,11 @@ void goto_convertt::convert_while(
 
   // do the x label
   goto_programt tmp_x;
-  convert(to_code(code.op1()), tmp_x);
+  convert(code.body(), tmp_x);
 
   // y: if(c) goto v;
   y->make_goto(v);
-  y->guard.make_true();
+  y->guard=true_exprt();
   y->location=code.location();
 
   dest.destructive_append(tmp_branch);
@@ -1109,7 +1080,7 @@ void goto_convertt::convert_while(
   dest.destructive_append(tmp_z);
 
   // restore break/continue
-  targets.restore(old_targets);
+  old_targets.restore(targets);
 }
 
 /*******************************************************************\
@@ -1188,7 +1159,7 @@ void goto_convertt::convert_dowhile(
   dest.destructive_append(tmp_z);
 
   // restore break/continue targets
-  targets.restore(old_targets);
+  old_targets.restore(targets);
 }
 
 /*******************************************************************\
@@ -1242,7 +1213,7 @@ Function: goto_convertt::convert_switch
 \*******************************************************************/
 
 void goto_convertt::convert_switch(
-  const codet &code,
+  const code_switcht &code,
   goto_programt &dest)
 {
   // switch(v) {
@@ -1266,13 +1237,13 @@ void goto_convertt::convert_switch(
     throw "switch takes at least two operands";
   }
   
-  exprt argument=code.op0();
+  exprt argument=code.value();
 
   goto_programt sideeffects;
   clean_expr(argument, sideeffects);
 
-  // save break/continue/default/cases targets
-  break_continue_switch_targetst old_targets(targets);
+  // save break/default/cases targets
+  break_switch_targetst old_targets(targets);
 
   // do the z label
   goto_programt tmp_z;
@@ -1322,7 +1293,7 @@ void goto_convertt::convert_switch(
   dest.destructive_append(tmp_z);
 
   // restore old targets
-  targets.restore(old_targets);
+  old_targets.restore(targets);
 }
 
 /*******************************************************************\
@@ -1347,6 +1318,10 @@ void goto_convertt::convert_break(
     throw "break without target";
   }
 
+  // need to process destructor stack
+  unwind_destructor_stack(code.location(), targets.break_stack_size, dest);
+
+  // add goto
   goto_programt::targett t=dest.add_instruction();
   t->make_goto(targets.break_target);
   t->location=code.location();
@@ -1385,12 +1360,19 @@ void goto_convertt::convert_return(
   
   if(new_code.has_return_value())
   {
+    bool result_is_used=
+      new_code.return_value().type().id()!=ID_empty;
+  
     goto_programt sideeffects;
-    clean_expr(new_code.return_value(), sideeffects);
+    clean_expr(new_code.return_value(), sideeffects, result_is_used);
     dest.destructive_append(sideeffects);
+
+    // remove void-typed return value
+    if(!result_is_used)
+      new_code.operands().resize(0);    
   }
 
-  if(targets.return_value)
+  if(targets.has_return_value)
   {
     if(!new_code.has_return_value())
     {
@@ -1407,7 +1389,13 @@ void goto_convertt::convert_return(
       throw "function must not return value";
     }
   }
-
+  
+  // Need to process entire destructor stack.
+  // We ignore 'dead' here, as we may need
+  // these for the return value.
+  unwind_destructor_stack(code.location(), 0, dest, false);
+  
+  // now add return
   goto_programt::targett t=dest.add_instruction();
   t->make_return();
   t->code=new_code;
@@ -1436,6 +1424,10 @@ void goto_convertt::convert_continue(
     throw "continue without target";
   }
 
+  // need to process destructor stack
+  unwind_destructor_stack(code.location(), targets.continue_stack_size, dest);
+
+  // add goto
   goto_programt::targett t=dest.add_instruction();
   t->make_goto(targets.continue_target);
   t->location=code.location();
@@ -1681,6 +1673,7 @@ void goto_convertt::convert_start_thread(
     
     goto_programt::targett goto_instruction=dest.add_instruction(GOTO);
     goto_instruction->guard=true_exprt();
+    goto_instruction->location=code.location();
 
     goto_programt tmp;
     convert(to_code(code.op0()), tmp);
@@ -1875,144 +1868,6 @@ void goto_convertt::convert_bp_abortif(
 
 /*******************************************************************\
 
-Function: goto_convertt::convert_msc_try_finally
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-void goto_convertt::convert_msc_try_finally(
-  const codet &code,
-  goto_programt &dest)
-{
-  if(code.operands().size()!=2)
-  {
-    err_location(code);
-    throw "msc_try_finally expects two arguments";
-  }
-
-  convert(to_code(code.op0()), dest);
-  
-  // todo: generate exception tracking
-}
-
-/*******************************************************************\
-
-Function: goto_convertt::convert_msc_try_except
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-void goto_convertt::convert_msc_try_except(
-  const codet &code,
-  goto_programt &dest)
-{
-  if(code.operands().size()!=3)
-  {
-    err_location(code);
-    throw "msc_try_except expects three arguments";
-  }
-
-  convert(to_code(code.op0()), dest);
-  
-  // todo: generate exception tracking
-}
-
-/*******************************************************************\
-
-Function: goto_convertt::convert_msc_leave
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-void goto_convertt::convert_msc_leave(
-  const codet &code,
-  goto_programt &dest)
-{
-  // todo: should throw (silent) exception
-}
-
-/*******************************************************************\
-
-Function: goto_convertt::convert_catch
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-void goto_convertt::convert_catch(
-  const codet &code,
-  goto_programt &dest)
-{
-  assert(code.operands().size()>=2);
-  
-  // add the CATCH-push instruction to 'dest'
-  goto_programt::targett catch_push_instruction=dest.add_instruction();
-  catch_push_instruction->make_catch();
-  catch_push_instruction->code.set_statement(ID_catch);
-  catch_push_instruction->location=code.location();
-  
-  // the CATCH-push instruction is annotated with a list of IDs,
-  // one per target
-  irept::subt &exception_list=
-    catch_push_instruction->code.add(ID_exception_list).get_sub();
-
-  // add a SKIP target for the end of everything
-  goto_programt end;
-  goto_programt::targett end_target=end.add_instruction();
-  end_target->make_skip();
-  
-  // the first operand is the 'try' block
-  convert(to_code(code.op0()), dest);
-  
-  // add the CATCH-pop to the end of the 'try' block
-  goto_programt::targett catch_pop_instruction=dest.add_instruction();
-  catch_pop_instruction->make_catch();
-  catch_pop_instruction->code.set_statement(ID_catch);
-  
-  // add a goto to the end of the 'try' block
-  dest.add_instruction()->make_goto(end_target);
-
-  for(unsigned i=1; i<code.operands().size(); i++)
-  {
-    const codet &block=to_code(code.operands()[i]);
-  
-    // grab the ID and add to CATCH instruction
-    exception_list.push_back(irept(block.get(ID_exception_id)));
-    
-    goto_programt tmp;
-    convert(block, tmp);
-    catch_push_instruction->targets.push_back(tmp.instructions.begin());
-    dest.destructive_append(tmp);
-
-    // add a goto to the end of the 'catch' block
-    dest.add_instruction()->make_goto(end_target);
-  }
-
-  // add end-target  
-  dest.destructive_append(end);
-}
-
-/*******************************************************************\
-
 Function: goto_convertt::convert_ifthenelse
 
   Inputs:
@@ -2024,35 +1879,53 @@ Function: goto_convertt::convert_ifthenelse
 \*******************************************************************/
 
 void goto_convertt::convert_ifthenelse(
-  const codet &code,
+  const code_ifthenelset &code,
   goto_programt &dest)
 {
-  if(code.operands().size()!=2 &&
-     code.operands().size()!=3)
+  if(code.operands().size()!=3)
   {
     err_location(code);
-    throw "ifthenelse takes two or three operands";
+    throw "ifthenelse takes three operands";
   }
   
+  assert(code.then_case().is_not_nil());
+  
   bool has_else=
-    code.operands().size()==3 &&
-    !code.op2().is_nil();
+    !code.else_case().is_nil();
 
   const locationt &location=code.location();
 
-  // convert 'then'-branch
-  goto_programt tmp_op1;
-  convert(to_code(code.op1()), tmp_op1);
+  // we do a bit of special treatment for && in the condition
+  // in case cleaning would be needed otherwise
+  if(code.cond().id()==ID_and &&
+     code.cond().operands().size()==2 &&
+     (needs_cleaning(code.cond().op0()) || needs_cleaning(code.cond().op1())) &&
+     !has_else)
+  {
+    // if(a && b) XX --> if(a) if(b) XX
+    code_ifthenelset new_if0, new_if1;
+    new_if0.cond()=code.cond().op0();
+    new_if1.cond()=code.cond().op1();
+    new_if0.location()=location;
+    new_if1.location()=location;
+    new_if1.then_case()=code.then_case();
+    new_if0.then_case()=new_if1;
+    return convert_ifthenelse(new_if0, dest);
+  }
 
-  goto_programt tmp_op2;  
+  // convert 'then'-branch
+  goto_programt tmp_then;
+  convert(to_code(code.then_case()), tmp_then);
+
+  goto_programt tmp_else;
 
   if(has_else)
-    convert(to_code(code.op2()), tmp_op2);
+    convert(to_code(code.else_case()), tmp_else);
 
-  exprt tmp_guard=code.op0();
+  exprt tmp_guard=code.cond();
   clean_expr(tmp_guard, dest);
 
-  generate_ifthenelse(tmp_guard, tmp_op1, tmp_op2, location, dest);
+  generate_ifthenelse(tmp_guard, tmp_then, tmp_else, location, dest);
 }
 
 /*******************************************************************\
@@ -2111,8 +1984,11 @@ void goto_convertt::generate_ifthenelse(
   if(false_case.instructions.empty() &&
      true_case.instructions.size()==1 &&
      true_case.instructions.back().is_goto() &&
-     true_case.instructions.back().guard.is_true())
+     true_case.instructions.back().guard.is_true() &&
+     true_case.instructions.back().labels.empty())
   {
+    // The above conjunction deliberately excludes the instance
+    // if(some) { label: goto somewhere; }
     true_case.instructions.back().guard=guard;
     dest.destructive_append(true_case);
     return;
@@ -2120,7 +1996,7 @@ void goto_convertt::generate_ifthenelse(
 
   if(true_case.instructions.empty())
     return generate_ifthenelse(
-      gen_not(guard), false_case, true_case, location, dest);
+      not_exprt(guard), false_case, true_case, location, dest);
 
   bool has_else=!false_case.instructions.empty();
 
@@ -2159,7 +2035,7 @@ void goto_convertt::generate_ifthenelse(
   // v: if(!c) goto z/y;
   goto_programt tmp_v;
   generate_conditional_branch(
-    gen_not(guard), has_else?y:z, location, tmp_v);
+    not_exprt(guard), has_else?y:z, location, tmp_v);
 
   // w: P;
   goto_programt tmp_w;
@@ -2282,7 +2158,7 @@ void goto_convertt::generate_conditional_branch(
   
     forall_expr_list(it, op)
       generate_conditional_branch(
-        gen_not(*it), target_false, location, dest);
+        not_exprt(*it), target_false, location, dest);
 
     goto_programt::targett t_true=dest.add_instruction();
     t_true->make_goto(target_true);
@@ -2449,16 +2325,16 @@ symbolt &goto_convertt::new_tmp_symbol(
   {
     new_symbol.base_name="tmp_"+suffix+"$"+i2string(++temporary_counter);
     new_symbol.name=tmp_symbol_prefix+id2string(new_symbol.base_name);
-    new_symbol.lvalue=true;
-    new_symbol.thread_local=true;
-    new_symbol.file_local=true;
+    new_symbol.is_lvalue=true;
+    new_symbol.is_thread_local=true;
+    new_symbol.is_file_local=true;
     new_symbol.type=type;    
-  } while(context.move(new_symbol, symbol_ptr));    
+  } while(symbol_table.move(new_symbol, symbol_ptr));    
   
   tmp_symbols.push_back(symbol_ptr->name);
   
   goto_programt::targett t=dest.add_instruction(DECL);
-  t->code=code_declt(symbol_expr(*symbol_ptr));
+  t->code=code_declt(symbol_ptr->symbol_expr());
   t->location=location;
 
   return *symbol_ptr;  
@@ -2487,12 +2363,125 @@ void goto_convertt::make_temp_symbol(
     new_tmp_symbol(expr.type(), suffix, dest, location);
 
   code_assignt assignment;
-  assignment.lhs()=symbol_expr(new_symbol);
+  assignment.lhs()=new_symbol.symbol_expr();
   assignment.rhs()=expr;
   assignment.location()=location;
 
   convert(assignment, dest);
 
-  expr=symbol_expr(new_symbol);  
+  expr=new_symbol.symbol_expr();
 }
 
+/*******************************************************************\
+
+Function: goto_convertt::new_name
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void goto_convertt::new_name(symbolt &symbol)
+{
+  // rename it
+  get_new_name(symbol, ns);
+
+  // store in symbol_table
+  symbol_table.add(symbol);
+}
+
+/*******************************************************************\
+
+Function: goto_convertt::lookup
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+const symbolt &goto_convertt::lookup(const irep_idt &identifier) const
+{
+  const symbolt *symbol;
+  if(ns.lookup(identifier, symbol))
+    throw "failed to find symbol "+id2string(identifier);
+  return *symbol;
+}
+
+/*******************************************************************\
+
+Function: goto_convert
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void goto_convert(
+  const codet &code,
+  symbol_tablet &symbol_table,
+  goto_programt &dest,
+  message_handlert &message_handler)
+{
+  goto_convertt goto_convert(symbol_table, message_handler);
+
+  try
+  {
+    goto_convert.goto_convert(code, dest);
+  }
+  
+  catch(int)
+  {
+    goto_convert.error();
+  }
+
+  catch(const char *e)
+  {
+    goto_convert.error(e);
+  }
+
+  catch(const std::string &e)
+  {
+    goto_convert.error(e);
+  }
+
+  if(goto_convert.get_error_found())
+    throw 0;
+}
+
+/*******************************************************************\
+
+Function: goto_convert
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void goto_convert(
+  symbol_tablet &symbol_table,
+  goto_programt &dest,
+  message_handlert &message_handler)
+{
+  // find main symbol
+  const symbol_tablet::symbolst::const_iterator s_it=
+    symbol_table.symbols.find("main");
+  
+  if(s_it==symbol_table.symbols.end())
+    throw "failed to find main symbol";
+  
+  const symbolt &symbol=s_it->second;
+  
+  ::goto_convert(to_code(symbol.value), symbol_table, dest, message_handler);
+}
