@@ -8,11 +8,12 @@ Date: June 2003
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
-#include <base_type.h>
-#include <std_code.h>
-#include <context.h>
+#include <util/base_type.h>
+#include <util/std_code.h>
+#include <util/symbol_table.h>
+#include <util/prefix.h>
 
 #include "goto_convert_functions.h"
 #include "goto_inline.h"
@@ -31,11 +32,10 @@ Function: goto_convert_functionst::goto_convert_functionst
 \*******************************************************************/
 
 goto_convert_functionst::goto_convert_functionst(
-  contextt &_context,
-  const optionst &_options,
+  symbol_tablet &_symbol_table,
   goto_functionst &_functions,
   message_handlert &_message_handler):
-  goto_convertt(_context, _options, _message_handler),
+  goto_convertt(_symbol_table, _message_handler),
   functions(_functions)
 {
 }
@@ -75,7 +75,7 @@ void goto_convert_functionst::goto_convert()
   typedef std::list<irep_idt> symbol_listt;
   symbol_listt symbol_list;
 
-  forall_symbols(it, context.symbols)
+  forall_symbols(it, symbol_table.symbols)
   {
     if(!it->second.is_type &&
        it->second.type.id()==ID_code)
@@ -94,7 +94,7 @@ void goto_convert_functionst::goto_convert()
 
   // this removes the parse tree of the bodies from memory
   #if 0
-  Forall_symbols(it, context.symbols)
+  Forall_symbols(it, symbol_table.symbols)
   {
     if(!it->second.is_type &&
        it->second.type.id()==ID_code &&
@@ -209,14 +209,13 @@ void goto_convert_functionst::convert_function(const irep_idt &identifier)
   locationt end_location;
 
   if(code.get_statement()==ID_block)
-    end_location=static_cast<const locationt &>(
-      code.find("#end_location"));
+    end_location=to_code_block(code).end_location();
   else
     end_location.make_nil();
 
   targets=targetst();
   targets.return_set=true;
-  targets.return_value=
+  targets.has_return_value=
     f.type.return_type().id()!=ID_empty &&
     f.type.return_type().id()!=ID_constructor &&
     f.type.return_type().id()!=ID_destructor;
@@ -224,9 +223,40 @@ void goto_convert_functionst::convert_function(const irep_idt &identifier)
   goto_convert_rec(code, f.body);
   
   // add non-det return value, if needed
-  if(targets.return_value)
+  if(targets.has_return_value)
     add_return(f, end_location);
       
+  // handle SV-COMP's __VERIFIER_atomic_
+  if(!f.body.instructions.empty() &&
+      has_prefix(id2string(identifier), "c::__VERIFIER_atomic_"))
+  {
+    goto_programt::instructiont a_begin;
+    a_begin.make_atomic_begin();
+    a_begin.location=f.body.instructions.front().location;
+    f.body.insert_before_swap(f.body.instructions.begin(), a_begin);
+
+    bool last_is_return=false;
+    Forall_goto_program_instructions(i_it, f.body)
+    {
+      last_is_return=i_it->is_return();
+      if(last_is_return)
+      {
+        goto_programt::instructiont a_end;
+        a_end.make_atomic_end();
+        a_end.location=i_it->location;
+        f.body.insert_before_swap(i_it, a_end);
+        ++i_it;
+      }
+    }
+
+    if(!last_is_return)
+    {
+      goto_programt::targett t=f.body.add_instruction();
+      t->make_atomic_end();
+      t->location=end_location;
+    }
+  }
+
   // add "end of function"
   goto_programt::targett t=f.body.add_instruction();
   t->type=END_FUNCTION;
@@ -261,13 +291,12 @@ Function: goto_convert
 \*******************************************************************/
 
 void goto_convert(
-  contextt &context,
-  const optionst &options,
+  symbol_tablet &symbol_table,
   goto_functionst &functions,
   message_handlert &message_handler)
 {
   goto_convert_functionst goto_convert_functions(
-    context, options, functions, message_handler);
+    symbol_table, functions, message_handler);
   
   try
   {  
@@ -307,13 +336,12 @@ Function: goto_convert
 
 void goto_convert(
   const irep_idt &identifier,
-  contextt &context,
-  const optionst &options,
+  symbol_tablet &symbol_table,
   goto_functionst &functions,
   message_handlert &message_handler)
 {
   goto_convert_functionst goto_convert_functions(
-    context, options, functions, message_handler);
+    symbol_table, functions, message_handler);
   
   try
   {  
