@@ -6,8 +6,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <std_expr.h>
-#include <i2string.h>
+#include <util/std_expr.h>
+#include <util/i2string.h>
 
 #include <langapi/language_util.h>
 
@@ -146,9 +146,10 @@ std::ostream& goto_programt::output_instruction(
     break;
     
   case CATCH:
-    out << "CATCH ";
-    
+    if(!it->targets.empty())
     {
+      out << "CATCH-PUSH ";
+    
       unsigned i=0;
       const irept::subt &exception_list=
         it->code.find(ID_exception_list).get_sub();
@@ -164,6 +165,8 @@ std::ostream& goto_programt::output_instruction(
             << (*gt_it)->target_number;
       }
     }
+    else
+      out << "CATCH-POP";
       
     out << std::endl;
     break;
