@@ -11,7 +11,7 @@ Date: May 2007
 #include <cstdlib>
 #include <iostream>
 
-#include <i2string.h>
+#include <util/i2string.h>
 
 #include "goto_program_irep.h"
 
@@ -195,8 +195,8 @@ void convert( const irept &irep, goto_programt &program )
       
       if (fit==program.instructions.end())
       {       
-        std::cout << "Warning: could not resolve target link " << 
-        "during irep->goto_program translation." << std::endl;
+        std::cout << "Warning: could not resolve target link "
+                  << "during irep->goto_program translation." << std::endl;
         throw 0;
       }
     }
