@@ -15,9 +15,10 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <ostream>
 #include <set>
 
-#include <namespace.h>
-#include <context.h>
-#include <location.h>
+#include <util/namespace.h>
+#include <util/symbol_table.h>
+#include <util/location.h>
+#include <util/std_expr.h>
 
 typedef enum { NO_INSTRUCTION_TYPE=0,
                GOTO=1,          // branch, possibly guarded
@@ -93,18 +94,27 @@ public:
     //! guard for gotos, assume, assert
     guardT guard;
     
+    // The below will eventually become a single target only.
     //! the target for gotos and for start_thread nodes
     typedef typename std::list<class instructiont>::iterator targett;
     typedef typename std::list<class instructiont>::const_iterator const_targett;
     typedef std::list<targett> targetst;
     typedef std::list<const_targett> const_targetst;
-
+    
     targetst targets;
+    
+    // for the usual case of a single target
+    inline targett get_target() const
+    {
+      assert(targets.size()==1);
+      return targets.front();
+    }
 
     //! goto target labels    
     typedef std::list<irep_idt> labelst;
     labelst labels;
 
+    // will go away
     std::set<targett> incoming_edges;
 
     //! is this node a branch target?
@@ -116,7 +126,7 @@ public:
     {
       type=_type;
       targets.clear();
-      guard.make_true();
+      guard=true_exprt();
       code.make_nil();
     }
      
@@ -172,19 +182,19 @@ public:
     inline instructiont():
       location(static_cast<const locationt &>(get_nil_irep())),
       type(NO_INSTRUCTION_TYPE),
+      guard(true_exprt()),
       location_number(0),
       target_number(unsigned(-1))
     {
-      guard.make_true();
     }
 
     inline instructiont(goto_program_instruction_typet _type):
       location(static_cast<const locationt &>(get_nil_irep())),
       type(_type),
+      guard(true_exprt()),
       location_number(0),
       target_number(unsigned(-1))
     {
-      guard.make_true();
     }
 
     //! swap two instructions    
@@ -247,13 +257,19 @@ public:
   void compute_incoming_edges();
 
   //! Insertion that preserves jumps to "target".
-  //! The instruction is destroyed.
-  void insert_before_swap(targett target, instructiont &instruction)
+  void insert_before_swap(targett target)
   {
     assert(target!=instructions.end());
     targett next=target;
     next++;
     instructions.insert(next, instructiont())->swap(*target);
+  }
+  
+  //! Insertion that preserves jumps to "target".
+  //! The instruction is destroyed.
+  void insert_before_swap(targett target, instructiont &instruction)
+  {
+    insert_before_swap(target);
     target->swap(instruction);
   }
   
@@ -268,7 +284,6 @@ public:
     next++;
     p.instructions.erase(p.instructions.begin());
     instructions.splice(next, p.instructions);
-    // BUG: The iterators to p-instructions are invalidated!
   }
   
   //! Insertion before the given target
@@ -331,7 +346,7 @@ public:
   //! Output goto-program to given stream  
   inline std::ostream &output(std::ostream &out) const
   {
-    return output(namespacet(contextt()), "", out);
+    return output(namespacet(symbol_tablet()), "", out);
   }
 
   //! Output a single instruction  
