@@ -6,8 +6,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include "std_expr.h"
-#include "expr_util.h"
+#include <util/std_expr.h>
+#include <util/expr_util.h>
 
 #include "pointer_arithmetic.h"
 
