@@ -11,16 +11,22 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <string>
 
-class contextt;
+class symbol_tablet;
 class goto_functionst;
 class message_handlert;
+class goto_modelt;
 
 bool read_goto_binary(
   const std::string &filename,
-  contextt &context,
+  symbol_tablet &symbol_table,
   goto_functionst &dest,
   message_handlert &message_handler);
   
+bool read_goto_binary(
+  const std::string &filename,
+  goto_modelt &dest,
+  message_handlert &message_handler);
+  
 bool is_goto_binary(const std::string &filename);
 
 #endif
