@@ -11,13 +11,17 @@ Date: June 2003
 #ifndef CPROVER_GOTO_FUNCTION_POINTERS_H
 #define CPROVER_GOTO_FUNCTION_POINTERS_H
 
-#include "goto_functions.h"
+#include "goto_model.h"
 
 // remove indirect function calls
 // and replace by case-split
 void remove_function_pointers(
-  const namespacet &ns,
-  goto_functionst &functions,
+  goto_modelt &goto_model,
+  bool add_safety_assertion);
+
+void remove_function_pointers(
+  symbol_tablet &symbol_table,
+  goto_functionst &goto_functions,
   bool add_safety_assertion);
 
 #endif
