@@ -6,15 +6,15 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <string.h>
+#include <cstring>
 
-#include <std_expr.h>
-#include <std_code.h>
-#include <expr_util.h>
-#include <message_stream.h>
-#include <arith_tools.h>
-#include <i2string.h>
-#include <type_eq.h>
+#include <util/std_expr.h>
+#include <util/std_code.h>
+#include <util/expr_util.h>
+#include <util/message_stream.h>
+#include <util/arith_tools.h>
+#include <util/i2string.h>
+#include <util/type_eq.h>
 
 #include <ansi-c/c_types.h>
 
@@ -95,11 +95,11 @@ Function: string_abstraction
 \*******************************************************************/
 
 void string_abstraction(
-  contextt &context,
+  symbol_tablet &symbol_table,
   message_handlert &message_handler,
   goto_programt &dest)
 {
-  string_abstractiont string_abstraction(context, message_handler);
+  string_abstractiont string_abstraction(symbol_table, message_handler);
   string_abstraction(dest);
 }
 
@@ -116,11 +116,11 @@ Function: string_abstraction
 \*******************************************************************/
 
 void string_abstraction(
-  contextt &context,
+  symbol_tablet &symbol_table,
   message_handlert &message_handler,
   goto_functionst &dest)
 {
-  string_abstractiont string_abstraction(context, message_handler);
+  string_abstractiont string_abstraction(symbol_table, message_handler);
   string_abstraction(dest);
 }
 
@@ -137,13 +137,13 @@ Function: string_abstractiont::string_abstractiont
 \*******************************************************************/
 
 string_abstractiont::string_abstractiont(
-  contextt &_context,
+  symbol_tablet &_symbol_table,
   message_handlert &_message_handler):
   message_streamt(_message_handler),
   arg_suffix("#strarg"),
   sym_suffix("#str$fcn"),
-  context(_context),
-  ns(_context),
+  symbol_table(_symbol_table),
+  ns(_symbol_table),
   temporary_counter(0)
 {
   struct_typet s;
@@ -184,8 +184,8 @@ typet string_abstractiont::build_type(whatt what)
   switch(what)
   {
   case IS_ZERO: type=bool_typet(); break;
-  case LENGTH:  type=uint_type(); break;
-  case SIZE:    type=uint_type(); break;
+  case LENGTH:  type=size_type(); break;
+  case SIZE:    type=size_type(); break;
   }
 
   return type;
@@ -215,7 +215,7 @@ void string_abstractiont::operator()(goto_functionst &dest)
 
   // do we have a main?
   goto_functionst::function_mapt::iterator
-    m_it=dest.function_map.find(dest.main_id());
+    m_it=dest.function_map.find(dest.entry_point());
 
   if(m_it!=dest.function_map.end())
   {
@@ -266,8 +266,8 @@ void string_abstractiont::add_str_arguments(
     const irep_idt & name,
     goto_functionst::goto_functiont &fct)
 {
-  contextt::symbolst::iterator sym_entry=context.symbols.find(name);
-  assert(sym_entry!=context.symbols.end());
+  symbol_tablet::symbolst::iterator sym_entry=symbol_table.symbols.find(name);
+  assert(sym_entry!=symbol_table.symbols.end());
   symbolt &fct_symbol=sym_entry->second;
 
   code_typet::argumentst &arguments=
@@ -344,13 +344,13 @@ void string_abstractiont::add_argument(
   new_symbol.base_name=str_args.back().get_base_name();
   new_symbol.mode=fct_symbol.mode;
   new_symbol.pretty_name=str_args.back().get_base_name();
-  new_symbol.is_statevar=true;
-  new_symbol.static_lifetime=false;
-  new_symbol.thread_local=true;
-  new_symbol.lvalue=true;
-  new_symbol.file_local=true;
+  new_symbol.is_state_var=true;
+  new_symbol.is_static_lifetime=false;
+  new_symbol.is_thread_local=true;
+  new_symbol.is_lvalue=true;
+  new_symbol.is_file_local=true;
 
-  context.move(new_symbol);
+  symbol_table.move(new_symbol);
 }
 
 /*******************************************************************\
@@ -446,7 +446,7 @@ void string_abstractiont::make_decl_and_def(goto_programt &dest,
     const irep_idt &source_sym)
 {
   const symbolt &symbol=ns.lookup(identifier);
-  symbol_exprt sym_expr=symbol_expr(symbol);
+  symbol_exprt sym_expr=symbol.symbol_expr();
 
   goto_programt::targett decl1=dest.add_instruction();
   decl1->make_decl();
@@ -536,7 +536,7 @@ exprt string_abstractiont::make_val_or_dummy_rec(goto_programt &dest,
             dest, ref_instr, symbol, it2->get_name(),
             it2->type(), ns.follow(it->type()));
 
-        member_exprt member(symbol_expr(symbol), it2->get_name(), it2->type());
+        member_exprt member(symbol.symbol_expr(), it2->get_name(), it2->type());
 
         goto_programt::targett assignment1=dest.add_instruction();
         assignment1->make_assignment();
@@ -592,13 +592,13 @@ symbol_exprt string_abstractiont::add_dummy_symbol_and_value(
   new_symbol.mode=symbol.mode;
   new_symbol.pretty_name=id2string(
       symbol.pretty_name.empty()?symbol.base_name:symbol.pretty_name)+suffix;
-  new_symbol.is_statevar=true;
-  new_symbol.static_lifetime=false;
-  new_symbol.thread_local=true;
-  new_symbol.lvalue=true;
-  new_symbol.file_local=true;
+  new_symbol.is_state_var=true;
+  new_symbol.is_static_lifetime=false;
+  new_symbol.is_thread_local=true;
+  new_symbol.is_lvalue=true;
+  new_symbol.is_file_local=true;
 
-  symbol_exprt sym_expr=symbol_expr(new_symbol);
+  symbol_exprt sym_expr=new_symbol.symbol_expr();
 
   // make sure it is declared before the recursive call
   goto_programt::targett decl=dest.add_instruction();
@@ -633,7 +633,7 @@ symbol_exprt string_abstractiont::add_dummy_symbol_and_value(
     assignment1->code.location()=ref_instr->location;
   }
 
-  context.move(new_symbol);
+  symbol_table.move(new_symbol);
 
   return sym_expr;
 }
@@ -857,7 +857,7 @@ goto_programt::targett string_abstractiont::abstract_return(
 
   irep_idt identifier=abstract_ret_val_name(fct_symbol);
   const symbolt &str_symbol=ns.lookup(identifier);
-  symbol_exprt sym_expr=symbol_expr(str_symbol);
+  symbol_exprt sym_expr=str_symbol.symbol_expr();
 
   goto_programt::instructiont is_null;
   is_null.function=target->function;
@@ -1007,7 +1007,7 @@ exprt string_abstractiont::build(
   if(what==LENGTH || what==SIZE)
   {
     // adjust for offset
-    exprt pointer_offset(ID_pointer_offset, uint_type());
+    exprt pointer_offset(ID_pointer_offset, size_type());
     pointer_offset.copy_to_operands(pointer);
     if(pointer_offset.is_not_nil() &&
         !pointer_offset.is_zero())
@@ -1384,15 +1384,15 @@ exprt string_abstractiont::build_unknown(const typet &type, bool write)
   new_symbol.base_name=identifier;
   new_symbol.mode=ID_C;
   new_symbol.pretty_name=identifier;
-  new_symbol.is_statevar=true;
-  new_symbol.static_lifetime=false;
-  new_symbol.thread_local=true;
-  new_symbol.lvalue=true;
-  new_symbol.file_local=true;
+  new_symbol.is_state_var=true;
+  new_symbol.is_static_lifetime=false;
+  new_symbol.is_thread_local=true;
+  new_symbol.is_lvalue=true;
+  new_symbol.is_file_local=true;
 
-  context.move(new_symbol);
+  symbol_table.move(new_symbol);
 
-  return symbol_expr(ns.lookup(identifier));
+  return ns.lookup(identifier).symbol_expr();
 }
 
 /*******************************************************************\
@@ -1421,12 +1421,12 @@ bool string_abstractiont::build_symbol(const symbol_exprt &sym, exprt &dest)
   else
   {
     identifier=id2string(symbol.name)+sym_suffix;
-    if(context.symbols.find(identifier)==context.symbols.end())
+    if(symbol_table.symbols.find(identifier)==symbol_table.symbols.end())
       build_new_symbol(symbol, identifier, abstract_type);
   }
 
   const symbolt &str_symbol=ns.lookup(identifier);
-  dest=symbol_expr(str_symbol);
+  dest=str_symbol.symbol_expr();
   if(current_args.find(symbol.name)!=current_args.end() &&
       !is_ptr_argument(abstract_type))
     dest=dereference_exprt(dest, dest.type().subtype());
@@ -1449,7 +1449,7 @@ Function: string_abstractiont::build_new_symbol
 void string_abstractiont::build_new_symbol(const symbolt &symbol,
     const irep_idt &identifier, const typet &type)
 {
-  if(!symbol.static_lifetime)
+  if(!symbol.is_static_lifetime)
     locals[symbol.name]=identifier;
 
   symbolt new_symbol;
@@ -1462,15 +1462,16 @@ void string_abstractiont::build_new_symbol(const symbolt &symbol,
   new_symbol.mode=symbol.mode;
   new_symbol.pretty_name=id2string(
       symbol.pretty_name.empty()?symbol.base_name:symbol.pretty_name)+sym_suffix;
-  new_symbol.is_statevar=true;
-  new_symbol.static_lifetime=symbol.static_lifetime;
-  new_symbol.thread_local=symbol.thread_local;
-  new_symbol.lvalue=true;
-  new_symbol.file_local=true;
+  new_symbol.is_state_var=true;
+  new_symbol.is_static_lifetime=symbol.is_static_lifetime;
+  new_symbol.is_thread_local=symbol.is_thread_local;
+  new_symbol.is_lvalue=true;
+  new_symbol.is_file_local=true;
 
-  context.move(new_symbol);
+  symbol_table.move(new_symbol);
 
-  if(symbol.static_lifetime) {
+  if(symbol.is_static_lifetime)
+  {
     goto_programt::targett dummy_loc=initialization.add_instruction();
     dummy_loc->location=symbol.location;
     make_decl_and_def(initialization, dummy_loc, identifier, symbol.name);
@@ -1497,8 +1498,8 @@ bool string_abstractiont::build_symbol_constant(const mp_integer &zero_length,
     +"_"+integer2string(buf_size);
   irep_idt identifier="string_abstraction::"+id2string(base);
 
-  if(context.symbols.find(identifier)==
-     context.symbols.end())
+  if(symbol_table.symbols.find(identifier)==
+     symbol_table.symbols.end())
   {
     symbolt new_symbol;
     new_symbol.type=string_struct;
@@ -1507,11 +1508,11 @@ bool string_abstractiont::build_symbol_constant(const mp_integer &zero_length,
     new_symbol.base_name=base;
     new_symbol.mode=ID_C;
     new_symbol.pretty_name=base;
-    new_symbol.is_statevar=true;
-    new_symbol.static_lifetime=true;
-    new_symbol.thread_local=false;
-    new_symbol.lvalue=true;
-    new_symbol.file_local=false;
+    new_symbol.is_state_var=true;
+    new_symbol.is_static_lifetime=true;
+    new_symbol.is_thread_local=false;
+    new_symbol.is_lvalue=true;
+    new_symbol.is_file_local=false;
 
     {
       struct_exprt value(string_struct);
@@ -1524,10 +1525,10 @@ bool string_abstractiont::build_symbol_constant(const mp_integer &zero_length,
       // initialization
       goto_programt::targett assignment1=initialization.add_instruction();
       assignment1->make_assignment();
-      assignment1->code=code_assignt(symbol_expr(new_symbol), value);
+      assignment1->code=code_assignt(new_symbol.symbol_expr(), value);
     }
 
-    context.move(new_symbol);
+    symbol_table.move(new_symbol);
   }
 
   dest=address_of_exprt(symbol_exprt(identifier, string_struct));
