@@ -12,13 +12,17 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "goto_functions.h"
 
 void string_instrumentation(
-  contextt &context,
+  symbol_tablet &symbol_table,
   message_handlert &message_handler,
   goto_programt &dest);
 
 void string_instrumentation(
-  contextt &context,
+  symbol_tablet &symbol_table,
   message_handlert &message_handler,
   goto_functionst &dest);
 
+exprt is_zero_string(const exprt &what, bool write=false);
+exprt zero_string_length(const exprt &what, bool write=false);
+exprt buffer_size(const exprt &what);
+
 #endif
