@@ -14,19 +14,21 @@ Author: CM Wintersteiger
 #include <ostream>
 #include <string>
 
-class contextt;
+#include "goto_functions.h"
+
+class symbol_tablet;
 class goto_functionst;
 class message_handlert;
 
 bool write_goto_binary(
   std::ostream &out,
-  const contextt &context,
+  const symbol_tablet &symbol_table,
   const goto_functionst &goto_functions,
   int version=GOTO_BINARY_VERSION);
 
 bool write_goto_binary(
   const std::string &filename,
-  const contextt &lcontext,
+  const symbol_tablet &lsymbol_table,
   const goto_functionst &goto_functions,
   message_handlert &message_handler);
 
