@@ -12,31 +12,56 @@ Author: Daniel Kroening, kroening@kroening.com
 /*! \defgroup goto_symex Symbolic execution of goto programs
 */
 
+#include <util/options.h>
+
 #include <goto-programs/goto_functions.h>
 
-#include "basic_symex.h"
 #include "goto_symex_state.h"
 
 class typet;
 class code_typet;
+class symbol_tablet;
+class code_assignt;
+class code_function_callt;
+class exprt;
+class goto_symex_statet;
+class guardt;
+class if_exprt;
+class index_exprt;
+class symbol_exprt;
+class member_exprt;
+class namespacet;
+class side_effect_exprt;
+class symex_targett;
+class typecast_exprt;
 
 /*! \brief The main class for the forward symbolic simulator
 */
-class goto_symext:
-  public basic_symext
+class goto_symext
 {
 public:
   goto_symext(
     const namespacet &_ns,
-    contextt &_new_context,
+    symbol_tablet &_new_symbol_table,
     symex_targett &_target):
-    basic_symext(_ns, _new_context, _target),
     total_claims(0),
     remaining_claims(0),
+    constant_propagation(true),
+    new_symbol_table(_new_symbol_table),
+    ns(_ns),
+    target(_target),
+    atomic_section_counter(0),
     guard_identifier("goto_symex::\\guard")
   {
+    options.set_option("simplify", true);
     options.set_option("assertions", true);
   }
+  
+  virtual ~goto_symext()
+  {
+  }
+
+  typedef goto_symex_statet statet;
 
   /** symex all at once, starting from entry point */
   virtual void operator()(
@@ -65,7 +90,16 @@ public:
   // statistics
   unsigned total_claims, remaining_claims;
 
+  bool constant_propagation;
+
+  optionst options;
+  symbol_tablet &new_symbol_table;
+
 protected:
+  const namespacet &ns;
+  symex_targett &target;  
+  unsigned atomic_section_counter;
+
   friend class symex_dereference_statet;
   
   void new_name(symbolt &symbol);
@@ -78,6 +112,7 @@ protected:
     exprt &expr, statet &state, bool write);
     
   void replace_array_equal(exprt &expr);
+  void adjust_float_expressions(exprt &expr);
   void trigger_auto_object(const exprt &expr, statet &state);
   void initialize_auto_object(const exprt &expr, statet &state);
   void process_array_expr(exprt &expr);
@@ -114,7 +149,10 @@ protected:
 
   virtual void symex_goto(statet &state);
   virtual void symex_start_thread(statet &state);
+  virtual void symex_atomic_begin(statet &state);
+  virtual void symex_atomic_end(statet &state);  
   virtual void symex_decl(statet &state);
+  virtual void symex_dead(statet &state);
   virtual void symex_return(statet &state);
 
   virtual void symex_other(
@@ -195,6 +233,41 @@ protected:
   
   void symex_throw(statet &state);
   void symex_catch(statet &state);
+
+  virtual void do_simplify(exprt &expr);
+  
+  //virtual void symex_block(statet &state, const codet &code);
+  virtual void symex_assign(statet &state, const code_assignt &code);
+  
+  typedef enum { VISIBLE, HIDDEN } visibilityt;
+  
+  void symex_assign_rec(statet &state, const exprt &lhs, const exprt &full_lhs, const exprt &rhs, guardt &guard, visibilityt visibility);
+  void symex_assign_symbol(statet &state, const symbol_exprt &lhs, const exprt &full_lhs, const exprt &rhs, guardt &guard, visibilityt visibility);
+  void symex_assign_typecast(statet &state, const typecast_exprt &lhs, const exprt &full_lhs, const exprt &rhs, guardt &guard, visibilityt visibility);
+  void symex_assign_array(statet &state, const index_exprt &lhs, const exprt &full_lhs, const exprt &rhs, guardt &guard, visibilityt visibility);
+  void symex_assign_member(statet &state, const member_exprt &lhs, const exprt &full_lhs, const exprt &rhs, guardt &guard, visibilityt visibility);
+  void symex_assign_if(statet &state, const if_exprt &lhs, const exprt &full_lhs, const exprt &rhs, guardt &guard, visibilityt visibility);
+  void symex_assign_byte_extract(statet &state, const exprt &lhs, const exprt &full_lhs, const exprt &rhs, guardt &guard, visibilityt visibility);
+  
+  static exprt add_to_lhs(const exprt &lhs, const exprt &what);
+  
+  virtual void symex_gcc_builtin_va_arg_next(statet &state, const exprt &lhs, const side_effect_exprt &code);
+  virtual void symex_malloc        (statet &state, const exprt &lhs, const side_effect_exprt &code);
+  virtual void symex_cpp_delete    (statet &state, const codet &code);
+  virtual void symex_cpp_new       (statet &state, const exprt &lhs, const side_effect_exprt &code);
+  virtual void symex_fkt           (statet &state, const code_function_callt &code);
+  virtual void symex_macro         (statet &state, const code_function_callt &code);
+  virtual void symex_trace         (statet &state, const code_function_callt &code);
+  virtual void symex_printf        (statet &state, const exprt &lhs, const exprt &rhs);
+  virtual void symex_input         (statet &state, const codet &code);
+  virtual void symex_output        (statet &state, const codet &code);
+
+  static unsigned nondet_count;
+  static unsigned dynamic_counter;
+  
+  void read(exprt &expr);
+  void replace_nondet(exprt &expr);
+  void rewrite_quantifiers(exprt &expr, statet &state);
 };
 
 #endif
