@@ -8,9 +8,10 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <cstdlib>
 #include <cassert>
+#include <iostream>
 
-#include <std_expr.h>
-#include <prefix.h>
+#include <util/std_expr.h>
+#include <util/prefix.h>
 
 #include "goto_symex_state.h"
 
@@ -28,7 +29,9 @@ Function: goto_symex_statet::goto_symex_statet
 
 goto_symex_statet::goto_symex_statet():
   depth(0),
-  atomic_section_count(0)
+  symex_target(NULL),
+  atomic_section_id(0),
+  record_events(true)
 {
   threads.resize(1);
   new_frame();
@@ -96,8 +99,9 @@ irep_idt goto_symex_statet::level0t::operator()(
     abort();
   }
   
-  // don't rename globals or code
-  if(s->type.id()==ID_code || is_global(*s))
+  // don't rename shared variables or functions
+  if(s->type.id()==ID_code ||
+     s->is_shared())
   {
     original_identifiers[identifier]=identifier;
     return identifier;
@@ -282,19 +286,21 @@ void goto_symex_statet::assignment(
   #if 0  
   assert(l1_identifier != get_original_name(l1_identifier)
       || l1_identifier=="goto_symex::\\guard"
-      || is_global(ns.lookup(l1_identifier))
-      || has_prefix(l1_identifier.as_string(), "symex::invalid_object")
-      || has_prefix(l1_identifier.as_string(), "symex_dynamic::dynamic_object"));
+      || ns.lookup(l1_identifier).is_shared()
+      || has_prefix(id2string(l1_identifier), "symex::invalid_object")
+      || has_prefix(id2string(l1_identifier), "symex_dynamic::dynamic_object"));
   #endif
 
   // do the l2 renaming 
-  unsigned new_count=level2.current_count(l1_identifier)+1;
-  level2.rename(l1_identifier, new_count);
-  lhs.set_identifier(level2.name(l1_identifier, new_count));
+  irep_idt new_l2_name=level2.increase_counter(l1_identifier);
+  lhs.set_identifier(new_l2_name);
+
+  // in case we happen to be multi-threaded, record the memory access
+  bool is_shared=l2_thread_write_encoding(lhs, ns);
 
   // for value propagation -- the RHS is L2
   
-  if(record_value && constant_propagation(rhs))
+  if(!is_shared && record_value && constant_propagation(rhs))
     propagation.values[l1_identifier]=rhs;
   else
     propagation.remove(l1_identifier);
@@ -308,7 +314,7 @@ void goto_symex_statet::assignment(
     symbol_exprt l1_lhs(l1_identifier, lhs.type());
     level2.get_original_name(l1_lhs.type());
 
-    value_set.assign(l1_lhs, l1_rhs, ns);  
+    value_set.assign(l1_lhs, l1_rhs, ns, is_shared);  
   }
   
   #if 0
@@ -417,30 +423,38 @@ void goto_symex_statet::rename(
 
   if(expr.id()==ID_symbol)
   {
-    irep_idt identifier=to_symbol_expr(expr).get_identifier();
+    const irep_idt identifier=to_symbol_expr(expr).get_identifier();
 
     if(level==L0 || level==L1)
     {
-      identifier=rename(identifier, ns, level);
-      to_symbol_expr(expr).set_identifier(identifier);
+      const irep_idt new_name=rename(identifier, ns, level);
+      to_symbol_expr(expr).set_identifier(new_name);
     }  
     else if(level==L2)
     {
-      if(!level2.is_renamed(identifier))
+      if(l2_thread_read_encoding(to_symbol_expr(expr), ns))
       {
-        identifier=rename(identifier, ns, L1);
+        // renaming taken care of by l2_thread_encoding
+      }
+      else if(level2.is_renamed(identifier))
+      {
+        // already at L2
+      }
+      else
+      {
+        irep_idt l1_identifier=rename(identifier, ns, L1);
 
         // We also consider propagation if we go up to L2.
         // L1 identifiers are used for propagation!
         propagationt::valuest::const_iterator p_it=
-          propagation.values.find(identifier);
+          propagation.values.find(l1_identifier);
 
         if(p_it!=propagation.values.end())
           expr=p_it->second; // already L2
         else
         {
-          identifier=level2(identifier); // L2
-          to_symbol_expr(expr).set_identifier(identifier);
+          irep_idt new_name=level2(l1_identifier); // L2
+          to_symbol_expr(expr).set_identifier(new_name);
         }
       }
     }
@@ -461,6 +475,136 @@ void goto_symex_statet::rename(
 
 /*******************************************************************\
 
+Function: goto_symex_statet::l2_thread_read_encoding
+
+  Inputs:
+
+ Outputs:
+
+ Purpose: thread encoding
+
+\*******************************************************************/
+
+bool goto_symex_statet::l2_thread_read_encoding(
+  symbol_exprt &expr,
+  const namespacet &ns)
+{
+  if(!record_events)
+    return false;
+
+  // do we have threads?
+  if(threads.size()<=1)
+    return false;
+
+  const irep_idt &identifier=expr.get_identifier();
+  const irep_idt &orig_identifier=get_original_name(identifier);
+
+  // is it a shared object?
+  if(orig_identifier=="goto_symex::\\guard" ||
+     !ns.lookup(orig_identifier).is_shared())
+    return false;
+
+  // see whether we are within an atomic section
+  if(atomic_section_id!=0)
+  {
+    const irep_idt l1_identifier=rename(orig_identifier, ns, L1);
+
+    written_in_atomic_sectiont::const_iterator a_s_writes=
+      written_in_atomic_section.find(orig_identifier);
+    if(a_s_writes!=written_in_atomic_section.end())
+    {
+      for(a_s_w_entryt::const_iterator it=a_s_writes->second.begin();
+          it!=a_s_writes->second.end();
+          ++it)
+        if(it->as_expr()==guard.as_expr())
+          // there has already been a write to orig_identifier within
+          // this atomic section under the same guard
+          return false;
+    }
+
+    // we cannot determine for sure that there has been a write already
+    // so generate a read even if orig_identifier has been written on
+    // all branches flowing into this read
+    // (from_read will ensure consistency)
+    a_s_r_entryt &a_s_read=read_in_atomic_section[orig_identifier];
+    if(a_s_read.second.empty())
+    {
+      propagation.remove(l1_identifier);
+      irep_idt new_l2_name=level2.increase_counter(l1_identifier);
+      a_s_read.first=new_l2_name;
+    }
+    a_s_read.second.push_back(guard);
+
+    return false;
+  }
+
+  irep_idt new_l2_name=identifier;
+
+  // produce a fresh L2 name
+  const irep_idt l1_identifier=rename(orig_identifier, ns, L1);
+  new_l2_name=level2.increase_counter(l1_identifier);
+  expr.set_identifier(new_l2_name);
+
+  // and record that
+  assert(symex_target!=NULL);
+  symbol_exprt original_symbol(orig_identifier, expr.type());
+  symex_target->shared_read(
+    guard.as_expr(),
+    expr,
+    original_symbol,
+    atomic_section_id,
+    source);
+
+  return true;
+}
+
+/*******************************************************************\
+
+Function: goto_symex_statet::l2_thread_write_encoding
+
+  Inputs:
+
+ Outputs:
+
+ Purpose: thread encoding
+
+\*******************************************************************/
+
+bool goto_symex_statet::l2_thread_write_encoding(
+  const symbol_exprt &expr,
+  const namespacet &ns)
+{
+  const irep_idt &identifier=expr.get_identifier();
+  const irep_idt &orig_identifier=get_original_name(identifier);
+
+  // is it a shared object?
+  if(orig_identifier=="goto_symex::\\guard" ||
+     !ns.lookup(orig_identifier).is_shared())
+    return false; // not shared
+    
+  // see whether we are within an atomic section
+  if(atomic_section_id!=0)
+  {
+    written_in_atomic_section[orig_identifier].push_back(guard);
+    return false;
+  }
+
+  // record a shared write
+  symbol_exprt original_symbol(orig_identifier, expr.type());
+
+  symex_target->shared_write(
+    guard.as_expr(),
+    expr,
+    original_symbol,
+    atomic_section_id,
+    source);
+
+  // do we have threads?
+  return threads.size()>1;
+}
+
+/*******************************************************************\
+
 Function: goto_symex_statet::rename_address
 
   Inputs:
@@ -709,7 +853,7 @@ Function: goto_symex_statet::renaming_levelt::get_original_name
 
 void goto_symex_statet::renaming_levelt::get_original_name(typet &type) const
 {
-  // rename all the symbols with their last known value
+  // rename all the symbols back to their original name
 
   if(type.id()==ID_array)
   {
@@ -743,8 +887,9 @@ Function: goto_symex_statet::get_original_name
 const irep_idt &goto_symex_statet::get_original_name(
   const irep_idt &identifier) const
 {
-  return level0.get_original_name(level1.get_original_name(
-        level2.get_original_name(identifier)));
+  return level0.get_original_name(
+         level1.get_original_name(
+         level2.get_original_name(identifier)));
 }
 
 /*******************************************************************\
@@ -766,6 +911,7 @@ void goto_symex_statet::switch_to_thread(unsigned t)
   
   // save PC
   threads[source.thread_nr].pc=source.pc;
+  threads[source.thread_nr].atomic_section_id=atomic_section_id;
 
   // get new PC
   source.thread_nr=t;
