@@ -10,8 +10,8 @@ Author: Daniel Kroening
 
 #include <cassert>
 
-#include <arith_tools.h>
-#include <symbol.h>
+#include <util/arith_tools.h>
+#include <util/symbol.h>
 
 #include <ansi-c/printf_formatter.h>
 #include <langapi/language_util.h>
@@ -67,16 +67,20 @@ void goto_trace_stept::output(
   case goto_trace_stept::DECL: out << "DECL"; break;
   case goto_trace_stept::OUTPUT: out << "OUTPUT"; break;
   case goto_trace_stept::INPUT: out << "INPUT"; break;
+  case goto_trace_stept::ATOMIC_BEGIN: out << "ATOMC_BEGIN"; break;
+  case goto_trace_stept::ATOMIC_END: out << "ATOMIC_END"; break;
+  case goto_trace_stept::SHARED_READ: out << "SHARED_READ"; break;
+  case goto_trace_stept::SHARED_WRITE: out << "SHARED WRITE"; break;
   default: assert(false);
   }
 
   if(type==ASSERT || type==ASSUME)
     out << " (" << cond_value << ")";
 
-  out << std::endl;
+  out << "\n";
 
   if(!pc->location.is_nil())
-    out << pc->location << std::endl;
+    out << pc->location << "\n";
 
   if(pc->is_goto())
     out << "GOTO   ";
@@ -97,31 +101,31 @@ void goto_trace_stept::output(
   else
     out << "(?)    ";
 
-  out << std::endl;
+  out << "\n";
 
   if(pc->is_other() || pc->is_assign())
   {
     irep_idt identifier=lhs_object.get_identifier();
     out << "  " << identifier
         << " = " << from_expr(ns, identifier, lhs_object_value)
-        << std::endl;
+        << "\n";
   }
   else if(pc->is_assert())
   {
     if(!cond_value)
     {
-      out << "Violated property:" << std::endl;
+      out << "Violated property:" << "\n";
       if(pc->location.is_nil())
-        out << "  " << pc->location << std::endl;
+        out << "  " << pc->location << "\n";
       
       if(comment!="")
-        out << "  " << comment << std::endl;
-      out << "  " << from_expr(ns, "", pc->guard) << std::endl;
-      out << std::endl;
+        out << "  " << comment << "\n";
+      out << "  " << from_expr(ns, "", pc->guard) << "\n";
+      out << "\n";
     }
   }
   
-  out << std::endl;
+  out << "\n";
 }
 
 /*******************************************************************\
@@ -149,7 +153,8 @@ std::string counterexample_value_binary(
        type.id()==ID_bv ||
        type.id()==ID_fixedbv ||
        type.id()==ID_floatbv ||
-       type.id()==ID_pointer)
+       type.id()==ID_pointer ||
+       type.id()==ID_c_enum)
     {
       return expr.get_string(ID_value);
     }
@@ -226,87 +231,7 @@ void counterexample_value(
   out << "  "
       << from_expr(ns, identifier, full_lhs)
       << "=" << value_string
-      << std::endl;
-}
-
-/*******************************************************************\
-
-Function: show_goto_trace_gui
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-void show_goto_trace_gui(
-  std::ostream &out,
-  const namespacet &ns,
-  const goto_tracet &goto_trace)
-{
-  locationt previous_location;
-
-  for(goto_tracet::stepst::const_iterator
-      it=goto_trace.steps.begin();
-      it!=goto_trace.steps.end();
-      it++)
-  {
-    const locationt &location=it->pc->location;
-    
-    if(it->type==goto_trace_stept::ASSERT &&
-       !it->cond_value)
-    {
-      out << "FAILED" << std::endl
-          << it->comment << std::endl // value
-          << std::endl // PC
-          << location.get_file() << std::endl
-          << location.get_line() << std::endl
-          << location.get_column() << std::endl;
-    }
-    else if(it->type==goto_trace_stept::ASSIGNMENT)
-    {
-      irep_idt identifier=it->lhs_object.get_identifier();
-      std::string value_string=from_expr(ns, identifier, it->lhs_object_value);
-      
-      const symbolt *symbol;
-      irep_idt base_name;
-      if(!ns.lookup(identifier, symbol))
-        base_name=symbol->base_name;
-
-      out << "TRACE" << std::endl;
-
-      out << identifier << ","
-          << base_name << ","
-          << it->lhs_object_value.type().to_string() << ","
-          << value_string << std::endl
-          << it->step_nr << std::endl
-          << it->pc->location.get_file() << std::endl
-          << it->pc->location.get_line() << std::endl
-          << it->pc->location.get_column() << std::endl;
-    }
-    else if(location!=previous_location)
-    {
-      // just the location
-      
-      if(location.get_file()!="")
-      {
-        out << "TRACE" << std::endl;
-
-        out << ","             // identifier
-            << ","             // base_name
-            << ","             // type
-            << "" << std::endl // value
-            << it->step_nr << std::endl
-            << location.get_file() << std::endl
-            << location.get_line() << std::endl
-            << location.get_column() << std::endl;
-      }
-    }
-
-    previous_location=location;
-  }
+      << "\n";
 }
 
 /*******************************************************************\
@@ -327,7 +252,7 @@ void show_state_header(
   const locationt &location,
   unsigned step_nr)
 {
-  out << std::endl;
+  out << "\n";
   
   if(step_nr==0)
     out << "Initial State";
@@ -335,13 +260,13 @@ void show_state_header(
     out << "State " << step_nr;
   
   out << " " << location
-      << " thread " << state.thread_nr << std::endl;
-  out << "----------------------------------------------------" << std::endl;
+      << " thread " << state.thread_nr << "\n";
+  out << "----------------------------------------------------" << "\n";
 }
 
 /*******************************************************************\
 
-Function: show_goto_trace
+Function: is_index_member_symbol
 
   Inputs:
 
@@ -363,6 +288,18 @@ bool is_index_member_symbol(const exprt &src)
     return false;
 }
 
+/*******************************************************************\
+
+Function: show_goto_trace
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
 void show_goto_trace(
   std::ostream &out,
   const namespacet &ns,
@@ -381,16 +318,16 @@ void show_goto_trace(
     case goto_trace_stept::ASSERT:
       if(!it->cond_value)
       {
-        out << std::endl;
-        out << "Violated property:" << std::endl;
+        out << "\n";
+        out << "Violated property:" << "\n";
         if(!it->pc->location.is_nil())
-          out << "  " << it->pc->location << std::endl;
-        out << "  " << it->comment << std::endl;
+          out << "  " << it->pc->location << "\n";
+        out << "  " << it->comment << "\n";
 
         if(it->pc->is_assert())
-          out << "  " << from_expr(ns, "", it->pc->guard) << std::endl;
+          out << "  " << from_expr(ns, "", it->pc->guard) << "\n";
         
-        out << std::endl;
+        out << "\n";
       }
       break;
       
@@ -438,7 +375,7 @@ void show_goto_trace(
         printf_formattert printf_formatter(ns);
         printf_formatter(id2string(it->format_string), it->io_args);
         printf_formatter.print(out);
-        out << std::endl;
+        out << "\n";
       }
       else
       {
@@ -457,7 +394,7 @@ void show_goto_trace(
           out << " (" << counterexample_value_binary(*l_it, ns) << ")";
         }
       
-        out << std::endl;
+        out << "\n";
       }
       break;
 
@@ -477,11 +414,24 @@ void show_goto_trace(
         out << " (" << counterexample_value_binary(*l_it, ns) << ")";
       }
       
-      out << std::endl;
+      out << "\n";
       break;
       
     case goto_trace_stept::FUNCTION_CALL:
     case goto_trace_stept::FUNCTION_RETURN:
+    case goto_trace_stept::SPAWN:
+    case goto_trace_stept::MEMORY_BARRIER:
+    case goto_trace_stept::ATOMIC_BEGIN:
+    case goto_trace_stept::ATOMIC_END:
+      break;
+    
+    case goto_trace_stept::CONSTRAINT:
+      assert(false);
+      break;
+      
+    case goto_trace_stept::SHARED_READ:
+    case goto_trace_stept::SHARED_WRITE:
+      assert(false);
       break;
       
     default:
