@@ -10,12 +10,13 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <cstdlib>
 #include <set>
 #include <fstream>
+#include <iostream>
 
-#include <simplify_expr.h>
-#include <i2string.h>
-#include <arith_tools.h>
-#include <std_expr.h>
-#include <guard.h>
+#include <util/simplify_expr.h>
+#include <util/i2string.h>
+#include <util/arith_tools.h>
+#include <util/std_expr.h>
+#include <util/guard.h>
 
 #include <langapi/language_util.h>
 
@@ -108,7 +109,7 @@ void symex_slice_by_tracet::slice_by_trace(std::string trace_files,
   equation.SSA_steps.push_front(symex_target_equationt::SSA_stept());
   symex_target_equationt::SSA_stept &SSA_step = equation.SSA_steps.front(); 
 
-  SSA_step.guard_expr=t_guard.as_expr();
+  SSA_step.guard=t_guard.as_expr();
   SSA_step.ssa_lhs.make_nil();
   SSA_step.cond_expr.swap(trace_condition);
   SSA_step.type=goto_trace_stept::ASSUME;
@@ -164,12 +165,12 @@ void symex_slice_by_tracet::read_trace(std::string filename)
   
   for (size_t i = 0; i < sigma.size(); i++) {
     exprt f_e = static_cast<const exprt &>(get_nil_irep());
-    f_e.make_false();
+    f_e=false_exprt();
     t.push_back(f_e);
   }
     
   exprt t_e = static_cast<const exprt &>(get_nil_irep());
-  t_e.make_true();
+  t_e=true_exprt();
   t.push_back(t_e);
 }  
 
@@ -290,7 +291,7 @@ void symex_slice_by_tracet::compute_ts_back(
 	  continue;
       }
       
-      exprt guard = i->guard_expr;
+      exprt guard = i->guard;
 
 #if 0
       std::cout << "EVENT:  " << event << std::endl;
@@ -355,11 +356,11 @@ void symex_slice_by_tracet::compute_ts_back(
 	  simplify(u_lhs, ns);
 	  
 	  if ((!u_lhs.is_false()) && implies_false(u_lhs))
-	    u_lhs.make_false();
+	    u_lhs=false_exprt();
 	  if (!u_lhs.is_false())
 	    slice_this = false;
 	} else {
-	  u_lhs.make_false();
+	  u_lhs=false_exprt();
 	}
 	exprt u_rhs = exprt (ID_and, typet(ID_bool));
 	if ((semantics != ":suffix") || (j != 0)) {
@@ -381,7 +382,7 @@ void symex_slice_by_tracet::compute_ts_back(
       }
       
       if (semantics == ":prefix")
-	t[t.size()-1].make_true();
+	t[t.size()-1]=true_exprt();
       
       if (slice_this) {
 	exprt guard_copy(guard);
@@ -444,7 +445,7 @@ void symex_slice_by_tracet::slice_SSA_steps(
     if (it->is_location())
       location_SSA_steps++;
     bool sliced_SSA_step = false;
-    exprt guard=it->guard_expr;
+    exprt guard=it->guard;
 
     simplify(guard, ns);
 
@@ -459,9 +460,9 @@ void symex_slice_by_tracet::slice_SSA_steps(
       simplify(guard, ns);
       
       if (implications.count(guard) != 0) {
-	it->cond_expr.make_true();
-	it->ssa_rhs.make_true();
-	it->guard_expr.make_false();
+	it->cond_expr=true_exprt();
+	it->ssa_rhs=true_exprt();
+	it->guard=false_exprt();
 	sliced_SSA_steps++;
 	if (it->is_output() || it->is_location())
 	  trace_loc_sliced++;
@@ -477,9 +478,9 @@ void symex_slice_by_tracet::slice_SSA_steps(
 	simplify(neg_expr, ns);
 	
 	if (implications.count(neg_expr) != 0) {
-	  it->cond_expr.make_true();
-	  it->ssa_rhs.make_true();
-	  it->guard_expr.make_false();
+	  it->cond_expr=true_exprt();
+	  it->ssa_rhs=true_exprt();
+	  it->guard=false_exprt();
 	  sliced_SSA_steps++;
 	  if (it->is_output() || it->is_location())
 	    trace_loc_sliced++;
@@ -582,7 +583,7 @@ void symex_slice_by_tracet::assign_merges(
     equation.SSA_steps.push_front(symex_target_equationt::SSA_stept());
     symex_target_equationt::SSA_stept &SSA_step = equation.SSA_steps.front();  
     
-    SSA_step.guard_expr=t_guard.as_expr();
+    SSA_step.guard=t_guard.as_expr();
     SSA_step.ssa_lhs=merge_sym;
     SSA_step.original_lhs_object=merge_symbol;
     SSA_step.ssa_rhs.swap(merge_copy);
