@@ -6,11 +6,11 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
-#include <expr_util.h>
-#include <std_expr.h>
-#include <i2string.h>
+#include <util/expr_util.h>
+#include <util/std_expr.h>
+#include <util/i2string.h>
 
 #include "goto_symex.h"
 
@@ -37,7 +37,7 @@ void goto_symext::symex_goto(statet &state)
   state.rename(new_guard, ns);
   do_simplify(new_guard);
   
-  target.location(state.guard, state.source);
+  target.location(state.guard.as_expr(), state.source);
   
   if(new_guard.is_false() ||
      state.guard.is_false())
@@ -57,7 +57,7 @@ void goto_symext::symex_goto(statet &state)
     throw "no support for non-deterministic gotos";
     
   goto_programt::const_targett goto_target=
-    instruction.targets.front();
+    instruction.get_target();
     
   bool forward=
     state.source.pc->location_number<
@@ -140,11 +140,11 @@ void goto_symext::symex_goto(statet &state)
       guardt guard;
 
       target.assignment(
-        guard,
+        guard.as_expr(),
         new_lhs, guard_symbol_expr, new_lhs, guard_symbol_expr,
         new_rhs,
         state.source,
-        symex_targett::HIDDEN);
+        symex_targett::GUARD);
       
       guard_expr=guard_symbol_expr;
       guard_expr.make_not();
@@ -191,7 +191,7 @@ void goto_symext::symex_step_goto(statet &state, bool taken)
   state.guard.guard_expr(guard);
   do_simplify(guard);
 
-  target.assumption(state.guard, guard, state.source);  
+  target.assumption(state.guard.as_expr(), guard, state.source);  
 }
 
 /*******************************************************************\
@@ -226,6 +226,16 @@ void goto_symext::merge_gotos(statet &state)
       list_it++)
   {
     statet::goto_statet &goto_state=*list_it;
+
+    // fix up atomic section
+    if(state.guard.is_false())
+      state.atomic_section_id=goto_state.atomic_section_id;
+    else
+    {
+      if(!goto_state.guard.is_false() &&
+         state.atomic_section_id!=goto_state.atomic_section_id)
+        throw "unmatched atomic section detected";
+    }
     
     // do SSA phi functions
     phi_function(goto_state, state);
@@ -309,21 +319,47 @@ void goto_symext::phi_function(
     irep_idt original_identifier=
       dest_state.get_original_name(l1_identifier);
 
-    // get type (may need renaming)      
+    // shared variables are renamed on every access anyway, we don't need to
+    // merge anything
     const symbolt &symbol=ns.lookup(original_identifier);
+    
+    // shared?
+    if(dest_state.atomic_section_id==0 &&
+       dest_state.threads.size()>=2 && symbol.is_shared())
+      continue; // no phi nodes for shared stuff
+    
+    // get type (may need renaming)      
     typet type=symbol.type;
     dest_state.rename(type, ns);
     
-    exprt rhs;
+    exprt goto_state_rhs, dest_state_rhs;
 
+    {
+      goto_symex_statet::propagationt::valuest::const_iterator p_it=
+        goto_state.propagation.values.find(l1_identifier);
+
+      if(p_it!=goto_state.propagation.values.end())
+        goto_state_rhs=p_it->second;
+      else
+        goto_state_rhs=symbol_exprt(goto_state.level2.current_name(l1_identifier), type);
+    }
+    
+    {
+      goto_symex_statet::propagationt::valuest::const_iterator p_it=
+        dest_state.propagation.values.find(l1_identifier);
+
+      if(p_it!=dest_state.propagation.values.end())
+        dest_state_rhs=p_it->second;
+      else
+        dest_state_rhs=symbol_exprt(dest_state.level2.current_name(l1_identifier), type);
+    }
+    
+    exprt rhs;
+    
     if(dest_state.guard.is_false())
-    {
-      rhs=symbol_exprt(goto_state.level2.current_name(l1_identifier), type);
-    }
+      rhs=goto_state_rhs;
     else if(goto_state.guard.is_false())
-    {
-      rhs=symbol_exprt(dest_state.level2.current_name(l1_identifier), type);
-    }
+      rhs=dest_state_rhs;
     else
     {      
       guardt tmp_guard(goto_state.guard);
@@ -331,25 +367,20 @@ void goto_symext::phi_function(
       // this gets the diff between the guards
       tmp_guard-=dest_state.guard;
       
-      rhs=if_exprt();
-      rhs.type()=type;
-      rhs.op0()=tmp_guard.as_expr();
-      rhs.op1()=symbol_exprt(goto_state.level2.current_name(l1_identifier), type);
-      rhs.op2()=symbol_exprt(dest_state.level2.current_name(l1_identifier), type);
+      rhs=if_exprt(tmp_guard.as_expr(), goto_state_rhs, dest_state_rhs, type);
+      do_simplify(rhs);
     }
 
-    symbol_exprt lhs=symbol_expr(symbol);
+    symbol_exprt lhs=symbol.symbol_expr();
     symbol_exprt new_lhs=symbol_exprt(l1_identifier, type);
-    dest_state.assignment(new_lhs, rhs, ns, false);
+    dest_state.assignment(new_lhs, rhs, ns, true);
     
-    guardt true_guard;
-
     target.assignment(
-      true_guard,
+      true_exprt(),
       new_lhs, lhs, new_lhs, lhs,
       rhs,
       dest_state.source,
-      symex_targett::HIDDEN);
+      symex_targett::PHI);
   }
 }
 
@@ -376,7 +407,7 @@ void goto_symext::loop_bound_exceeded(
   if(guard.is_true())
     negated_cond=false_exprt();
   else
-    negated_cond=gen_not(guard);
+    negated_cond=not_exprt(guard);
 
   bool unwinding_assertions=
     options.get_bool_option("unwinding-assertions");
@@ -398,7 +429,7 @@ void goto_symext::loop_bound_exceeded(
       // generate unwinding assumption, unless we permit partial loops
       exprt guarded_expr=negated_cond;
       state.guard.guard_expr(guarded_expr);
-      target.assumption(state.guard, guarded_expr, state.source);
+      target.assumption(state.guard.as_expr(), guarded_expr, state.source);
     }
 
     // add to state guard to prevent further assignments
