@@ -8,9 +8,9 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <cassert>
 
-#include <i2string.h>
-#include <std_expr.h>
-#include <expr_util.h>
+#include <util/i2string.h>
+#include <util/std_expr.h>
+#include <util/expr_util.h>
 
 #include <langapi/language_util.h>
 #include <solvers/prop/prop_conv.h>
@@ -21,6 +21,195 @@ Author: Daniel Kroening, kroening@kroening.com
 
 /*******************************************************************\
 
+Function: symex_target_equationt::symex_target_equationt
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+symex_target_equationt::symex_target_equationt(
+  const namespacet &_ns):ns(_ns)
+{
+}
+
+/*******************************************************************\
+
+Function: symex_target_equationt::~symex_target_equationt
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+symex_target_equationt::~symex_target_equationt()
+{
+}
+
+/*******************************************************************\
+
+Function: symex_target_equationt::shared_read
+
+  Inputs:
+
+ Outputs:
+
+ Purpose: read from a shared variable
+
+\*******************************************************************/
+
+void symex_target_equationt::shared_read(
+  const exprt &guard,
+  const symbol_exprt &ssa_object,
+  const symbol_exprt &original_object,
+  unsigned atomic_section_id,
+  const sourcet &source)
+{
+  SSA_steps.push_back(SSA_stept());
+  SSA_stept &SSA_step=SSA_steps.back();
+  
+  SSA_step.guard=guard;
+  SSA_step.ssa_lhs=ssa_object;
+  SSA_step.original_lhs_object=original_object;
+  SSA_step.type=goto_trace_stept::SHARED_READ;
+  SSA_step.atomic_section_id=atomic_section_id;
+  SSA_step.source=source;
+}
+
+/*******************************************************************\
+
+Function: symex_target_equationt::shared_write
+
+  Inputs:
+
+ Outputs:
+
+ Purpose: write to a sharedvariable
+
+\*******************************************************************/
+
+void symex_target_equationt::shared_write(
+  const exprt &guard,
+  const symbol_exprt &ssa_object,
+  const symbol_exprt &original_object,
+  unsigned atomic_section_id,
+  const sourcet &source)
+{
+  SSA_steps.push_back(SSA_stept());
+  SSA_stept &SSA_step=SSA_steps.back();
+  
+  SSA_step.guard=guard;
+  SSA_step.ssa_lhs=ssa_object;
+  SSA_step.original_lhs_object=original_object;
+  SSA_step.type=goto_trace_stept::SHARED_WRITE;
+  SSA_step.atomic_section_id=atomic_section_id;
+  SSA_step.source=source;
+}
+
+/*******************************************************************\
+
+Function: symex_target_equationt::spawn
+
+  Inputs:
+
+ Outputs:
+
+ Purpose: spawn a new thread
+
+\*******************************************************************/
+
+void symex_target_equationt::spawn(
+  const exprt &guard,
+  const sourcet &source)
+{
+  SSA_steps.push_back(SSA_stept());
+  SSA_stept &SSA_step=SSA_steps.back();
+  SSA_step.guard=guard;
+  SSA_step.type=goto_trace_stept::SPAWN;
+  SSA_step.source=source;
+}
+
+/*******************************************************************\
+
+Function: symex_target_equationt::memory_barrier
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void symex_target_equationt::memory_barrier(
+  const exprt &guard,
+  const sourcet &source)
+{
+  SSA_steps.push_back(SSA_stept());
+  SSA_stept &SSA_step=SSA_steps.back();
+  SSA_step.guard=guard;
+  SSA_step.type=goto_trace_stept::MEMORY_BARRIER;
+  SSA_step.source=source;
+}
+
+/*******************************************************************\
+
+Function: symex_target_equationt::atomic_begin
+
+  Inputs:
+
+ Outputs:
+
+ Purpose: start an atomic section
+
+\*******************************************************************/
+
+void symex_target_equationt::atomic_begin(
+  const exprt &guard,
+  unsigned atomic_section_id,
+  const sourcet &source)
+{
+  SSA_steps.push_back(SSA_stept());
+  SSA_stept &SSA_step=SSA_steps.back();
+  SSA_step.guard=guard;
+  SSA_step.type=goto_trace_stept::ATOMIC_BEGIN;
+  SSA_step.atomic_section_id=atomic_section_id;
+  SSA_step.source=source;
+}
+
+/*******************************************************************\
+
+Function: symex_target_equationt::atomic_end
+
+  Inputs:
+
+ Outputs:
+
+ Purpose: end an atomic section
+
+\*******************************************************************/
+
+void symex_target_equationt::atomic_end(
+  const exprt &guard,
+  unsigned atomic_section_id,
+  const sourcet &source)
+{
+  SSA_steps.push_back(SSA_stept());
+  SSA_stept &SSA_step=SSA_steps.back();
+  SSA_step.guard=guard;
+  SSA_step.type=goto_trace_stept::ATOMIC_END;
+  SSA_step.atomic_section_id=atomic_section_id;
+  SSA_step.source=source;
+}
+
+/*******************************************************************\
+
 Function: symex_target_equationt::assignment
 
   Inputs:
@@ -32,7 +221,7 @@ Function: symex_target_equationt::assignment
 \*******************************************************************/
 
 void symex_target_equationt::assignment(
-  const guardt &guard,
+  const exprt &guard,
   const symbol_exprt &ssa_lhs,
   const symbol_exprt &original_lhs_object,
   const exprt &ssa_full_lhs,
@@ -46,7 +235,7 @@ void symex_target_equationt::assignment(
   SSA_steps.push_back(SSA_stept());
   SSA_stept &SSA_step=SSA_steps.back();
   
-  SSA_step.guard_expr=guard.as_expr();
+  SSA_step.guard=guard;
   SSA_step.ssa_lhs=ssa_lhs;
   SSA_step.original_lhs_object=original_lhs_object;
   SSA_step.ssa_full_lhs=ssa_full_lhs;
@@ -72,7 +261,7 @@ Function: symex_target_equationt::decl
 \*******************************************************************/
 
 void symex_target_equationt::decl(
-  const guardt &guard,
+  const exprt &guard,
   const symbol_exprt &ssa_lhs,
   const symbol_exprt &original_lhs_object,
   const sourcet &source)
@@ -82,7 +271,7 @@ void symex_target_equationt::decl(
   SSA_steps.push_back(SSA_stept());
   SSA_stept &SSA_step=SSA_steps.back();
   
-  SSA_step.guard_expr=guard.as_expr();
+  SSA_step.guard=guard;
   SSA_step.ssa_lhs=ssa_lhs;
   SSA_step.ssa_full_lhs=ssa_lhs;
   SSA_step.original_lhs_object=original_lhs_object;
@@ -108,7 +297,7 @@ Function: symex_target_equationt::dead
 \*******************************************************************/
 
 void symex_target_equationt::dead(
-  const guardt &guard,
+  const exprt &guard,
   const symbol_exprt &ssa_lhs,
   const symbol_exprt &original_lhs_object,
   const sourcet &source)
@@ -129,13 +318,13 @@ Function: symex_target_equationt::location
 \*******************************************************************/
 
 void symex_target_equationt::location(
-  const guardt &guard,
+  const exprt &guard,
   const sourcet &source)
 {
   SSA_steps.push_back(SSA_stept());
   SSA_stept &SSA_step=SSA_steps.back();
   
-  SSA_step.guard_expr=guard.as_expr();
+  SSA_step.guard=guard;
   SSA_step.type=goto_trace_stept::LOCATION;
   SSA_step.source=source;
 }
@@ -153,14 +342,14 @@ Function: symex_target_equationt::function_call
 \*******************************************************************/
 
 void symex_target_equationt::function_call(
-  const guardt &guard,
+  const exprt &guard,
   const irep_idt &identifier,
   const sourcet &source)
 {
   SSA_steps.push_back(SSA_stept());
   SSA_stept &SSA_step=SSA_steps.back();
   
-  SSA_step.guard_expr=guard.as_expr();
+  SSA_step.guard=guard;
   SSA_step.type=goto_trace_stept::FUNCTION_CALL;
   SSA_step.source=source;
   SSA_step.identifier=identifier;
@@ -179,14 +368,14 @@ Function: symex_target_equationt::function_return
 \*******************************************************************/
 
 void symex_target_equationt::function_return(
-  const guardt &guard,
+  const exprt &guard,
   const irep_idt &identifier,
   const sourcet &source)
 {
   SSA_steps.push_back(SSA_stept());
   SSA_stept &SSA_step=SSA_steps.back();
   
-  SSA_step.guard_expr=guard.as_expr();
+  SSA_step.guard=guard;
   SSA_step.type=goto_trace_stept::FUNCTION_RETURN;
   SSA_step.source=source;
   SSA_step.identifier=identifier;
@@ -205,7 +394,7 @@ Function: symex_target_equationt::output
 \*******************************************************************/
 
 void symex_target_equationt::output(
-  const guardt &guard,
+  const exprt &guard,
   const sourcet &source,
   const irep_idt &output_id,
   const std::list<exprt> &args)
@@ -213,7 +402,7 @@ void symex_target_equationt::output(
   SSA_steps.push_back(SSA_stept());
   SSA_stept &SSA_step=SSA_steps.back();
   
-  SSA_step.guard_expr=guard.as_expr();
+  SSA_step.guard=guard;
   SSA_step.type=goto_trace_stept::OUTPUT;
   SSA_step.source=source;
   SSA_step.io_args=args;
@@ -233,7 +422,7 @@ Function: symex_target_equationt::output_fmt
 \*******************************************************************/
 
 void symex_target_equationt::output_fmt(
-  const guardt &guard,
+  const exprt &guard,
   const sourcet &source,
   const irep_idt &output_id,
   const irep_idt &fmt,
@@ -242,7 +431,7 @@ void symex_target_equationt::output_fmt(
   SSA_steps.push_back(SSA_stept());
   SSA_stept &SSA_step=SSA_steps.back();
   
-  SSA_step.guard_expr=guard.as_expr();
+  SSA_step.guard=guard;
   SSA_step.type=goto_trace_stept::OUTPUT;
   SSA_step.source=source;
   SSA_step.io_args=args;
@@ -264,7 +453,7 @@ Function: symex_target_equationt::input
 \*******************************************************************/
 
 void symex_target_equationt::input(
-  const guardt &guard,
+  const exprt &guard,
   const sourcet &source,
   const irep_idt &input_id,
   const std::list<exprt> &args)
@@ -272,7 +461,7 @@ void symex_target_equationt::input(
   SSA_steps.push_back(SSA_stept());
   SSA_stept &SSA_step=SSA_steps.back();
   
-  SSA_step.guard_expr=guard.as_expr();
+  SSA_step.guard=guard;
   SSA_step.type=goto_trace_stept::INPUT;
   SSA_step.source=source;
   SSA_step.io_args=args;
@@ -292,14 +481,14 @@ Function: symex_target_equationt::assumption
 \*******************************************************************/
 
 void symex_target_equationt::assumption(
-  const guardt &guard,
+  const exprt &guard,
   const exprt &cond,
   const sourcet &source)
 {
   SSA_steps.push_back(SSA_stept());
   SSA_stept &SSA_step=SSA_steps.back();
   
-  SSA_step.guard_expr=guard.as_expr();
+  SSA_step.guard=guard;
   SSA_step.cond_expr=cond;
   SSA_step.type=goto_trace_stept::ASSUME;
   SSA_step.source=source;
@@ -318,7 +507,7 @@ Function: symex_target_equationt::assertion
 \*******************************************************************/
 
 void symex_target_equationt::assertion(
-  const guardt &guard,
+  const exprt &guard,
   const exprt &cond,
   const std::string &msg,
   const sourcet &source)
@@ -326,7 +515,7 @@ void symex_target_equationt::assertion(
   SSA_steps.push_back(SSA_stept());
   SSA_stept &SSA_step=SSA_steps.back();
   
-  SSA_step.guard_expr=guard.as_expr();
+  SSA_step.guard=guard;
   SSA_step.cond_expr=cond;
   SSA_step.type=goto_trace_stept::ASSERT;
   SSA_step.source=source;
@@ -335,6 +524,34 @@ void symex_target_equationt::assertion(
 
 /*******************************************************************\
 
+Function: symex_target_equationt::constraint
+
+  Inputs: 
+
+ Outputs:
+
+ Purpose: record a constraint
+
+\*******************************************************************/
+
+void symex_target_equationt::constraint(
+  const exprt &cond,
+  const std::string &msg,
+  const sourcet &source)
+{
+  // like assumption, but with global effect
+  SSA_steps.push_back(SSA_stept());
+  SSA_stept &SSA_step=SSA_steps.back();
+  
+  SSA_step.guard=true_exprt();
+  SSA_step.cond_expr=cond;
+  SSA_step.type=goto_trace_stept::CONSTRAINT;
+  SSA_step.source=source;
+  SSA_step.comment=msg;
+}
+
+/*******************************************************************\
+
 Function: symex_target_equationt::convert
 
   Inputs:
@@ -354,6 +571,7 @@ void symex_target_equationt::convert(
   convert_assumptions(prop_conv);
   convert_assertions(prop_conv);
   convert_io(prop_conv);
+  convert_constraints(prop_conv);
 }
 
 /*******************************************************************\
@@ -427,7 +645,7 @@ void symex_target_equationt::convert_guards(
     if(it->ignore)
       it->guard_literal=const_literal(false);
     else
-      it->guard_literal=prop_conv.convert(it->guard_expr);
+      it->guard_literal=prop_conv.convert(it->guard);
   }
 }
 
@@ -461,6 +679,35 @@ void symex_target_equationt::convert_assumptions(
 
 /*******************************************************************\
 
+Function: symex_target_equationt::convert_constraints
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void symex_target_equationt::convert_constraints(
+  decision_proceduret &decision_procedure) const
+{
+  for(SSA_stepst::const_iterator it=SSA_steps.begin();
+      it!=SSA_steps.end();
+      it++)
+  {
+    if(it->is_constraint())
+    {
+      if(it->ignore)
+        continue;
+
+      decision_procedure.set_to_true(it->cond_expr);
+    }
+  }
+}
+
+/*******************************************************************\
+
 Function: symex_target_equationt::convert_assertions
 
   Inputs:
@@ -481,7 +728,7 @@ void symex_target_equationt::convert_assertions(
 
   if(number_of_assertions==0)
     return;
-  
+    
   if(number_of_assertions==1)
   {
     for(SSA_stepst::iterator it=SSA_steps.begin();
@@ -489,7 +736,7 @@ void symex_target_equationt::convert_assertions(
       if(it->is_assert())
       {
         prop_conv.set_to_false(it->cond_expr);
-        it->cond_literal=prop_conv.convert(it->cond_expr);
+        it->cond_literal=const_literal(false);
         return; // prevent further assumptions!
       }
       else if(it->is_assume())
@@ -635,6 +882,8 @@ void symex_target_equationt::SSA_stept::output(
     {
     case HIDDEN: out << "HIDDEN"; break;
     case STATE: out << "STATE"; break;
+    case PHI: out << "PHI"; break;
+    case GUARD: out << "GUARD"; break; 
     default:;
     }
 
@@ -644,17 +893,27 @@ void symex_target_equationt::SSA_stept::output(
   case goto_trace_stept::DEAD: out << "DEAD" << std::endl; break;
   case goto_trace_stept::FUNCTION_CALL: out << "FUNCTION_CALL" << std::endl; break;
   case goto_trace_stept::FUNCTION_RETURN: out << "FUNCTION_RETURN" << std::endl; break;
+  case goto_trace_stept::CONSTRAINT: out << "CONSTRAINT" << std::endl; break;
+  case goto_trace_stept::SHARED_READ: out << "SHARED READ" << std::endl; break;
+  case goto_trace_stept::SHARED_WRITE: out << "SHARED WRITE" << std::endl; break;
+  case goto_trace_stept::ATOMIC_BEGIN: out << "ATOMIC_BEGIN" << std::endl; break;
+  case goto_trace_stept::ATOMIC_END: out << "AUTOMIC_END" << std::endl; break;
+  case goto_trace_stept::SPAWN: out << "SPAWN" << std::endl; break;
+  case goto_trace_stept::MEMORY_BARRIER: out << "MEMORY_BARRIER" << std::endl; break;
 
   default: assert(false);
   }
 
-  if(is_assert() || is_assume() || is_assignment())
+  if(is_assert() || is_assume() || is_assignment() || is_constraint())
     out << from_expr(ns, "", cond_expr) << std::endl;
   
-  if(is_assert())
+  if(is_assert() || is_constraint())
     out << comment << std::endl;
 
-  out << "Guard: " << from_expr(ns, "", guard_expr) << std::endl;
+  if(is_shared_read() || is_shared_write())
+    out << from_expr(ns, "", ssa_lhs) << std::endl;
+
+  out << "Guard: " << from_expr(ns, "", guard) << std::endl;
 }
 
 /*******************************************************************\
@@ -676,3 +935,27 @@ std::ostream &operator<<(
   equation.output(out);
   return out;
 }
+
+/*******************************************************************\
+
+Function: operator <<
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+std::ostream &operator<<(
+  std::ostream &out,
+  const symex_target_equationt::SSA_stept &step)
+{
+  // may cause lookup failures, since it's blank
+  symbol_tablet symbol_table;
+  namespacet ns(symbol_table);
+  step.output(ns, out);
+  return out;
+}
+
