@@ -10,6 +10,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #define CPROVER_BASIC_SYMEX_EQUATION_H
 
 #include <list>
+#include <ostream>
 
 #include <goto-programs/goto_program.h>
 #include <solvers/prop/literal.h>
@@ -24,11 +25,28 @@ class prop_convt;
 class symex_target_equationt:public symex_targett
 {
 public:
-  symex_target_equationt(const namespacet &_ns):ns(_ns) { }
+  explicit symex_target_equationt(const namespacet &_ns);
+  virtual ~symex_target_equationt();
+
+  // read event
+  virtual void shared_read(
+    const exprt &guard,
+    const symbol_exprt &ssa_object,
+    const symbol_exprt &original_object,
+    unsigned atomic_section_id,
+    const sourcet &source);
+
+  // write event
+  virtual void shared_write(
+    const exprt &guard,
+    const symbol_exprt &ssa_object,
+    const symbol_exprt &original_object,
+    unsigned atomic_section_id,
+    const sourcet &source);
 
   // assignment to a variable - lhs must be symbol
   virtual void assignment(
-    const guardt &guard,
+    const exprt &guard,
     const symbol_exprt &ssa_lhs,
     const symbol_exprt &original_lhs,
     const exprt &ssa_full_lhs,
@@ -39,45 +57,45 @@ public:
     
   // declare fresh variable - lhs must be symbol
   virtual void decl(
-    const guardt &guard,
+    const exprt &guard,
     const symbol_exprt &ssa_lhs,
     const symbol_exprt &original_lhs_object,
     const sourcet &source);
 
   // note the death of a variable - lhs must be symbol
   virtual void dead(
-    const guardt &guard,
+    const exprt &guard,
     const symbol_exprt &ssa_lhs,
     const symbol_exprt &original_lhs_object,
     const sourcet &source);
 
   // record a function call
   virtual void function_call(
-    const guardt &guard,
+    const exprt &guard,
     const irep_idt &identifier,
     const sourcet &source);
 
   // record return from a function
   virtual void function_return(
-    const guardt &guard,
+    const exprt &guard,
     const irep_idt &identifier,
     const sourcet &source);
 
   // just record a location
   virtual void location(
-    const guardt &guard,
+    const exprt &guard,
     const sourcet &source);
   
   // output
   virtual void output(
-    const guardt &guard,
+    const exprt &guard,
     const sourcet &source,
     const irep_idt &fmt,
     const std::list<exprt> &args);
   
   // output, formatted
   virtual void output_fmt(
-    const guardt &guard,
+    const exprt &guard,
     const sourcet &source,
     const irep_idt &output_id,
     const irep_idt &fmt,
@@ -85,29 +103,56 @@ public:
   
   // input
   virtual void input(
-    const guardt &guard,
+    const exprt &guard,
     const sourcet &source,
     const irep_idt &input_id,
     const std::list<exprt> &args);
   
   // record an assumption
   virtual void assumption(
-    const guardt &guard,
+    const exprt &guard,
     const exprt &cond,
     const sourcet &source);
 
   // record an assertion
   virtual void assertion(
-    const guardt &guard,
+    const exprt &guard,
     const exprt &cond,
     const std::string &msg,
     const sourcet &source);
 
+  // record a (global) constraint
+  virtual void constraint(
+    const exprt &cond,
+    const std::string &msg,
+    const sourcet &source);
+
+  // record thread spawn
+  virtual void spawn(
+    const exprt &guard,
+    const sourcet &source);
+
+  // record memory barrier
+  virtual void memory_barrier(
+    const exprt &guard,
+    const sourcet &source);
+
+  // record atomic section
+  virtual void atomic_begin(
+    const exprt &guard,
+    unsigned atomic_section_id,
+    const sourcet &source);
+  virtual void atomic_end(
+    const exprt &guard,
+    unsigned atomic_section_id,
+    const sourcet &source);
+
   void convert(prop_convt &prop_conv);
   void convert_assignments(decision_proceduret &decision_procedure) const;
   void convert_decls(prop_convt &prop_conv) const;
   void convert_assumptions(prop_convt &prop_conv);
   void convert_assertions(prop_convt &prop_conv);
+  void convert_constraints(decision_proceduret &decision_procedure) const;
   void convert_guards(prop_convt &prop_conv);
   void convert_io(decision_proceduret &decision_procedure);
 
@@ -122,13 +167,20 @@ public:
     bool is_assert() const          { return type==goto_trace_stept::ASSERT; }
     bool is_assume() const          { return type==goto_trace_stept::ASSUME; }
     bool is_assignment() const      { return type==goto_trace_stept::ASSIGNMENT; }
+    bool is_constraint() const      { return type==goto_trace_stept::CONSTRAINT; }
     bool is_location() const        { return type==goto_trace_stept::LOCATION; }
     bool is_output() const          { return type==goto_trace_stept::OUTPUT; }
     bool is_decl() const            { return type==goto_trace_stept::DECL; }
     bool is_function_call() const   { return type==goto_trace_stept::FUNCTION_CALL; }
     bool is_function_return() const { return type==goto_trace_stept::FUNCTION_RETURN; }
+    bool is_shared_read() const     { return type==goto_trace_stept::SHARED_READ; }
+    bool is_shared_write() const    { return type==goto_trace_stept::SHARED_WRITE; }
+    bool is_spawn() const           { return type==goto_trace_stept::SPAWN; }
+    bool is_memory_barrier() const  { return type==goto_trace_stept::MEMORY_BARRIER; }
+    bool is_atomic_begin() const    { return type==goto_trace_stept::ATOMIC_BEGIN; }
+    bool is_atomic_end() const      { return type==goto_trace_stept::ATOMIC_END; }
     
-    exprt guard_expr;
+    exprt guard;
     literalt guard_literal;
 
     // for ASSIGNMENT and DECL
@@ -137,7 +189,7 @@ public:
     exprt ssa_rhs;
     assignment_typet assignment_type;
     
-    // for ASSUME/ASSERT
+    // for ASSUME/ASSERT/CONSTRAINT
     exprt cond_expr; 
     literalt cond_literal;
     std::string comment;
@@ -150,12 +202,15 @@ public:
     
     // for function call/return
     irep_idt identifier;
+
+    // for SHARED_READ/SHARED_WRITE and ATOMIC_BEGIN/ATOMIC_END
+    unsigned atomic_section_id;
     
     // for slicing
     bool ignore;
     
     SSA_stept():
-      guard_expr(static_cast<const exprt &>(get_nil_irep())),
+      guard(static_cast<const exprt &>(get_nil_irep())),
       ssa_lhs(static_cast<const symbol_exprt &>(get_nil_irep())),
       original_lhs_object(static_cast<const symbol_exprt &>(get_nil_irep())),
       ssa_full_lhs(static_cast<const exprt &>(get_nil_irep())),
@@ -163,6 +218,7 @@ public:
       ssa_rhs(static_cast<const exprt &>(get_nil_irep())),
       cond_expr(static_cast<const exprt &>(get_nil_irep())),
       formatted(false),
+      atomic_section_id(0),
       ignore(false)
     {
     }
@@ -213,6 +269,17 @@ public:
     SSA_steps.clear();
   }
   
+  bool has_threads() const
+  {
+    for(SSA_stepst::const_iterator it=SSA_steps.begin();
+        it!=SSA_steps.end();
+        it++)
+      if(it->source.thread_nr!=0)
+        return true;
+
+    return false;
+  }
+  
 protected:
   const namespacet &ns;
 };
