@@ -11,9 +11,9 @@ Date: June 2011
 #ifndef CPROVER_GOTO_SYMEX_VCD_GOTO_TRACE_H
 #define CPROVER_GOTO_SYMEX_VCD_GOTO_TRACE_H
 
-#include <iostream>
+#include <ostream>
 
-#include <namespace.h>
+#include <util/namespace.h>
 
 #include "goto_trace.h"
 
