@@ -9,10 +9,10 @@ Author: Daniel Kroening, kroening@cs.cmu.edu
 #ifndef CPROVER_LANGUAGE_UI_H
 #define CPROVER_LANGUAGE_UI_H
 
-#include <message.h>
-#include <language_file.h>
-#include <context.h>
-#include <ui_message.h>
+#include <util/message.h>
+#include <util/language_file.h>
+#include <util/symbol_table.h>
+#include <util/ui_message.h>
 
 class cmdlinet;
 
@@ -20,7 +20,7 @@ class language_uit:public messaget
 {
 public:
   language_filest language_files;
-  contextt context;
+  symbol_tablet symbol_table;
   
   language_uit(
     const std::string &program,
