@@ -6,20 +6,20 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#ifndef CPROVER_C_MAIN_H
-#define CPROVER_C_MAIN_H
+#ifndef CPROVER_LINKING_ENTRY_POINT_H
+#define CPROVER_LINKING_ENTRY_POINT_H
 
-#include <context.h>
-#include <message.h>
-#include <std_code.h>
+#include <util/symbol_table.h>
+#include <util/message.h>
+#include <util/location.h>
 
 bool entry_point(
-  contextt &context,
+  symbol_tablet &symbol_table,
   const std::string &standard_main,
   message_handlert &message_handler);
 
 bool static_lifetime_init(
-  contextt &context,
+  symbol_tablet &symbol_table,
   const locationt &location,
   message_handlert &message_handler);
 
