@@ -10,6 +10,6 @@ Author: Daniel Kroening
 #define CPROVER_LINKING_REMOVE_INTERNAL_SYMBOLS_H
 
 void remove_internal_symbols(
-  class contextt &context);
+  class symbol_tablet &symbol_table);
 
 #endif
