@@ -9,25 +9,25 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <cassert>
 #include <cstdlib>
 
-#include <expr_util.h>
-#include <c_misc.h>
-#include <base_type.h>
-#include <arith_tools.h>
-#include <rename.h>
-#include <i2string.h>
-#include <array_name.h>
-#include <config.h>
-#include <std_expr.h>
-#include <cprover_prefix.h>
-#include <pointer_offset_size.h>
-#include <context.h>
-#include <guard.h>
-#include <options.h>
+#include <util/expr_util.h>
+#include <util/c_misc.h>
+#include <util/base_type.h>
+#include <util/arith_tools.h>
+#include <util/rename.h>
+#include <util/i2string.h>
+#include <util/array_name.h>
+#include <util/config.h>
+#include <util/std_expr.h>
+#include <util/cprover_prefix.h>
+#include <util/pointer_offset_size.h>
+#include <util/symbol_table.h>
+#include <util/guard.h>
+#include <util/options.h>
+#include <util/pointer_predicates.h>
 
 #include <ansi-c/c_types.h>
 #include <ansi-c/c_typecast.h>
 
-#include <goto-programs/dynamic_memory.h>
 #include <pointer-analysis/value_set.h>
 
 #include <langapi/language_util.h>
@@ -119,7 +119,7 @@ exprt dereferencet::dereference(
     guardt false_guard=guard;
     
     true_guard.add(if_expr.cond());
-    false_guard.add(gen_not(if_expr.cond()));
+    false_guard.add(not_exprt(if_expr.cond()));
     
     exprt true_case=dereference(if_expr.true_case(), true_guard, mode);
     exprt false_case=dereference(if_expr.true_case(), false_guard, mode);
@@ -195,7 +195,7 @@ exprt dereferencet::dereference(
          pointer, failed_symbol))
     {
       // yes!
-      failure_value=symbol_expr(*failed_symbol);
+      failure_value=failed_symbol->symbol_expr();
       failure_value.set(ID_C_invalid_object, true);
     }
     else
@@ -208,19 +208,19 @@ exprt dereferencet::dereference(
       symbol.type=type;
 
       // make it a lvalue, so we can assign to it
-      symbol.lvalue=true;
+      symbol.is_lvalue=true;
       
       get_new_name(symbol, ns);
 
-      failure_value=symbol_expr(symbol);
+      failure_value=symbol.symbol_expr();
       failure_value.set(ID_C_invalid_object, true);
       
-      new_context.move(symbol);
+      new_symbol_table.move(symbol);
     }
 
     valuet value;
     value.value=failure_value;
-    value.pointer_guard.make_true();    
+    value.pointer_guard=true_exprt();
     values.push_front(value);
   }
   
@@ -315,14 +315,8 @@ void dereferencet::invalid_pointer(
     return;
     
   // constraint that it actually is an invalid pointer
-
-  exprt invalid_pointer_expr("invalid-pointer", bool_typet());
-  invalid_pointer_expr.copy_to_operands(pointer);
-  
-  // produce new guard
-  
   guardt tmp_guard(guard);
-  tmp_guard.add(invalid_pointer_expr);
+  tmp_guard.add(::invalid_pointer(pointer));
   
   dereference_callback.dereference_failure(
     "pointer dereference",
@@ -376,13 +370,11 @@ dereferencet::valuet dereferencet::build_reference_to(
   {
     if(options.get_bool_option("pointer-check"))
     {
-      null_pointer_exprt null_pointer(to_pointer_type(pointer_expr.type()));
-
       guardt tmp_guard(guard);
       
       if(o.offset().is_zero())
       {
-        tmp_guard.add(equal_exprt(pointer_expr, null_pointer));
+        tmp_guard.add(null_pointer(pointer_expr));
 
         dereference_callback.dereference_failure(
           "pointer dereference",
@@ -390,9 +382,7 @@ dereferencet::valuet dereferencet::build_reference_to(
       }
       else
       {
-        exprt pointer_guard(ID_same_object, bool_typet());
-        pointer_guard.copy_to_operands(pointer_expr, null_pointer);
-        tmp_guard.add(pointer_guard);
+        tmp_guard.add(null_object(pointer_expr));
 
         dereference_callback.dereference_failure(
           "pointer dereference",
@@ -407,7 +397,7 @@ dereferencet::valuet dereferencet::build_reference_to(
 
     // the object produced by malloc
     exprt malloc_object=
-      symbol_expr(ns.lookup(CPROVER_PREFIX "malloc_object"));
+      ns.lookup(CPROVER_PREFIX "malloc_object").symbol_expr();
 
     exprt is_malloc_object=exprt(ID_same_object, bool_typet());
     is_malloc_object.copy_to_operands(pointer_expr, malloc_object);
@@ -428,7 +418,7 @@ dereferencet::valuet dereferencet::build_reference_to(
       {
         // check if it is still alive
         guardt tmp_guard(guard);
-        tmp_guard.add(deallocated(ns, pointer_expr));
+        tmp_guard.add(deallocated(pointer_expr, ns));
         dereference_callback.dereference_failure(
           "pointer dereference",
           "dynamic object deallocated", 
@@ -440,18 +430,9 @@ dereferencet::valuet dereferencet::build_reference_to(
         if(!o.offset().is_zero())
         {
           // check lower bound
-          exprt zero=gen_zero(index_type());
-          assert(zero.is_not_nil());
-
-          exprt object_offset=unary_exprt(
-            ID_pointer_offset, pointer_expr, index_type());
-
-          binary_relation_exprt
-            inequality(object_offset, ID_lt, zero);
-
           guardt tmp_guard(guard);
           tmp_guard.add(is_malloc_object);
-          tmp_guard.add(inequality);
+          tmp_guard.add(dynamic_object_lower_bound(pointer_expr));
           dereference_callback.dereference_failure(
             "pointer dereference",
             "dynamic object lower bound", tmp_guard);
@@ -463,34 +444,9 @@ dereferencet::valuet dereferencet::build_reference_to(
           // we check SAME_OBJECT(__CPROVER_malloc_object, p) &&
           //          POINTER_OFFSET(p)+size>__CPROVER_malloc_size
 
-          exprt malloc_size=
-            symbol_expr(ns.lookup(CPROVER_PREFIX "malloc_size"));
-          
-          assert(ns.follow(malloc_object.type()).id()==ID_pointer);
-          
-          exprt object_offset=unary_exprt(
-            ID_pointer_offset, pointer_expr, index_type());
-            
-          mp_integer element_size=
-            pointer_offset_size(ns, dereference_type);
-            
-          if(element_size<0) element_size=0;
-          
-          exprt size=from_integer(element_size, object_offset.type());
-          
-          // need to add size
-          exprt sum=plus_exprt(object_offset, size);
-          
-          if(ns.follow(sum.type())!=
-             ns.follow(malloc_size.type()))
-            sum.make_typecast(malloc_size.type());
-
-          binary_relation_exprt
-            inequality(sum, ID_gt, malloc_size);
-
           guardt tmp_guard(guard);
           tmp_guard.add(is_malloc_object);
-          tmp_guard.add(inequality);
+          tmp_guard.add(dynamic_object_upper_bound(pointer_expr, dereference_type, ns));
           dereference_callback.dereference_failure(
             "pointer dereference",
             "dynamic object upper bound", tmp_guard);
