@@ -9,7 +9,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_POINTER_ANALYSIS_GOTO_PROGRAM_DEREFERENCE_H
 #define CPROVER_POINTER_ANALYSIS_GOTO_PROGRAM_DEREFERENCE_H
 
-#include <namespace.h>
+#include <util/namespace.h>
+
 #include <goto-programs/goto_functions.h>
 
 #include "value_sets.h"
@@ -20,13 +21,13 @@ class goto_program_dereferencet:protected dereference_callbackt
 public:
   goto_program_dereferencet(
     const namespacet &_ns,
-    contextt &_new_context,
+    symbol_tablet &_new_symbol_table,
     const optionst &_options,
     value_setst &_value_sets):
     options(_options),
     ns(_ns),
     value_sets(_value_sets),
-    dereference(_ns, _new_context, _options, *this) { }
+    dereference(_ns, _new_symbol_table, _options, *this) { }
 
   void dereference_program(
     goto_programt &goto_program,
@@ -90,23 +91,23 @@ void dereference(
 
 void remove_pointers(
   goto_programt &goto_program,
-  contextt &context,
+  symbol_tablet &symbol_table,
   value_setst &value_sets);
 
 void remove_pointers(
   goto_functionst &goto_functions,
-  contextt &context,
+  symbol_tablet &symbol_table,
   value_setst &value_sets);
 
 void pointer_checks(
   goto_programt &goto_program,
-  contextt &context,
+  symbol_tablet &symbol_table,
   const optionst &options,
   value_setst &value_sets);
 
 void pointer_checks(
   goto_functionst &goto_functions,
-  contextt &context,
+  symbol_tablet &symbol_table,
   const optionst &options,
   value_setst &value_sets);
 
