@@ -9,7 +9,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_POINTER_ANALYSIS_POINTER_OFFSET_SUM_H
 #define CPROVER_POINTER_ANALYSIS_POINTER_OFFSET_SUM_H
 
-#include <expr.h>
+#include <util/expr.h>
 
 exprt pointer_offset_sum(const exprt &a, const exprt &b);
 
