@@ -6,7 +6,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <std_expr.h>
+#include <util/std_expr.h>
 
 #include "rewrite_index.h"
 
