@@ -6,19 +6,20 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
+#include <iostream>
 
-#include <context.h>
-#include <simplify_expr.h>
-#include <expr_util.h>
-#include <base_type.h>
-#include <std_expr.h>
-#include <i2string.h>
-#include <prefix.h>
-#include <std_code.h>
-#include <arith_tools.h>
-#include <pointer_offset_size.h>
-#include <cprover_prefix.h>
+#include <util/symbol_table.h>
+#include <util/simplify_expr.h>
+#include <util/expr_util.h>
+#include <util/base_type.h>
+#include <util/std_expr.h>
+#include <util/i2string.h>
+#include <util/prefix.h>
+#include <util/std_code.h>
+#include <util/arith_tools.h>
+#include <util/pointer_offset_size.h>
+#include <util/cprover_prefix.h>
 
 #include <langapi/language_util.h>
 #include <ansi-c/c_types.h>
@@ -334,6 +335,69 @@ bool value_sett::make_union(object_mapt &dest, const object_mapt &src) const
 
 /*******************************************************************\
 
+Function: value_sett::eval_pointer_offset
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void value_sett::eval_pointer_offset(
+  exprt &expr,
+  const namespacet &ns) const
+{
+  if(expr.id()==ID_pointer_offset)
+  {
+    assert(expr.operands().size()==1);
+
+    object_mapt reference_set;
+    get_value_set(expr.op0(), reference_set, ns);
+
+    exprt new_expr;
+    new_expr.make_nil();
+
+    const object_map_dt &object_map=reference_set.read();
+    for(object_map_dt::const_iterator
+        it=object_map.begin();
+        it!=object_map.end();
+        it++)
+      if(!it->second.offset_is_set)
+        return;
+      else
+      {
+        const exprt &object=object_numbering[it->first];
+        mp_integer ptr_offset=compute_pointer_offset(ns, object);
+        exprt offset=from_integer(it->second.offset, index_type());
+
+        if(ptr_offset<0 || offset.id()!=ID_constant)
+          return;
+
+        plus_exprt offset_sum(
+          offset,
+          from_integer(ptr_offset, index_type()));
+        simplify(offset_sum, ns);
+
+        if(new_expr.is_not_nil() && offset_sum!=new_expr)
+          return;
+
+        new_expr=offset_sum;
+      }
+
+    if(new_expr.is_not_nil())
+      expr.swap(new_expr);
+  }
+  else
+  {
+    Forall_operands(it, expr)
+      eval_pointer_offset(*it, ns);
+  }
+}
+
+/*******************************************************************\
+
 Function: value_sett::get_value_set
 
   Inputs:
@@ -460,6 +524,21 @@ void value_sett::get_value_set_rec(
       valuest::const_iterator v_it=
         values.find(expr.get_string(ID_identifier)+suffix);
 
+      // try first component name as suffix if not yet found
+      if(v_it==values.end() &&
+          (expr_type.id()==ID_struct ||
+           expr_type.id()==ID_union))
+      {
+        const struct_union_typet &struct_union_type=
+          to_struct_union_type(expr_type);
+
+        const std::string first_component_name=
+          struct_union_type.components().front().get_string(ID_name);
+
+        v_it=values.find(
+            expr.get_string(ID_identifier)+"."+first_component_name+suffix);
+      }
+
       // not found? try without suffix
       if(v_it==values.end())
         v_it=values.find(expr.get_string(ID_identifier));
@@ -542,7 +621,7 @@ void value_sett::get_value_set_rec(
             expr_type.id()==ID_signedbv)
     {
       // an integer constant got turned into a pointer
-      insert(dest, exprt(ID_integer_address, uchar_type()));
+      insert(dest, exprt(ID_integer_address, unsigned_char_type()));
     }
     else
       insert(dest, exprt(ID_unknown, original_type));
@@ -578,13 +657,13 @@ void value_sett::get_value_set_rec(
         if(tmp.read().size()==0)
         {
           // if not, throw in integer
-          insert(dest, exprt(ID_integer_address, uchar_type()));        
+          insert(dest, exprt(ID_integer_address, unsigned_char_type()));        
         }
         else if(tmp.read().size()==1 &&
                 object_numbering[tmp.read().begin()->first].id()==ID_unknown)
         {
           // if not, throw in integer
-          insert(dest, exprt(ID_integer_address, uchar_type()));        
+          insert(dest, exprt(ID_integer_address, unsigned_char_type()));        
         }
         else
         {
@@ -827,8 +906,47 @@ void value_sett::get_value_set_rec(
     if(expr.operands().size()!=2)
       throw "byte_extract takes two operands";
       
-    // we just pass through
-    get_value_set_rec(expr.op0(), dest, suffix, original_type, ns);
+    bool found=false;
+
+    exprt op1=expr.op1();
+    eval_pointer_offset(op1, ns);
+    simplify(op1, ns);
+
+    const typet &op0_type=ns.follow(expr.op0().type());
+    if(op1.id()==ID_constant && op0_type.id()==ID_struct)
+    {
+      const struct_typet &struct_type=to_struct_type(op0_type);
+
+      for(struct_union_typet::componentst::const_iterator
+          c_it=struct_type.components().begin();
+          !found && c_it!=struct_type.components().end();
+          c_it++)
+      {
+        const irep_idt &name=c_it->get_name();
+
+        exprt offset=member_offset_expr(struct_type, name, ns);
+        if(offset.is_nil())
+          continue;
+
+        equal_exprt eq(offset, op1);
+        simplify(eq, ns);
+
+        if(!eq.is_true())
+          continue;
+
+        found=true;
+
+        member_exprt member(c_it->type());
+        member.set_component_name(name);
+        member.struct_op()=expr.op0();
+
+        get_value_set_rec(member, dest, suffix, original_type, ns);
+      }
+    }
+
+    if(!found)
+      // we just pass through
+      get_value_set_rec(expr.op0(), dest, suffix, original_type, ns);
   }
   else if(expr.id()==ID_byte_update_little_endian ||
           expr.id()==ID_byte_update_big_endian)
@@ -1649,10 +1767,15 @@ void value_sett::apply_code(
   {
     guard(to_code_assume(code).op0(), ns);
   }
-  else if(statement==ID_user_specified_predicate || statement==ID_user_specified_parameter_predicates || statement == ID_user_specified_return_predicates)
+  else if(statement==ID_user_specified_predicate ||
+          statement==ID_user_specified_parameter_predicates ||
+          statement==ID_user_specified_return_predicates)
   {
     // doesn't do anything
   }
+  else if(statement==ID_fence)
+  {
+  }
   else
   {
     std::cerr << code.pretty() << std::endl;
@@ -1694,7 +1817,7 @@ void value_sett::guard(
   {
     assert(expr.operands().size()==1);
 
-    dynamic_object_exprt dynamic_object(uchar_type());
+    dynamic_object_exprt dynamic_object(unsigned_char_type());
     //dynamic_object.instance()=from_integer(location_number, typet(ID_natural));
     dynamic_object.valid()=true_exprt();
 
@@ -1722,13 +1845,13 @@ exprt value_sett::make_member(
   const irep_idt &component_name,
   const namespacet &ns)
 {
-  const struct_union_typet &struct_type=
-    to_struct_type(ns.follow(src.type()));
+  const struct_union_typet &struct_union_type=
+    to_struct_union_type(ns.follow(src.type()));
 
   if(src.id()==ID_struct ||
      src.id()==ID_constant)
   {
-    unsigned no=struct_type.component_number(component_name);
+    unsigned no=struct_union_type.component_number(component_name);
     assert(no<src.operands().size());
     return src.operands()[no];
   }
@@ -1754,7 +1877,7 @@ exprt value_sett::make_member(
   }
 
   // give up
-  typet subtype=struct_type.component_type(component_name);
+  typet subtype=struct_union_type.component_type(component_name);
   member_exprt member_expr(subtype);
   member_expr.op0()=src;
   member_expr.set_component_name(component_name);
