@@ -6,7 +6,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <std_code.h>
+#include <util/std_code.h>
 
 #include "value_set_domain.h"
 
