@@ -9,7 +9,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_POINTER_ANALYSIS_VALUE_SET_DOMAIN_H
 #define CPROVER_POINTER_ANALYSIS_VALUE_SET_DOMAIN_H
 
-#include <goto-programs/static_analysis.h>
+#include <analyses/static_analysis.h>
 
 #include "value_set.h"
 
@@ -20,7 +20,7 @@ public:
 
   // overloading  
 
-  virtual bool merge(const value_set_domaint &other)
+  inline bool merge(const value_set_domaint &other, locationt to)
   {
     return value_set.make_union(other.value_set);
   }
