@@ -6,17 +6,18 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
+#include <iostream>
 
-#include <context.h>
-#include <simplify_expr.h>
-#include <expr_util.h>
-#include <base_type.h>
-#include <std_expr.h>
-#include <i2string.h>
-#include <prefix.h>
-#include <std_code.h>
-#include <arith_tools.h>
+#include <util/symbol_table.h>
+#include <util/simplify_expr.h>
+#include <util/expr_util.h>
+#include <util/base_type.h>
+#include <util/std_expr.h>
+#include <util/i2string.h>
+#include <util/prefix.h>
+#include <util/std_code.h>
+#include <util/arith_tools.h>
 
 #include <langapi/language_util.h>
 #include <ansi-c/c_types.h>
@@ -193,7 +194,7 @@ void value_set_fit::flatten_rec(
   std::cout << "FLATTEN_REC: " << e.identifier << e.suffix << std::endl;
   #endif
 
-  std::string identifier = e.identifier.as_string();
+  std::string identifier = id2string(e.identifier);
   assert(seen.find(identifier + e.suffix)==seen.end());
   
   bool generalize_index = false; 
@@ -1551,7 +1552,7 @@ void value_set_fit::do_function_call(
 
   for(unsigned i=0; i<arguments.size(); i++)
   {
-    const std::string identifier="value_set::" + function.as_string() + "::" +  
+    const std::string identifier="value_set::" + id2string(function) + "::" +  
                                  "argument$"+i2string(i);
     add_var(identifier, "");
     exprt dummy_lhs=symbol_exprt(identifier, arguments[i].type());
@@ -1573,7 +1574,7 @@ void value_set_fit::do_function_call(
     add_var(identifier, "");
   
     const exprt v_expr=
-      symbol_exprt("value_set::" + function.as_string() + "::" + 
+      symbol_exprt("value_set::" + id2string(function) + "::" + 
                    "argument$"+i2string(i), it->type());
     
     exprt actual_lhs=symbol_exprt(identifier, it->type());
