@@ -7,17 +7,17 @@ Author: Daniel Kroening, kroening@kroening.com
 \*******************************************************************/
 
 #include <cassert>
-#include <ctype.h>
+#include <cctype>
 #include <cstdlib>
 
-#include <arith_tools.h>
-#include <std_types.h>
-#include <std_expr.h>
-#include <config.h>
-#include <i2string.h>
-#include <expr_util.h>
-#include <find_symbols.h>
-#include <pointer_offset_size.h>
+#include <util/arith_tools.h>
+#include <util/std_types.h>
+#include <util/std_expr.h>
+#include <util/config.h>
+#include <util/i2string.h>
+#include <util/expr_util.h>
+#include <util/find_symbols.h>
+#include <util/pointer_offset_size.h>
 
 #include <ansi-c/string_constant.h>
 
@@ -598,10 +598,7 @@ void cvc_convt::convert_cvc_expr(const exprt &expr)
       cvc_prop.out << expr.get(ID_value);
     }
     else
-    {
-      std::cerr << expr.pretty() << std::endl;
       throw "unknown constant: "+expr.type().id_string();
-    }
   }
   else if(expr.id()==ID_concatenation || 
           expr.id()==ID_bitand ||
