@@ -6,11 +6,15 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <string.h>
-#include <stdlib.h>
-#include <assert.h>
+#include <cstring>
+#include <cstdlib>
+#include <cassert>
 
-#ifndef _WIN32
+#if defined(__linux__) || \
+    defined(__FreeBSD_kernel__) || \
+    defined(__GNU__) || \
+    defined(__CYGWIN__) || \
+    defined(__MACH__)
 #include <unistd.h>
 #endif
 
@@ -19,8 +23,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #define getpid _getpid
 #endif
 
-#include <i2string.h>
-#include <prefix.h>
+#include <util/i2string.h>
+#include <util/prefix.h>
 
 #include "cvc_dec.h"
 
@@ -141,10 +145,11 @@ void cvc_dect::read_assert(std::istream &in, std::string &line)
     if(pos2==std::string::npos) return;    
     
     std::string value=std::string(line, pos, pos2-pos);
-    
+
+    #if 0    
     std::cout << ">" << identifier << "< = >" << value << "<";
-    
     std::cout << std::endl;
+    #endif
   }
   else
   {
