@@ -12,24 +12,22 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <set>
 #include <iostream>
 
-#include <symbol.h>
-#include <mp_arith.h>
-#include <i2string.h>
-#include <arith_tools.h>
-#include <replace_expr.h>
-#include <std_types.h>
-#include <prefix.h>
-#include <std_expr.h>
-#include <threeval.h>
+#include <util/symbol.h>
+#include <util/mp_arith.h>
+#include <util/i2string.h>
+#include <util/arith_tools.h>
+#include <util/replace_expr.h>
+#include <util/std_types.h>
+#include <util/prefix.h>
+#include <util/std_expr.h>
+#include <util/threeval.h>
 
 #include <ansi-c/string_constant.h>
 
 #include "boolbv.h"
 #include "boolbv_type.h"
 
-#ifdef HAVE_FLOATBV
 #include "../floatbv/float_utils.h"
-#endif
 
 //#define DEBUG
 
@@ -218,6 +216,8 @@ void boolbvt::convert_bitvector(const exprt &expr, bvt &bv)
     return convert_member(to_member_expr(expr), bv);
   else if(expr.id()==ID_with)
     return convert_with(expr, bv);
+  else if(expr.id()==ID_update)
+    return convert_update(expr, bv);
   else if(expr.id()==ID_width)
   {
     unsigned result_width=boolbv_width(expr.type());
@@ -276,6 +276,11 @@ void boolbvt::convert_bitvector(const exprt &expr, bvt &bv)
     return convert_mod(expr, bv);
   else if(expr.id()==ID_shl || expr.id()==ID_ashr || expr.id()==ID_lshr)
     return convert_shift(expr, bv);
+  else if(expr.id()==ID_floatbv_plus || expr.id()==ID_floatbv_minus ||
+          expr.id()==ID_floatbv_mult || expr.id()==ID_floatbv_div)
+    return convert_floatbv_op(expr, bv);
+  else if(expr.id()==ID_floatbv_typecast)
+    return convert_floatbv_typecast(expr, bv);
   else if(expr.id()==ID_concatenation)
     return convert_concatenation(expr, bv);
   else if(expr.id()==ID_replication)
@@ -335,7 +340,6 @@ void boolbvt::convert_bitvector(const exprt &expr, bvt &bv)
     
     return;
   }
-  #ifdef HAVE_FLOATBV
   else if(expr.id()==ID_float_debug1 ||
           expr.id()==ID_float_debug2)
   {
@@ -350,7 +354,6 @@ void boolbvt::convert_bitvector(const exprt &expr, bvt &bv)
       float_utils.debug2(bv0, bv1);
     return;
   }
-  #endif
 
   return conversion_failed(expr, bv);
 }
@@ -599,11 +602,9 @@ literalt boolbvt::convert_rest(const exprt &expr)
     
     if(expr.op0().type().id()==ID_floatbv)
     {
-      #ifdef HAVE_FLOATBV
       float_utilst float_utils(prop);
       float_utils.spec=to_floatbv_type(expr.op0().type());
       return float_utils.is_NaN(bv);
-      #endif
     }
     else if(expr.op0().type().id()==ID_fixedbv)
       return const_literal(false);
@@ -617,13 +618,11 @@ literalt boolbvt::convert_rest(const exprt &expr)
     
     if(expr.op0().type().id()==ID_floatbv)
     {
-      #ifdef HAVE_FLOATBV
       float_utilst float_utils(prop);
       float_utils.spec=to_floatbv_type(expr.op0().type());
       return prop.land(
         prop.lnot(float_utils.is_infinity(bv)),
         prop.lnot(float_utils.is_NaN(bv)));
-      #endif
     }
     else if(expr.op0().type().id()==ID_fixedbv)
       return const_literal(true);
@@ -637,11 +636,9 @@ literalt boolbvt::convert_rest(const exprt &expr)
     
     if(expr.op0().type().id()==ID_floatbv)
     {
-      #ifdef HAVE_FLOATBV
       float_utilst float_utils(prop);
       float_utils.spec=to_floatbv_type(expr.op0().type());
       return float_utils.is_infinity(bv);
-      #endif
     }
     else if(expr.op0().type().id()==ID_fixedbv)
       return const_literal(false);
@@ -655,11 +652,9 @@ literalt boolbvt::convert_rest(const exprt &expr)
     
     if(expr.op0().type().id()==ID_floatbv)
     {
-      #ifdef HAVE_FLOATBV
       float_utilst float_utils(prop);
       float_utils.spec=to_floatbv_type(expr.op0().type());
       return float_utils.is_normal(bv);
-      #endif
     }
     else if(expr.op0().type().id()==ID_fixedbv)
       return const_literal(true);
