@@ -13,9 +13,9 @@ Author: Daniel Kroening, kroening@kroening.com
 // convert expression to boolean formula
 //
 
-#include <hash_cont.h>
-#include <mp_arith.h>
-#include <expr.h>
+#include <util/hash_cont.h>
+#include <util/mp_arith.h>
+#include <util/expr.h>
 
 #include "bv_utils.h"
 #include "boolbv_width.h"
@@ -139,8 +139,11 @@ protected:
   virtual void convert_mult(const exprt &expr, bvt &bv);
   virtual void convert_div(const exprt &expr, bvt &bv);
   virtual void convert_mod(const exprt &expr, bvt &bv);
+  virtual void convert_floatbv_op(const exprt &expr, bvt &bv);
+  virtual void convert_floatbv_typecast(const exprt &expr, bvt &bv);
   virtual void convert_member(const member_exprt &expr, bvt &bv);
   virtual void convert_with(const exprt &expr, bvt &bv);
+  virtual void convert_update(const exprt &expr, bvt &bv);
   virtual void convert_case(const exprt &expr, bvt &bv);
   virtual void convert_cond(const exprt &expr, bvt &bv);
   virtual void convert_shift(const exprt &expr, bvt &bv);
@@ -192,6 +195,14 @@ protected:
     const bvt &prev_bv,
     bvt &next_bv);
 
+  void convert_update_rec(
+    const exprt::operandst &designator,
+    unsigned d,
+    const typet &type,
+    unsigned offset,
+    const exprt &new_value,
+    bvt &bv);
+
   virtual exprt bv_get_unbounded_array(
     const irep_idt &identifier,
     const array_typet &type) const;
