@@ -34,8 +34,8 @@ void boolbvt::convert_case(const exprt &expr, bvt &bv)
   bv.resize(width);
 
   // make it free variables
-  for(unsigned i=0; i<bv.size(); i++)
-    bv[i]=prop.new_variable();
+  Forall_literals(it, bv)
+    *it=prop.new_variable();
 
   if(operands.size()<3)
     throw "case takes at least three operands";
@@ -50,9 +50,7 @@ void boolbvt::convert_case(const exprt &expr, bvt &bv)
 
   forall_operands(it, expr)
   {
-    bvt op;
-
-    convert_bv(*it, op);
+    bvt op=convert_bv(*it);
 
     switch(what)
     {
