@@ -6,8 +6,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <arith_tools.h>
-#include <i2string.h>
+#include <util/arith_tools.h>
+#include <util/i2string.h>
 
 #include "boolbv.h"
 
