@@ -8,9 +8,9 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <cassert>
 
-#include <arith_tools.h>
-#include <std_expr.h>
-#include <threeval.h>
+#include <util/arith_tools.h>
+#include <util/std_expr.h>
+#include <util/threeval.h>
 
 #include "boolbv.h"
 #include "boolbv_type.h"
@@ -210,6 +210,22 @@ exprt boolbvt::bv_get_rec(
           value.operands()[i]=
             bv_get_rec(bv, unknown, i*sub_width, subtype);
             
+        return value;
+      }
+    }
+    else if(type.id()==ID_complex)
+    {
+      const typet &subtype=ns.follow(type.subtype());
+      unsigned sub_width=boolbv_width(subtype);
+
+      if(sub_width!=0 && width==sub_width*2)
+      {
+        exprt value(ID_complex, type);
+        value.operands().resize(2);
+
+        value.op0()=bv_get_rec(bv, unknown, 0*sub_width, subtype);
+        value.op1()=bv_get_rec(bv, unknown, 1*sub_width, subtype);
+            
         return value;
       }
     }
