@@ -6,13 +6,11 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <std_types.h>
+#include <util/std_types.h>
 
 #include "boolbv.h"
 
-#ifdef HAVE_FLOATBV
 #include "../floatbv/float_utils.h"
-#endif
 
 /*******************************************************************\
 
@@ -82,7 +80,6 @@ void boolbvt::convert_mult(const exprt &expr, bvt &bv)
   }
   else if(expr.type().id()==ID_floatbv)
   {
-    #ifdef HAVE_FLOATBV
     if(op0.type()!=expr.type())
       throw "multiplication with mixed types";
     
@@ -109,7 +106,6 @@ void boolbvt::convert_mult(const exprt &expr, bvt &bv)
     }
     
     return;
-    #endif
   }
   else if(expr.type().id()==ID_unsignedbv ||
           expr.type().id()==ID_signedbv)
