@@ -22,11 +22,13 @@ Function: boolbvt::convert_shift
 
 void boolbvt::convert_shift(const exprt &expr, bvt &bv)
 {
-  if(expr.type().id()!=ID_unsignedbv &&
-     expr.type().id()!=ID_signedbv &&
-     expr.type().id()!=ID_floatbv &&
-     expr.type().id()!=ID_pointer &&
-     expr.type().id()!=ID_bv)
+  const irep_idt &type_id=expr.type().id();
+
+  if(type_id!=ID_unsignedbv &&
+     type_id!=ID_signedbv &&
+     type_id!=ID_floatbv &&
+     type_id!=ID_pointer &&
+     type_id!=ID_bv)
     return conversion_failed(expr, bv);
 
   unsigned width=boolbv_width(expr.type());
@@ -37,10 +39,8 @@ void boolbvt::convert_shift(const exprt &expr, bvt &bv)
   if(expr.operands().size()!=2)
     throw "shifting takes two operands";
 
-  bvt op, dist;
-
-  convert_bv(expr.op0(), op);
-  convert_bv(expr.op1(), dist);
+  const bvt &op=convert_bv(expr.op0());
+  const bvt &dist=convert_bv(expr.op1());
 
   if(op.size()!=width)
     throw "convert_shift: unexpected operand width";
