@@ -6,16 +6,14 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
-#include <std_types.h>
+#include <util/std_types.h>
 
 #include "boolbv.h"
 #include "boolbv_type.h"
 
-#ifdef HAVE_FLOATBV
 #include "../floatbv/float_utils.h"
-#endif
 
 /*******************************************************************\
 
@@ -98,7 +96,25 @@ bool boolbvt::type_conversion(
       return false;
     }
   }
-
+  
+  if(src_type.id()==ID_complex)
+  {
+    assert(dest_type.id()!=ID_complex);
+    if(dest_type.id()==ID_signedbv ||
+       dest_type.id()==ID_unsignedbv ||
+       dest_type.id()==ID_floatbv ||
+       dest_type.id()==ID_fixedbv ||
+       dest_type.id()==ID_c_enum ||
+       dest_type.id()==ID_bool)
+    {
+      // A cast from complex x to real T
+      // is (T) __real__ x.
+      bvt tmp_src(src);
+      tmp_src.resize(src.size()/2); // cut off imag part
+      return type_conversion(src_type.subtype(), tmp_src, dest_type, dest);
+    }
+  }
+  
   switch(dest_bvtype)
   {
   case IS_RANGE:
@@ -139,7 +155,6 @@ bool boolbvt::type_conversion(
     
   case IS_FLOAT: // to float
     {
-      #ifdef HAVE_FLOATBV
       float_utilst float_utils(prop);
       
       switch(src_bvtype)
@@ -185,7 +200,6 @@ bool boolbvt::type_conversion(
           return false;
         }
       }
-      #endif
     }
     break;
 
@@ -293,7 +307,6 @@ bool boolbvt::type_conversion(
     {
     case IS_FLOAT: // float to integer
       {
-        #ifdef HAVE_FLOATBV
         // note that float to int conversion in ANSI-C is hardwired
         // to ROUND TO ZERO, also known as truncate.
         float_utilst float_utils(prop);
@@ -301,9 +314,6 @@ bool boolbvt::type_conversion(
         float_utils.spec=to_floatbv_type(src_type);
         dest=float_utils.to_integer(src, dest_width, dest_bvtype!=IS_UNSIGNED);
         return false;
-        #else
-        return true;
-        #endif
       }
      
     case IS_FIXED: // fixed to integer
