@@ -6,7 +6,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <arith_tools.h>
+#include <util/arith_tools.h>
 
 #include "boolbv.h"
 
