@@ -9,7 +9,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_BV_UTILS_H
 #define CPROVER_BV_UTILS_H
 
-#include <mp_arith.h>
+#include <util/mp_arith.h>
 
 #include <solvers/prop/prop.h>
 
@@ -159,6 +159,8 @@ protected:
     const bvt &op0, const bvt &op1);
 
   bvt cond_negate_no_overflow(const bvt &bv, const literalt cond);
+
+  bvt wallace_tree(const std::vector<bvt> &pps);
 };
 
 #endif
