@@ -6,11 +6,11 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <expr.h>
-#include <std_types.h>
-#include <std_expr.h>
-#include <arith_tools.h>
-#include <pointer_offset_size.h>
+#include <util/expr.h>
+#include <util/std_types.h>
+#include <util/std_expr.h>
+#include <util/arith_tools.h>
+#include <util/pointer_offset_size.h>
 
 #include "flatten_byte_operators.h"
 
@@ -118,9 +118,12 @@ exprt flatten_byte_extract(
       }
 
       // the new offset is width%offset
-      exprt new_offset=
-        (element_width==1)?from_integer(0, offset_type):
-        mod_exprt(offset, from_integer(element_width, offset_type));
+      exprt new_offset;
+      
+      if(element_width==1)
+        new_offset=from_integer(0, offset_type);
+      else
+        new_offset=mod_exprt(offset, from_integer(element_width, offset_type));
 
       // build new byte-extract expression
       exprt tmp(src.id(), src.type());
