@@ -12,7 +12,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 /*******************************************************************\
 
-Function: aig_propt::set_l
+Function: aig_prop_baset::land
 
   Inputs:
 
@@ -22,100 +22,7 @@ Function: aig_propt::set_l
 
 \*******************************************************************/
 
-#if 0
-void aig_propt::set_l(propt &dest, literalt a, literalt l)
-{
-  unsigned v=a.var_no();
-  bool sign=a.sign();
-  
-  assert(v<vars.size());
-  aig_nodet &var=vars[v];
-
-  assert(!var.l_is_set);
-
-  var.l_is_set=true;
-  var.l=sign?dest.lnot(l):l;
-}
-#endif
-
-/*******************************************************************\
-
-Function: aig_propt::convert_prop
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-#if 0
-literalt aig_propt::convert_prop(propt &dest, literalt a)
-{
-  if(a==const_literal(false)) return dest.constant(false);
-  if(a==const_literal(true)) return dest.constant(true);
-
-  unsigned v=a.var_no();
-  bool sign=a.sign();
-  
-  assert(v<vars.size());
-  aig_nodet &var=vars[v];
-
-  if(!var.l_is_set)
-  {
-    assert(var.is_and);
-
-    var.l_is_set=true;
-    var.l=dest.land(convert_prop(dest, var.a),
-                    convert_prop(dest, var.b));
-  }
-
-  return sign?dest.lnot(var.l):var.l;
-}
-#endif
-
-/*******************************************************************\
-
-Function: aig_propt::can_convert
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-#if 0
-bool aig_propt::can_convert(literalt a) const
-{
-  if(a==const_literal(false) || a==const_literal(true)) return true;
-
-  unsigned v=a.var_no();
-  
-  assert(v<vars.size());
-  const aig_nodet &var=vars[v];
-
-  if(var.l_is_set) return true;
-  
-  return var.is_and;
-}
-#endif
-
-/*******************************************************************\
-
-Function: aig_propt::land
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-literalt aig_propt::land(const bvt &bv)
+literalt aig_prop_baset::land(const bvt &bv)
 {
   literalt literal=const_literal(true);
 
@@ -127,7 +34,7 @@ literalt aig_propt::land(const bvt &bv)
   
 /*******************************************************************\
 
-Function: aig_propt::lor
+Function: aig_prop_baset::lor
 
   Inputs:
 
@@ -137,7 +44,7 @@ Function: aig_propt::lor
 
 \*******************************************************************/
 
-literalt aig_propt::lor(const bvt &bv)
+literalt aig_prop_baset::lor(const bvt &bv)
 {
   literalt literal=const_literal(false);
 
@@ -149,7 +56,7 @@ literalt aig_propt::lor(const bvt &bv)
   
 /*******************************************************************\
 
-Function: aig_propt::lxor
+Function: aig_prop_baset::lxor
 
   Inputs:
 
@@ -159,7 +66,7 @@ Function: aig_propt::lxor
 
 \*******************************************************************/
 
-literalt aig_propt::lxor(const bvt &bv)
+literalt aig_prop_baset::lxor(const bvt &bv)
 {
   literalt literal=const_literal(false);
 
@@ -171,7 +78,7 @@ literalt aig_propt::lxor(const bvt &bv)
   
 /*******************************************************************\
 
-Function: aig_propt::land
+Function: aig_prop_baset::land
 
   Inputs:
 
@@ -181,28 +88,22 @@ Function: aig_propt::land
 
 \*******************************************************************/
 
-literalt aig_propt::land(literalt a, literalt b)
+literalt aig_prop_baset::land(literalt a, literalt b)
 {
-  if(a==const_literal(true)) return b;
-  if(b==const_literal(true)) return a;
-  if(a==const_literal(false)) return const_literal(false);
-  if(b==const_literal(false)) return const_literal(false);
+  if(a.is_true()) return b;
+  if(b.is_true()) return a;
+  if(a.is_false()) return a;
+  if(b.is_false()) return b;
 
-  if(a==lnot(b)) return const_literal(false);
-  if(a == b) return a;
-  literalt l=new_variable();
-  unsigned n=l.var_no();
-
-  assert(n<dest.number_of_nodes());
-  aig_nodet &node=dest.get_node(n);
-  node.make_and(a, b);
-
-  return l;
+  if(a==neg(b)) return const_literal(false);
+  if(a==b) return a;
+  
+  return dest.new_and_node(a, b);
 }
 
 /*******************************************************************\
 
-Function: aig_propt::lor
+Function: aig_prop_baset::lor
 
   Inputs:
 
@@ -212,14 +113,14 @@ Function: aig_propt::lor
 
 \*******************************************************************/
 
-literalt aig_propt::lor(literalt a, literalt b)
+literalt aig_prop_baset::lor(literalt a, literalt b)
 {
-  return lnot(land(lnot(a), lnot(b)));
+  return neg(land(neg(a), neg(b))); // De Morgan's
 }
 
 /*******************************************************************\
 
-Function: aig_propt::lnot
+Function: aig_prop_baset::lnot
 
   Inputs:
 
@@ -229,15 +130,14 @@ Function: aig_propt::lnot
 
 \*******************************************************************/
 
-literalt aig_propt::lnot(literalt a)
+literalt aig_prop_baset::lnot(literalt a)
 {
-  a.invert();
-  return a;
+  return neg(a);
 }
 
 /*******************************************************************\
 
-Function: aig_propt::lxor
+Function: aig_prop_baset::lxor
 
   Inputs:
 
@@ -247,22 +147,23 @@ Function: aig_propt::lxor
 
 \*******************************************************************/
 
-literalt aig_propt::lxor(literalt a, literalt b)
+literalt aig_prop_baset::lxor(literalt a, literalt b)
 {
-  if(a==const_literal(false)) return b;
-  if(b==const_literal(false)) return a;
-  if(a==const_literal(true)) return lnot(b);
-  if(b==const_literal(true)) return lnot(a);
+  if(a.is_false()) return b;
+  if(b.is_false()) return a;
+  if(a.is_true()) return neg(b);
+  if(b.is_true()) return neg(a);
 
   if(a==b) return const_literal(false);
-  if(a==lnot(b)) return const_literal(true);
+  if(a==neg(b)) return const_literal(true);
 
-  return lor(land(a, lnot(b)), land(lnot(a), b));
+  // This produces up to three nodes!
+  return lor(land(a, neg(b)), land(neg(a), b));
 }
 
 /*******************************************************************\
 
-Function: aig_propt::lnand
+Function: aig_prop_baset::lnand
 
   Inputs:
 
@@ -272,14 +173,14 @@ Function: aig_propt::lnand
 
 \*******************************************************************/
 
-literalt aig_propt::lnand(literalt a, literalt b)
+literalt aig_prop_baset::lnand(literalt a, literalt b)
 {
-  return lnot(land(a, b));
+  return land(a, b).negation();
 }
 
 /*******************************************************************\
 
-Function: aig_propt::lnor
+Function: aig_prop_baset::lnor
 
   Inputs:
 
@@ -289,14 +190,14 @@ Function: aig_propt::lnor
 
 \*******************************************************************/
 
-literalt aig_propt::lnor(literalt a, literalt b)
+literalt aig_prop_baset::lnor(literalt a, literalt b)
 {
-  return lnot(lor(a, b));
+  return lor(a, b).negation();
 }
 
 /*******************************************************************\
 
-Function: aig_propt::lequal
+Function: aig_prop_baset::lequal
 
   Inputs:
 
@@ -306,14 +207,14 @@ Function: aig_propt::lequal
 
 \*******************************************************************/
 
-literalt aig_propt::lequal(literalt a, literalt b)
+literalt aig_prop_baset::lequal(literalt a, literalt b)
 {
-  return lnot(lxor(a, b));
+  return lxor(a, b).negation();
 }
 
 /*******************************************************************\
 
-Function: aig_propt::limplies
+Function: aig_prop_baset::limplies
 
   Inputs:
 
@@ -323,14 +224,14 @@ Function: aig_propt::limplies
 
 \*******************************************************************/
 
-literalt aig_propt::limplies(literalt a, literalt b)
+literalt aig_prop_baset::limplies(literalt a, literalt b)
 {
-  return lor(lnot(a), b);
+  return lor(neg(a), b);
 }
 
 /*******************************************************************\
 
-Function: aig_propt::lselect
+Function: aig_prop_baset::lselect
 
   Inputs:
 
@@ -340,12 +241,148 @@ Function: aig_propt::lselect
 
 \*******************************************************************/
 
-literalt aig_propt::lselect(literalt a, literalt b, literalt c)
+literalt aig_prop_baset::lselect(literalt a, literalt b, literalt c)
 {  // a?b:c = (a AND b) OR (/a AND c)
-  if(a==const_literal(true)) return b;
-  if(a==const_literal(false)) return c;
+  if(a.is_true()) return b;
+  if(a.is_false()) return c;
   if(b==c) return b;
 
-  return lor(land(a, b), land(lnot(a), c));
+  return lor(land(a, b), land(neg(a), c));
 }
 
+/*******************************************************************\
+
+Function: aig_prop_constraintt::lcnf
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void aig_prop_constraintt::lcnf(const bvt &clause)
+{
+  l_set_to_true(lor(clause));
+}
+  
+/*******************************************************************\
+
+Function: aig_prop_baset::set_equal
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void aig_prop_baset::set_equal(literalt a, literalt b)
+{
+  // we produce two constraints:
+  // a|!b   !a|b
+
+  l_set_to_true(lor(pos(a), neg(b)));
+  l_set_to_true(lor(neg(a), pos(b)));
+}
+
+/*******************************************************************\
+
+Function: aig_prop_solvert::l_get
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+tvt aig_prop_solvert::l_get(literalt a) const
+{
+  return solver.l_get(a);
+}
+
+/*******************************************************************\
+
+Function: aig_prop_solvert::prop_solve
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+propt::resultt aig_prop_solvert::prop_solve()
+{
+  status() << "converting AIG, "
+           << aig.nodes.size() << " nodes" << eom;
+  convert_aig();
+
+  return solver.prop_solve();
+}
+
+/*******************************************************************\
+
+Function: aig_prop_solvert::convert_aig
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void aig_prop_solvert::convert_aig()
+{
+  // 1. Do variables
+  
+  while(solver.no_variables()<=aig.nodes.size())
+    solver.new_variable();
+
+  // 2. Do nodes
+
+  for(unsigned n=0; n<aig.nodes.size(); n++)
+  {
+    const aigt::nodet &node=aig.nodes[n];
+
+    if(node.is_and())
+    {
+      literalt o=literalt(n, false);
+      literalt a=node.a;
+      literalt b=node.b;
+      
+      bvt lits(2);
+   
+      lits[0]=pos(a);
+      lits[1]=neg(o);
+      solver.lcnf(lits);
+
+      lits[0]=pos(b);
+      lits[1]=neg(o);
+      solver.lcnf(lits);
+
+      lits.clear();
+      lits.reserve(3);
+      lits.push_back(neg(a));
+      lits.push_back(neg(b));
+      lits.push_back(pos(o));
+      solver.lcnf(lits);
+    }
+  }
+  
+  // 3. Do constraints
+  
+  for(constraintst::const_iterator
+      c_it=constraints.begin();
+      c_it!=constraints.end();
+      c_it++)
+  {
+    solver.l_set_to(*c_it, true);
+  }
+}
