@@ -10,6 +10,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #define CPROVER_PROPDEC_LITERAL_H
 
 #include <vector>
+#include <ostream>
 
 // a pair of a variable number and a sign, encoded as follows:
 //
@@ -150,9 +151,7 @@ public:
 
   friend inline literalt const_literal(bool value)
   {
-    literalt l;
-    l.set(literalt::const_var_no(), value);
-    return l;
+    return literalt(literalt::const_var_no(), value);
   }
   
   inline bool is_constant() const
@@ -177,12 +176,18 @@ protected:
   unsigned l;  
 };
 
+static inline std::ostream & operator << (std::ostream &out, literalt l)
+{
+  return out << (l.sign()?"-":"") << l.var_no();
+}
+
 // constants
 literalt const_literal(bool value);
 
 literalt neg(literalt a);
 literalt pos(literalt a);
 
+// bit-vectors
 typedef std::vector<literalt> bvt;
 
 #define forall_literals(it, bv) \
@@ -193,5 +198,4 @@ typedef std::vector<literalt> bvt;
   for(bvt::iterator it=(bv).begin(); \
       it!=(bv).end(); ++it)
 
-
 #endif
