@@ -10,9 +10,9 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <cstdlib>
 #include <map>
 
-#include <std_expr.h>
-#include <symbol.h>
-#include <threeval.h>
+#include <util/std_expr.h>
+#include <util/symbol.h>
+#include <util/threeval.h>
 
 #include "prop.h"
 #include "prop_conv.h"
@@ -428,7 +428,7 @@ void prop_convt::set_to(const exprt &expr, bool value)
     msg+=expr.to_string();
     throw msg;
   }
-
+  
   bool boolean=true;
 
   forall_operands(it, expr)
@@ -498,16 +498,16 @@ void prop_convt::set_to(const exprt &expr, bool value)
         // set_to_false
         if(expr.id()==ID_implies) // !(a=>b)  ==  (a && !b)
         {
-          if(expr.operands().size()==2)
-          {
-            set_to_true(expr.op0());
-            set_to_false(expr.op1());
-          }
+          assert(expr.operands().size()==2);
+          set_to_true(expr.op0());
+          set_to_false(expr.op1());
+          return;
         }
         else if(expr.id()==ID_or) // !(a || b)  ==  (!a && !b)
         {
           forall_operands(it, expr)
             set_to_false(*it);
+          return;
         }
       }
     }
@@ -639,6 +639,6 @@ void prop_convt::print_assignment(std::ostream &out) const
   for(symbolst::const_iterator it=symbols.begin();
       it!=symbols.end();
       it++)
-    out << it->first << " = " << prop.l_get(it->second) << std::endl;
+    out << it->first << " = " << prop.l_get(it->second) << "\n";
 }
 
