@@ -51,8 +51,8 @@ protected:
 
   static bool is_all(const bvt &bv, literalt l)
   {
-    for(unsigned i=0; i<bv.size(); i++)
-      if(bv[i]!=l) return false;
+    forall_literals(it, bv)
+      if(*it!=l) return false;
     return true;
   }
 };
