@@ -9,8 +9,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <cassert>
 #include <stack>
 
-#include <i2string.h>
-#include <threeval.h>
+#include <util/i2string.h>
+#include <util/threeval.h>
 
 #include "satcheck_minisat.h"
 
