@@ -13,8 +13,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <cassert>
 #include <stack>
 
-#include <i2string.h>
-#include <threeval.h>
+#include <util/i2string.h>
+#include <util/threeval.h>
 
 #include "satcheck_minisat2.h"
 
@@ -41,9 +41,9 @@ void convert(const bvt &bv, Minisat::vec<Minisat::Lit> &dest)
 {
   dest.capacity(bv.size());
 
-  for(unsigned i=0; i<bv.size(); i++)
-    if(!bv[i].is_false())
-      dest.push(Minisat::mkLit(bv[i].var_no(), bv[i].sign()));
+  forall_literals(it, bv)
+    if(!it->is_false())
+      dest.push(Minisat::mkLit(it->var_no(), it->sign()));
 }
 
 /*******************************************************************\
@@ -99,7 +99,7 @@ Function: satcheck_minisat_no_simplifiert::solver_text
 
 const std::string satcheck_minisat_no_simplifiert::solver_text()
 {
-  return "MiniSAT2 without simplifier";
+  return "MiniSAT 2.2.0 without simplifier";
 }
 
 /*******************************************************************\
@@ -116,7 +116,7 @@ Function: satcheck_minisat_simplifiert::solver_text
 
 const std::string satcheck_minisat_simplifiert::solver_text()
 {
-  return "MiniSAT2 with simplifier";
+  return "MiniSAT 2.2.0 with simplifier";
 }
 
 /*******************************************************************\
@@ -167,16 +167,9 @@ void satcheck_minisat2_baset<T>::lcnf(const bvt &bv)
 
   convert(bv, c);
 
-  // Minisat can't do empty clauses
-  if(c.size()==0)
-  {
-    empty_clause_added=true;
-    return;
-  }
-
   // Note the underscore.
   // Add a clause to the solver without making superflous internal copy.
-  
+
   solver->addClause_(c);
 
   clause_counter++;
@@ -210,12 +203,7 @@ propt::resultt satcheck_minisat2_baset<T>::prop_solve()
   
   std::string msg;
 
-  if(empty_clause_added)
-  {
-    msg="empty clause: negated claim is UNSATISFIABLE, i.e., holds";
-    messaget::status(msg);
-  }
-  else if(!solver->okay())
+  if(!solver->okay())
   {
     msg="SAT checker inconsistent: negated claim is UNSATISFIABLE, i.e., holds";
     messaget::status(msg);
@@ -283,44 +271,9 @@ Function: satcheck_minisat2_baset::satcheck_minisat2_baset
 \*******************************************************************/
 
 template<typename T>
-satcheck_minisat2_baset<T>::satcheck_minisat2_baset()
+satcheck_minisat2_baset<T>::satcheck_minisat2_baset(T *_solver):
+  solver(_solver)
 {
-  empty_clause_added=false;
-  solver=NULL;
-}
-
-/*******************************************************************\
-
-Function: satcheck_minisat_no_simplifiert::satcheck_minisat_no_simplifiert
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-satcheck_minisat_no_simplifiert::satcheck_minisat_no_simplifiert()
-{
-  solver=new Minisat::Solver;
-}
-
-/*******************************************************************\
-
-Function: satcheck_minisat_simplifiert::satcheck_minisat_simplifiert
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-satcheck_minisat_simplifiert::satcheck_minisat_simplifiert()
-{
-  solver=new Minisat::SimpSolver;
 }
 
 /*******************************************************************\
@@ -335,8 +288,14 @@ Function: satcheck_minisat2_baset::~satcheck_minisat2_baset
 
 \*******************************************************************/
 
-template<typename T>
-satcheck_minisat2_baset<T>::~satcheck_minisat2_baset()
+template<>
+satcheck_minisat2_baset<Minisat::Solver>::~satcheck_minisat2_baset()
+{
+  delete solver;
+}
+
+template<>
+satcheck_minisat2_baset<Minisat::SimpSolver>::~satcheck_minisat2_baset()
 {
   delete solver;
 }
@@ -388,6 +347,40 @@ void satcheck_minisat2_baset<T>::set_assumptions(const bvt &bv)
 
 /*******************************************************************\
 
+Function: satcheck_minisat_no_simplifiert::satcheck_minisat_no_simplifiert
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+satcheck_minisat_no_simplifiert::satcheck_minisat_no_simplifiert():
+  satcheck_minisat2_baset<Minisat::Solver>(new Minisat::Solver)
+{
+}
+
+/*******************************************************************\
+
+Function: satcheck_minisat_simplifiert::satcheck_minisat_simplifiert
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+satcheck_minisat_simplifiert::satcheck_minisat_simplifiert():
+  satcheck_minisat2_baset<Minisat::SimpSolver>(new Minisat::SimpSolver)
+{
+}
+
+/*******************************************************************\
+
 Function: satcheck_minisat_simplifiert::set_frozen
 
   Inputs:
@@ -405,3 +398,22 @@ void satcheck_minisat_simplifiert::set_frozen(literalt a)
   solver->setFrozen(a.var_no(), true);
 }
 
+/*******************************************************************\
+
+Function: satcheck_minisat_simplifiert::is_eliminated
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+bool satcheck_minisat_simplifiert::is_eliminated(literalt a) const
+{
+  assert(!a.is_constant());
+
+  return solver->isEliminated(a.var_no());
+}
+
