@@ -6,17 +6,17 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
-#include <arith_tools.h>
-#include <expr_util.h>
-#include <std_types.h>
-#include <std_expr.h>
-#include <i2string.h>
-#include <fixedbv.h>
-#include <pointer_offset_size.h>
-#include <base_type.h>
-#include <ieee_float.h>
+#include <util/arith_tools.h>
+#include <util/expr_util.h>
+#include <util/std_types.h>
+#include <util/std_expr.h>
+#include <util/i2string.h>
+#include <util/fixedbv.h>
+#include <util/pointer_offset_size.h>
+#include <util/base_type.h>
+#include <util/ieee_float.h>
 
 #include <ansi-c/string_constant.h>
 
@@ -190,6 +190,11 @@ exprt smt1_convt::ce_value(
         return array_list;
       }
     }
+    else if(ns.follow(type.subtype()).id()==ID_array) 
+    {
+      // a 2 dimensional array - second dimension is flattened
+      return ce_value(type.subtype(), "", value, true);
+    }
     else
     {
       exprt value_expr=ce_value(type.subtype(), "", value, in_struct);
@@ -420,16 +425,17 @@ void smt1_convt::convert_byte_update(
 
   mp_integer upper, lower; // of the byte
   mp_integer max=w-1;
+  unsigned op_w = boolbv_width(expr.op2().type());
 
   if(expr.id()==ID_byte_update_little_endian)
   {
-    upper = ((i+1)*8)-1;
     lower = i*8;
+    upper = lower+op_w-1;
   }
   else
   {
     upper = max-(i*8);
-    lower = max-((i+1)*8-1);
+    lower = max-(i*8+op_w-1);
   }
 
   if(upper==max)
@@ -496,15 +502,15 @@ literalt smt1_convt::convert(const exprt &expr)
   else if(expr.is_false())
     return const_literal(false);
 
-  smt1_prop.out << std::endl;
+  smt1_prop.out << "\n";
 
   find_symbols(expr);
 
   literalt l=smt1_prop.new_variable();
-  smt1_prop.out << ":assumption ; convert " << std::endl
+  smt1_prop.out << ":assumption ; convert " << "\n"
                 << " (iff " << smt1_prop.smt1_literal(l) << " ";
   convert_expr(expr, false);
-  smt1_prop.out << ")" << std::endl;
+  smt1_prop.out << ")" << "\n";
 
   return l;
 }
@@ -809,14 +815,26 @@ void smt1_convt::convert_expr(const exprt &expr, bool bool_as_bv)
   {
     convert_plus(to_plus_expr(expr));
   }
+  else if(expr.id()==ID_floatbv_plus)
+  {
+    convert_floatbv_plus(expr);
+  }
   else if(expr.id()==ID_minus)
   {
     convert_minus(to_minus_expr(expr));
   }
+  else if(expr.id()==ID_floatbv_minus)
+  {
+    convert_floatbv_minus(expr);
+  }
   else if(expr.id()==ID_div)
   {
     convert_div(to_div_expr(expr));
   }
+  else if(expr.id()==ID_floatbv_div)
+  {
+    convert_floatbv_div(expr);
+  }
   else if(expr.id()==ID_mod)
   {
     convert_mod(to_mod_expr(expr));
@@ -825,6 +843,10 @@ void smt1_convt::convert_expr(const exprt &expr, bool bool_as_bv)
   {
     convert_mult(to_mult_expr(expr));
   }
+  else if(expr.id()==ID_floatbv_mult)
+  {
+    convert_floatbv_mult(expr);
+  }
   else if(expr.id()==ID_address_of)
   {
     const typet &type=expr.type();
@@ -914,6 +936,10 @@ void smt1_convt::convert_expr(const exprt &expr, bool bool_as_bv)
   {
     convert_with(expr);
   }
+  else if(expr.id()==ID_update)
+  {
+    convert_update(expr);
+  }
   else if(expr.id()==ID_member)
   {
     convert_member(to_member_expr(expr), bool_as_bv);
@@ -1410,7 +1436,9 @@ Function: smt1_convt::convert_typecast
 
 \*******************************************************************/
 
-void smt1_convt::convert_typecast(const typecast_exprt &expr, bool bool_as_bv)
+void smt1_convt::convert_typecast(
+  const typecast_exprt &expr,
+  bool bool_as_bv)
 {
   assert(expr.operands().size()==1);
   const exprt &op=expr.op0();
@@ -1475,13 +1503,13 @@ void smt1_convt::convert_typecast(const typecast_exprt &expr, bool bool_as_bv)
         smt1_prop.out << ")";
       }
     }
-    else if(op_type.id()==ID_fixedbv) // from fixedbv
+    else if(op_type.id()==ID_fixedbv) // from fixedbv to integer
     {
-      const fixedbv_typet &fixedbv_type=to_fixedbv_type(op_type);
+      const fixedbv_typet &fixedbv_op_type=to_fixedbv_type(op_type);
 
-      unsigned from_width=fixedbv_type.get_width();
-      unsigned from_integer_bits=fixedbv_type.get_integer_bits();
-      unsigned from_fraction_bits=fixedbv_type.get_fraction_bits();
+      unsigned from_width=fixedbv_op_type.get_width();
+      unsigned from_integer_bits=fixedbv_op_type.get_integer_bits();
+      unsigned from_fraction_bits=fixedbv_op_type.get_fraction_bits();
 
       if(to_width>from_integer_bits)
       {
@@ -1566,20 +1594,24 @@ void smt1_convt::convert_typecast(const typecast_exprt &expr, bool bool_as_bv)
        op_type.id()==ID_signedbv ||
        op_type.id()==ID_c_enum)
     {
+      // integer to fixedbv
       unsigned from_width=to_bitvector_type(op_type).get_width();
+      
+      // we just concatenate a zero-valued fractional part
       smt1_prop.out << "(concat";
 
       if(from_width==to_integer_bits)
         convert_expr(op, true);
       else if(from_width>to_integer_bits)
       {
-        smt1_prop.out << " (extract[" << (to_integer_bits-1) << ":"
-                      << to_fraction_bits << "] ";
+        // too many integer bits, chop some off
+        smt1_prop.out << " (extract[" << (to_integer_bits-1) << ":0] ";
         convert_expr(op, true);
         smt1_prop.out << ")";
       }
       else
       {
+        // too few integer bits
         assert(from_width<to_integer_bits);
         if(expr_type.id()==ID_unsignedbv)
         {
@@ -1597,10 +1629,12 @@ void smt1_convt::convert_typecast(const typecast_exprt &expr, bool bool_as_bv)
         }
       }
 
-      smt1_prop.out << " bv0[" << to_fraction_bits << "])";
+      smt1_prop.out << " bv0[" << to_fraction_bits << "]";
+      smt1_prop.out << ")"; // concat
     }
     else if(op_type.id()==ID_bool)
     {
+      // bool to fixedbv
       smt1_prop.out << "(concat (concat bv0[" << (to_integer_bits-1) << "]"
                     << " ";
       convert_expr(op, true); // this returns a 1-bit bit-vector
@@ -1610,6 +1644,7 @@ void smt1_convt::convert_typecast(const typecast_exprt &expr, bool bool_as_bv)
     }
     else if(op_type.id()==ID_fixedbv)
     {
+      // fixedbv to fixedbv
       const fixedbv_typet &from_fixedbv_type=to_fixedbv_type(op_type);
       unsigned from_fraction_bits=from_fixedbv_type.get_fraction_bits();
       unsigned from_integer_bits=from_fixedbv_type.get_integer_bits();
@@ -1735,7 +1770,14 @@ void smt1_convt::convert_struct(const exprt &expr)
   assert(!components.empty());
 
   if(components.size()==1)
-    convert_expr(expr.op0(), true);
+  {
+    const exprt &op=expr.op0();
+
+    if(op.type().id()==ID_array)
+      flatten_array(op);
+    else
+      convert_expr(op, true);
+  }
   else
   {
     unsigned nr_ops=0;
@@ -1846,7 +1888,7 @@ void smt1_convt::convert_constant(
   {
     fixedbv_spect spec(to_fixedbv_type(expr.type()));
 
-    std::string v_str=id2string(expr.get(ID_value));
+    std::string v_str=expr.get_string(ID_value);
     mp_integer v=binary2integer(v_str, false);
 
     smt1_prop.out << "bv" << v << "[" << spec.width << "]";
@@ -1855,7 +1897,7 @@ void smt1_convt::convert_constant(
   {
     ieee_float_spect spec(to_floatbv_type(expr.type()));
 
-    std::string v_str=id2string(expr.get(ID_value));
+    std::string v_str=expr.get_string(ID_value);
     mp_integer v=binary2integer(v_str, false);
 
     smt1_prop.out << "bv" << v << "[" << spec.width() << "]";
@@ -1893,7 +1935,7 @@ void smt1_convt::convert_constant(
   }
   else if(expr.type().id()==ID_rational)
   {
-    std::string value=expr.get(ID_value).as_string();
+    std::string value=expr.get_string(ID_value);
     size_t pos=value.find("/");
 
     if(pos==std::string::npos)
@@ -1907,7 +1949,7 @@ void smt1_convt::convert_constant(
   else if(expr.type().id()==ID_integer ||
           expr.type().id()==ID_natural)
   {
-    std::string value=expr.get(ID_value).as_string();
+    std::string value=expr.get_string(ID_value);
     
     if(value[0]=='-') 
       smt1_prop.out << "(~ " << value.substr(1) << ")";
@@ -2184,6 +2226,26 @@ void smt1_convt::convert_plus(const plus_exprt &expr)
 
 /*******************************************************************\
 
+Function: smt1_convt::convert_floatbv_plus
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void smt1_convt::convert_floatbv_plus(const exprt &expr)
+{
+  assert(expr.operands().size()==3);
+  assert(expr.type().id()==ID_floatbv);
+
+  throw "todo: floatbv_plus";
+}
+
+/*******************************************************************\
+
 Function: smt1_convt::convert_minus
 
   Inputs:
@@ -2235,6 +2297,26 @@ void smt1_convt::convert_minus(const minus_exprt &expr)
 
 /*******************************************************************\
 
+Function: smt1_convt::convert_floatbv_minus
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void smt1_convt::convert_floatbv_minus(const exprt &expr)
+{
+  assert(expr.operands().size()==3); 
+  assert(expr.type().id()==ID_floatbv);
+    
+  throw "todo: floatbv_minus";
+}
+
+/*******************************************************************\
+
 Function: smt1_convt::convert_div
 
   Inputs:
@@ -2286,6 +2368,26 @@ void smt1_convt::convert_div(const div_exprt &expr)
 
 /*******************************************************************\
 
+Function: smt1_convt::convert_floatbv_div
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void smt1_convt::convert_floatbv_div(const exprt &expr)
+{
+  assert(expr.operands().size()==3); 
+  assert(expr.type().id()==ID_floatbv);
+    
+  throw "todo: floatbv_div";
+}
+      
+/*******************************************************************\
+
 Function: smt1_convt::convert_mult
 
   Inputs:
@@ -2330,6 +2432,7 @@ void smt1_convt::convert_mult(const mult_exprt &expr)
     fixedbv_spect spec(to_fixedbv_type(expr.type()));
     unsigned fraction_bits=spec.get_fraction_bits();
 
+    // strip away faction_bits off the result
     smt1_prop.out << "(extract[" << spec.width+fraction_bits-1 << ":"
                                  << fraction_bits << "] ";
 
@@ -2343,8 +2446,8 @@ void smt1_convt::convert_mult(const mult_exprt &expr)
     convert_expr(expr.op1(), true);
     smt1_prop.out << ") ";
 
-    smt1_prop.out << ")"; // bvmul
-    smt1_prop.out << ")"; // extract
+    smt1_prop.out << ")"; // bvmul, fraction_bits+width wide
+    smt1_prop.out << ")"; // extract, width bits wide
   }
   else if(expr.type().id()==ID_rational)
   {
@@ -2360,6 +2463,26 @@ void smt1_convt::convert_mult(const mult_exprt &expr)
 
 /*******************************************************************\
 
+Function: smt1_convt::convert_floatbv_mult
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void smt1_convt::convert_floatbv_mult(const exprt &expr)
+{
+  assert(expr.operands().size()==3); 
+  assert(expr.type().id()==ID_floatbv);
+    
+  throw "todo: floatbv_mult";
+}
+        
+/*******************************************************************\
+
 Function: smt1_convt::convert_with
 
   Inputs:
@@ -2403,21 +2526,31 @@ void smt1_convt::convert_with(const exprt &expr)
 
     if(array.id()==ID_member)
     {
-      // arrays in structs are flattened!
+      // arrays in structs and unions are flattened!
+
       const typet &array_type=to_array_type(expr.type());
       const typet &elem_type=array_type.subtype();
 
       const member_exprt &member_expr=to_member_expr(array);
       const exprt &struct_op=member_expr.struct_op();
       const irep_idt &name=member_expr.get_component_name();
+      
+      const typet &struct_op_type=ns.follow(struct_op.type());
 
-      unsigned total_width=boolbv_width(struct_op.type());
+      unsigned total_width=boolbv_width(struct_op_type);
       
       if(total_width==0)
         throw "failed to get struct width";
 
-      unsigned offset=boolbv_width.get_member(
-        to_struct_type(struct_op.type()), name).offset;
+      unsigned offset;
+      
+      if(struct_op_type.id()==ID_struct)
+        offset=boolbv_width.get_member(
+          to_struct_type(struct_op_type), name).offset;
+      else if(struct_op_type.id()==ID_union)
+        offset=0;
+      else
+        assert(false);
 
       unsigned width=boolbv_width(expr.type());
 
@@ -2467,6 +2600,73 @@ void smt1_convt::convert_with(const exprt &expr)
       // the new value
       smt1_prop.out << " (bvshl (zero_extend[" << array_bits-elem_width << "] ";
       convert_expr(value, true);
+
+      // shift it to the index
+      smt1_prop.out << ")";
+      if (width>=array_index_bits)
+        smt1_prop.out << " (zero_extend[" << width-array_index_bits << "]";
+      else
+        smt1_prop.out << " (extract[" << width-1 << ":0]";
+      smt1_prop.out << " (bvmul ";
+      convert_expr(index_tc, true);
+      smt1_prop.out << " bv" << elem_width << "[" << array_index_bits << "]";
+      smt1_prop.out << ")))"; // bvmul, bvshl, ze
+
+      smt1_prop.out << ")"; // bvor
+    }
+    else if(array.id()==ID_index)
+    {
+      // arrays in structs are flattened!
+
+      const typet &array_type=to_array_type(expr.type());
+      const typet &elem_type=array_type.subtype();
+
+      const index_exprt &index_expr=to_index_expr(array);
+
+      unsigned width=boolbv_width(expr.type());
+
+      if(width==0)
+        throw "failed to get width of 2nd dimension array";
+
+      unsigned elem_width=boolbv_width(elem_type);
+
+      if(elem_width==0)
+        throw "failed to get array element width";
+
+      assert(expr.operands().size()==3);
+      const exprt &index_2nd=expr.operands()[1];
+      const exprt &value=expr.operands()[2];
+      typecast_exprt index_tc(index_2nd, array_index_type());
+
+      smt1_prop.out << "(bvor ";
+      smt1_prop.out << "(bvand ";
+
+      // this gets us the array
+      convert_expr(index_expr, true);
+
+      // the mask
+      smt1_prop.out << " (bvnot (bvshl";
+
+      smt1_prop.out << " (concat";
+      smt1_prop.out << " (repeat[" << width-elem_width << "] bv0[1])";
+      smt1_prop.out << " (repeat[" << elem_width << "] bv1[1])";
+      smt1_prop.out << ")"; // concat
+
+      // shift it to the index
+      if (width>=array_index_bits)
+        smt1_prop.out << " (zero_extend[" << width-array_index_bits << "]";
+      else
+        smt1_prop.out << " (extract[" << width-1 << ":0]";
+      smt1_prop.out << " (bvmul ";
+      convert_expr(index_tc, true);
+      smt1_prop.out << " bv" << elem_width << "[" << array_index_bits << "]";
+      smt1_prop.out << "))))"; // bvmul, zero_extend, bvshl, bvneg
+
+      smt1_prop.out << ")"; // bvand
+
+      // the new value
+      smt1_prop.out << " (bvshl (zero_extend[" << width-elem_width << "] ";
+      convert_expr(value, true);
       // shift it to the index
       smt1_prop.out << ")";
       if (width>=array_index_bits)
@@ -2633,6 +2833,26 @@ void smt1_convt::convert_with(const exprt &expr)
 
 /*******************************************************************\
 
+Function: smt1_convt::convert_update
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+void smt1_convt::convert_update(const exprt &expr)
+{
+  assert(expr.operands().size()==3);
+
+  // todo
+  throw "smt1_convt::convert_update to be implemented";
+}
+
+/*******************************************************************\
+
 Function: smt1_convt::convert_index
 
   Inputs:
@@ -2647,28 +2867,18 @@ void smt1_convt::convert_index(const index_exprt &expr, bool bool_as_bv)
 {
   assert(expr.operands().size()==2);
 
-  if(expr.array().id()==ID_member)
+  if(expr.array().id()==ID_member ||
+     expr.array().id()==ID_index)
   {
+    // arrays inside structs and 2-dimensional arrays
     // these were flattened
+
     const typet &array_type=to_array_type(expr.array().type());
     const typet &elem_type=array_type.subtype();
 
-    const member_exprt &member_expr=to_member_expr(expr.array());
-    const exprt &struct_op=member_expr.struct_op();
-    //const irep_idt &name=member_expr.get_component_name();
-
-    unsigned total_width=boolbv_width(struct_op.type());
-    
-    if(total_width==0)
-      throw "failed to get struct width";
-
-    //unsigned offset=boolbv_width.get_member(
-    //     to_struct_type(struct_op.type()), name).offset;
-
-    unsigned width=boolbv_width(member_expr.type());
-    
+    unsigned width=boolbv_width(expr.array().type());
     if(width==0)
-      throw "failed to get struct member width";
+      throw "failed to get array width";
 
     unsigned elem_width=boolbv_width(elem_type);
 
@@ -2809,17 +3019,17 @@ void smt1_convt::set_to(const exprt &expr, bool value)
     return set_to(expr.op0(), !value);
   }
 
-  smt1_prop.out << std::endl;
+  smt1_prop.out << "\n";
 
   find_symbols(expr);
 
   #if 0
   smt1_prop.out << "; CONV: "
-                << from_expr(expr) << std::endl;
+                << from_expr(expr) << "\n";
   #endif
 
   smt1_prop.out << ":assumption ; set_to "
-                << (value?"true":"false") << std::endl
+                << (value?"true":"false") << "\n"
                 << " ";
 
   assert(expr.type().id()==ID_bool);
@@ -2833,7 +3043,7 @@ void smt1_convt::set_to(const exprt &expr, bool value)
   else
     convert_expr(expr, false);
 
-  smt1_prop.out << std::endl;
+  smt1_prop.out << "\n";
 }
 
 /*******************************************************************\
@@ -2879,7 +3089,7 @@ void smt1_convt::find_symbols(const exprt &expr)
     if(expr.id()==ID_symbol)
       identifier=to_symbol_expr(expr).get_identifier();
     else
-      identifier="nondet_"+id2string(expr.get(ID_identifier));
+      identifier="nondet_"+expr.get_string(ID_identifier);
 
     if(quantified_symbols.find(identifier)!=quantified_symbols.end())
       return; // Symbol is quantified, i.e., it doesn't require declaration.
@@ -2894,7 +3104,7 @@ void smt1_convt::find_symbols(const exprt &expr)
       {
         smt1_prop.out << ":extrapreds(("
                       << convert_identifier(identifier)
-                      << "))" << std::endl;
+                      << "))" << "\n";
       }
       else
       {
@@ -2902,7 +3112,7 @@ void smt1_convt::find_symbols(const exprt &expr)
                       << convert_identifier(identifier)
                       << " ";
         convert_type(type);
-        smt1_prop.out << "))" << std::endl;
+        smt1_prop.out << "))" << "\n";
       }
     }
   }
@@ -2911,12 +3121,12 @@ void smt1_convt::find_symbols(const exprt &expr)
     if(array_of_map.find(expr)==array_of_map.end())
     {
       irep_idt id="array_of'"+i2string(array_of_map.size());
-      smt1_prop.out << "; the following is a poor substitute for lambda i. x" << std::endl;
+      smt1_prop.out << "; the following is a poor substitute for lambda i. x" << "\n";
       smt1_prop.out << ":extrafuns(("
                     << id
                     << " ";
       convert_type(type);
-      smt1_prop.out << "))" << std::endl;
+      smt1_prop.out << "))" << "\n";
 
       // we can initialize array_ofs if they have
       // a constant size and a constant element
@@ -2934,7 +3144,7 @@ void smt1_convt::find_symbols(const exprt &expr)
             smt1_prop.out << ":assumption (= (select " << id << " bv" <<
              i << "[" << array_index_bits << "]) ";
             convert_expr(expr.op0(), true);
-            smt1_prop.out << ")" << std::endl;
+            smt1_prop.out << ")" << "\n";
           }
         }
       }
@@ -2952,7 +3162,7 @@ void smt1_convt::find_symbols(const exprt &expr)
                     << id
                     << " ";
       convert_type(type);
-      smt1_prop.out << "))" << std::endl;
+      smt1_prop.out << "))" << "\n";
       array_expr_map[expr]=id;
     }
   }
@@ -2969,7 +3179,7 @@ void smt1_convt::find_symbols(const exprt &expr)
                     << id
                     << " ";
       convert_type(t.type());
-      smt1_prop.out << "))" << std::endl;
+      smt1_prop.out << "))" << "\n";
       array_expr_map[t]=id;
     }
   }
