@@ -13,8 +13,8 @@ Revision: Roberto Bruttomesso, roberto.bruttomesso@unisi.ch
 #include <sstream>
 #include <set>
 
-#include <hash_cont.h>
-#include <std_expr.h>
+#include <util/hash_cont.h>
+#include <util/std_expr.h>
 
 #include <solvers/prop/prop_conv.h>
 #include <solvers/flattening/pointer_logic.h>
@@ -63,14 +63,14 @@ public:
   virtual ~smt1_convt() { }
   virtual resultt dec_solve();
 
-protected:
-  boolbv_widtht boolbv_width;
-
-  // overloading
+  // overloading interfaces
   virtual literalt convert(const exprt &expr);
   virtual void set_to(const exprt &expr, bool value);
   virtual exprt get(const exprt &expr) const;
 
+protected:
+  boolbv_widtht boolbv_width;
+
   // new stuff
   void convert_expr(const exprt &expr, bool bool_as_bv);
   void convert_type(const typet &type);
@@ -88,11 +88,16 @@ protected:
   void convert_minus(const minus_exprt &expr);
   void convert_div(const div_exprt &expr);
   void convert_mult(const mult_exprt &expr);
+  void convert_floatbv_plus(const exprt &expr);
+  void convert_floatbv_minus(const exprt &expr);
+  void convert_floatbv_div(const exprt &expr);
+  void convert_floatbv_mult(const exprt &expr);
   void convert_mod(const mod_exprt &expr);
   void convert_index(const index_exprt &expr, bool bool_as_bv);
   void convert_member(const member_exprt &expr, bool bool_as_bv);
   void convert_overflow(const exprt &expr);
   void convert_with(const exprt &expr);
+  void convert_update(const exprt &expr);
   
   std::string convert_identifier(const irep_idt &identifier);
   
