@@ -6,23 +6,23 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <stdlib.h>
+#include <cstdlib>
 
-#ifdef __linux__
+#if defined(__linux__) || \
+    defined(__FreeBSD_kernel__) || \
+    defined(__GNU__) || \
+    defined(__CYGWIN__) || \
+    defined(__MACH__)
 #include <unistd.h>
 #endif
 
-#ifdef __APPLE__
-#include <unistd.h>
-#endif
-
-#include <std_expr.h>
-#include <std_types.h>
-#include <tempfile.h>
-#include <arith_tools.h>
-#include <i2string.h>
-#include <string2int.h>
-#include <prefix.h>
+#include <util/std_expr.h>
+#include <util/std_types.h>
+#include <util/tempfile.h>
+#include <util/arith_tools.h>
+#include <util/i2string.h>
+#include <util/string2int.h>
+#include <util/prefix.h>
 
 #include "smt1_dec.h"
 
@@ -134,11 +134,11 @@ decision_proceduret::resultt smt1_dect::dec_solve()
     break;
 
   case BOOLECTOR:
-    // –rwl0 disables rewriting, which makes things slower,
+    // -rwl0 disables rewriting, which makes things slower,
     // but in return values for arrays appear
     command = "boolector -rwl0 --smt "
             + temp_out_filename
-            + " -fm --output "
+            + " -m --output "
             + temp_result_filename;
     break;
 
@@ -739,9 +739,9 @@ decision_proceduret::resultt smt1_dect::read_result_cvc3(std::istream &in)
       set_value(it->second, "", binary);
     }
     else if(value=="false")
-      it->second.value.make_false();
+      it->second.value=false_exprt();
     else if(value=="true")
-      it->second.value.make_true();
+      it->second.value=true_exprt();
     else if(value.substr(0,8)=="array_of")
     {
       // We assume that array_of has only concrete arguments...
