@@ -8,9 +8,9 @@ Revisions: Roberto Bruttomesso, roberto.bruttomesso@unisi.ch
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
-#include <i2string.h>
+#include <util/i2string.h>
 
 #include "smt1_prop.h"
 
@@ -32,10 +32,10 @@ smt1_propt::smt1_propt(
   const std::string &logic,
   std::ostream &_out):out(_out)
 {
-  out << "(benchmark " << benchmark << std::endl;
-  out << ":source { " << source << " }" << std::endl;
-  out << ":status unknown" << std::endl;
-  out << ":logic " << logic << " ; SMT1" << std::endl;
+  out << "(benchmark " << benchmark << "\n";
+  out << ":source { " << source << " }" << "\n";
+  out << ":status unknown" << "\n";
+  out << ":logic " << logic << " ; SMT1" << "\n";
   _no_variables=0;
 }
 
@@ -69,9 +69,9 @@ Function: smt1_propt::finalize
 
 void smt1_propt::finalize()
 {
-  out << std::endl;
-  out << ":formula true" << std::endl;
-  out << ") ; benchmark" << std::endl;
+  out << "\n";
+  out << ":formula true" << "\n";
+  out << ") ; benchmark" << "\n";
 }
 
 /*******************************************************************\
@@ -88,17 +88,17 @@ Function: smt1_propt::land
 
 literalt smt1_propt::land(const bvt &bv)
 {
-  out << std::endl;
+  out << "\n";
 
   literalt l=new_variable();
 
-  out << ":assumption ; land" << std::endl;
+  out << ":assumption ; land" << "\n";
   out << " (iff " << smt1_literal(l) << " (and";
   
   forall_literals(it, bv)
     out << " " << smt1_literal(*it);
 
-  out << "))" << std::endl;
+  out << "))" << "\n";
 
   return l;
 }
@@ -117,17 +117,17 @@ Function: smt1_propt::lor
 
 literalt smt1_propt::lor(const bvt &bv)
 {
-  out << std::endl;
+  out << "\n";
 
   literalt l=new_variable();
 
-  out << ":assumption ; lor" << std::endl;
+  out << ":assumption ; lor" << "\n";
   out << " (iff " << smt1_literal(l) << " (or";
 
   forall_literals(it, bv)
     out << " " << smt1_literal(*it);
 
-  out << "))" << std::endl;
+  out << "))" << "\n";
 
   return l;
 }
@@ -149,17 +149,17 @@ literalt smt1_propt::lxor(const bvt &bv)
   if(bv.size()==0) return const_literal(false);
   if(bv.size()==1) return bv[0];
 
-  out << std::endl;
+  out << "\n";
 
   literalt l=new_variable();
 
-  out << ":assumption ; lxor" << std::endl;
+  out << ":assumption ; lxor" << "\n";
   out << " (iff " << smt1_literal(l) << " (xor";
 
   forall_literals(it, bv)
     out << " " << smt1_literal(*it);
 
-  out << "))" << std::endl;
+  out << "))" << "\n";
 
   return l;
 }
@@ -184,15 +184,15 @@ literalt smt1_propt::land(literalt a, literalt b)
   if(b==const_literal(false)) return const_literal(false);
   if(a==b) return a;
 
-  out << std::endl;
+  out << "\n";
 
   literalt l=new_variable();
 
-  out << ":assumption ; land" << std::endl;
+  out << ":assumption ; land" << "\n";
   out << " (iff " << smt1_literal(l) << " (and";
   out << " " << smt1_literal(a);
   out << " " << smt1_literal(b);
-  out << "))" << std::endl;
+  out << "))" << "\n";
 
   return l;
 }
@@ -217,15 +217,15 @@ literalt smt1_propt::lor(literalt a, literalt b)
   if(b==const_literal(true)) return const_literal(true);
   if(a==b) return a;
   
-  out << std::endl;
+  out << "\n";
 
   literalt l=new_variable();
 
-  out << ":assumption ; lor" << std::endl;
+  out << ":assumption ; lor" << "\n";
   out << " (iff " << smt1_literal(l) << " (or";
   out << " " << smt1_literal(a);
   out << " " << smt1_literal(b);
-  out << "))" << std::endl;
+  out << "))" << "\n";
 
   return l;
 }
@@ -267,15 +267,15 @@ literalt smt1_propt::lxor(literalt a, literalt b)
   if(a==const_literal(true)) return lnot(b);
   if(b==const_literal(true)) return lnot(a);
 
-  out << std::endl;
+  out << "\n";
 
   literalt l=new_variable();
 
-  out << ":assumption ; lxor" << std::endl;
+  out << ":assumption ; lxor" << "\n";
   out << " (iff " << smt1_literal(l) << " (xor";
   out << " " << smt1_literal(a);
   out << " " << smt1_literal(b);
-  out << "))" << std::endl;
+  out << "))" << "\n";
 
   return l;
 }
@@ -371,14 +371,14 @@ literalt smt1_propt::lselect(literalt a, literalt b, literalt c)
   if(a==const_literal(true)) return lnot(b);
   if(b==const_literal(true)) return lnot(a);
 
-  out << std::endl;
+  out << "\n";
 
   literalt l=new_variable();
 
-  out << ":assumption ; lselect" << std::endl;
+  out << ":assumption ; lselect" << "\n";
   out << " (iff " << smt1_literal(l) << "(if_then_else "
       << smt1_literal(a) << " " << smt1_literal(b) << " "
-      << smt1_literal(c) << ")" << std::endl;
+      << smt1_literal(c) << ")" << "\n";
 
   return l;
 }
@@ -401,7 +401,7 @@ literalt smt1_propt::new_variable()
   l.set(_no_variables, false);
   _no_variables++;
   
-  out << ":extrapreds((" << smt1_literal(l) << "))" << std::endl;
+  out << ":extrapreds((" << smt1_literal(l) << "))" << "\n";
 
   return l;
 }
@@ -420,8 +420,8 @@ Function: smt1_propt::lcnf
 
 void smt1_propt::lcnf(const bvt &bv)
 {
-  out << std::endl;
-  out << ":assumption ; lcnf" << std::endl;
+  out << "\n";
+  out << ":assumption ; lcnf" << "\n";
   out << " ";
 
   if(bv.empty())
@@ -438,7 +438,7 @@ void smt1_propt::lcnf(const bvt &bv)
     out << ")";
   }
 
-  out << std::endl;
+  out << "\n";
 }
 
 /*******************************************************************\
