@@ -9,9 +9,9 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_PROP_SMT_PROP_H
 #define CPROVER_PROP_SMT_PROP_H
 
-#include <iostream>
+#include <ostream>
 
-#include <threeval.h>
+#include <util/threeval.h>
 
 #include <solvers/prop/prop.h>
 
