@@ -12,8 +12,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <sstream>
 #include <set>
 
-#include <hash_cont.h>
-#include <std_expr.h>
+#include <util/hash_cont.h>
+#include <util/std_expr.h>
 
 #include <solvers/prop/prop_conv.h>
 #include <solvers/flattening/boolbv_width.h>
@@ -28,8 +28,9 @@ public:
     const std::string &_benchmark,
     const std::string &_notes,
     const std::string &_logic,
+    bool _core_enabled,
     std::ostream &_out):
-    smt2_prop(_benchmark, _notes, _logic, _out) 
+    smt2_prop(_benchmark, _notes, _logic, _core_enabled, _out) 
   { }
 
 protected:
@@ -50,14 +51,31 @@ public:
     const namespacet &_ns,
     const std::string &_benchmark,
     const std::string &_notes,
-    const std::string &_logic,    
+    const std::string &_logic,
     std::ostream &_out):
-    smt2_prop_wrappert(_benchmark, _notes, _logic, _out),
+    smt2_prop_wrappert(_benchmark, _notes, _logic, false, _out),
     prop_convt(_ns, smt2_prop),
     use_FPA_theory(false),
     boolbv_width(_ns),
     pointer_logic(_ns),
-    array_index_bits(32)
+    array_index_bits(32),
+    num_core_constraints(0)
+  { }
+
+  smt2_convt(
+    const namespacet &_ns,
+    const std::string &_benchmark,
+    const std::string &_notes,
+    const std::string &_logic,
+    bool _core_enabled,
+    std::ostream &_out):
+    smt2_prop_wrappert(_benchmark, _notes, _logic, _core_enabled, _out),
+    prop_convt(_ns, smt2_prop),
+    use_FPA_theory(false),
+    boolbv_width(_ns),
+    pointer_logic(_ns),
+    array_index_bits(32),
+    num_core_constraints(0)
   { }
 
   virtual ~smt2_convt() { }
@@ -65,13 +83,14 @@ public:
 
   bool use_FPA_theory;
 
-protected:
-  boolbv_widtht boolbv_width;
-
-  // overloading
+  // overloading interfaces
   virtual literalt convert(const exprt &expr);
   virtual void set_to(const exprt &expr, bool value);
   virtual exprt get(const exprt &expr) const;
+  virtual bool in_core(const exprt &expr);
+
+protected:
+  boolbv_widtht boolbv_width;
 
   // new stuff
   void convert_expr(const exprt &expr);
@@ -90,11 +109,16 @@ protected:
   void convert_minus(const minus_exprt &expr);
   void convert_div(const div_exprt &expr);
   void convert_mult(const mult_exprt &expr);
+  void convert_floatbv_plus(const exprt &expr);
+  void convert_floatbv_minus(const exprt &expr);
+  void convert_floatbv_div(const exprt &expr);
+  void convert_floatbv_mult(const exprt &expr);
   void convert_mod(const mod_exprt &expr);
   void convert_index(const index_exprt &expr);
   void convert_member(const member_exprt &expr);
   void convert_overflow(const exprt &expr);
   void convert_with(const exprt &expr);
+  void convert_update(const exprt &expr);
   
   std::string convert_identifier(const irep_idt &identifier);
   
@@ -102,6 +126,9 @@ protected:
   void find_symbols(const exprt &expr);
   void find_symbols(const typet &type);
   void find_symbols_rec(const typet &type, std::set<irep_idt> &recstack);
+
+  constant_exprt parse_literal(const std::string &s, const typet &type);
+  exprt parse_struct(const std::string &s, const typet &type);
   
   // arrays
   typet array_index_type() const;
@@ -133,6 +160,9 @@ protected:
     identifier_mapt;
 
   identifier_mapt identifier_map;
+
+  typedef std::map<typet, std::string> type_mapt;
+  type_mapt type_map;
   
   unsigned array_index_bits;
   
@@ -144,6 +174,10 @@ protected:
 
   typedef std::map<exprt, irep_idt> defined_expressionst;
   defined_expressionst defined_expressions;
+
+  int num_core_constraints;
+  std::map<std::string, exprt> core_map;
+  std::set<exprt> unsat_core;
 };
 
 #endif
