@@ -1,3 +1,4 @@
+
 /*******************************************************************\
 
 Module:
@@ -6,21 +7,22 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <stdlib.h>
+#include <cstdlib>
 
-#ifdef __linux__
+#if defined(__linux__) || \
+    defined(__FreeBSD_kernel__) || \
+    defined(__GNU__) || \
+    defined(__CYGWIN__) || \
+    defined(__MACH__)
 #include <unistd.h>
 #endif
 
-#ifdef __APPLE__
-#include <unistd.h>
-#endif
-
-#include <std_expr.h>
-#include <std_types.h>
-#include <tempfile.h>
-#include <arith_tools.h>
-#include <i2string.h>
+#include <util/std_expr.h>
+#include <util/std_types.h>
+#include <util/tempfile.h>
+#include <util/arith_tools.h>
+#include <util/i2string.h>
+#include <util/ieee_float.h>
 
 #include "smt2_dec.h"
 
@@ -38,7 +40,9 @@ Function: smt2_dect::decision_procedure_text
 
 std::string smt2_dect::decision_procedure_text() const
 {
-  return "SMT "+logic+" using "+
+  return "SMT "+logic+
+    (use_FPA_theory?" (with FPA)":"")+
+    " using "+
     (solver==BOOLECTOR?"Boolector":
      solver==CVC3?"CVC3":
      solver==MATHSAT?"MathSAT":
@@ -141,13 +145,28 @@ decision_proceduret::resultt smt2_dect::dec_solve()
     break;
 
   case MATHSAT:
-    command = "mathsat -input=smt2"
+    // The options below were recommended by Alberto Griggio
+    // on 10 July 2013
+    command = "mathsat -input=smt2 \
+ -preprocessor.toplevel_propagation=true \
+ -preprocessor.simplification=7 \
+ -dpll.branching_random_frequency=0.01 \
+ -dpll.branching_random_invalidate_phase_cache=true \
+ -dpll.restart_strategy=3 \
+ -dpll.glucose_var_activity=true \
+ -dpll.glucose_learnt_minimization=true \
+ -theory.bv.eager=true \
+ -theory.bv.bit_blast_mode=1 \
+ -theory.bv.delay_propagated_eqs=true \
+ -theory.fp.mode=1 \
+ -theory.fp.bit_blast_mode=2 \
+ -theory.arr.mode=1"
               " < "+temp_out_filename
             + " > "+temp_result_filename;
     break;
 
   case Z3:
-    command = "z3 -m "
+    command = "z3 -smt2 "
             + temp_out_filename
             + " > "
             + temp_result_filename;
@@ -312,11 +331,26 @@ decision_proceduret::resultt smt2_dect::read_result_mathsat(std::istream &in)
       std::size_t pos1=line.find('(', 1);
       std::size_t pos2=line.find(' ', pos1);
       std::size_t pos3=line.find(')', pos2);
+
       if(pos1!=std::string::npos &&
          pos2!=std::string::npos &&
          pos3!=std::string::npos)
       {
         std::string id=std::string(line, pos1+1, pos2-pos1-1);
+        int op = 0;
+        for (size_t i = pos2+1; i < line.size(); ++i)
+        {
+            if (line[i] == '(') {
+                ++op;
+            } else if (line[i] == ')') {
+                --op;
+            }
+            if (op < 0) {
+              pos3 = i;
+              break;
+            }
+        }
+
         std::string value=std::string(line, pos2+1, pos3-pos2-1);
         values[id]=value;
       }
@@ -329,20 +363,74 @@ decision_proceduret::resultt smt2_dect::read_result_mathsat(std::istream &in)
       it++)
   {
     it->second.value.make_nil();
+
     std::string conv_id=convert_identifier(it->first);
     std::string value=values[conv_id];
     if(value=="") continue;
-    if (value.substr(0, 5) == "(_ bv") {
-      // value is "(_ bvDECIMAL_VALUE SIZE"
+
+    if(value.substr(0, 5) == "(_ bv")
+    {
+      // value is "(_ bvDECIMAL_VALUE SIZE)"
       // convert to binary
       value = value.substr(5);
       size_t pos = value.find(' ');
       std::string v = value.substr(0, pos);
-      std::string w = value.substr(pos+1);
-      value = integer2binary(string2integer(v, 10),
-                             string2integer(w, 10).to_ulong());
+      //std::string w = value.substr(pos+1, value.size()-pos-2);
+      //integer2binary(string2integer(v, 10),
+      //                       string2integer(w, 10).to_ulong());
+      it->second.value=parse_literal(v, it->second.type);
+    }
+    else if(value.substr(0, 4) == "(fp ")
+    {
+      // value is "(fpnum signbv exponentbv significandbv)"
+      value = value.substr(4);
+      size_t pos = value.find(' ');
+      std::string e = value.substr(0, pos);
+      size_t pos2 = value.find(')');
+      std::string m = value.substr(pos+1, pos2-pos-1);
+      //unsigned long w = (string2integer(e, 10).to_ulong() +
+      //                   string2integer(m, 10).to_ulong() + 1);
+      std::string v=value.substr(pos2+2, value.size()-pos2-3);
+      //value = integer2binary(
+      //    string2integer(, 10), w);
+      it->second.value=parse_literal(v, it->second.type);
+    }
+    else if(value.substr(0, 8) == "(_ +oo ")
+    {
+      value = value.substr(8);
+      size_t pos = value.find(' ');
+      unsigned long e = string2integer(value.substr(0, pos), 10).to_ulong();
+      size_t pos2 = value.find(')');
+      unsigned long m =
+          string2integer(value.substr(pos+1, pos2-pos-1), 10).to_ulong();
+      //value = integer2binary(
+      //    ieee_floatt::plus_infinity(ieee_float_spect(m, e)).pack(), e+m+1);
+      it->second.value=ieee_floatt::plus_infinity(ieee_float_spect(m, e)).to_expr();
+    }
+    else if(value.substr(0, 8) == "(_ -oo ")
+    {
+      value = value.substr(8);
+      size_t pos = value.find(' ');
+      unsigned long e = string2integer(value.substr(0, pos), 10).to_ulong();
+      size_t pos2 = value.find(')');
+      unsigned long m =
+          string2integer(value.substr(pos+1, pos2-pos-1), 10).to_ulong();
+      //value = integer2binary(
+      //    ieee_floatt::minus_infinity(ieee_float_spect(m, e)).pack(), e+m+1);
+      it->second.value=ieee_floatt::minus_infinity(ieee_float_spect(m, e)).to_expr();
+    }
+    else if (value.substr(0, 7) == "(_ NaN ")
+    {
+      value = value.substr(7);
+      size_t pos = value.find(' ');
+      unsigned long e = string2integer(value.substr(0, pos), 10).to_ulong();
+      size_t pos2 = value.find(')');
+      unsigned long m =
+          string2integer(value.substr(pos+1, pos2-pos-1), 10).to_ulong();
+      //value = integer2binary(
+      //    ieee_floatt::NaN(ieee_float_spect(m, e)).pack(), e+m+1);
+      it->second.value=ieee_floatt::NaN(ieee_float_spect(m, e)).to_expr();
     }
-    set_value(it->second, value);
   }
 
   // Booleans
@@ -384,12 +472,49 @@ decision_proceduret::resultt smt2_dect::read_result_z3(std::istream &in)
       res = D_SATISFIABLE;
     else if(line=="unsat")
       res = D_UNSATISFIABLE;
+    // XXX -- this is a really nasty hack.  It'll disappear when I write a
+    // proper parser (Matt).
+    else if(line.substr(0, 11) == "(core_expr_")
+    {
+      // This is the unsat core.  It looks like
+      //
+      // (core_expr_N1 core_expr_N2 ... core_expr_Nk)
+      //
+      // where each Ni is an integer.
+      size_t start, end;
+
+      // Start scanning from the 2nd character, so we skip over the leading
+      // '('.
+      start = 1;
+
+      while (start < line.size()) {
+        for (end = start; line[end] != ' ' && line[end] != ')'; end++);
+
+        std::string core_name = line.substr(start, end-start);
+        exprt &core_expr = core_map[core_name];
+        unsat_core.insert(core_expr);
+
+        start = end+1;
+      }
+    }
     else
     {
-      std::size_t pos=line.find(" -> ");
-      if(pos!=std::string::npos)
-        values[std::string(line, 0, pos)]=
-          std::string(line, pos+4, std::string::npos);
+      // Values look like:
+      //
+      // ((identifer value))
+      size_t start, mid, end;
+
+      for (start = 0; start < line.size() && line[start] == '('; start++);
+      for (mid = start; mid < line.size() && line[mid] != ' '; mid++);
+      for (end = mid; end < line.size() && line[end] != ')'; end++);
+
+      if (start < line.size() && mid < line.size() && end < line.size())
+      {
+        std::string identifier = line.substr(start, mid-start);
+        std::string value = line.substr(mid+1, end-(mid+1));
+
+        values[identifier] = value;
+      }
     }
   }
 
@@ -403,12 +528,12 @@ decision_proceduret::resultt smt2_dect::read_result_z3(std::istream &in)
     std::string value=values[conv_id];
     if(value=="") continue;
 
-//    std::cout << it->first << " := " << value << std::endl;
+//    std::cout << it->first << " := " << value << "\n";
 
     exprt e;
     if(string_to_expr_z3(it->second.type, value, e))
     {
-//      std::cout << "E: " << e << std::endl;
+//      std::cout << "E: " << e << "\n";
       it->second.value=e;
     }
     else
@@ -484,7 +609,7 @@ bool smt2_dect::string_to_expr_z3(
   }
   else if(value.substr(0,6)=="(store")
   {
-//    std::cout << "STR: " << value << std::endl;
+//    std::cout << "STR: " << value << "\n";
 
     size_t p1=value.rfind(' ')+1;
     size_t p2=value.rfind(' ', p1-2)+1;
@@ -495,9 +620,9 @@ bool smt2_dect::string_to_expr_z3(
     std::string inx = value.substr(p2, p1-p2-1);
     std::string array = value.substr(7, p2-8);
 
-//    std::cout << "ELEM: " << elem << std::endl;
-//    std::cout << "INX: " << inx << std::endl;
-//    std::cout << "ARR: " << array << std::endl;
+//    std::cout << "ELEM: " << elem << "\n";
+//    std::cout << "INX: " << inx << "\n";
+//    std::cout << "ARR: " << array << "\n";
 
     exprt old;
     if(!string_to_expr_z3(type, array, old)) return false;
@@ -610,7 +735,7 @@ decision_proceduret::resultt smt2_dect::read_result_cvc3(std::istream &in)
             std::string t=var; var=val; val=t;
           }
 
-//          std::cout << "OPS: " << ops << std::endl;
+//          std::cout << "OPS: " << ops << "\n";
         }
         else if(line.substr(pos+1,3)=="not")
         {
@@ -624,8 +749,8 @@ decision_proceduret::resultt smt2_dect::read_result_cvc3(std::istream &in)
           val = "true";
         }
 
-//        std::cout << "VAR: " << var << std::endl;
-//        std::cout << "VAL: " << val << std::endl;
+//        std::cout << "VAR: " << var << "\n";
+//        std::cout << "VAL: " << val << "\n";
 
         values[var]=val;
       }
@@ -642,7 +767,7 @@ decision_proceduret::resultt smt2_dect::read_result_cvc3(std::istream &in)
     std::string value=values[conv_id];
     if(value=="") continue;
 
-//    std::cout << it->first << " := " << value << std::endl;
+//    std::cout << it->first << " := " << value << "\n";
 
     if(value.substr(0,2)=="bv")
     {
@@ -656,9 +781,9 @@ decision_proceduret::resultt smt2_dect::read_result_cvc3(std::istream &in)
       set_value(it->second, binary);
     }
     else if(value=="false")
-      it->second.value.make_false();
+      it->second.value=false_exprt();
     else if(value=="true")
-      it->second.value.make_true();
+      it->second.value=true_exprt();
     else if(value.substr(0,8)=="array_of")
     {
       // We assume that array_of has only concrete arguments...
