@@ -9,10 +9,10 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_PROP_SMT2_PROP_H
 #define CPROVER_PROP_SMT2_PROP_H
 
-#include <iostream>
+#include <ostream>
 #include <set>
 
-#include <threeval.h>
+#include <util/threeval.h>
 
 #include <solvers/prop/prop.h>
 
@@ -23,6 +23,7 @@ public:
     const std::string &_benchmark,
     const std::string &_source,
     const std::string &_logic,
+    bool _core_enabled,
     std::ostream &_out);
   virtual ~smt2_propt();
 
@@ -82,6 +83,8 @@ protected:
   
   typedef std::set<std::string> smt2_identifierst;
   smt2_identifierst smt2_identifiers;
+
+  bool core_enabled;
 };
 
 #endif
