@@ -6,10 +6,11 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
 #include "arith_tools.h"
 #include "std_types.h"
+#include "std_expr.h"
 
 /*******************************************************************\
 
@@ -26,13 +27,29 @@ Function: to_integer
 bool to_integer(const exprt &expr, mp_integer &int_value)
 {
   if(!expr.is_constant()) return true;
+  return to_integer(to_constant_expr(expr), int_value);
+}
 
-  const std::string &value=expr.get_string(ID_value);
+/*******************************************************************\
+
+Function: to_integer
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+bool to_integer(const constant_exprt &expr, mp_integer &int_value)
+{
+  const irep_idt &value=expr.get_value();
   const irep_idt &type_id=expr.type().id();
 
   if(type_id==ID_pointer)
   {
-    if(value=="NULL")
+    if(value==ID_NULL)
     {
       int_value=0;
       return false;
@@ -42,17 +59,17 @@ bool to_integer(const exprt &expr, mp_integer &int_value)
           type_id==ID_natural ||
           type_id==ID_c_enum)
   {
-    int_value=string2integer(value);
+    int_value=string2integer(id2string(value));
     return false;
   }
   else if(type_id==ID_unsignedbv)
   {
-    int_value=binary2integer(value, false);
+    int_value=binary2integer(id2string(value), false);
     return false;
   }
   else if(type_id==ID_signedbv)
   {
-    int_value=binary2integer(value, true);
+    int_value=binary2integer(id2string(value), true);
     return false;
   }
 
@@ -71,57 +88,61 @@ Function: from_integer
 
 \*******************************************************************/
 
-exprt from_integer(
+constant_exprt from_integer(
   const mp_integer &int_value,
   const typet &type)
 {
-  exprt expr;
-
-  expr.clear();
-  expr.type()=type;
-  expr.id(ID_constant);
-
   const irep_idt &type_id=type.id();
 
   if(type_id==ID_integer)
   {
-    expr.set(ID_value, integer2string(int_value));
-    return expr;
+    constant_exprt result(type);
+    result.set_value(integer2string(int_value));
+    return result;
   }
   else if(type_id==ID_natural)
   {
-    if(int_value<0) { expr.make_nil(); return expr; }
-    expr.set(ID_value, integer2string(int_value));
-    return expr;
+    if(int_value<0) { constant_exprt r; r.make_nil(); return r; }
+    constant_exprt result(type);
+    result.set_value(integer2string(int_value));
+    return result;
   }
   else if(type_id==ID_unsignedbv)
   {
     unsigned width=to_unsignedbv_type(type).get_width();
-    expr.set(ID_value, integer2binary(int_value, width));
-    return expr;
+    constant_exprt result(type);
+    result.set_value(integer2binary(int_value, width));
+    return result;
   }
   else if(type_id==ID_signedbv)
   {
     unsigned width=to_signedbv_type(type).get_width();
-    expr.set(ID_value, integer2binary(int_value, width));
-    return expr;
+    constant_exprt result(type);
+    result.set_value(integer2binary(int_value, width));
+    return result;
   }
   else if(type_id==ID_bool)
   {
     if(int_value==0)
-    {
-      expr.make_false();
-      return expr;
-    }
+      return false_exprt();
     else if(int_value==1)
+      return true_exprt();
+  }
+  else if(type_id==ID_pointer)
+  {
+    if(int_value==0)
     {
-      expr.make_true();
-      return expr;
+      constant_exprt result(type);
+      result.set_value(ID_NULL);
+      return result;
     }
   }
 
-  expr.make_nil();
-  return expr;
+  {
+    constant_exprt r;
+    r.make_nil();
+    return r;
+  }
 }
 
 /*******************************************************************\
