@@ -12,10 +12,16 @@ Author: Daniel Kroening, kroening@kroening.com
 #include "mp_arith.h"
 
 class exprt;
+class constant_exprt;
 class typet;
 
+// this one will go away
 bool to_integer(const exprt &expr, mp_integer &int_value);
-exprt from_integer(const mp_integer &int_value, const typet &type);
+
+// use this one
+bool to_integer(const constant_exprt &expr, mp_integer &int_value);
+
+constant_exprt from_integer(const mp_integer &int_value, const typet &type);
 
 // ceil(log2(size))
 mp_integer address_bits(const mp_integer &size);
