@@ -12,9 +12,10 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <list>
 
 #include "ieee_float.h"
+#include "irep.h"
 
 class cmdlinet;
-class contextt;
+class symbol_tablet;
 class namespacet;
 
 /*! \brief Globally accessible architectural configuration
@@ -37,8 +38,11 @@ public:
     unsigned long_double_width;
     unsigned wchar_t_width;
     
+    // various language options
     bool char_is_unsigned, wchar_t_is_unsigned;
     bool use_fixed_for_float;
+    bool for_has_scope;
+    bool single_precision_constant;
     
     ieee_floatt::rounding_modet rounding_mode;
 
@@ -53,7 +57,7 @@ public:
     void set_ILP32(); // int=32, long=32, pointer=32
     void set_LP32();  // int=16, long=32, pointer=32
 
-    void set_from_context(const contextt &context);
+    void set_from_symbol_table(const symbol_tablet &symbol_table);
     
     // minimum alignment (in structs) measured in bytes
     unsigned alignment;
@@ -69,12 +73,30 @@ public:
     ost os;
 
     typedef enum { NO_ARCH, ARCH_I386, ARCH_X86_64, ARCH_POWER, ARCH_ARM,
-                   ARCH_ALPHA, ARCH_MIPS } archt;
+                   ARCH_ALPHA, ARCH_MIPS, ARCH_S390, ARCH_S390X, ARCH_SPARC,
+                   ARCH_IA64, ARCH_X32 } archt;
     archt arch;
+
+    void set_arch_spec_i386();
+    void set_arch_spec_x86_64();
+    void set_arch_spec_power(const irep_idt &subarch);
+    void set_arch_spec_arm(const irep_idt &subarch);
+    void set_arch_spec_alpha();
+    void set_arch_spec_mips(const irep_idt &subarch);
+    void set_arch_spec_s390();
+    void set_arch_spec_s390x();
+    void set_arch_spec_sparc();
+    void set_arch_spec_ia64();
+    void set_arch_spec_x32();
     
-    typedef enum { NO_MODE, MODE_ANSI, MODE_GCC, MODE_VISUAL_STUDIO,
-                   MODE_CODEWARRIOR, MODE_ARM } modet;
-    modet mode;
+    typedef enum { NO_MODE, MODE_ANSI_C_CPP, MODE_GCC_C, MODE_GCC_CPP,
+                   MODE_VISUAL_STUDIO_C_CPP,
+                   MODE_CODEWARRIOR_C_CPP, MODE_ARM_C_CPP } flavourt;
+    flavourt mode; // the syntax of source files
+
+    typedef enum { NO_PP, PP_GCC, PP_VISUAL_STUDIO,
+                   PP_CODEWARRIOR, PP_ARM } preprocessort;
+    preprocessort preprocessor; // the preprocessor to use
 
     std::list<std::string> defines;
     std::list<std::string> undefines;
@@ -99,6 +121,9 @@ public:
   std::string main;
   
   bool set(const cmdlinet &cmdline);
+  
+  static irep_idt this_architecture();
+  static irep_idt this_operating_system();
 };
 
 extern configt config;
