@@ -9,7 +9,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_DECISION_PROCEDURE_H
 #define CPROVER_DECISION_PROCEDURE_H
 
-#include <message.h>
+#include "message.h"
 
 class exprt;
 class namespacet;
@@ -32,10 +32,10 @@ public:
   // the expression must be of Boolean type  
   virtual void set_to(const exprt &expr, bool value)=0;
   
-  void set_to_true(const exprt &expr)
+  inline void set_to_true(const exprt &expr)
   { set_to(expr, true); }
    
-  void set_to_false(const exprt &expr)
+  inline void set_to_false(const exprt &expr)
   { set_to(expr, false); }
   
   // solve the problem
@@ -43,7 +43,8 @@ public:
                  D_UNSATISFIABLE, D_ERROR } resultt;
   
   virtual resultt dec_solve()=0;
-  
+
+  // old-style, will go away  
   virtual bool in_core(const exprt &expr);
   
   // return a textual description of the decision procedure
@@ -53,4 +54,12 @@ protected:
   const namespacet &ns;
 };
 
+static inline decision_proceduret & operator << (
+  decision_proceduret &dest,
+  const exprt &src)
+{
+  dest.set_to_true(src);
+  return dest;
+}
+
 #endif
