@@ -37,7 +37,6 @@ exprt gen_zero(const typet &type)
      type_id==ID_real ||
      type_id==ID_integer ||
      type_id==ID_natural ||
-     type_id==ID_complex ||
      type_id==ID_c_enum)
   {
     result.set(ID_value, ID_0);
@@ -64,7 +63,7 @@ exprt gen_zero(const typet &type)
   }
   else if(type_id==ID_bool)
   {
-    result.make_false();
+    result=false_exprt();
   }
   else if(type_id==ID_pointer)
   {
@@ -202,74 +201,6 @@ exprt gen_binary(
 
 /*******************************************************************\
 
-Function: gen_binary
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-void gen_binary(exprt &expr, const irep_idt &id, bool default_value)
-{
-  if(expr.operands().size()==0)
-  {
-    if(default_value)
-      expr.make_true();
-    else
-      expr.make_false();
-  }
-  else if(expr.operands().size()==1)
-  {
-    exprt tmp;
-    tmp.swap(expr.op0());
-    expr.swap(tmp);
-  }
-  else
-  {
-    expr.id(id);
-    expr.type()=bool_typet();
-  }
-}
-
-/*******************************************************************\
-
-Function: gen_and
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-void gen_and(exprt &expr)
-{
-  gen_binary(expr, ID_and, true);
-}
-
-/*******************************************************************\
-
-Function: gen_or
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-void gen_or(exprt &expr)
-{
-  gen_binary(expr, ID_or, false);
-}
-
-/*******************************************************************\
-
 Function: symbol_expr
 
   Inputs:
