@@ -7,6 +7,7 @@ Author: Daniel Kroening, kroening@kroening.com
 \*******************************************************************/
 
 #include "std_types.h"
+#include "std_expr.h"
 
 #include "find_symbols.h"
 
@@ -146,6 +147,31 @@ Function: find_symbols
 
 \*******************************************************************/
 
+void find_symbols(
+  const exprt &src,
+  std::set<symbol_exprt> &dest)
+{
+  if(src.id()==ID_symbol)
+    dest.insert(to_symbol_expr(src));
+  else
+  {
+    forall_operands(it, src)
+      find_symbols(*it, dest);
+  }
+}
+
+/*******************************************************************\
+
+Function: find_symbols
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
 void find_symbols(kindt kind, const typet &src, find_symbols_sett &dest);
 
 void find_symbols(kindt kind, const exprt &src, find_symbols_sett &dest)
@@ -159,6 +185,11 @@ void find_symbols(kindt kind, const exprt &src, find_symbols_sett &dest)
     if(src.id()==ID_symbol ||
        src.id()==ID_next_symbol)
       dest.insert(src.get(ID_identifier));
+
+  const irept &c_sizeof_type=src.find(ID_C_c_sizeof_type);
+
+  if(c_sizeof_type.is_not_nil())
+    find_symbols(kind, static_cast<const typet &>(c_sizeof_type), dest);
 }
 
 /*******************************************************************\
