@@ -10,5 +10,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 std::string i2string(int i);
 std::string i2string(signed long int i);
-std::string i2string(unsigned i);
+std::string i2string(signed long long int i);
+std::string i2string(unsigned int i);
 std::string i2string(unsigned long int i);
+std::string i2string(unsigned long long int i);
+
