@@ -6,14 +6,15 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <assert.h>
-
-#include <arith_tools.h>
-#include <std_types.h>
-#include <math.h>
+// <cstdint> is yet to come
 #include <stdint.h>
+
+#include <cassert>
+#include <cmath>
 #include <limits>
 
+#include "arith_tools.h"
+#include "std_types.h"
 #include "std_expr.h"
 #include "ieee_float.h"
 
@@ -656,7 +657,7 @@ ieee_floatt &ieee_floatt::operator /= (const ieee_floatt &other)
   // NaN/x = NaN
   if(NaN_flag) return *this;
   
-  // x/Nan = NaN
+  // x/NaN = NaN
   if(other.NaN_flag) { make_NaN(); return *this; }
   
   // 0/0 = NaN
