@@ -6,8 +6,12 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <stdlib.h>
-#include <assert.h>
+#include <cstdlib>
+#include <cassert>
+
+#ifdef IREP_DEBUG
+#include <iostream>
+#endif
 
 #include "irep.h"
 #include "i2string.h"
@@ -88,6 +92,43 @@ void irept::detatch()
 
 /*******************************************************************\
 
+Function: irept::recursive_detatch
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+#ifdef SHARING
+void irept::recursive_detatch()
+{
+  detatch();
+  
+  for(named_subt::iterator
+      it=data->named_sub.begin();
+      it!=data->named_sub.end();
+      it++)
+    it->second.recursive_detatch();
+    
+  for(named_subt::iterator
+      it=data->comments.begin();
+      it!=data->comments.end();
+      it++)
+    it->second.recursive_detatch();
+    
+  for(subt::iterator
+      it=data->sub.begin();
+      it!=data->sub.end();
+      it++)
+    it->recursive_detatch();
+}
+#endif
+
+/*******************************************************************\
+
 Function: irept::remove_ref
 
   Inputs:
@@ -98,8 +139,6 @@ Function: irept::remove_ref
 
 \*******************************************************************/
 
-#include <iostream>
-  
 #ifdef SHARING
 void irept::remove_ref(dt *old_data)
 {
@@ -298,23 +337,6 @@ void irept::remove(const irep_namet &name)
 
 /*******************************************************************\
 
-Function: irept::set
-
-  Inputs:
-
- Outputs:
-
- Purpose:
-
-\*******************************************************************/
-
-void irept::set(const irep_namet &name, const irept &irep)
-{
-  add(name)=irep;
-}  
-
-/*******************************************************************\
-
 Function: irept::find
 
   Inputs:
@@ -360,6 +382,32 @@ irept &irept::add(const irep_namet &name)
 
 /*******************************************************************\
 
+Function: irept::add
+
+  Inputs:
+
+ Outputs:
+
+ Purpose:
+
+\*******************************************************************/
+
+irept &irept::add(const irep_namet &name, const irept &irep)
+{
+  named_subt &s=
+    is_comment(name)?get_comments():get_named_sub();
+
+  std::pair<named_subt::iterator, bool> entry=
+    s.insert(std::make_pair(name, irep));
+
+  if(!entry.second)
+    entry.first->second=irep;
+
+  return entry.first->second;
+}
+
+/*******************************************************************\
+
 Function: operator==
 
   Inputs:
@@ -648,6 +696,16 @@ Function: irept::hash
 
 \*******************************************************************/
 
+static inline size_t hash_rotl(const size_t value, int shift)
+{
+  return (value << shift) | (value >> (sizeof(value)*8 - shift));
+}
+
+static inline size_t hash_combine(size_t h1, size_t h2)
+{
+  return hash_rotl(h1, 7)^h2;
+}
+
 size_t irept::hash() const
 {
   const irept::subt &sub=get_sub();
@@ -655,12 +713,12 @@ size_t irept::hash() const
 
   size_t result=hash_string(id());
 
-  forall_irep(it, sub) result=result^it->hash();
+  forall_irep(it, sub) result=hash_combine(result, it->hash());
 
   forall_named_irep(it, named_sub)
   {
-    result=result^hash_string(it->first);
-    result=result^it->second.hash();
+    result=hash_combine(result, hash_string(it->first));
+    result=hash_combine(result, it->second.hash());
   }
 
   return result;
@@ -686,18 +744,18 @@ size_t irept::full_hash() const
 
   size_t result=hash_string(id());
 
-  forall_irep(it, sub) result=result^it->full_hash();
+  forall_irep(it, sub) result=hash_combine(result, it->full_hash());
 
   forall_named_irep(it, named_sub)
   {
-    result=result^hash_string(it->first);
-    result=result^it->second.full_hash();
+    result=hash_combine(result, hash_string(it->first));
+    result=hash_combine(result, it->second.full_hash());
   }
 
   forall_named_irep(it, comments)
   {
-    result=result^hash_string(it->first);
-    result=result^it->second.full_hash();
+    result=hash_combine(result, hash_string(it->first));
+    result=hash_combine(result, it->second.full_hash());
   }
 
   return result;
@@ -752,11 +810,7 @@ std::string irept::pretty(unsigned indent, unsigned max_indent) const
     indent_str(result, indent);
 
     result+="* ";
-    #ifdef USE_DSTRING
-    result+=it->first.as_string();
-    #else
-    result+=it->first;
-    #endif
+    result+=id2string(it->first);
     result+=": ";
 
     result+=it->second.pretty(indent+2, max_indent);
@@ -768,11 +822,7 @@ std::string irept::pretty(unsigned indent, unsigned max_indent) const
     indent_str(result, indent);
 
     result+="* ";
-    #ifdef USE_DSTRING
-    result+=it->first.as_string();
-    #else
-    result+=it->first;
-    #endif
+    result+=id2string(it->first);
     result+=": ";
 
     result+=it->second.pretty(indent+2, max_indent);
