@@ -13,7 +13,7 @@ Author: Daniel Kroening, kroening@kroening.com
 #include <map>
 #include <string>
 #include <cassert>
-#include <ostream>
+#include <iosfwd>
 
 #define USE_DSTRING
 #define SHARING
@@ -35,6 +35,24 @@ typedef std::string irep_namet;
 typedef string_hash irep_id_hash;
 #endif
 
+extern inline const std::string &id2string(const irep_idt &d)
+{
+  #ifdef USE_DSTRING
+  return as_string(d);
+  #else
+  return d;
+  #endif
+}
+
+extern inline const std::string &name2string(const irep_namet &n)
+{
+  #ifdef USE_DSTRING
+  return as_string(n);
+  #else
+  return n;
+  #endif
+}
+
 #define forall_irep(it, irep) \
   for(irept::subt::const_iterator it=(irep).begin(); \
       it!=(irep).end(); ++it)
@@ -72,10 +90,12 @@ public:
   }
 
   #ifdef SHARING
+  // constructor for blank irep
   inline irept():data(&empty_d)
   {
   }
 
+  // copy constructor
   inline irept(const irept &irep):data(irep.data)
   {
     if(data!=&empty_d)
@@ -88,6 +108,19 @@ public:
     }
   }
 
+  #ifdef USE_MOVE
+  // Copy from rvalue reference.
+  // Note that this does avoid a branch compared to the
+  // standard copy constructor above.
+  inline irept(const irept &&irep):data(irep.data)
+  {
+    #ifdef IREP_DEBUG
+    std::cout << "COPY MOVE\n";
+    #endif
+    irep.data=&empty_d;
+  }
+  #endif
+
   inline irept &operator=(const irept &irep)
   {
     #ifdef IREP_DEBUG
@@ -105,11 +138,26 @@ public:
     return *this;
   }
 
+  #ifdef USE_MOVE
+  // Note that the move assignment operator does avoid
+  // three branches compared to standard operator above.
+  inline irept &operator=(const irept &&irep)
+  {
+    #ifdef IREP_DEBUG
+    std::cout << "ASSIGN MOVE\n";
+    #endif
+    // we simply swap two pointers
+    std::swap(data, irep.data);
+    return *this;
+  }
+  #endif
+  
   inline ~irept()
   {
     remove_ref(data);
     data=NULL;
   }
+  
   #else
   inline irept()
   {
@@ -119,27 +167,19 @@ public:
   inline const irep_idt &id() const
   { return read().data; }
   
-  #ifdef USE_DSTRING
   inline const std::string &id_string() const
-  { return read().data.as_string(); }
-  #else
-  inline const std::string &id_string() const
-  { return read().data; }
-  #endif
+  { return id2string(read().data); }
 
   inline void id(const irep_idt &_data)
   { write().data=_data; }
 
   const irept &find(const irep_namet &name) const;
   irept &add(const irep_namet &name);
+  irept &add(const irep_namet &name, const irept &irep);
 
   inline const std::string &get_string(const irep_namet &name) const
   {
-    #ifdef USE_DSTRING
-    return get(name).as_string();
-    #else
-    return get(name);
-    #endif
+    return id2string(get(name));
   }
   
   const irep_idt &get(const irep_namet &name) const;
@@ -148,9 +188,10 @@ public:
 
   inline void set(const irep_namet &name, const irep_idt &value)
   { add(name).id(value); }
-  
+  inline void set(const irep_namet &name, const irept &irep)
+  { add(name, irep); }
   void set(const irep_namet &name, const long value);
-  void set(const irep_namet &name, const irept &irep);
+  
   void remove(const irep_namet &name);
   void move_to_sub(irept &irep);
   void move_to_named_sub(const irep_namet &name, irept &irep);
@@ -265,6 +306,8 @@ public:
     return *data;
   }
   
+  void recursive_detatch();
+  
   #else
   dt data;
 
@@ -281,24 +324,6 @@ public:
   #endif
 };
 
-extern inline const std::string &id2string(const irep_idt &d)
-{
-  #ifdef USE_DSTRING
-  return d.as_string();
-  #else
-  return d;
-  #endif
-}
-
-extern inline const std::string &name2string(const irep_namet &n)
-{
-  #ifdef USE_DSTRING
-  return n.as_string();
-  #else
-  return n;
-  #endif
-}
-
 struct irep_hash
 {
   inline size_t operator()(const irept &irep) const { return irep.hash(); }
