@@ -8,8 +8,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <fstream>
 
-#include <i2string.h>
-
+#include "i2string.h"
 #include "language.h"
 #include "language_file.h"
   
@@ -142,14 +141,14 @@ Function: language_filest::typecheck
 
 \*******************************************************************/
 
-bool language_filest::typecheck(contextt &context)
+bool language_filest::typecheck(symbol_tablet &symbol_table)
 {
   // typecheck interfaces
 
   for(filemapt::iterator it=filemap.begin();
       it!=filemap.end(); it++)
   {
-    if(it->second.language->interfaces(context, get_message_handler()))
+    if(it->second.language->interfaces(symbol_table, get_message_handler()))
       return true;
   }
 
@@ -191,7 +190,7 @@ bool language_filest::typecheck(contextt &context)
       it!=filemap.end(); it++)
   {
     if(it->second.modules.empty())
-      if(it->second.language->typecheck(context, "", get_message_handler()))
+      if(it->second.language->typecheck(symbol_table, "", get_message_handler()))
         return true;
   }
 
@@ -200,7 +199,7 @@ bool language_filest::typecheck(contextt &context)
   for(modulemapt::iterator it=modulemap.begin();
       it!=modulemap.end(); it++)
   {
-    if(typecheck_module(context, it->second))
+    if(typecheck_module(symbol_table, it->second))
       return true;
   }
 
@@ -220,7 +219,7 @@ Function: language_filest::final
 \*******************************************************************/
 
 bool language_filest::final(
-  contextt &context)
+  symbol_tablet &symbol_table)
 {
   std::set<std::string> languages;
 
@@ -228,7 +227,7 @@ bool language_filest::final(
       it!=filemap.end(); it++)
   {
     if(languages.insert(it->second.language->id()).second)
-      if(it->second.language->final(context, get_message_handler()))
+      if(it->second.language->final(symbol_table, get_message_handler()))
         return true;
   }
 
@@ -248,12 +247,12 @@ Function: language_filest::interfaces
 \*******************************************************************/
 
 bool language_filest::interfaces(
-  contextt &context)
+  symbol_tablet &symbol_table)
 {
   for(filemapt::iterator it=filemap.begin();
       it!=filemap.end(); it++)
   {
-    if(it->second.language->interfaces(context, get_message_handler()))
+    if(it->second.language->interfaces(symbol_table, get_message_handler()))
       return true;
   }
 
@@ -273,7 +272,7 @@ Function: language_filest::typecheck_module
 \*******************************************************************/
 
 bool language_filest::typecheck_module(
-  contextt &context,
+  symbol_tablet &symbol_table,
   const std::string &module)
 {
   // check module map
@@ -286,7 +285,7 @@ bool language_filest::typecheck_module(
     return true;
   }
 
-  return typecheck_module(context, it->second);
+  return typecheck_module(symbol_table, it->second);
 }
 
 /*******************************************************************\
@@ -302,7 +301,7 @@ Function: language_filest::typecheck_module
 \*******************************************************************/
 
 bool language_filest::typecheck_module(
-  contextt &context,
+  symbol_tablet &symbol_table,
   language_modulet &module)
 {
   // already typechecked?
@@ -331,7 +330,7 @@ bool language_filest::typecheck_module(
       it!=dependency_set.end();
       it++)
   {
-    if(typecheck_module(context, *it))
+    if(typecheck_module(symbol_table, *it))
     {
       module.in_progress=false;
       return true;
@@ -342,7 +341,7 @@ bool language_filest::typecheck_module(
 
   status("Type-checking "+module.name);
 
-  if(module.file->language->typecheck(context, module.name, get_message_handler()))
+  if(module.file->language->typecheck(symbol_table, module.name, get_message_handler()))
   {
     module.in_progress=false;
     return true;
