@@ -41,6 +41,11 @@ public:
     return get(ID_property);
   }
 
+  inline const irep_idt &get_source() const
+  {
+    return get(ID_source);
+  }
+
   inline const irep_idt &get_comment() const
   {
     return get(ID_comment);
@@ -86,6 +91,11 @@ public:
     set(ID_property, property);
   }
 
+  inline void set_source(const irep_idt &source)
+  {
+    set(ID_source, source);
+  }
+
   inline void set_comment(const irep_idt &comment)
   {
     set(ID_comment, comment);
