@@ -6,10 +6,11 @@ Author: Daniel Kroening, kroening@kroening.com
 
 \*******************************************************************/
 
-#include <stdlib.h>
-#include <ctype.h>
+#include <cstdlib>
+#include <cctype>
 
 #include <sstream>
+#include <ostream>
 
 #include "mp_arith.h"
 #include "arith_tools.h"
@@ -26,8 +27,6 @@ Function: >>
 
 \*******************************************************************/
 
-#include <iostream>
-
 mp_integer operator>>(const mp_integer &a, const mp_integer &b)
 {
   mp_integer power=::power(2, b);
@@ -192,31 +191,64 @@ Function: binary2integer
 
 const mp_integer binary2integer(const std::string &n, bool is_signed)
 {
-  if(n.size()==0) return 0;
+  if(n.size()==0 || n.find_first_not_of("01")!=std::string::npos)
+    return 0;
+
+  if(n.size()<=(sizeof(unsigned long)*8))
+  {
+    // this is a tuned implementation for short integers
+
+    unsigned long mask=1;
+    mask=mask << (n.size()-1);
+    mp_integer result=(n[0]=='1') ? mask : 0;
+    if(is_signed) result.negate();
+    mask>>=1;
+
+    for(std::string::const_iterator it=++n.begin();
+        it!=n.end();
+        ++it)
+    {
+      if(*it=='1')
+        result+=mask;
+
+      mask>>=1;
+    }
+
+    return result;
+  }
+
+  #if 0
 
-  mp_integer result=0;
   mp_integer mask=1;
   mask=mask << (n.size()-1);
+  mp_integer result=(n[0]=='1') ? mask : 0;
+  if(is_signed) result.negate();
+  mask=mask>>1;
 
-  for(unsigned i=0; i<n.size(); i++)
+  for(std::string::const_iterator it=++n.begin();
+      it!=n.end();
+      ++it)
   {
-    if(n[i]=='0')
-    {
-    }
-    else if(n[i]=='1')
-    {
-      if(is_signed && i==0)
-        result=-mask;
-      else
-        result=result+mask;
-    }
-    else
-      return 0;
+    if(*it=='1')
+      result+=mask;
 
     mask=mask>>1;
   }
 
   return result;
+
+  #else
+
+  if(is_signed && n[0]=='1')
+  {
+    mp_integer result(n.c_str()+1, 2);
+    result-=mp_integer(1)<<(n.size()-1);
+    return result;
+  }
+  else
+    return BigInt(n.c_str(), 2);
+
+  #endif
 }
 
 /*******************************************************************\
