@@ -9,8 +9,8 @@ Author: Daniel Kroening, kroening@kroening.com
 #ifndef CPROVER_POINTER_OFFSET_SIZE_H
 #define CPROVER_POINTER_OFFSET_SIZE_H
 
-#include <mp_arith.h>
-#include <irep.h>
+#include "mp_arith.h"
+#include "irep.h"
 
 class exprt;
 class namespacet;
@@ -18,6 +18,7 @@ class struct_typet;
 class typet;
 
 // these return -1 on failure
+
 mp_integer member_offset(
   const namespacet &ns,
   const struct_typet &type,
@@ -31,4 +32,15 @@ mp_integer compute_pointer_offset(
   const namespacet &ns,
   const exprt &expr);
 
+// these return 'nil' on failure
+
+exprt member_offset_expr(
+  const struct_typet &type,
+  const irep_idt &member,
+  const namespacet &ns);
+
+exprt size_of_expr(
+  const typet &type,
+  const namespacet &ns);
+
 #endif
