@@ -14,8 +14,8 @@ Author: Daniel Kroening, kroening@kroening.com
 // false: replaced something
 //
 
-#include <hash_cont.h>
-#include <expr.h>
+#include "hash_cont.h"
+#include "expr.h"
 
 class replace_symbolt
 {
@@ -23,20 +23,33 @@ public:
   typedef hash_map_cont<irep_idt, exprt, irep_id_hash> expr_mapt;
   typedef hash_map_cont<irep_idt, typet, irep_id_hash> type_mapt;
   
-  void insert(const irep_idt &identifier,
-              const exprt &expr)
+  inline void insert(const irep_idt &identifier,
+                     const exprt &expr)
   {
     expr_map.insert(std::pair<irep_idt, exprt>(identifier, expr));
   }
   
-  void insert(const irep_idt &identifier,
-              const typet &type)
+  void insert(const class symbol_exprt &old_expr,
+              const exprt &new_expr);
+
+  inline void insert(const irep_idt &identifier,
+                     const typet &type)
   {
     type_map.insert(std::pair<irep_idt, typet>(identifier, type));
   }
   
   virtual bool replace(exprt &dest) const;
   virtual bool replace(typet &dest) const;
+  
+  inline void operator()(exprt &dest) const
+  {
+    replace(dest);
+  }
+
+  inline void operator()(typet &dest) const
+  {
+    replace(dest);
+  }
 
   replace_symbolt();
   virtual ~replace_symbolt();
