@@ -11,7 +11,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <cassert>
 
-#include <expr.h>
+#include "expr.h"
 
 /*! \brief A statement in a programming language
 */
@@ -81,6 +81,12 @@ public:
   {
     copy_to_operands(code);
   }
+  
+  // This is the closing '}' or 'END' at the end of a block
+  locationt end_location() const
+  {
+    return static_cast<const locationt &>(find(ID_C_end_location));
+  }
 };
 
 extern inline const code_blockt &to_code_block(const codet &code)
@@ -177,27 +183,31 @@ public:
   {
     return op0();
   }
-  
+
+  #if 0  
   inline exprt &initializer()
   {
-    return op0();
+    return op1();
   }
 
   inline const exprt &initializer() const
   {
-    return op0();
+    return op1();
   }
+  #endif
   
   const irep_idt &get_identifier() const;
 
   friend inline const code_declt &to_code_decl(const codet &code)
   {
+    // will be size()==1 in the future
     assert(code.get_statement()==ID_decl && code.operands().size()>=1);
     return static_cast<const code_declt &>(code);
   }
 
   friend inline code_declt &to_code_decl(codet &code)
   {
+    // will be size()==1 in the future
     assert(code.get_statement()==ID_decl && code.operands().size()>=1);
     return static_cast<code_declt &>(code);
   }
@@ -207,6 +217,50 @@ public:
 const code_declt &to_code_decl(const codet &code);
 code_declt &to_code_decl(codet &code);
 
+/*! \brief A removal of a local variable
+*/
+class code_deadt:public codet
+{
+public:
+  inline code_deadt():codet(ID_dead)
+  {
+    operands().resize(1);
+  }
+  
+  inline explicit code_deadt(const exprt &symbol):codet(ID_dead)
+  {
+    copy_to_operands(symbol);
+  }
+  
+  inline exprt &symbol()
+  {
+    return op0();
+  }
+
+  inline const exprt &symbol() const
+  {
+    return op0();
+  }
+
+  const irep_idt &get_identifier() const;
+
+  friend inline const code_deadt &to_code_dead(const codet &code)
+  {
+    assert(code.get_statement()==ID_dead && code.operands().size()==1);
+    return static_cast<const code_deadt &>(code);
+  }
+
+  friend inline code_deadt &to_code_dead(codet &code)
+  {
+    assert(code.get_statement()==ID_dead && code.operands().size()==1);
+    return static_cast<code_deadt &>(code);
+  }
+
+};
+
+const code_deadt &to_code_dead(const codet &code);
+code_deadt &to_code_dead(codet &code);
+
 /*! \brief An assumption
 */
 class code_assumet:public codet
@@ -214,6 +268,7 @@ class code_assumet:public codet
 public:
   inline code_assumet():codet(ID_assume)
   {
+    // will change to resize(1) in the future
     operands().reserve(1);
   }
 
@@ -222,12 +277,12 @@ public:
     copy_to_operands(expr);
   }
 
-  inline const exprt assumption() const
+  inline const exprt &assumption() const
   {
     return op0();
   }
 
-  inline exprt assumption()
+  inline exprt &assumption()
   {
     return op0();
   }
@@ -252,15 +307,21 @@ class code_assertt:public codet
 public:
   inline code_assertt():codet(ID_assert)
   {
+    // will change to resize(1) in the future
     operands().reserve(1);
   }
   
-  inline const exprt assertion() const
+  inline explicit code_assertt(const exprt &expr):codet(ID_assert)
+  {
+    copy_to_operands(expr);
+  }
+
+  inline const exprt &assertion() const
   {
     return op0();
   }
 
-  inline exprt assertion()
+  inline exprt &assertion()
   {
     return op0();
   }
@@ -285,7 +346,9 @@ class code_ifthenelset:public codet
 public:
   inline code_ifthenelset():codet(ID_ifthenelse)
   {
-    operands().reserve(3);
+    operands().resize(3);
+    op1().make_nil();
+    op2().make_nil();
   }
   
   inline const exprt &cond() const
@@ -300,12 +363,12 @@ public:
   
   inline const codet &then_case() const
   {
-    return to_code(op1());
+    return static_cast<const codet &>(op1());
   }
 
-  inline const codet &else_case() const
+  inline const exprt &else_case() const
   {
-    return to_code(op2());
+    return static_cast<const codet &>(op2());
   }
 
   inline codet &then_case()
@@ -313,7 +376,7 @@ public:
     return static_cast<codet &>(op1());
   }
 
-  inline codet &else_case()
+  inline exprt &else_case()
   {
     return static_cast<codet &>(op2());
   }
@@ -321,13 +384,15 @@ public:
 
 extern inline const code_ifthenelset &to_code_ifthenelse(const codet &code)
 {
-  assert(code.get_statement()==ID_ifthenelse);
+  assert(code.get_statement()==ID_ifthenelse &&
+         code.operands().size()==3);
   return static_cast<const code_ifthenelset &>(code);
 }
 
 extern inline code_ifthenelset &to_code_ifthenelse(codet &code)
 {
-  assert(code.get_statement()==ID_ifthenelse);
+  assert(code.get_statement()==ID_ifthenelse &&
+         code.operands().size()==3);
   return static_cast<code_ifthenelset &>(code);
 }
 
@@ -376,7 +441,6 @@ extern inline code_switcht &to_code_switch(codet &code)
   return static_cast<code_switcht &>(code);
 }
 
-
 /*! \brief A `while' instruction
 */
 class code_whilet:public codet
@@ -476,15 +540,16 @@ public:
   {
     operands().resize(4);
   }
-  
-  inline const codet &init() const
+
+  // nil or a statement  
+  inline const exprt &init() const
   {
-    return to_code(op0());
+    return op0();
   }
 
-  inline codet &init()
+  inline exprt &init()
   {
-    return static_cast<codet &>(op0());
+    return op0();
   }
 
   inline const exprt &cond() const
@@ -532,6 +597,45 @@ extern inline code_fort &to_code_for(codet &code)
   return static_cast<code_fort &>(code);
 }
 
+/*! \brief A `goto' instruction
+*/
+class code_gotot:public codet
+{
+public:
+  inline code_gotot():codet(ID_goto)
+  {
+  }
+
+  explicit inline code_gotot(const irep_idt &label):codet(ID_goto)
+  {
+    set_destination(label);
+  }
+  
+  void set_destination(const irep_idt &label)
+  {
+    set(ID_destination, label);
+  }
+
+  const irep_idt &get_destination() const
+  {
+    return get(ID_destination);
+  }
+};
+
+extern inline const code_gotot &to_code_goto(const codet &code)
+{
+  assert(code.get_statement()==ID_goto &&
+         code.operands().size()==0);
+  return static_cast<const code_gotot &>(code);
+}
+
+extern inline code_gotot &to_code_goto(codet &code)
+{
+  assert(code.get_statement()==ID_goto &&
+         code.operands().size()==0);
+  return static_cast<code_gotot &>(code);
+}
+
 /*! \brief A function call
     
     The function call instruction has three operands.
@@ -648,6 +752,20 @@ public:
     operands().resize(1);
   }
 
+  explicit inline code_labelt(const irep_idt &_label):codet(ID_label)
+  {
+    operands().resize(1);
+    set_label(_label);
+  }
+
+  inline code_labelt(
+    const irep_idt &_label, const codet &_code):codet(ID_label)
+  {
+    operands().resize(1);
+    set_label(_label);
+    code()=_code;
+  }
+
   inline bool is_default() const
   {
     return get_bool(ID_default);
@@ -885,4 +1003,98 @@ public:
 side_effect_expr_function_callt &to_side_effect_expr_function_call(exprt &expr);
 const side_effect_expr_function_callt &to_side_effect_expr_function_call(const exprt &expr);
 
+/*! \brief A side effect that throws an exception
+*/
+class side_effect_expr_throwt:public side_effect_exprt
+{
+public:
+  inline side_effect_expr_throwt():side_effect_exprt(ID_throw)
+  {
+  }
+
+  inline explicit side_effect_expr_throwt(const irept &exception_list):
+    side_effect_exprt(ID_throw)
+  {
+    set(ID_exception_list, exception_list);
+  }
+};
+
+extern inline side_effect_expr_throwt &to_side_effect_expr_throw(exprt &expr)
+{
+  assert(expr.id()==ID_sideeffect);
+  assert(expr.get(ID_statement)==ID_throw);
+  return static_cast<side_effect_expr_throwt &>(expr);
+}
+
+extern inline const side_effect_expr_throwt &to_side_effect_expr_throw(const exprt &expr)
+{
+  assert(expr.id()==ID_sideeffect);
+  assert(expr.get(ID_statement)==ID_throw);
+  return static_cast<const side_effect_expr_throwt &>(expr);
+}
+
+/*! \brief A try/catch block
+*/
+class code_try_catcht:public codet
+{
+public:
+  inline code_try_catcht():codet(ID_try_catch)
+  {
+    operands().resize(1);
+  }
+
+  codet &try_code()
+  {
+    return static_cast<codet &>(op0());
+  }
+
+  const codet &try_code() const
+  {
+    return static_cast<const codet &>(op0());
+  }
+
+  code_declt &get_catch_decl(unsigned i)
+  {
+    assert((2*i+2)<operands().size());
+    return to_code_decl(to_code(operands()[2*i+1]));
+  }
+
+  const code_declt &get_catch_decl(unsigned i) const
+  {
+    assert((2*i+2)<operands().size());
+    return to_code_decl(to_code(operands()[2*i+1]));
+  }
+
+  codet &get_catch_code(unsigned i)
+  {
+    assert((2*i+2)<operands().size());
+    return to_code(operands()[2*i+2]);
+  }
+
+  const codet &get_catch_code(unsigned i) const
+  {
+    assert((2*i+2)<operands().size());
+    return to_code(operands()[2*i+2]);
+  }
+
+  void add_catch(const code_declt &to_catch, const codet &code_catch)
+  {
+    operands().reserve(operands().size()+2);
+    copy_to_operands(to_catch);
+    copy_to_operands(code_catch);
+  }
+};
+
+extern inline const code_try_catcht &to_code_try_catch(const codet &code)
+{
+  assert(code.get_statement()==ID_try_catch && code.operands().size()>=3);
+  return static_cast<const code_try_catcht &>(code);
+}
+
+extern inline code_try_catcht &to_code_try_catch(codet &code)
+{
+  assert(code.get_statement()==ID_try_catch && code.operands().size()>=3);
+  return static_cast<code_try_catcht &>(code);
+}
+
 #endif
