@@ -18,7 +18,7 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <cassert>
 
-#include <std_types.h>
+#include "std_types.h"
 
 /*! \defgroup gr_std_expr Conversion to specific expressions
  *  Conversion to subclasses of @ref exprt
@@ -196,9 +196,8 @@ public:
 class predicate_exprt:public exprt
 {
 public:
-  inline predicate_exprt()
+  inline predicate_exprt():exprt("", typet(ID_bool))
   {
-    type()=typet(ID_bool);
   }
 
   explicit inline predicate_exprt(const irep_idt &_id):
@@ -526,6 +525,90 @@ extern inline mod_exprt &to_mod_expr(exprt &expr)
   return static_cast<mod_exprt &>(expr);
 }
 
+/*! \brief exponentiation
+ */
+class power_exprt:public binary_exprt
+{
+ public:
+  inline power_exprt():binary_exprt(ID_power)
+  {
+  }
+
+  inline power_exprt(
+      const exprt &_base,
+      const exprt &_exp):
+      binary_exprt(_base, ID_power, _exp)
+  {
+  }
+};
+
+/*! \brief Cast a generic exprt to a \ref power_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * power_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref power_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const power_exprt &to_power_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_power && expr.operands().size()==2);
+  return static_cast<const power_exprt &>(expr);
+}
+
+/*! \copydoc to_power_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline power_exprt &to_power_expr(exprt &expr)
+{
+  assert(expr.id()==ID_power && expr.operands().size()==2);
+  return static_cast<power_exprt &>(expr);
+}
+
+/*! \brief falling factorial power
+ */
+class factorial_power_exprt:public binary_exprt
+{
+ public:
+  inline factorial_power_exprt():binary_exprt(ID_factorial_power)
+  {
+  }
+
+  inline factorial_power_exprt(
+      const exprt &_base,
+      const exprt &_exp):
+      binary_exprt(_base, ID_factorial_power, _exp)
+  {
+  }
+};
+
+/*! \brief Cast a generic exprt to a \ref factorial_power_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * factorial_power_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref factorial_power_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const factorial_power_exprt &to_factorial_power_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_factorial_power && expr.operands().size()==2);
+  return static_cast<const factorial_power_exprt &>(expr);
+}
+
+/*! \copydoc to_factorial_power_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline factorial_power_exprt &to_factorial_expr(exprt &expr)
+{
+  assert(expr.id()==ID_factorial_power && expr.operands().size()==2);
+  return static_cast<factorial_power_exprt &>(expr);
+}
+
 /*! \brief equality
 */
 class equal_exprt:public binary_relation_exprt
@@ -566,6 +649,46 @@ extern inline equal_exprt &to_equal_expr(exprt &expr)
   return static_cast<equal_exprt &>(expr);
 }
 
+/*! \brief inequality
+*/
+class notequal_exprt:public binary_relation_exprt
+{
+public:
+  inline notequal_exprt():binary_relation_exprt(ID_notequal)
+  {
+  }
+
+  inline notequal_exprt(const exprt &_lhs, const exprt &_rhs):
+    binary_relation_exprt(_lhs, ID_notequal, _rhs)
+  {
+  }
+};
+
+/*! \brief Cast a generic exprt to an \ref notequal_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * notequal_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref notequal_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const notequal_exprt &to_notequal_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_notequal && expr.operands().size()==2);
+  return static_cast<const notequal_exprt &>(expr);
+}
+
+/*! \copydoc to_notequal_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline notequal_exprt &to_notequal_expr(exprt &expr)
+{
+  assert(expr.id()==ID_notequal && expr.operands().size()==2);
+  return static_cast<notequal_exprt &>(expr);
+}
+
 /*! \brief array index operator
 */
 class index_exprt:public exprt
@@ -656,11 +779,9 @@ public:
   }
  
   explicit inline array_of_exprt(
-    const exprt &_what, const typet &_type):exprt(ID_array_of)
+    const exprt &_what, const typet &_type):exprt(ID_array_of, _type)
   {
-    operands().resize(1);
-    op0()=_what;
-    type()=_type;
+    copy_to_operands(_what);
   }
  
   inline exprt &what()
@@ -1113,17 +1234,54 @@ public:
     copy_to_operands(op0, op1);
   }
 
-  and_exprt(const exprt::operandst &op):exprt(ID_and, bool_typet())
+  inline and_exprt(const exprt &op0, const exprt &op1, const exprt &op2):exprt(ID_and, typet(ID_bool))
   {
-    if(op.empty())
-      make_true();
-    else if(op.size()==1)
-      *this=static_cast<const and_exprt &>(op.front());
-    else
-      operands()=op;
+    copy_to_operands(op0, op1, op2);
+  }
+
+  inline and_exprt(const exprt &op0, const exprt &op1, const exprt &op2, const exprt &op3):exprt(ID_and, typet(ID_bool))
+  {
+    exprt::operandst &op=operands();
+    op.resize(4);
+    op[0]=op0;
+    op[1]=op1;
+    op[2]=op2;
+    op[3]=op3;
   }
 };
 
+/*! 1) generates a conjunction for two or more operands
+ *  2) for one operand, returns the operand
+ *  3) returns true otherwise 
+*/
+
+exprt conjunction(const exprt::operandst &op);
+
+/*! \brief Cast a generic exprt to a \ref typecast_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * and_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref and_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const and_exprt &to_and_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_and);
+  return static_cast<const and_exprt &>(expr);
+}
+
+/*! \copydoc to_and_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline and_exprt &to_and_expr(exprt &expr)
+{
+  assert(expr.id()==ID_and);
+  return static_cast<and_exprt &>(expr);
+}
+
 /*! \brief boolean implication
 */
 class implies_exprt:public binary_exprt
@@ -1139,6 +1297,31 @@ public:
   }
 };
 
+/*! \brief Cast a generic exprt to a \ref implies_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * implies_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref implies_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const implies_exprt &to_implies_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_implies && expr.operands().size()==2);
+  return static_cast<const implies_exprt &>(expr);
+}
+
+/*! \copydoc to_implies_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline implies_exprt &to_implies_expr(exprt &expr)
+{
+  assert(expr.id()==ID_implies && expr.operands().size()==2);
+  return static_cast<implies_exprt &>(expr);
+}
+
 /*! \brief boolean OR
 */
 class or_exprt:public exprt
@@ -1153,17 +1336,54 @@ public:
     copy_to_operands(op0, op1);
   }
 
-  or_exprt(const exprt::operandst &op):exprt(ID_or, bool_typet())
+  inline or_exprt(const exprt &op0, const exprt &op1, const exprt &op2):exprt(ID_or, typet(ID_bool))
   {
-    if(op.empty())
-      make_false();
-    else if(op.size()==1)
-      *this=static_cast<const or_exprt &>(op.front());
-    else
-      operands()=op;
+    copy_to_operands(op0, op1, op2);
+  }
+
+  inline or_exprt(const exprt &op0, const exprt &op1, const exprt &op2, const exprt &op3):exprt(ID_or, typet(ID_bool))
+  {
+    exprt::operandst &op=operands();
+    op.resize(4);
+    op[0]=op0;
+    op[1]=op1;
+    op[2]=op2;
+    op[3]=op3;
   }
 };
 
+/*! 1) generates a disjunction for two or more operands
+ *  2) for one operand, returns the operand
+ *  3) returns false otherwise 
+*/
+
+exprt disjunction(const exprt::operandst &op);
+
+/*! \brief Cast a generic exprt to a \ref or_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * or_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref or_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const or_exprt &to_or_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_or);
+  return static_cast<const or_exprt &>(expr);
+}
+
+/*! \copydoc to_or_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline or_exprt &to_or_expr(exprt &expr)
+{
+  assert(expr.id()==ID_or);
+  return static_cast<or_exprt &>(expr);
+}
+
 /*! \brief Bit-wise negation of bit-vectors
 */
 class bitnot_exprt:public unary_exprt
@@ -1195,6 +1415,31 @@ public:
   }
 };
 
+/*! \brief Cast a generic exprt to a \ref bitor_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * bitor_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref bitor_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const bitor_exprt &to_bitor_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_bitor);
+  return static_cast<const bitor_exprt &>(expr);
+}
+
+/*! \copydoc to_bitor_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline bitor_exprt &to_bitor_expr(exprt &expr)
+{
+  assert(expr.id()==ID_bitor);
+  return static_cast<bitor_exprt &>(expr);
+}
+
 /*! \brief Bit-wise XOR
 */
 class bitxor_exprt:public exprt
@@ -1211,6 +1456,31 @@ public:
   }
 };
 
+/*! \brief Cast a generic exprt to a \ref bitxor_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * bitxor_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref bitxor_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const bitxor_exprt &to_bitxor_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_bitxor);
+  return static_cast<const bitxor_exprt &>(expr);
+}
+
+/*! \copydoc to_bitxor_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline bitxor_exprt &to_bitxor_expr(exprt &expr)
+{
+  assert(expr.id()==ID_bitxor);
+  return static_cast<bitxor_exprt &>(expr);
+}
+
 /*! \brief Bit-wise AND
 */
 class bitand_exprt:public exprt
@@ -1227,6 +1497,31 @@ public:
   }
 };
 
+/*! \brief Cast a generic exprt to a \ref bitand_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * bitand_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref bitand_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const bitand_exprt &to_bitand_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_bitand);
+  return static_cast<const bitand_exprt &>(expr);
+}
+
+/*! \copydoc to_bitand_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline bitand_exprt &to_bitand_expr(exprt &expr)
+{
+  assert(expr.id()==ID_bitand);
+  return static_cast<bitand_exprt &>(expr);
+}
+
 /*! \brief A base class for shift operators
 */
 class shift_exprt:public binary_exprt
@@ -1266,6 +1561,31 @@ public:
   }
 };
 
+/*! \brief Cast a generic exprt to a \ref shift_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * shift_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref shift_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const shift_exprt &to_shift_expr(const exprt &expr)
+{
+  assert(expr.operands().size()==2);
+  return static_cast<const shift_exprt &>(expr);
+}
+
+/*! \copydoc to_shift_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline shift_exprt &to_shift_expr(exprt &expr)
+{
+  assert(expr.operands().size()==2);
+  return static_cast<shift_exprt &>(expr);
+}
+
 /*! \brief Left shift
 */
 class shl_exprt:public shift_exprt
@@ -1453,9 +1773,8 @@ class address_of_exprt:public exprt
 {
 public:
   explicit address_of_exprt(const exprt &op):
-    exprt(ID_address_of, pointer_typet())
+    exprt(ID_address_of, pointer_typet(op.type()))
   {
-    type().subtype()=op.type();
     copy_to_operands(op);
   }
 
@@ -1515,8 +1834,43 @@ public:
   {
     operands().resize(1);
   }
+  
+  inline exprt &op()
+  {
+    return op0();
+  }
+
+  inline const exprt &op() const
+  {
+    return op0();
+  }
 };
 
+/*! \brief Cast a generic exprt to an \ref not_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * not_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref not_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const not_exprt &to_not_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_not && expr.operands().size()==1);
+  return static_cast<const not_exprt &>(expr);
+}
+
+/*! \copydoc to_not_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline not_exprt &to_not_expr(exprt &expr)
+{
+  assert(expr.id()==ID_not && expr.operands().size()==1);
+  return static_cast<not_exprt &>(expr);
+}
+
 /*! \brief Operator to dereference a pointer
 */
 class dereference_exprt:public exprt
@@ -1733,6 +2087,123 @@ extern inline with_exprt &to_with_expr(exprt &expr)
   return static_cast<with_exprt &>(expr);
 }
 
+class index_designatort:public exprt
+{
+public:
+  inline explicit index_designatort(const exprt &_index):
+    exprt(ID_index_designator)
+  {
+    copy_to_operands(_index);
+  }
+  
+  inline const exprt &index() const
+  {
+    return op0();
+  }
+
+  inline exprt &index()
+  {
+    return op0();
+  }
+};
+
+class member_designatort:public exprt
+{
+public:
+  inline explicit member_designatort(const irep_idt &_component_name):
+    exprt(ID_member_designator)
+  {
+    set(ID_component_name, _component_name);
+  }
+
+  irep_idt get_component_name() const
+  {
+    return get(ID_component_name);
+  }
+};
+
+/*! \brief Operator to update elements in structs and arrays
+*/
+class update_exprt:public exprt
+{
+public:
+  update_exprt(
+    const exprt &_old,
+    const exprt &_designator,
+    const exprt &_new_value):
+    exprt(ID_update, _old.type())
+  {
+    copy_to_operands(_old, _designator, _new_value);
+  }
+
+  inline explicit update_exprt(const typet _type):
+    exprt(ID_update, _type)
+  {
+    operands().resize(3);
+  }
+
+  inline update_exprt():exprt(ID_update)
+  {
+    operands().resize(3);
+    op1().id(ID_designator);
+  }
+  
+  inline exprt &old()
+  {
+    return op0();
+  }
+
+  inline const exprt &old() const
+  {
+    return op0();
+  }
+
+  inline exprt::operandst &designator()
+  {
+    return op1().operands();
+  }
+
+  inline const exprt::operandst &designator() const
+  {
+    return op1().operands();
+  }
+
+  inline exprt &new_value()
+  {
+    return op2();
+  }
+
+  inline const exprt &new_value() const
+  {
+    return op2();
+  }
+};
+
+/*! \brief Cast a generic exprt to an \ref update_exprt
+ *
+ * This is an unchecked conversion. \a expr must be known to be \ref
+ * update_exprt.
+ *
+ * \param expr Source expression
+ * \return Object of type \ref with_exprt
+ *
+ * \ingroup gr_std_expr
+*/
+extern inline const update_exprt &to_update_expr(const exprt &expr)
+{
+  assert(expr.id()==ID_update && expr.operands().size()==3);
+  return static_cast<const update_exprt &>(expr);
+}
+
+/*! \copydoc to_update_expr(const exprt &)
+ * \ingroup gr_std_expr
+*/
+extern inline update_exprt &to_update_expr(exprt &expr)
+{
+  assert(expr.id()==ID_update && expr.operands().size()==3);
+  return static_cast<update_exprt &>(expr);
+}
+
 /*! \brief update of one element of an array
 */
 class array_update_exprt:public exprt
@@ -1954,6 +2425,12 @@ public:
   {
   }
   
+  inline constant_exprt(const irep_idt &_value, const typet &_type):
+    exprt(ID_constant, _type)
+  {
+    set_value(_value);
+  }
+
   inline const irep_idt &get_value() const
   {
     return get(ID_value);
@@ -1963,6 +2440,8 @@ public:
   {
     set(ID_value, value);
   }
+
+  bool value_is_zero_string() const;
 };
 
 /*! \brief Cast a generic exprt to a \ref constant_exprt
@@ -2197,4 +2676,15 @@ public:
   }
 };
 
+/*! \brief An expression denoting infinity
+*/
+class infinity_exprt:public exprt
+{
+public:
+  inline explicit infinity_exprt(const typet &_type):
+    exprt(ID_infinity, _type)
+  {
+  }
+};
+
 #endif
