@@ -18,8 +18,8 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include <cassert>
 
-#include <expr.h>
-#include <mp_arith.h>
+#include "expr.h"
+#include "mp_arith.h"
 
 class constant_exprt;
 
@@ -57,6 +57,12 @@ public:
   }
 };
 
+/*! \brief The void type
+*/
+class void_typet:public empty_typet
+{
+};
+
 /*! \brief Unbounded, signed integers
 */
 class integer_typet:public typet
@@ -252,6 +258,9 @@ public:
 
   unsigned component_number(const irep_idt &component_name) const;
   typet component_type(const irep_idt &component_name) const;
+  
+  irep_idt get_tag() const { return get(ID_tag); }
+  void set_tag(const irep_idt &tag) { set(ID_tag, tag); }
 };
 
 /*! \brief Cast a generic typet to a \ref struct_union_typet
@@ -267,8 +276,7 @@ public:
 extern inline const struct_union_typet &to_struct_union_type(const typet &type)
 {
   assert(type.id()==ID_struct ||
-         type.id()==ID_union ||
-         type.id()==ID_class);
+         type.id()==ID_union);
   return static_cast<const struct_union_typet &>(type);
 }
 
@@ -278,8 +286,7 @@ extern inline const struct_union_typet &to_struct_union_type(const typet &type)
 extern inline struct_union_typet &to_struct_union_type(typet &type)
 {
   assert(type.id()==ID_struct ||
-         type.id()==ID_union ||
-         type.id()==ID_class);
+         type.id()==ID_union);
   return static_cast<struct_union_typet &>(type);
 }
 
@@ -288,11 +295,11 @@ extern inline struct_union_typet &to_struct_union_type(typet &type)
 class struct_typet:public struct_union_typet
 {
 public:
-  struct_typet():struct_union_typet(ID_struct)
+  inline struct_typet():struct_union_typet(ID_struct)
   {
   }
 
-  /// returns true if the object is a prefix of \a other    
+  // returns true if the object is a prefix of \a other    
   bool is_prefix_of(const struct_typet &other) const;
 };
 
@@ -308,9 +315,7 @@ public:
 */
 extern inline const struct_typet &to_struct_type(const typet &type)
 {
-  assert(type.id()==ID_struct ||
-         type.id()==ID_union ||
-         type.id()==ID_class);
+  assert(type.id()==ID_struct);
   return static_cast<const struct_typet &>(type);
 }
 
@@ -319,18 +324,95 @@ extern inline const struct_typet &to_struct_type(const typet &type)
 */
 extern inline struct_typet &to_struct_type(typet &type)
 {
-  assert(type.id()==ID_struct ||
-         type.id()==ID_union ||
-         type.id()==ID_class);
+  assert(type.id()==ID_struct);
   return static_cast<struct_typet &>(type);
 }
 
+/*! \brief C++ class type
+*/
+class class_typet:public struct_typet
+{
+public:
+  inline class_typet():struct_typet()
+  {
+    set(ID_C_class, true);
+  }
+  
+  typedef componentt methodt;
+  typedef componentst methodst;
+
+  inline const methodst &methods() const
+  {
+    return (const methodst &)(find(ID_methods).get_sub());
+  }
+  
+  inline componentst &methods()
+  {
+    return (methodst &)(add(ID_methods).get_sub());
+  }
+
+  inline bool is_class() const
+  {
+    return get_bool(ID_C_class);    
+  }
+  
+  inline irep_idt default_access() const
+  {
+    return is_class()?ID_private:ID_public;
+  }
+
+  inline const irept::subt &bases() const  
+  {
+    return find(ID_bases).get_sub();
+  }
+  
+  bool has_base(const irep_idt &id) const
+  {
+    const irept::subt &b=bases();
+    forall_irep(it, b)
+    {
+      assert(it->id()==ID_base);
+      const irept &type=it->find(ID_type);
+      assert(type.id()==ID_symbol);
+      if(type.get(ID_identifier)==id) return true;
+    }
+    
+    return false;
+  }
+
+};
+
+/*! \brief Cast a generic typet to a \ref class_typet
+ *
+ * This is an unchecked conversion. \a type must be known to be \ref
+ * class_typet.
+ *
+ * \param type Source type
+ * \return Object of type \ref class_typet
+ *
+ * \ingroup gr_std_types
+*/
+extern inline const class_typet &to_class_type(const typet &type)
+{
+  assert(type.id()==ID_struct);
+  return static_cast<const class_typet &>(type);
+}
+
+/*! \copydoc to_class_type(const typet &)
+ * \ingroup gr_std_types
+*/
+extern inline class_typet &to_class_type(typet &type)
+{
+  assert(type.id()==ID_struct);
+  return static_cast<class_typet &>(type);
+}
+
 /*! \brief The union type
 */
 class union_typet:public struct_union_typet
 {
 public:
-  union_typet():struct_union_typet(ID_union)
+  inline union_typet():struct_union_typet(ID_union)
   {
   }
 };
@@ -369,14 +451,16 @@ public:
   {
   }
   
-  class argumentt:public exprt
+  // will rename arguments -> parameters to use standard terminology
+
+  class parametert:public exprt
   {
   public:
-    inline argumentt():exprt(ID_argument)
+    inline parametert():exprt(ID_argument)
     {
     }
     
-    explicit inline argumentt(const typet &type):exprt(ID_argument, type)
+    explicit inline parametert(const typet &type):exprt(ID_argument, type)
     {
     }
     
@@ -416,6 +500,8 @@ public:
     }
   };
   
+  typedef parametert argumentt;
+  
   inline bool has_ellipsis() const
   {
     return find(ID_arguments).get_bool(ID_ellipsis);
@@ -431,7 +517,8 @@ public:
     add(ID_arguments).set(ID_ellipsis, true);
   }
 
-  typedef std::vector<argumentt> argumentst;
+  typedef std::vector<parametert> argumentst;
+  typedef std::vector<parametert> parameterst;
 
   inline const typet &return_type() const
   {
@@ -442,15 +529,25 @@ public:
   {
     return add_type(ID_return_type);
   }
-
-  inline const argumentst &arguments() const
+  
+  inline const parameterst &arguments() const
   {
-    return (const argumentst &)find(ID_arguments).get_sub();
+    return (const parameterst &)find(ID_arguments).get_sub();
   }
 
-  inline argumentst &arguments()
+  inline parameterst &arguments()
   {
-    return (argumentst &)add(ID_arguments).get_sub();
+    return (parameterst &)add(ID_arguments).get_sub();
+  }
+  
+  inline const parameterst &parameters() const
+  {
+    return (const parameterst &)find(ID_arguments).get_sub();
+  }
+
+  inline parameterst &parameters()
+  {
+    return (parameterst &)add(ID_arguments).get_sub();
   }
   
   inline bool get_inlined() const
@@ -462,6 +559,18 @@ public:
   {
     set(ID_C_inlined, value);
   }
+  
+  // this produces the list of parameter identifiers
+  std::vector<irep_idt> parameter_identifiers() const
+  {
+    std::vector<irep_idt> result;
+    const parameterst &p=parameters();
+    result.reserve(p.size());
+    for(parameterst::const_iterator it=p.begin();
+        it!=p.end(); it++)
+      result.push_back(it->get_identifier());
+    return result;
+  }
 };
 
 /*! \brief Cast a generic typet to a \ref code_typet
@@ -499,10 +608,9 @@ public:
   }
   
   inline array_typet(const typet &_subtype,
-                     const exprt &_size):typet(ID_array)
+                     const exprt &_size):typet(ID_array, _subtype)
   {
     size()=_size;
-    subtype()=_subtype;
   }
   
   inline const exprt &size() const
@@ -561,6 +669,11 @@ public:
   {
   }
 
+  inline bitvector_typet(const irep_idt &_id, const typet &_subtype):
+    typet(_id, _subtype)
+  {
+  }
+
   unsigned get_width() const;
 
   inline void set_width(unsigned width)
@@ -589,11 +702,11 @@ inline const bitvector_typet &to_bitvector_type(const typet &type)
 class bv_typet:public bitvector_typet
 {
 public:
-  bv_typet():bitvector_typet(ID_bv)
+  inline bv_typet():bitvector_typet(ID_bv)
   {
   }
 
-  explicit bv_typet(unsigned width):bitvector_typet(ID_bv)
+  inline explicit bv_typet(unsigned width):bitvector_typet(ID_bv)
   {
     set_width(width);
   }
@@ -629,11 +742,11 @@ inline bv_typet &to_bv_type(typet &type)
 class unsignedbv_typet:public bitvector_typet
 {
 public:
-  unsignedbv_typet():bitvector_typet(ID_unsignedbv)
+  inline unsignedbv_typet():bitvector_typet(ID_unsignedbv)
   {
   }
 
-  explicit unsignedbv_typet(unsigned width):bitvector_typet(ID_unsignedbv)
+  inline explicit unsignedbv_typet(unsigned width):bitvector_typet(ID_unsignedbv)
   {
     set_width(width);
   }
@@ -674,11 +787,11 @@ inline unsignedbv_typet &to_unsignedbv_type(typet &type)
 class signedbv_typet:public bitvector_typet
 {
 public:
-  signedbv_typet():bitvector_typet(ID_signedbv)
+  inline signedbv_typet():bitvector_typet(ID_signedbv)
   {
   }
 
-  explicit signedbv_typet(unsigned width):bitvector_typet(ID_signedbv)
+  inline explicit signedbv_typet(unsigned width):bitvector_typet(ID_signedbv)
   {
     set_width(width);
   }
@@ -719,23 +832,23 @@ inline signedbv_typet &to_signedbv_type(typet &type)
 class fixedbv_typet:public bitvector_typet
 {
 public:
-  fixedbv_typet():bitvector_typet(ID_fixedbv)
+  inline fixedbv_typet():bitvector_typet(ID_fixedbv)
   {
   }
 
-  unsigned get_fraction_bits() const
+  inline unsigned get_fraction_bits() const
   {
     return get_width()-get_integer_bits();
   }
 
   unsigned get_integer_bits() const;
 
-  void set_integer_bits(unsigned b)
+  inline void set_integer_bits(unsigned b)
   {
     set(ID_integer_bits, b);
   }
 
-  friend const fixedbv_typet &to_fixedbv_type(const typet &type)
+  inline friend const fixedbv_typet &to_fixedbv_type(const typet &type)
   {
     assert(type.id()==ID_fixedbv);
     return static_cast<const fixedbv_typet &>(type);
@@ -759,11 +872,11 @@ const fixedbv_typet &to_fixedbv_type(const typet &type);
 class floatbv_typet:public bitvector_typet
 {
 public:
-  floatbv_typet():bitvector_typet(ID_floatbv)
+  inline floatbv_typet():bitvector_typet(ID_floatbv)
   {
   }
 
-  unsigned get_e() const
+  inline unsigned get_e() const
   {
     // subtract one for sign bit
     return get_width()-get_f()-1;
@@ -771,12 +884,12 @@ public:
 
   unsigned get_f() const;
 
-  void set_f(unsigned b)
+  inline void set_f(unsigned b)
   {
     set(ID_f, b);
   }
 
-  friend const floatbv_typet &to_floatbv_type(const typet &type)
+  inline friend const floatbv_typet &to_floatbv_type(const typet &type)
   {
     assert(type.id()==ID_floatbv);
     return static_cast<const floatbv_typet &>(type);
@@ -804,15 +917,14 @@ public:
   {
   }
 
-  inline explicit pointer_typet(const typet &_subtype):bitvector_typet(ID_pointer)
+  inline explicit pointer_typet(const typet &_subtype):
+    bitvector_typet(ID_pointer, _subtype)
   {
-    subtype()=_subtype;
   }
 
   inline explicit pointer_typet(const typet &_subtype, unsigned width):
-    bitvector_typet(ID_pointer)
+    bitvector_typet(ID_pointer, _subtype)
   {
-    subtype()=_subtype;
     set_width(width);
   }
   
@@ -852,10 +964,14 @@ extern inline pointer_typet &to_pointer_type(typet &type)
 class reference_typet:public pointer_typet
 {
 public:
-  reference_typet()
+  inline reference_typet()
   {
     set(ID_C_reference, true);
   }
+
+  inline explicit reference_typet(const typet &_subtype):pointer_typet(_subtype)
+  {
+  }
 };
 
 /*! \brief TO_BE_DOCUMENTED
@@ -870,11 +986,11 @@ bool is_rvalue_reference(const typet &type);
 class string_typet:public typet
 {
 public:
-  string_typet():typet(ID_string)
+  inline string_typet():typet(ID_string)
   {
   }
 
-  friend const string_typet &to_string_type(const typet &type)
+  inline friend const string_typet &to_string_type(const typet &type)
   {
     assert(type.id()==ID_string);
     return static_cast<const string_typet &>(type);
@@ -898,17 +1014,17 @@ const string_typet &to_string_type(const typet &type);
 class range_typet:public typet
 {
 public:
-  range_typet():typet(ID_range)
+  inline range_typet():typet(ID_range)
   {
   }
   
-  range_typet(const mp_integer &_from, const mp_integer &_to)
+  inline range_typet(const mp_integer &_from, const mp_integer &_to)
   {
     set_from(_from);
     set_to(_to);
   }
 
-  friend const range_typet &to_range_type(const typet &type)
+  inline friend const range_typet &to_range_type(const typet &type)
   {
     assert(type.id()==ID_range);
     return static_cast<const range_typet &>(type);
@@ -938,23 +1054,22 @@ const range_typet &to_range_type(const typet &type);
 class vector_typet:public typet
 {
 public:
-  vector_typet():typet(ID_vector)
+  inline vector_typet():typet(ID_vector)
   {
   }
   
-  vector_typet(const typet &_subtype,
-               const exprt &_size):typet(ID_vector)
+  inline vector_typet(const typet &_subtype,
+                      const exprt &_size):typet(ID_vector, _subtype)
   {
     size()=_size;
-    subtype()=_subtype;
   }
   
-  const exprt &size() const
+  inline const exprt &size() const
   {
     return static_cast<const exprt &>(find(ID_size));
   }
   
-  exprt &size()
+  inline exprt &size()
   {
     return static_cast<exprt &>(add(ID_size));
   }
@@ -991,13 +1106,12 @@ extern inline vector_typet &to_vector_type(typet &type)
 class complex_typet:public typet
 {
 public:
-  complex_typet():typet(ID_complex)
+  inline complex_typet():typet(ID_complex)
   {
   }
   
-  complex_typet(const typet &_subtype):typet(ID_complex)
+  inline complex_typet(const typet &_subtype):typet(ID_complex, _subtype)
   {
-    subtype()=_subtype;
   }
 };
 
