@@ -14,88 +14,108 @@ Author: Daniel Kroening, kroening@kroening.com
 
 #include "numbering.h"
 
+// Standard union find with weighting and path compression.
+// See http://www.cs.princeton.edu/~rs/AlgsDS07/01UnionFind.pdf
+
 class unsigned_union_find
 {
 public:
+  // merge the sets 'a' and 'b'
   void make_union(unsigned a, unsigned b);
+
+  // find the root of the set 'a' belongs to
   unsigned find(unsigned a) const;  
 
+  // Makes 'this' the union-find with the following:
+  // any union in 'this' will be present in both source sets,
+  // i.e., this is the strongest implication of the two
+  // data structures.
   void intersection(const unsigned_union_find &other);
 
   // remove from any sets
   void isolate(unsigned a);
 
-  void swap(unsigned_union_find &other)
+  inline void swap(unsigned_union_find &other)
   {
     other.nodes.swap(nodes);
   }
 
-  void resize(unsigned size)
+  inline void resize(unsigned size)
   {
-    // only enlarge
+    // We only enlarge. Shrinking is yet to be implemented.
     assert(nodes.size()<=size);
-    nodes.resize(size);
+    nodes.reserve(size);
+    while(nodes.size()<size)
+      nodes.push_back(nodet(nodes.size()));
   }
   
-  void clear()
+  inline void clear()
   {
     nodes.clear();
   }
-  
-  bool is_root(unsigned a) const
+
+  // is 'a' a root?  
+  inline bool is_root(unsigned a) const
   {
     if(a>=size()) return true;
-    return nodes[a].root;
+    // a root is its own parent
+    return nodes[a].parent==a;
   }
-  
-  bool same_set(unsigned a, unsigned b) const
+
+  // are 'a' and 'b' in the same set?
+  inline bool same_set(unsigned a, unsigned b) const
   {
     return find(a)==find(b);
   }
-  
-  unsigned size() const
+
+  // total number of elements  
+  inline unsigned size() const
   {
     return nodes.size();
   }
-  
-  unsigned count(unsigned a) const
+
+  // size of the set that 'a' is in  
+  inline unsigned count(unsigned a) const
   {
     if(a>=size()) return 1;
     return nodes[find(a)].count;
   }
-  
-  void check_index(unsigned a)
+
+  // make the array large enough to contain 'a'  
+  inline void check_index(unsigned a)
   {
     if(a>=size()) resize(a+1);
   }
-  
+
+  // number of disjoint sets  
   unsigned count_roots() const
   {
     unsigned c=0;
     for(unsigned i=0; i<nodes.size(); i++)
-      if(nodes[i].root) c++;
+      if(is_root(i)) c++;
     return c;
   }
-  
+
+  // makes 'new_root' the root of the set 'old'  
   void re_root(unsigned old, unsigned new_root);
-  
+
+  // find a different member of the same set
   unsigned get_other(unsigned a);
   
 protected:  
   struct nodet
   {
-    // these two could be shared
-    unsigned count;
+    unsigned count; // set size
     unsigned parent;
 
-    bool root;
-    
-    nodet():count(1), parent(0), root(true)
+    // constructs a root node        
+    explicit nodet(unsigned index):count(1), parent(index)
     {
     }
   };
 
-  std::vector<nodet> nodes;
+  // This is mutable to allow path compression in find().
+  mutable std::vector<nodet> nodes;
 };
 
 template <typename T>
@@ -111,32 +131,39 @@ public:
     return is_union;
   }
   
-  const T &find(const T &a)
+  // are 'a' and 'b' in the same set?
+  inline bool same_set(const T &a, const T &b)
+  {
+    unsigned na=number(a), nb=number(b);
+    return uuf.same_set(na, nb);
+  }
+
+  inline const T &find(const T &a)
   {
     return find(number(a));
   }
   
-  unsigned find_number(unsigned a) const
+  inline unsigned find_number(unsigned a) const
   {
     return uuf.find(a);
   }
 
-  unsigned find_number(const T &a)
+  inline unsigned find_number(const T &a)
   {
     return uuf.find(number(a));
   }
   
-  bool is_root_number(unsigned a) const
+  inline bool is_root_number(unsigned a) const
   {
     return uuf.is_root(a);
   }
 
-  bool is_root(const T &a)
+  inline bool is_root(const T &a)
   {
     return is_root(number(a));
   }
 
-  unsigned number(const T &a)
+  inline unsigned number(const T &a)
   {
     unsigned n=subt::number(a);
   
@@ -147,7 +174,13 @@ public:
     
     return n;
   }
-  
+
+  inline void clear()
+  {
+    subt::clear();
+    uuf.clear();
+  }  
+
 protected:
   unsigned_union_find uuf;
   typedef numbering<T> subt;
